SystemOrganization addCategory: #'XML-Tests-Parser'!
SystemOrganization addCategory: 'XML-Tests-Parser-DOM'!
SystemOrganization addCategory: 'XML-Tests-Parser-Factories'!
SystemOrganization addCategory: 'XML-Tests-Parser-Namespaces'!
SystemOrganization addCategory: 'XML-Tests-Parser-Streams'!
SystemOrganization addCategory: 'XML-Tests-Parser-URI'!
SystemOrganization addCategory: 'XML-Tests-Parser-Utils'!

XMLDocument subclass: #XMLFDocument
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLFDocument commentStamp: '<historical>' prior: 0!
A dummy subclass of XMLDocument!

TestCase subclass: #XMLCharacterMethodsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLCharacterMethodsTest methodsFor: 'tests' stamp: 'monty 3/12/2016 19:13'!
testIsXMLSeparator
	| separators |

	separators := Character xmlSeparators.
	0 to: 255 do: [:each | | char |
		char := each asCharacter.
		(separators includes: char)
			ifTrue: [self assert: char isXMLSeparator]
			ifFalse: [self deny: char isXMLSeparator]].
	self
		assert: (nil respondsTo: #isXMLSeparator);
		deny: nil isXMLSeparator.! !

!XMLCharacterMethodsTest methodsFor: 'tests' stamp: 'monty 3/12/2016 19:13'!
testIsXMLWhitespace
	| whitespace |

	whitespace := Character xmlWhitespace.
	0 to: 255 do: [:each | | char |
		char := each asCharacter.
		(whitespace includes: char)
			ifTrue: [self assert: char isXMLWhitespace]
			ifFalse: [self deny: char isXMLWhitespace]].
	self
		assert: (nil respondsTo: #isXMLWhitespace);
		deny: nil isXMLWhitespace.! !

!XMLCharacterMethodsTest methodsFor: 'tests' stamp: 'monty 3/14/2016 00:25'!
testXMLSplit
	self
		assert: ($: xmlSplit: '') = (OrderedCollection with: '');
		assert: ($: xmlSplit: 'abc') = (OrderedCollection with: 'abc');
		assert: ($: xmlSplit: ':') = #('' '') asOrderedCollection;
		assert: ($: xmlSplit: '::') = #('' '' '') asOrderedCollection;
		assert: ($: xmlSplit: 'a:bc:') = #('a' 'bc' '') asOrderedCollection;
		assert: ($: xmlSplit: ':ab:c:') = #('' 'ab' 'c' '') asOrderedCollection;
		assert: ($: xmlSplit: 'a:bc:') = #('a' 'bc' '') asOrderedCollection;
		assert: ($: xmlSplit: 'ab::cd::ef') = #('ab'  '' 'cd' '' 'ef') asOrderedCollection;
		assert:
			($: xmlSplit: '::ab:::cd:::ef::') =
				#('' '' 'ab'  '' '' 'cd' '' '' 'ef' '' '') asOrderedCollection! !

TestCase subclass: #XMLDFALimitsTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLDFALimitsTests methodsFor: 'tests' stamp: 'monty 11/14/2015 07:22'!
testDFAStateCacheLimit
	| names contentModel firstState currentState visitedStates |

	names :=
		(1 to: XMLDFAStateCache new maxCachedDFAStates) collect: [:each |
			'test', each printString].

	contentModel := String streamContents: [:stream |
		stream nextPut: $(.
		names withIndexDo: [:each :i |
			i > 1
				ifTrue: [stream nextPut: $,].
			stream nextPutAll: each].
		stream nextPut: $)].

	firstState :=
		(DTDDFAContentModelParser
			element: 'test'
			contentModel: contentModel) parse dfa.
	currentState := firstState.
	visitedStates := Set new.
	names withIndexDo: [:name :i | | previousState |
		visitedStates add: currentState.
		previousState := currentState.
		currentState := currentState nextDFAStateMatching: name.
		self assert: currentState notNil.
		i = names size
			ifTrue: [
				visitedStates do: [:each |
					"the cache clearing was triggered by the last match, which
					added one transition from previousState to currentState,
					but other transitions should be cleared"
					each = previousState
						ifTrue: [self assert: each dfaStateTransitions size = 1]
						ifFalse: [self assert: each dfaStateTransitions isEmpty]].
				self
					assert: firstState dfaStateTransitions isEmpty;
					assert: firstState dfaStateCache cachedDFAStates size = 2;
					assert: currentState isAccepting]
			ifFalse: [
				visitedStates do: [:each | self deny: each dfaStateTransitions isEmpty].
				self
					assert: firstState dfaStateTransitions notEmpty;
					assert: firstState dfaStateCache cachedDFAStates size = (i + 1);
					deny: currentState isAccepting]].

	currentState := firstState.
	names withIndexDo: [:each :i |
		currentState := currentState nextDFAStateMatching: each.
		self deny: currentState isNil.
		i = names size
			ifTrue: [self assert: currentState isAccepting]
			ifFalse: [self deny: currentState isAccepting]].! !

!XMLDFALimitsTests methodsFor: 'tests' stamp: 'monty 11/10/2015 22:26'!
testDFAStateTransitionLimit
	| dfaState elementName |

	dfaState :=
		(DTDDFAContentModelParser
			element: 'test'
			contentModel: '(test)') parse dfa.
	1 to: dfaState maxDFAStateTransitions do: [:i |
		elementName := 'test', i printString.
		dfaState nextDFAStateMatching: elementName.
		self
			assert: (dfaState dfaStateTransitions includesKey: elementName);
			assert: dfaState dfaStateTransitions size = i].

	elementName := 'test', (dfaState maxDFAStateTransitions + 1) printString.
	dfaState nextDFAStateMatching: elementName.
	self
		assert: (dfaState dfaStateTransitions includesKey: elementName);
		assert: dfaState dfaStateTransitions size = 1.! !

TestCase subclass: #XMLEncodingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLEncodingTest methodsFor: 'converting' stamp: 'monty 2/12/2016 05:58'!
asciiToUTF16BE: anASCIIString
	^ XMLStreamAdapterTest asciiToUTF16BE: anASCIIString! !

!XMLEncodingTest methodsFor: 'converting' stamp: 'monty 2/12/2016 05:58'!
asciiToUTF16LE: anASCIIString
	^ XMLStreamAdapterTest asciiToUTF16LE: anASCIIString! !

!XMLEncodingTest methodsFor: 'converting' stamp: 'monty 2/12/2016 05:58'!
asciiToUTF32BE: anASCIIString
	^ XMLStreamAdapterTest asciiToUTF32BE: anASCIIString! !

!XMLEncodingTest methodsFor: 'converting' stamp: 'monty 2/12/2016 05:58'!
asciiToUTF32LE: anASCIIString
	^ XMLStreamAdapterTest asciiToUTF32LE: anASCIIString! !

!XMLEncodingTest methodsFor: 'converting' stamp: 'monty 12/13/2014 20:16'!
convertableStreamOn: aString
	^ aString readStream! !

!XMLEncodingTest methodsFor: 'source' stamp: 'monty 11/8/2014 19:48'!
encodingDeclaration: anEncoding
	^ '<?xml version="1.0" encoding="', anEncoding, '"?>'! !

!XMLEncodingTest methodsFor: 'source' stamp: 'monty 2/11/2016 02:18'!
sourceContent
	^ 'encoding test'! !

!XMLEncodingTest methodsFor: 'source' stamp: 'monty 2/11/2016 02:18'!
sourceXML
	^ self sourceXMLWithContent: self sourceContent! !

!XMLEncodingTest methodsFor: 'source' stamp: 'monty 2/10/2016 23:39'!
sourceXMLWithContent: aCharacterOrString
	^ '<encoding-test>', aCharacterOrString asString, '</encoding-test>'! !

!XMLEncodingTest methodsFor: 'source' stamp: 'monty 2/12/2016 00:20'!
sourceXMLWithEncoding: anEncoding andASCIIContentAsUTF16BE: anASCIIContentString
	^ ((self encodingDeclaration: anEncoding) allButLast: 2),
		(self asciiToUTF16BE: '?>', anASCIIContentString)! !

!XMLEncodingTest methodsFor: 'source' stamp: 'monty 2/12/2016 00:20'!
sourceXMLWithEncoding: anEncoding andASCIIContentAsUTF16LE: anASCIIContentString
	^ ((self encodingDeclaration: anEncoding) allButLast: 2),
		(self asciiToUTF16LE: '?>', anASCIIContentString)! !

!XMLEncodingTest methodsFor: 'source' stamp: 'monty 2/12/2016 00:20'!
sourceXMLWithEncoding: anEncoding andASCIIContentAsUTF32BE: anASCIIContentString
	^ ((self encodingDeclaration: anEncoding) allButLast: 2),
		(self asciiToUTF32BE: '?>', anASCIIContentString)! !

!XMLEncodingTest methodsFor: 'source' stamp: 'monty 2/12/2016 00:20'!
sourceXMLWithEncoding: anEncoding andASCIIContentAsUTF32LE: anASCIIContentString
	^ ((self encodingDeclaration: anEncoding) allButLast: 2),
		(self asciiToUTF32LE: '?>', anASCIIContentString)! !

!XMLEncodingTest methodsFor: 'converting' stamp: 'monty 2/14/2016 04:02'!
stringFrom: aCollection
	^ XMLStreamAdapterTest stringFrom: aCollection! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/13/2016 04:15'!
testDecodingBinary
	| source binarySource |
	
	source := self sourceXML.
	binarySource := source asArray collect: [:each | each asciiValue].
	self assert:
		(XMLDOMParser parse: binarySource) printString =
			(XMLDOMParser parse: source) printString.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/11/2016 02:48'!
testDecodingCharacterReferences
	| document |

	document :=
		XMLDOMParser parse:
			(self sourceXMLWithContent:
				(String streamContents: [:stream |
					self sourceContent do: [:each |
						stream
							nextPutAll: '&#';
							nextPutAll: each asciiValue asString;
							nextPut: $;]])).
	self assert: document root contentString = self sourceContent.

	document :=
		XMLDOMParser parse:
			(self sourceXMLWithContent:
				(String streamContents: [:stream |
					self sourceContent do: [:each |
						stream
							nextPutAll: '&#x';
							nextPutAll: each asciiValue printStringHex asLowercase;
							nextPut: $;]])).
	self assert: document root contentString = self sourceContent.

	document :=
		XMLDOMParser parse:
			(self sourceXMLWithContent:
				(String streamContents: [:stream |
					self sourceContent do: [:each |
						stream
							nextPutAll: '&#x';
							nextPutAll: each asciiValue printStringHex asUppercase;
							nextPut: $;]])).
	self assert: document root contentString = self sourceContent.

	#('&#0;' '&#-1;' '&#9:;' '&#xFFG;') do: [:each |
		self
			should: [XMLDOMParser parse: (self sourceXMLWithContent: each)]
			raise: XMLWellFormednessException].! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:22'!
testDecodingDisabledEncodingDeclaration
	| xml |

	xml := 
		self
			sourceXMLWithEncoding: 'UTF-16-BE'
			andASCIIContentAsUTF16BE: self sourceXML.
	self
		should: [
			(SAXHandler on: (self convertableStreamOn: xml))
				decodesCharacters: false;
				parseDocument]
		raise: XMLParserException.
	self
		shouldnt: [
			(SAXHandler on: (self convertableStreamOn: xml))
				decodesCharacters: true;
				parseDocument]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:23'!
testDecodingDisabledEncodingDetection
	| xml |

	xml := self asciiToUTF16LE: self sourceXML.
	self
		should: [
			(SAXHandler on: (self convertableStreamOn: xml))
				decodesCharacters: false;
				parseDocument]
		raise: XMLParserException.
	self
		shouldnt: [
			(SAXHandler on: (self convertableStreamOn: xml))
				decodesCharacters: true;
				parseDocument]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/10/2016 23:40'!
testEncodingDeclarationLatin1
	| xml |

	xml := self sourceXMLWithContent: (self stringFrom: #(126 127 128 255)).
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf8BOM, (self encodingDeclaration: 'UTF-8'), xml)]
		raise: XMLEncodingException.
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf8BOM, (self encodingDeclaration: 'latin-1'), xml)]
		raise: XMLEncodingException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:25'!
testEncodingDeclarationUTF16BE
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					(self
						sourceXMLWithEncoding: 'UTF-16-BE'
						andASCIIContentAsUTF16LE: self sourceXML))]
		raise: XMLParserException.
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn:
					(self
						sourceXMLWithEncoding: 'UTF-16-BE'
						andASCIIContentAsUTF16BE: self sourceXML))]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:26'!
testEncodingDeclarationUTF16LE
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					(self
						sourceXMLWithEncoding: 'UTF-16LE'
						andASCIIContentAsUTF16BE: self sourceXML))]
		raise: XMLParserException.
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn:
					(self
						sourceXMLWithEncoding: 'UTF-16BE'
						andASCIIContentAsUTF16BE: self sourceXML))]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:27'!
testEncodingDeclarationUTF32BE
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					(self
						sourceXMLWithEncoding: 'utf-32be'
						andASCIIContentAsUTF32LE: self sourceXML))]
		raise: XMLParserException.
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn:
					(self
						sourceXMLWithEncoding: 'utf-32be'
						andASCIIContentAsUTF32BE: self sourceXML))]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:28'!
testEncodingDeclarationUTF32LE
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					(self
						sourceXMLWithEncoding: 'utf-32-le'
						andASCIIContentAsUTF32BE: self sourceXML))]
		raise: XMLParserException.
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn:
					(self
						sourceXMLWithEncoding: 'utf-32-le'
						andASCIIContentAsUTF32LE: self sourceXML))]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:29'!
testEncodingDeclarationUTF8
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					(self
						sourceXMLWithEncoding: 'UTF8'
						andASCIIContentAsUTF16BE: self sourceXML))]
		raise: XMLParserException.
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					(self
						sourceXMLWithEncoding: 'UTF-8'
						andASCIIContentAsUTF16LE: self sourceXML))]
		raise: XMLParserException.
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn: (self encodingDeclaration: 'UTF-8'), self sourceXML)]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:30'!
testExternalReadStreamDecoder
	| readStreamAdapter |

	readStreamAdapter :=
		XMLDecodingReadStreamAdapter on: 
			(self convertableStreamOn:
				self utf16LittleEndianBOM, (self asciiToUTF16LE: self sourceXML)).

	self
		shouldnt: [SAXHandler parse: readStreamAdapter]
		raise: XMLParserException.

	self
		assert: readStreamAdapter atEnd;
		assert: (readStreamAdapter streamConverter isKindOf: XMLUTF16StreamConverter).! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:31'!
testExternalSubsetByteOrderMarkDetection
	| xml uri |

	xml := '<!!ELEMENT name EMPTY>'.
	uri := '/external.xml'.
	self
		shouldnt: [
			((SAXHandler on: '') parser driver
				externalSubsetParserOn:
					(self convertableStreamOn:
						self utf16LittleEndianBOM, (self asciiToUTF16LE: xml))
				documentURI: uri) parseDocument]
		raise: XMLParserException.
	self
		shouldnt: [
			((SAXHandler on: '') parser driver
				externalSubsetParserOn:
					(self convertableStreamOn:
						self utf16BigEndianBOM, (self asciiToUTF16BE: xml))
				documentURI: uri) parseDocument]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:32'!
testExternalSubsetImplicitEncodingDetection
	| xml uri |

	xml := '<!!ELEMENT name EMPTY>'.
	uri := '/external.xml'.
	self
		shouldnt: [
			((SAXHandler on: '') parser driver
				externalSubsetParserOn:
					(self convertableStreamOn: (self asciiToUTF16LE: xml))
				documentURI: uri) parseDocument]
		raise: XMLParserException.
	self
		shouldnt: [
			((SAXHandler on: '') parser driver
				externalSubsetParserOn:
					(self convertableStreamOn: (self asciiToUTF16BE: xml))
				documentURI: uri) parseDocument]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:32'!
testImplicitUTF16EncodingDetection
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn: (self asciiToUTF16LE: self sourceXML))]
		raise: XMLParserException.
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn: (self asciiToUTF16BE: self sourceXML))]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:52'!
testImplicitUTF32EncodingDetection
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn:
					(self asciiToUTF32LE: self sourceXML))]
		raise: XMLParserException.
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn:
					(self asciiToUTF32BE: self sourceXML))]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:36'!
testUTF16BigEndianByteOrderMarkDetection
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn: self utf16BigEndianBOM, self sourceXML)]
		raise: XMLParserException.
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf16BigEndianBOM, (self asciiToUTF16LE: self sourceXML))]
		raise: XMLParserException.
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf16BigEndianBOM, (self asciiToUTF16BE: self sourceXML))]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:36'!
testUTF16LittleEndianByteOrderMarkDetection
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf16LittleEndianBOM, self sourceXML)]
		raise: XMLParserException.
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf16LittleEndianBOM, (self asciiToUTF16BE: self sourceXML))]
		raise: XMLParserException.
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf16LittleEndianBOM, (self asciiToUTF16LE: self sourceXML))]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:52'!
testUTF32BigEndianByteOrderMarkDetection
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf32BigEndianBOM, self sourceXML)]
		raise: XMLParserException.
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf32BigEndianBOM, (self asciiToUTF32LE: self sourceXML))]
		raise: XMLParserException.
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf32BigEndianBOM, (self asciiToUTF32BE: self sourceXML))]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:34'!
testUTF32LittleEndianByteOrderMarkDetection
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn: self utf32LittleEndianBOM, self sourceXML)]
		raise: XMLParserException.
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf32LittleEndianBOM, (self asciiToUTF32BE: self sourceXML))]
		raise: XMLParserException.
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf32LittleEndianBOM, (self asciiToUTF32LE: self sourceXML))]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/12/2016 00:33'!
testUTF8ByteOrderMarkDetection
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf8BOM, (self asciiToUTF16LE: self sourceXML))]
		raise: XMLParserException.
	self
		should: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf8BOM, (self asciiToUTF16BE: self sourceXML))]
		raise: XMLParserException.
	self
		shouldnt: [
			SAXHandler parse:
				(self convertableStreamOn:
					self utf8BOM, self sourceXML)]
		raise: XMLParserException.! !

!XMLEncodingTest methodsFor: 'tests' stamp: 'monty 2/10/2016 23:32'!
testWideCharacterNames
	| wideNames document |

	(wideNames := OrderedCollection new)
		addLast: (self stringFrom: #(16rC0 16rD6 16rD8 16rF6 16rF8 16r2FF $- $.));
		addLast: (self stringFrom: #(16r370 16r37D 16r37F 16r1FFF 16r200C $0 $9));
		addLast: (self stringFrom: #(16r200D 16r2070 16r218F 16r2C00 16r2FEF 16rB7));
		addLast: (self stringFrom: #(16r3001 16rD7FF 16rF900 16rFDCF 16rFDF0 16r0300));
		addLast: (self stringFrom: #(16rFFFD 16r10000 16rEFFFF 16r036F 16r203F 16r2040)).

	self
		shouldnt: [| source |
			source :=
				'<!!DOCTYPE {1} [
					<!!ELEMENT {1} ({2}, {3}, {4})>
					<!!ELEMENT {2} EMPTY>
					<!!ELEMENT {3} (#PCDATA)*>
					<!!ELEMENT {4} EMPTY>
					<!!ATTLIST {1} {5} ({3} | {4} | {5}) #IMPLIED>
				]>
				<{1} {5}="{4}">
					<{2}/>
					<{3}>{5}</{3}>
					<{4}/>
				</{1}>' format: wideNames.

			"a char-wise copy of source is parsed because wide chars aren't immediate
			types in pre-Spur Pharo/Squeak, so this can catch improper use of == on
			wide chars in those images"
			document := XMLDOMParser parse: (source collect: [:each | each copy])]
		raise: XMLParserException.

	self
		assert: document root name = wideNames first;
		assert: (document root attributeAt: wideNames fifth) = wideNames fourth;
		assert: (document root elementNames) asOrderedCollection = (wideNames copyFrom: 2 to: 4);
		assert: document contentString = wideNames fifth.! !

!XMLEncodingTest methodsFor: 'accessing' stamp: 'monty 1/22/2016 21:04'!
utf16BigEndianBOM
	^ XMLUTF16BigEndianStreamConverter byteOrderMark! !

!XMLEncodingTest methodsFor: 'accessing' stamp: 'monty 1/22/2016 21:05'!
utf16LittleEndianBOM
	^ XMLUTF16LittleEndianStreamConverter byteOrderMark! !

!XMLEncodingTest methodsFor: 'accessing' stamp: 'monty 1/22/2016 21:06'!
utf32BigEndianBOM
	^ XMLUTF32BigEndianStreamConverter byteOrderMark! !

!XMLEncodingTest methodsFor: 'accessing' stamp: 'monty 1/22/2016 21:06'!
utf32LittleEndianBOM
	^ XMLUTF32LittleEndianStreamConverter byteOrderMark! !

!XMLEncodingTest methodsFor: 'accessing' stamp: 'monty 1/22/2016 21:05'!
utf8BOM
	^ XMLUTF8StreamConverter byteOrderMark! !

TestCase subclass: #XMLExternalEntityTest
	instanceVariableNames: ''
	classVariableNames: 'IsSkipping'
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLExternalEntityTest commentStamp: 'monty 10/14/2014 00:42' prior: 0!
These tests use HTTP and the file system and are skipped by default. This enables them:

self isSkipping: false.!

XMLExternalEntityTest subclass: #XMLExternalEntityLimitsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLExternalEntityLimitsTest methodsFor: 'tests' stamp: 'monty 1/27/2015 23:06'!
testDefaultLimits
	| resolver |

	resolver := (SAXHandler on: '') externalEntityResolver.
	self
		assert: resolver externalSubsetReadLimit = DTDExternalEntityLimits defaultExternalSubsetReadLimit;
		assert: resolver maxExternalEntities > 0;
		assert: resolver maxExternalEntitySize = DTDExternalEntityLimits defaultMaxExternalEntitySize;
		deny: resolver resolvesLocalFromNonLocal.! !

!XMLExternalEntityLimitsTest methodsFor: 'tests' stamp: 'monty 3/9/2016 19:02'!
testExternalEntitiesLimit
	| entityLimit xml parser externalEntityResolver |

	self isSkipping
		ifTrue: [self skipOrReturnWith: [^ self]].

	entityLimit := 2.
	xml :=
		(self doctypeWithDeclarationsForEntityURIs:
			(self musicXMLExternalEntityURLs copyFrom: 1 to: entityLimit)),
		'<test/>'.
	(parser := SAXHandler on: xml)
		resolvesExternalEntities: true.
	(externalEntityResolver := parser externalEntityResolver)
		maxExternalEntities: entityLimit;
		clearEntityCache.
	self
		shouldnt: [parser parseDocument]
		raise: XMLLimitException.
	self assert: externalEntityResolver totalResolvedEntities = entityLimit.

	"reusing the parser shouldn't trigger the limit"
	parser on: xml.
	externalEntityResolver := parser externalEntityResolver.
	self
		assert: parser resolvesExternalEntities;
		assert: externalEntityResolver maxExternalEntities = entityLimit;
		assert: externalEntityResolver totalResolvedEntities = 0.
	externalEntityResolver clearEntityCache.
	self
		shouldnt: [parser parseDocument]
		raise: XMLLimitException.
	self assert: externalEntityResolver totalResolvedEntities = entityLimit.

	((parser := SAXHandler on: xml)
		resolvesExternalEntities: true;
		externalEntityResolver)
			clearEntityCache;
			maxExternalEntities: entityLimit - 1.
	self
		should: [parser parseDocument]
		raise: XMLLimitException.! !

!XMLExternalEntityLimitsTest methodsFor: 'tests' stamp: 'monty 3/9/2016 19:03'!
testExternalEntitySizeLimit
	| parser url xml size |

	self isSkipping
		ifTrue: [self skipOrReturnWith: [^ self]].

	url := self musicXMLExternalEntityURLs first asXMLURI.
	xml :=
		(self doctypeWithDeclarationsForEntityURIs: (Array with: url asString)),
		'<test/>'.
	size :=
		(url
			getUpToLimit: nil
			decoding: false) size.
	((parser := SAXHandler on: xml)
		resolvesExternalEntities: true;
		externalEntityResolver)
			clearEntityCache;
			maxExternalEntitySize: size.
	self
		shouldnt: [parser parseDocument]
		raise: XMLLimitException.

	((parser := SAXHandler on: xml)
		resolvesExternalEntities: true;
		externalEntityResolver)
			clearEntityCache;
			maxExternalEntitySize: size - 1.
	self
		should: [parser parseDocument]
		raise: XMLLimitException.! !

!XMLExternalEntityLimitsTest methodsFor: 'tests' stamp: 'monty 3/9/2016 20:53'!
testExternalSubsetReadLimit
	| xml fileHandle externalSubset parser |

	self isSkipping
		ifTrue: [self skipOrReturnWith: [^ self]].

	fileHandle := self fileHandleForNewFileSuffixed: 'externalSubset.dtd'.
	externalSubset := '<?xml encoding="UTF-8"?><!!ELEMENT test EMPTY>'.
	fileHandle writeStreamDo: [:writeStream |
		writeStream nextPutAll: externalSubset].
	xml := '<!!DOCTYPE test SYSTEM "', fileHandle path, '"><test/>'.

	self
		shouldnt: [
			(parser := SAXHandler on: xml)
				externalEntityResolver externalSubsetReadLimit: externalSubset size.
			parser
				resolvesExternalEntities: true;
				parseDocument]
		raise: XMLLimitException.

	self
		should: [
			(parser := SAXHandler on: xml)
				externalEntityResolver externalSubsetReadLimit: externalSubset size - 1.
			parser
				resolvesExternalEntities: true;
				parseDocument]
		raise: XMLLimitException.

	fileHandle delete.! !

!XMLExternalEntityLimitsTest methodsFor: 'tests' stamp: 'monty 12/3/2015 02:34'!
testOnURLUpToLimit
	| parser size |

	self isSkipping
		ifTrue: [self skipOrReturnWith: [^ self]].

	size :=
		(self feedURL asXMLURI
			getUpToLimit: nil
			decoding: false) size.
	self
		shouldnt: [
			parser :=
				SAXHandler
					onURL: self feedURL
					upToLimit: size]
		raise: XMLLimitException.
	self assert: parser documentReadLimit = size.

	self
		should: [
			(SAXHandler
				onURL: self feedURL
				upToLimit: size - 1)
				parseDocument]
		raise: XMLLimitException.! !

!XMLExternalEntityLimitsTest methodsFor: 'tests' stamp: 'monty 5/25/2015 02:39'!
testRemoveLimits
	|  readLimit entityLimit entitySizeLimit parser resolver |

	readLimit := 100.
	entityLimit := 10.
	entitySizeLimit := 100.
	parser := SAXHandler on: ''.
	(resolver := parser externalEntityResolver)
		externalSubsetReadLimit: readLimit;
		maxExternalEntities: entityLimit;
		maxExternalEntitySize: entitySizeLimit;
		resolvesLocalFromNonLocal: true.
	self
		assert: resolver externalSubsetReadLimit = readLimit;
		assert: resolver maxExternalEntities = entityLimit;
		assert: resolver maxExternalEntitySize = entitySizeLimit;
		assert: resolver resolvesLocalFromNonLocal.

	parser removeLimits.
	self
		assert: resolver externalSubsetReadLimit isNil;
		assert: resolver maxExternalEntities isNil;
		assert: resolver maxExternalEntitySize isNil;
		assert: resolver resolvesLocalFromNonLocal.! !

!XMLExternalEntityLimitsTest methodsFor: 'tests' stamp: 'monty 1/24/2014 17:33'!
testResolvingLocalFromNonLocal
	| parser |

	self
		should: [
			(parser :=
				SAXHandler
					on: '<!!DOCTYPE root SYSTEM "file://local/uri">'
					documentURI: 'http://www.example.com')
				externalEntityResolver resolvesLocalFromNonLocal: false.
			parser
				resolvesExternalEntities: true;
				parseDocument]
		raise: XMLLimitException.! !

!XMLExternalEntityTest class methodsFor: 'testing' stamp: 'monty 2/11/2014 02:25'!
isSkipping
	"HTTP/file tests are skipped by default"
	^ IsSkipping ifNil: [IsSkipping := true]! !

!XMLExternalEntityTest class methodsFor: 'testing' stamp: 'monty 11/24/2014 21:08'!
isSkipping: aBoolean
	"HTTP/file tests are skipped by default.

	self isSkipping: false.
	self isSkipping: true.
	"
	IsSkipping := aBoolean! !

!XMLExternalEntityTest methodsFor: 'source' stamp: 'monty 4/26/2015 23:59'!
docBookArticleSource
	^ '<?xml version="1.0" encoding="UTF-8"?>
<!!DOCTYPE article PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN" "http://docbook.org/xml/4.5/docbookx.dtd">
<article>
	<articleinfo>
		<title>About Pharo</title>
		<author>
			<honorific>Dr.</honorific>
			<firstname>Stephane</firstname>
			<surname>Ducasse</surname>
		</author>
	</articleinfo>
	<section>
		<title>Mission</title>
		<para>Pharo''s goal is to deliver a clean, innovative, free open-source environment. Here is the to be revised Pharo vision document.</para>
		<para>By providing a stable and small core system, excellent developing tools, and maintained releases, Pharo is an attractive platform to build and deploy mission critical applications.</para>
		<para>Pharo fosters a healthy ecosystem of both private and commercial contributors who advance and maintain the core system and its external packages.</para>
	</section>
	<section>
		<title>Pharo features</title>
		<para>A dynamic, pure object-oriented programming language in the tradition of Smalltalk</para>
		<para>An IDE (integrated development environment)</para>
		<para>A huge library and set of external packages</para>
	</section>
	<section>
		<title>Pharo and Business</title>
		<para>The goal of Pharo is to sustain and grow business.</para>
		<para>Pharo is supported by the industrial Pharo consortium.</para>
		<para>Pharo has also an association of users: Pharo association</para>
    </section>
</article>'! !

!XMLExternalEntityTest methodsFor: 'source' stamp: 'monty 4/26/2015 14:52'!
docBookExampleSource
	^ '<!!DOCTYPE example
	PUBLIC "-//OASIS//DTD DocBook XML V4.1.2//EN"
	"http://www.oasis-open.org/docbook/xml/4.1.2/docbookx.dtd">
	<example>
		<title>An Example</title>
		<programlisting>
		XMLDocument root:
			((XMLElement named: ''test'')
				addNode: ''An example'' asXMLString;
				yourself)
		</programlisting>
	</example>'! !

!XMLExternalEntityTest methodsFor: 'source' stamp: 'monty 3/9/2016 19:04'!
doctypeWithDeclarationsForEntityURIs: aURICollection
	^ String streamContents: [:stream |
		stream nextPutAll: '<!!DOCTYPE test ['.
		aURICollection withIndexDo: [:each :i |
			stream nextPutAll: '<!!ENTITY entity', i printString, ' SYSTEM "', each, '">'].
		stream nextPutAll: ']>']! !

!XMLExternalEntityTest methodsFor: 'urls' stamp: 'monty 9/29/2014 12:33'!
feedURL
	^ 'http://pharo.org/news/atom.xml'! !

!XMLExternalEntityTest methodsFor: 'files' stamp: 'monty 3/13/2016 00:39'!
fileHandleForNewFileSuffixed: aFileSuffix
	| fileHandle |
	
	fileHandle := (self class name asString, '-', aFileSuffix) asXMLFileHandle.
	fileHandle exists
		ifTrue: [fileHandle delete].
	^ fileHandle.! !

!XMLExternalEntityTest methodsFor: 'accessing' stamp: 'monty 2/11/2014 00:55'!
isSkipping
	^ self class isSkipping! !

!XMLExternalEntityTest methodsFor: 'source' stamp: 'monty 4/26/2015 23:59'!
musicXMLDocument
	^ '<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!!DOCTYPE score-partwise PUBLIC
    "-//Recordare//DTD MusicXML 3.0 Partwise//EN"
    "http://www.musicxml.org/dtds/partwise.dtd">
<score-partwise version="3.0">
  <part-list>
    <score-part id="P1">
      <part-name>Music</part-name>
    </score-part>
  </part-list>
  <part id="P1">
    <measure number="1">
      <attributes>
        <divisions>1</divisions>
        <key>
          <fifths>0</fifths>
        </key>
        <time>
          <beats>4</beats>
          <beat-type>4</beat-type>
        </time>
        <clef>
          <sign>G</sign>
          <line>2</line>
        </clef>
      </attributes>
      <note>
        <pitch>
          <step>C</step>
          <octave>4</octave>
        </pitch>
        <duration>4</duration>
        <type>whole</type>
      </note>
    </measure>
  </part>
</score-partwise>
'.! !

!XMLExternalEntityTest methodsFor: 'urls' stamp: 'monty 4/26/2015 14:40'!
musicXMLExternalEntityURLs
	^  #('http://www.musicxml.org/dtds/note.mod'
		'http://www.musicxml.org/dtds/isolat1.ent'
		'http://www.musicxml.org/dtds/isolat2.ent'
		'http://www.musicxml.org/dtds/layout.mod'
		'http://www.musicxml.org/dtds/identity.mod'
		'http://www.musicxml.org/dtds/attributes.mod'
		'http://www.musicxml.org/dtds/partwise.dtd'
		'http://www.musicxml.org/dtds/barline.mod'
		'http://www.musicxml.org/dtds/direction.mod'
		'http://www.musicxml.org/dtds/score.mod'
		'http://www.musicxml.org/dtds/link.mod'
		'http://www.musicxml.org/dtds/common.mod')! !

!XMLExternalEntityTest methodsFor: 'private' stamp: 'monty 4/4/2015 06:03'!
skipOrReturnWith: aBlock
	"skip is not portable to squeak"
	(self respondsTo: #skip)
		ifTrue: [self skip]
		ifFalse: [aBlock value]! !

!XMLExternalEntityTest methodsFor: 'tests' stamp: 'monty 3/9/2016 18:55'!
testOnFileNamed
	| parser fileHandle |

	self isSkipping
		ifTrue: [self skipOrReturnWith: [^ self]].

	fileHandle := self fileHandleForNewFileSuffixed: 'testOnFileNamed.xml'.
	fileHandle writeStreamDo: [:writeStream |
		writeStream nextPutAll: '<test/>'].
	self
		shouldnt: [parser := SAXHandler onFileNamed: fileHandle path]
		raise: XMLFileException.
	self
		assert: parser documentURI = fileHandle path;
		assert: parser documentURI asXMLURI isLocalURI.

	fileHandle delete.! !

!XMLExternalEntityTest methodsFor: 'tests' stamp: 'monty 4/4/2015 06:01'!
testOnURL
	| parser |

	self isSkipping
		ifTrue: [self skipOrReturnWith: [^ self]].

	self
		shouldnt: [parser := SAXHandler onURL: self feedURL]
		raise: XMLHTTPException.
	self
		assert: parser documentURI = self feedURL;
		deny: parser documentURI asXMLURI isLocalURI.! !

!XMLExternalEntityTest methodsFor: 'tests' stamp: 'monty 4/23/2015 18:17'!
testParsingDocBookArticle
	"This uses HTTP so is skipped by default"

	self isSkipping
		ifTrue: [self skipOrReturnWith: [^ self]].

	self
		shouldnt: [
			(SAXHandler on: self docBookArticleSource)
				resolvesExternalEntities: true;
				removeLimits;
				parseDocument]
		raise: XMLParserException.! !

!XMLExternalEntityTest methodsFor: 'tests' stamp: 'monty 4/26/2015 14:52'!
testParsingDocBookExample
	"This uses HTTP so is skipped by default"

	self isSkipping
		ifTrue: [self skipOrReturnWith: [^ self]].

	self
		shouldnt: [
			(SAXHandler on: self docBookExampleSource)
				resolvesExternalEntities: true;
				removeLimits;
				parseDocument]
		raise: XMLParserException.! !

!XMLExternalEntityTest methodsFor: 'tests' stamp: 'monty 3/9/2016 19:20'!
testParsingEncodedFile
	"This test creates/deletes a file so is skipped by default"
	| fileHandle doc docFromFile |

	self isSkipping
		ifTrue: [self skipOrReturnWith: [^ self]].

	fileHandle := self fileHandleForNewFileSuffixed: 'testParsingEncodedFile.xml'.
	doc := XMLDOMParser parse: self docBookArticleSource.
	doc
		encoding: 'UTF-16BE';
		printToFileNamed: fileHandle path.
	self
		shouldnt: [docFromFile := XMLDOMParser parseFileNamed: fileHandle path]
		raise: XMLFileException.
	self
		assert: docFromFile encoding = 'UTF-16BE';
		assert: doc printString = docFromFile printString.

	fileHandle delete.! !

!XMLExternalEntityTest methodsFor: 'tests' stamp: 'monty 4/26/2015 14:41'!
testParsingExternalEntities
	"This test uses HTTP so is skipped by default."
	| entityCache entityCacheCopy |

	self isSkipping
		ifTrue: [self skipOrReturnWith: [^ self]].

	DTDCachingExternalEntityResolver clearEntityCache.
	entityCache := DTDCachingExternalEntityResolver entityCache.
	self musicXMLExternalEntityURLs do: [:each |
		self deny: (entityCache includesKey: each asXMLURI)].

	SAXHandler
		parse: self musicXMLDocument
		usingNamespaces: true
		validation: true
		externalEntities: true.
	self musicXMLExternalEntityURLs do: [:each |
		self assert: (entityCache at: each asXMLURI ifAbsent: ['']) notEmpty].

	"check for re-requesting"
	entityCacheCopy := entityCache copy.
	SAXHandler
		parse: self musicXMLDocument
		usingNamespaces: true
		validation: true
		externalEntities: true.
	self musicXMLExternalEntityURLs do: [:each |
		self assert: (entityCache at: each asXMLURI) == (entityCacheCopy at: each asXMLURI)].! !

!XMLExternalEntityTest methodsFor: 'tests' stamp: 'monty 3/9/2016 20:52'!
testParsingFile
	"This test creates/deletes a file so is skipped by default"
	| fileHandle doc |

	self isSkipping
		ifTrue: [self skipOrReturnWith: [^ self]].

	fileHandle := self fileHandleForNewFileSuffixed: 'testParsingFile.xml'.
	self
		should: [XMLDOMParser parseFileNamed: fileHandle path]
		raise: XMLFileException.

	fileHandle writeStreamDo: [:writeStream |
		writeStream nextPutAll: self musicXMLDocument].
	self
		shouldnt: [doc := XMLDOMParser parseFileNamed: fileHandle path]
		raise: XMLFileException.
	self assert: (doc root isNamed: 'score-partwise').

	fileHandle delete.! !

!XMLExternalEntityTest methodsFor: 'tests' stamp: 'monty 12/3/2015 06:32'!
testParsingRetrievedURL
	| source |

	self isSkipping
		ifTrue: [self skipOrReturnWith: [^ self]].

	source := self feedURL asXMLURI get.
	self assert: source isString.
	self
		shouldnt: [SAXHandler parse: source]
		raise: XMLParserException.

	source := self feedURL asXMLURI streamGet.
	self
		assert: source isStream;
		assert: source peek isCharacter.
	self
		shouldnt: [SAXHandler parse: source]
		raise: XMLParserException.! !

!XMLExternalEntityTest methodsFor: 'tests' stamp: 'monty 4/4/2015 06:02'!
testParsingURL
	"This uses HTTP so is skipped by default"
	| doc |

	self isSkipping
		ifTrue: [self skipOrReturnWith: [^ self]].

	self
		shouldnt: [doc := XMLDOMParser parseURL: self feedURL]
		raise: XMLHTTPException.
	self assert: (doc root isNamed: 'feed').! !

TestCase subclass: #XMLIntegerReaderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Utils'!

!XMLIntegerReaderTest methodsFor: 'private' stamp: 'monty 10/24/2014 20:31'!
maxBase
	^ XMLIntegerReader maxBase! !

!XMLIntegerReaderTest methodsFor: 'tests' stamp: 'monty 10/27/2014 13:32'!
testBaseMarker
	2 to: self maxBase do: [:base | | stream defaultBase |
		defaultBase := (base - 1) max: 2.
		stream := ((base printStringBase: base) copyWithFirst: $#) readStream.
		self
			assert:
				(XMLIntegerReader
					readFrom: stream
					withBase: base
					startingWithAny: '@#$'
					orBase: defaultBase) = base;
			assert: stream atEnd.

		stream := (base printStringBase: defaultBase) readStream.
		self
			assert:
				(XMLIntegerReader
					readFrom: stream
					withBase: base
					startingWithAny: '@#$'
					orBase: defaultBase) = base;
			assert: stream atEnd].! !

!XMLIntegerReaderTest methodsFor: 'tests' stamp: 'monty 10/26/2014 10:24'!
testBases
	2 to: self maxBase do: [:base | | stream |
		(0 to: base), (base + 1 to: base * 10 by: base) do: [:i |
			stream := (i printStringBase: base) readStream.
			self
				assert: (XMLIntegerReader readFrom: stream withBase: base) = i;
				assert: stream atEnd]].
	self
		should: [XMLIntegerReader readFrom: '1' readStream withBase: 1]
		raise: Error.
	self
		should: [XMLIntegerReader readFrom: '1' readStream withBase: self maxBase + 1]
		raise: Error.! !

!XMLIntegerReaderTest methodsFor: 'tests' stamp: 'monty 1/7/2016 05:49'!
testMaxIntegerValue
	| maxValue |

	maxValue := XMLIntegerReader maxIntegerValue.
	2 to: self maxBase do: [:base |
		self assert:
			(XMLIntegerReader
				readFrom: (maxValue printStringBase: base) readStream
				withBase: base) = maxValue.
		1 to: base do: [:i |
			self  assert:
				(XMLIntegerReader
					readFrom: ((maxValue + i) printStringBase: base) readStream
					withBase: base) = maxValue]].! !

!XMLIntegerReaderTest methodsFor: 'tests' stamp: 'monty 1/8/2016 08:47'!
testNonDigit
	2 to: self maxBase do: [:base | | digitInterval letterInterval uppercaseLetterInterval |
		base > 10
			ifTrue: [
				digitInterval := $0 asciiValue to: $9 asciiValue.
				letterInterval := $a asciiValue to: $a asciiValue + (base - 11).
				uppercaseLetterInterval := $A asciiValue to: $A asciiValue + (base - 11)]
			ifFalse: [
				digitInterval := $0 asciiValue to: $0 asciiValue + base - 1.
				letterInterval := #().
				uppercaseLetterInterval := #()].
		0 to: 256 do: [:each | | char reader |
			char := each asCharacter.
			reader := XMLIntegerReader on: char asString readStream.
			((digitInterval includes: each)
				or: [(letterInterval includes: each)
					or: [uppercaseLetterInterval includes: each]])
				ifTrue: [
					self
						assert: (reader nextIntegerWithBase: base) notNil;
						assert: reader atEnd]
				ifFalse: [
					self
						assert: (reader nextIntegerWithBase: base) isNil;
						deny: reader atEnd;
						assert: reader next = char;
						assert: reader atEnd]]]! !

!XMLIntegerReaderTest methodsFor: 'tests' stamp: 'monty 10/27/2014 08:50'!
testPositioning
	| reader string |

	string := 'test'.
	reader := XMLIntegerReader on: string readStream.
	string do: [:each |
		self
			deny: reader atEnd;
			assert: reader peek = each;
			assert: reader next = each].
	self assert: reader atEnd.! !

!XMLIntegerReaderTest methodsFor: 'tests' stamp: 'monty 12/4/2015 03:49'!
testPrintStream
	| printStream reader |

	printStream := String new writeStream.
	(reader := XMLIntegerReader on: '123xABChdefO11' readStream)
		printStream: printStream.
	self
		assert: (reader nextIntegerWithBase: 10) = 123;
		assert: printStream contents = '123';
		assert: (reader nextIntegerWithBase: 16 startingWithAny: 'x' orBase: 10) = 2748;
		assert: printStream contents = '123xABC';
		assert: (reader nextIntegerWithBase: 16 startingWithAny: 'h' orBase: 10) = 3567;
		assert: printStream contents = '123xABChdef'.

	reader printStream: nil.
	self
		assert: (reader nextIntegerWithBase: 8 startingWithAny: 'oO' orBase: 10) = 9;
		assert: printStream contents = '123xABChdef';
		assert: reader atEnd.! !

!XMLIntegerReaderTest methodsFor: 'tests' stamp: 'monty 10/24/2014 20:40'!
testReadLimit
	2 to: self maxBase do: [:base | | stream |
		"101 regardless of base"
		stream := (base * base + 1 printStringBase: base) readStream.
		self
			assert: (XMLIntegerReader readFrom: stream withBase: base upTo: 2) = base;
			deny: stream atEnd;
			assert: stream next == $1]! !

!XMLIntegerReaderTest methodsFor: 'tests' stamp: 'monty 2/11/2016 05:03'!
testStreamReadingPortability
	| stream integerReader integer atEnd |

	stream := XMLExceptionRaisingReadStream on: '123abc'.
	integerReader := XMLIntegerReader on: stream.
	self
		assert: (integerReader nextIntegerWithBase: 16) = 16r123ABC;
		assert: integerReader atEnd.
	self
		shouldnt: [
			integerReader
				nextInteger;
				nextIntegerWithBase: 16;
				nextIntegerWithBase: 16 upTo: 1;
				nextIntegerWithBase: 16 startingWithAny: 'x' orBase: 10;
				peek;
				next]
		raise: Error.

	self assert: stream atEnd.
	self should: [stream peek] raise: Error.
	self should: [stream next] raise: Error.! !

TestCase subclass: #XMLKeyValueCacheTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Utils'!

!XMLKeyValueCacheTest methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:42'!
cacheClass
	^ XMLKeyValueCache! !

!XMLKeyValueCacheTest methodsFor: 'accessing' stamp: 'monty 11/15/2014 21:19'!
keysAndValuesDo: aBlock
	1 to: self maxSize do: [:i |
		aBlock
			value: ('key', i asString)
			value: ('value', i asString)]! !

!XMLKeyValueCacheTest methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:57'!
maxSize
	^ 20! !

!XMLKeyValueCacheTest methodsFor: 'instance creation' stamp: 'monty 11/15/2014 20:58'!
newCache
	^ self cacheClass new! !

!XMLKeyValueCacheTest methodsFor: 'instance creation' stamp: 'monty 11/15/2014 21:18'!
newCacheWithEntries
	| cache |

	cache := self cacheClass maxSize: self maxSize.
	self keysAndValuesDo: [:key :value |
		cache
			at: key
			put: value].
	^ cache.! !

!XMLKeyValueCacheTest methodsFor: 'tests' stamp: 'monty 2/16/2016 06:26'!
testAt
	| cache |

	cache := self newCacheWithEntries.
	self keysAndValuesDo: [:key :value |
		self assert: (cache at: key) = value].
	self assert: (cache at: 'newkey') isNil.! !

!XMLKeyValueCacheTest methodsFor: 'tests' stamp: 'monty 11/15/2014 21:35'!
testAtIfAbsent
	| cache |

	cache := self newCacheWithEntries.
	self keysAndValuesDo: [:key :value |
		self assert: (cache at: key ifAbsent: ['absent']) = value].
	self assert: (cache at: 'newkey' ifAbsent: ['absent']) = 'absent'.! !

!XMLKeyValueCacheTest methodsFor: 'tests' stamp: 'monty 11/15/2014 21:19'!
testAtIfAbsentPut
	| cache |

	cache := self newCacheWithEntries.
	self keysAndValuesDo: [:key :value |
		cache at: key ifAbsentPut: ['newkey'].
		self
			assert: (cache at: key) = value;
			assert: cache size = self maxSize].
	self
		assert: (cache at: 'newkey' ifAbsentPut: ['newvalue']) = 'newvalue';
		assert: (cache at: 'newkey') = 'newvalue';
		assert: cache size = 1.! !

!XMLKeyValueCacheTest methodsFor: 'tests' stamp: 'monty 11/15/2014 21:38'!
testAtPut
	| cache i |

	cache := self cacheClass maxSize: self maxSize.
	i := 0.
	self keysAndValuesDo: [:key :value |
		self
			assert: (cache at: key put: value) = value;
			assert: (cache at: key) = value.
		i := i + 1.
		self assert: cache size = i].
	self
		assert: (cache at: 'newkey' put: 'newvalue') = 'newvalue';
		assert: (cache at: 'newkey') = 'newvalue';
		assert: cache size = 1.! !

!XMLKeyValueCacheTest methodsFor: 'tests' stamp: 'monty 11/15/2014 21:03'!
testClear
	| cache |

	cache := self newCacheWithEntries.
	self deny: cache isEmpty.
	cache clear.
	self assert: cache isEmpty.! !

!XMLKeyValueCacheTest methodsFor: 'tests' stamp: 'monty 11/15/2014 20:56'!
testDefaultMaxSize
	self assert: self newCache maxSize = self cacheClass defaultMaxSize! !

!XMLKeyValueCacheTest methodsFor: 'tests' stamp: 'monty 11/16/2014 00:02'!
testIncludesKey
	| cache |

	cache := self newCacheWithEntries.
	self keysAndValuesDo: [:key :value | self assert: (cache includesKey: key)].
	self deny: (cache includesKey: 'newkey').! !

!XMLKeyValueCacheTest methodsFor: 'tests' stamp: 'monty 11/15/2014 20:53'!
testIsEmpty
	self
		assert: self newCache isEmpty;
		deny: self newCacheWithEntries isEmpty! !

!XMLKeyValueCacheTest methodsFor: 'tests' stamp: 'monty 11/15/2014 20:57'!
testMaxSize
	| cache |

	cache := self newCacheWithEntries.
	self assert: cache size = self maxSize.
	cache maxSize: self maxSize + 1.
	self assert: cache size = self maxSize.
	cache maxSize: 0.
	self assert: cache size = 0.! !

!XMLKeyValueCacheTest methodsFor: 'tests' stamp: 'monty 2/16/2016 08:00'!
testNotEmpty
	self
		deny: self newCache notEmpty;
		assert: self newCacheWithEntries notEmpty! !

!XMLKeyValueCacheTest methodsFor: 'tests' stamp: 'monty 11/15/2014 20:57'!
testSize
	self
		assert: self newCache size = 0;
		assert: self newCacheWithEntries size = self maxSize! !

TestCase subclass: #XMLNamespaceScopeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Namespaces'!

!XMLNamespaceScopeTest methodsFor: 'accessing' stamp: 'monty 12/29/2014 17:54'!
namespaces
	^ OrderPreservingStringDictionary new
		at: 'prefix1' put: 'ns1';
		at: 'prefix2' put: 'ns2';
		at: '' put: 'defaultNS';
		yourself! !

!XMLNamespaceScopeTest methodsFor: 'instance creation' stamp: 'monty 12/28/2014 23:18'!
newNamespaceScope
	^ XMLNamespaceScope new! !

!XMLNamespaceScopeTest methodsFor: 'instance creation' stamp: 'monty 12/28/2014 23:24'!
newNamespaceScopeWithNamespaces
	| scope |

	scope := self newNamespaceScope.
	self namespaces keysAndValuesDo: [:prefix :uri |
		scope
			mapPrefix: prefix
			to: uri].
	^ scope.! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 4/21/2015 13:27'!
testCopy
	| scope oldPrefixMappings |

	scope := self newNamespaceScopeWithNamespaces.
	oldPrefixMappings := scope prefixMappings.
	self assert: scope copy prefixMappings == oldPrefixMappings.! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 3/6/2016 19:57'!
testDefaultNamespace
	| scope oldPrefixMappings |

	scope := self newNamespaceScope.
	self assert: scope defaultNamespace isEmpty.

	oldPrefixMappings := scope prefixMappings.
	scope defaultNamespace: 'test'.
	self
		assert: scope defaultNamespace = 'test';
		assert: (scope prefixMappings at: '') = 'test';
		deny: scope prefixMappings = oldPrefixMappings.

	scope unmapPrefix: ''.
	self assert: scope defaultNamespace isEmpty.! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 12/28/2014 23:31'!
testEquals
	self
		assert: self newNamespaceScope = self newNamespaceScope;
		assert: self newNamespaceScopeWithNamespaces = self newNamespaceScopeWithNamespaces;
		deny: self newNamespaceScope = self newNamespaceScopeWithNamespaces;
		deny: self newNamespaceScopeWithNamespaces = self newNamespaceScope! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 3/6/2016 19:57'!
testHasDefaultNamespace
	| scope |

	scope := self newNamespaceScope.
	self deny: scope hasDefaultNamespace.

	scope defaultNamespace: 'test'.
	self assert: scope hasDefaultNamespace.

	scope defaultNamespace: ''.
	self assert: scope hasDefaultNamespace.

	scope unmapPrefix: ''.
	self deny: scope hasDefaultNamespace.! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 3/6/2016 19:57'!
testHasNamespaces
	| scope |

	scope := self newNamespaceScope.
	self deny: scope hasNamespaces.

	scope mapPrefixesAndURIs: self namespaces.
	self assert: scope hasNamespaces.
	self namespaces keysDo: [:each |
		self assert: scope hasNamespaces.
		scope unmapPrefix: each].
	self deny: scope hasNamespaces.! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 2/26/2016 18:30'!
testHash
	self assert:
		self newNamespaceScope hash =
			self newNamespaceScope hash.
	self assert:
		self newNamespaceScopeWithNamespaces hash =
			self newNamespaceScopeWithNamespaces hash.
	self assert:
		self newNamespaceScope hash =
			(self newNamespaceScopeWithNamespaces
				unmapPrefixes;
				hash).! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 3/6/2016 19:58'!
testIncludesPrefix
	| scope |

	scope := self newNamespaceScope.
	self
		deny: (scope includesPrefix: 'xml');
		deny: (scope includesPrefix: 'xmlns').

	"cannot map xmlns prefix, but can map xml"
	scope mapPrefix: 'xml' to: XMLNamespaceScope xmlNamespaceURI.
	self assert: (scope includesPrefix: 'xml').

	self namespaces keysAndValuesDo: [:prefix :uri |
		self deny: (scope includesPrefix: prefix).
		scope mapPrefix: prefix to: uri.
		self assert: (scope includesPrefix: prefix)].! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 3/6/2016 19:58'!
testIncludesPrefixMappedTo
	| scope |

	scope := self newNamespaceScope.
	self
		deny: (scope includesPrefix: 'xml' mappedTo: XMLNamespaceScope xmlNamespaceURI);
		deny: (scope includesPrefix: 'xmlns' mappedTo: XMLNamespaceScope xmlnsNamespaceURI).

	"cannot map xmlns prefix, but can map xml"
	scope mapPrefix: 'xml' to: XMLNamespaceScope xmlNamespaceURI.
	self assert: (scope includesPrefix: 'xml' mappedTo: XMLNamespaceScope xmlNamespaceURI).

	self namespaces keysAndValuesDo: [:prefix :uri |
		self deny: (scope includesPrefix: prefix mappedTo: uri).
		scope mapPrefix: prefix to: uri.
		self assert: (scope includesPrefix: prefix mappedTo: uri)].! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 12/28/2014 23:57'!
testInheritMappingsFrom
	| outerScope innerScope |

	outerScope := self newNamespaceScopeWithNamespaces.
	innerScope := self newNamespaceScope.
	innerScope inheritMappingsFrom: outerScope.
	self assert: innerScope prefixMappings == outerScope prefixMappings.

	innerScope := self newNamespaceScopeWithNamespaces.
	innerScope inheritMappingsFrom: outerScope.
	self
		assert: innerScope prefixMappings = outerScope prefixMappings;
		deny: innerScope prefixMappings == outerScope prefixMappings.! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 3/6/2016 18:38'!
testMapPrefixTo
	| scope oldPrefixMappings |

	scope := self newNamespaceScope.
	self namespaces keysAndValuesDo: [:prefix :uri |
		"save to ensure prefixMappings is copied, even if unchanged"
		oldPrefixMappings := scope prefixMappings.
		scope
			mapPrefix: prefix
			to: uri.
		self
			assert: (scope prefixMappings at: prefix) = uri;
			deny: scope prefixMappings = oldPrefixMappings;
			deny: scope prefixMappings == oldPrefixMappings.

		"save to ensure prefixMappings is copied, even if unchanged"
		oldPrefixMappings := scope prefixMappings.
		scope
			mapPrefix: prefix
			to: uri.
		self
			assert: (scope prefixMappings at: prefix) = uri;
			assert: scope prefixMappings = oldPrefixMappings;
			deny: scope prefixMappings == oldPrefixMappings].
	self assert:
		scope prefixMappings associations asArray =
			self namespaces associations asArray.

	self namespaces keysDo: [:each |
		"save to ensure prefixMappings is copied, even if unchanged"
		oldPrefixMappings := scope prefixMappings.
		scope
			mapPrefix: each
			to: 'newNS'.
		self
			assert: (scope prefixMappings at: each) = 'newNS';
			deny: scope prefixMappings = oldPrefixMappings;
			deny: scope prefixMappings == oldPrefixMappings].

	"save to ensure prefixMappings is copied, even if unchanged"
	oldPrefixMappings := scope prefixMappings.
	scope
		mapPrefix: 'xml'
		to: scope xmlNamespaceURI.
	self
		assert: (scope prefixMappings at: 'xml') = scope xmlNamespaceURI;
		deny: scope prefixMappings = oldPrefixMappings;
		deny: scope prefixMappings == oldPrefixMappings.

	"cannot rebind xmlns prefix at all, even to the same URI"
	self
		should: [
			scope
				mapPrefix: 'xmlns'
				to: scope xmlnsNamespaceURI]
		raise: XMLNamespaceException.

	"cannot bind a new prefix to the xml, xmlns, or empty namespace URI"
	(Array
		with: scope xmlNamespaceURI
		with: scope xmlnsNamespaceURI
		with: '') do: [:each |
		self
			should: [
				scope
					mapPrefix: 'newPrefix'
					to: each]
			raise: XMLNamespaceException].! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 4/21/2015 12:13'!
testMapPrefixesAndURIs
	| outerScope innerScope namespaces |

	namespaces := self namespaces.
	(outerScope := self newNamespaceScope) mapPrefixesAndURIs: namespaces.
	self
		assert: outerScope prefixMappings = namespaces;
		deny: outerScope prefixMappings == namespaces.

	"copying shares dictionary until modified"
	innerScope := outerScope copy.
	self
		assert: innerScope prefixMappings = outerScope prefixMappings;
		assert: innerScope prefixMappings == outerScope prefixMappings.

	"modifying creates new dictionary"
	innerScope mapPrefixesAndURIs: namespaces.
	self
		assert: innerScope prefixMappings = outerScope prefixMappings;
		deny: innerScope prefixMappings == outerScope prefixMappings.! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 2/24/2016 06:14'!
testMappedPrefixes
	| scope prefixes |

	scope := self newNamespaceScope.
	prefixes := OrderedCollection new.
	self namespaces keysAndValuesDo: [:prefix :uri |
		scope
			mapPrefix: prefix
			to: uri.
		prefixes addLast: prefix.
		self assert: scope mappedPrefixes = prefixes asArray].! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 3/6/2016 19:58'!
testPrefixMappingAt
	| scope |

	scope := self newNamespaceScope.
	self
		assert: (scope prefixMappingAt: 'xml') isEmpty;
		assert: (scope prefixMappingAt: 'xmlns') isEmpty.

	"cannot map xmlns prefix, but can map xml"
	scope mapPrefix: 'xml' to: XMLNamespaceScope xmlNamespaceURI.
	self assert: (scope prefixMappingAt: 'xml') = XMLNamespaceScope xmlNamespaceURI.

	self namespaces keysAndValuesDo: [:prefix :uri |
		self assert: (scope prefixMappingAt: prefix) isEmpty.
		scope mapPrefix: prefix to: uri.
		self assert: (scope prefixMappingAt: prefix) = uri].! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 3/6/2016 19:58'!
testPrefixMappingAtIfAbsent
	| scope |

	scope := self newNamespaceScope.
	self
		assert: (scope prefixMappingAt: 'xml' ifAbsent: [#absent]) = #absent;
		assert: (scope prefixMappingAt: 'xmlns'ifAbsent: [#absent]) = #absent.

	"cannot map xmlns prefix, but can map xml"
	scope mapPrefix: 'xml' to: XMLNamespaceScope xmlNamespaceURI.
	self assert:
		(scope
			prefixMappingAt: 'xml'
			ifAbsent: [#absent]) = XMLNamespaceScope xmlNamespaceURI.

	self namespaces keysAndValuesDo: [:prefix :uri |
		self assert:
			(scope
				prefixMappingAt: prefix
				ifAbsent: [#absent]) = #absent.
		scope mapPrefix: prefix to: uri.
		self assert:
			(scope
				prefixMappingAt: prefix
				ifAbsent: [#absent]) = uri].! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 4/21/2015 11:49'!
testPrefixesAndURIsDo
	| associations |

	associations := OrderedCollection new.
	self newNamespaceScope prefixesAndURIsDo: [:prefix :uri |
		associations addLast: prefix -> uri].
	self assert: associations isEmpty.

	self newNamespaceScopeWithNamespaces prefixesAndURIsDo: [:prefix :uri |
		associations addLast: prefix -> uri].
	self assert: associations asArray = self namespaces associations asArray.! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 3/6/2016 19:57'!
testResolvePrefix
	| scope |

	scope := self newNamespaceScope.
	self namespaces keysAndValuesDo: [:prefix :uri |
		self assert: (scope resolvePrefix: prefix) isEmpty.
		scope mapPrefix: prefix to: uri.
		self assert: (scope resolvePrefix: prefix) = uri].

	self assert:
		(scope resolvePrefix: 'xml') = scope xmlNamespaceURI.

	self assert:
		(scope resolvePrefix: 'xmlns') = scope xmlnsNamespaceURI.! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 3/6/2016 19:51'!
testResolvePrefixIfUnresolvable
	| scope |

	scope := self newNamespaceScope.
	self namespaces keysAndValuesDo: [:prefix :uri |
		self assert:
			(scope
				resolvePrefix: prefix
				ifUnresolvable: [#unresolvable]) = #unresolvable.
		scope mapPrefix: prefix to: uri.
		self assert:
			(scope
				resolvePrefix: prefix
				ifUnresolvable: [#unresolvable]) = uri].

	self assert:
		(scope
			resolvePrefix: 'xml'
			ifUnresolvable: [#unresolvable]) = scope xmlNamespaceURI.

	self  assert:
		(scope
			resolvePrefix: 'xmlns'
			ifUnresolvable: [#unresolvable]) = scope xmlnsNamespaceURI.! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 3/6/2016 18:41'!
testResolvePrefixesOfNamespaceURI
	| scope |

	scope := self newNamespaceScope.

	self namespaces keysAndValuesDo: [:prefix :uri |
		self assert:
			(scope resolvePrefixesOfNamespaceURI: uri) isEmpty.

		scope
			mapPrefix: prefix
			to: uri.
		self assert:
			(scope resolvePrefixesOfNamespaceURI: uri) =
				(Set with: prefix).

		scope
			mapPrefix: 'newPrefix'
			to: uri.
		self assert:
			(scope resolvePrefixesOfNamespaceURI: uri) =
				(Set with: prefix with: 'newPrefix')].

	self assert:
		(scope resolvePrefixesOfNamespaceURI: scope xmlNamespaceURI) =
			(Set with: 'xml').
	self assert:
		(scope resolvePrefixesOfNamespaceURI: scope xmlnsNamespaceURI) =
			(Set with: 'xmlns').

	"cannot map xmlns prefix at all, but can map xml to the same URI"
	scope
		mapPrefix: 'xml'
		to: scope xmlNamespaceURI.
	self assert:
		(scope resolvePrefixesOfNamespaceURI: scope xmlNamespaceURI) =
			(Set with: 'xml').! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 12/28/2014 23:30'!
testSize
	self
		assert: self newNamespaceScope size = 0;
		assert: self newNamespaceScopeWithNamespaces size = self namespaces size! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 2/26/2016 05:51'!
testUnmapPrefix
	| scope |

	scope := self newNamespaceScope.
	self namespaces keysAndValuesDo: [:prefix :uri |  | oldPrefixMappings |
		self
			should: [scope unmapPrefix: prefix]
			raise: XMLNamespaceException.
		scope
			mapPrefix: prefix
			to: uri.
	
		oldPrefixMappings := scope prefixMappings.
		self assert: (scope unmapPrefix: prefix) = uri.

		"ensure prefixMappings is copied when removing"
		self
			deny: (scope prefixMappings includesKey: prefix);
			deny: scope prefixMappings = oldPrefixMappings;
			deny: scope prefixMappings == oldPrefixMappings.
		self
			should: [scope unmapPrefix: prefix]
			raise: XMLNamespaceException].
	self assert: scope prefixMappings isEmpty.! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 2/24/2016 13:42'!
testUnmapPrefixIfAbsent
	| scope |

	scope := self newNamespaceScope.
	self namespaces keysAndValuesDo: [:prefix :uri |  | oldPrefixMappings |
		self assert:
			(scope
				unmapPrefix: prefix
				ifAbsent: [#absent]) = #absent.
		scope
			mapPrefix: prefix
			to: uri.

		oldPrefixMappings := scope prefixMappings.
		self assert:
			(scope
				unmapPrefix: prefix
				ifAbsent: [#absent]) = uri.

		"ensure prefixMappings is copied when removing"
		self
			deny: (scope prefixMappings includesKey: prefix);
			deny: scope prefixMappings = oldPrefixMappings;
			deny: scope prefixMappings == oldPrefixMappings.

		self
			assert:
				(scope
					unmapPrefix: prefix
					ifAbsent: [#absent]) = #absent].
	self assert: scope prefixMappings isEmpty.! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 2/17/2016 00:33'!
testXMLNSNamespaceURI
	| namespaceURI |

	namespaceURI := 'http://www.w3.org/2000/xmlns/'.
	self
		assert: self newNamespaceScope xmlnsNamespaceURI = namespaceURI;
		assert: XMLNamespaceScope xmlnsNamespaceURI = namespaceURI.! !

!XMLNamespaceScopeTest methodsFor: 'tests' stamp: 'monty 2/17/2016 00:33'!
testXMLNamespaceURI
	| namespaceURI |

	namespaceURI := 'http://www.w3.org/XML/1998/namespace'.
	self
		assert: self newNamespaceScope xmlNamespaceURI = namespaceURI;
		assert: XMLNamespaceScope xmlNamespaceURI = namespaceURI.! !

TestCase subclass: #XMLNodeFactoryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLNodeFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 15:59'!
cdataClass
	^ XMLCData! !

!XMLNodeFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 15:59'!
commentClass
	^ XMLComment! !

!XMLNodeFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 16:00'!
documentClass
	^ XMLDocument! !

!XMLNodeFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 16:01'!
elementClass
	^ XMLElement! !

!XMLNodeFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 15:56'!
factoryClass
	^ XMLNodeFactory! !

!XMLNodeFactoryTest methodsFor: 'instance creation' stamp: 'monty 11/1/2014 15:56'!
newFactory
	^ self factoryClass new! !

!XMLNodeFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 16:02'!
piClass
	^ XMLPI! !

!XMLNodeFactoryTest methodsFor: 'source XML' stamp: 'monty 11/1/2014 21:32'!
sampleUserXML
	^
	XMLWriter new
		enablePrettyPrinting;
		writeWith: [:writer |
			writer pi
				target: 'xml-stylesheet';
				data: 'type="text/css" href="sample.css"'.
			writer comment: 'sample'.
			writer tag
				name: 'user:user';
				xmlnsAt: 'user' put: 'urn:user';
				with: [
					writer
						tag: 'user:username' with: 'jsmith';
						tag: 'user:real-name' with: 'John Smith';
						tag: 'user:date-registered' with: [writer cdata: DateAndTime now printString];
						tag: 'user:date-last-active' with: [writer cdata: DateAndTime now printString];
						tag: 'table-name' with: 'users';
						tag: 'row-id' with: '1000';
						tag: 'serializationFormat' with: 'XML']];
		contents! !

!XMLNodeFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 15:59'!
stringClass
	^ XMLString! !

!XMLNodeFactoryTest methodsFor: 'tests' stamp: 'monty 4/17/2015 10:49'!
testAsPluggableNodeFactory
	| factory pluggableFactory |

	factory := self factoryClass new.
	pluggableFactory := factory asPluggableNodeFactory.
	self
		assert: (pluggableFactory isKindOf: XMLPluggableNodeFactory);
		deny: pluggableFactory == factory.! !

!XMLNodeFactoryTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:02'!
testNewCData
	| cdata |

	cdata := self newFactory newCData: 'test'.
	self
		assert: cdata class == self cdataClass;
		assert: cdata string = 'test'.! !

!XMLNodeFactoryTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:02'!
testNewComment
	| comment |

	comment := self newFactory newComment: 'test'.
	self
		assert: comment class == self commentClass;
		assert: comment string = 'test'.! !

!XMLNodeFactoryTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:03'!
testNewDocument
	| newDocument |

	newDocument := self newFactory newDocument.
	self assert: newDocument class == self documentClass.! !

!XMLNodeFactoryTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:05'!
testNewElement
	| element |

	element := self newFactory newElement.
	self assert: element class == self elementClass.! !

!XMLNodeFactoryTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:05'!
testNewElementNamed
	| element |

	element := self newFactory newElementNamed: 'test'.
	self
		assert: element class == self elementClass;
		assert: element name = 'test'.! !

!XMLNodeFactoryTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:07'!
testNewElementNamedNamespaceURI
	| element |

	element := self newFactory newElementNamed: 'prefix:test' namespaceURI: 'prefixNS'.
	self
		assert: element class == self elementClass;
		assert: element name = 'prefix:test';
		assert: element namespaceURI = 'prefixNS'.! !

!XMLNodeFactoryTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:06'!
testNewPI
	self assert: self newFactory newPI class == self piClass! !

!XMLNodeFactoryTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:06'!
testNewString
	| string |

	string := self newFactory newString: 'test'.
	self
		assert: string class == self stringClass;
		assert: string string = 'test'.! !

!XMLNodeFactoryTest methodsFor: 'tests' stamp: 'monty 11/4/2014 20:23'!
testParsingSample
	| document factory |

	factory := self newFactory.
	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory: factory;
			preservesCDataNodes: true;
			preservesCommentNodes: true;
			parseDocument.
	self
		assert: document class == self documentClass;
		assert: document nodeFactory == factory.

	document allElementsDo: [:each |
		self
			assert: each class == self elementClass;
			assert: each nodeFactory == factory].

	document allNodesDo: [:each |
		each isStringNode
			ifTrue: [
				self assert:
					each class == 
						(each isCData
							ifTrue: [self cdataClass]
							ifFalse: [self stringClass])].
		each isComment
			ifTrue: [self assert: each class == self commentClass].
		each isPI
			ifTrue: [self assert: each class == self piClass]].! !

XMLNodeFactoryTest subclass: #XMLPluggableNodeFactoryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

XMLPluggableNodeFactoryTest subclass: #XMLPluggableElementFactoryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLPluggableElementFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 22:37'!
dateElementClass
	^ XMLFDateElement! !

!XMLPluggableElementFactoryTest methodsFor: 'accessing' stamp: 'monty 11/2/2014 09:38'!
defaultElementClass
	^ self elementClass! !

!XMLPluggableElementFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 15:58'!
factoryClass
	^ XMLPluggableElementFactory! !

!XMLPluggableElementFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 22:38'!
realNameElementClass
	^ XMLFRealNameElement! !

!XMLPluggableElementFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 22:38'!
serializationFormatElementClass
	^ XMLFSerializationFormatElement! !

!XMLPluggableElementFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 22:39'!
tableElementClass
	^ XMLFTableElement! !

!XMLPluggableElementFactoryTest methodsFor: 'tests' stamp: 'monty 7/20/2015 21:34'!
testAddElementNamed
	| element userElement dataElement |

	element := XMLElement name: 'root'.
	element
		defaultNamespace: 'dataNS';
		declarePrefix: 'prefix' uri: 'prefixNS'.
	element nodeFactory:
		(XMLPluggableElementFactory new
			handleElement: 'data'
			namespaceURI: 'dataNS'
			withClass: XMLFElement;
			handleElement: 'user'
			namespaceURI: 'prefixNS'
			withClass: XMLFUserElement).

	userElement := element addElementNamed: 'prefix:user'.
	self
		assert: userElement class == XMLFUserElement;
		assert: (userElement isNamed: 'prefix:user');
		assert: userElement namespaceURI = 'prefixNS'.

	dataElement := element addElementNamed: 'data'.
	self
		assert: dataElement class == XMLFElement;
		assert: (dataElement isNamed: 'data');
		assert: dataElement namespaceURI = 'dataNS'.

	self assert: (element addElementNamed: 'default') class == XMLElement.! !

!XMLPluggableElementFactoryTest methodsFor: 'tests' stamp: 'monty 7/15/2015 09:26'!
testCopy
	| factory copy |

	factory := self newFactory.
	factory handleElement: 'user' withClass: XMLFUserElement.
	copy := factory copy.
	self
		deny: copy == factory;
		deny: copy elementClasses == factory elementClasses;
		assert: copy elementClasses = factory elementClasses.

	copy handleElement: 'user' withClass: XMLElement.
	self deny: copy elementClasses = factory elementClasses.! !

!XMLPluggableElementFactoryTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 18:25'!
testElementClassAccessor
	| document |

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory: XMLPluggableElementFactory new;
			parseDocument.
	self assert: (document allElementsDetect: [:each | (each isMemberOf: XMLElement) not]) isNil.

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory: (XMLPluggableElementFactory new elementClass: self defaultElementClass);
			parseDocument.
	self assert: (document allElementsDetect: [:each | (each isMemberOf: self defaultElementClass) not]) isNil.! !

!XMLPluggableElementFactoryTest methodsFor: 'tests' stamp: 'monty 11/1/2014 22:27'!
testElementClassNameMatching
	| document |

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory:
				(self newFactory
					handleElementsMatchingClass: self userElementClass;
					handleElementsMatchingClasses:
						(Array
							with: self realNameElementClass
							with: self serializationFormatElementClass));
			parseDocument.

	self
		assert: ((document root) isMemberOf: self userElementClass);
		assert: ((document root elementAt: 'real-name') isMemberOf: self realNameElementClass);
		assert:
			((document root elementAt: 'serializationFormat')
				isMemberOf: self serializationFormatElementClass).! !

!XMLPluggableElementFactoryTest methodsFor: 'tests' stamp: 'monty 11/1/2014 22:28'!
testElementNameAndNamespaceMatching
	| factory document root |

	(factory := self newFactory)
		elementClass: self defaultElementClass;
		handleElement: 'user:user' namespaceURI: 'urn:user' withClass: self userElementClass;
		handleElement: 'user:user' withClass: self wrongElementClass;
		handleElement: 'user' namespaceURI: 'urn:user' withClass: self wrongElementClass;
		handleElement: 'user' withClass: self wrongElementClass;
		handleElement: 'user:username' withClass: self usernameElementClass;
		handleElement: 'username' namespaceURI: 'urn:user' withClass: self wrongElementClass;
		handleElement: 'username' withClass: self wrongElementClass;
		handleElement: 'real-name' namespaceURI: 'urn:user' withClass: self realNameElementClass;
		handleElement: 'real-name' withClass: self wrongElementClass;
		handleElement: 'date-registered' withClass: self dateElementClass;
		handleElement: 'user:date-last-active' namespaceURI: 'urn:notuser' withClass: self wrongElementClass;
		handleElement: 'notuser:date-last-active' namespaceURI: 'urn:user' withClass: self wrongElementClass;
		handleElement: 'date-last-active' namespaceURI: 'urn:notuser' withClass: self wrongElementClass;
		handleElement: 'notuser:date-last-active' withClass: self wrongElementClass.

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory: factory;
			parseDocument.
	self
		assert: ((root := document root) isMemberOf: self userElementClass);
		assert: ((root elementAt: 'username') isMemberOf: self usernameElementClass);
		assert: ((root elementAt: 'real-name') isMemberOf: self realNameElementClass);
		assert: ((root elementAt: 'date-registered') isMemberOf: self dateElementClass);
		assert:
			((root elements copyAfter: (root elementAt: 'date-registered'))
				allSatisfy: [:each | each isMemberOf: self defaultElementClass]).! !

!XMLPluggableElementFactoryTest methodsFor: 'tests' stamp: 'monty 11/1/2014 22:28'!
testElementsNameAndNamespaceMatching
	| factory document root |

	factory := self newFactory.
	factory
		handleElements: #('date-registered' 'date-last-active')
		namespaceURI: 'urn:user'
		withClass: self dateElementClass.
	factory
		handleElements: #('date-registered' 'date-last-active')
		withClass: self wrongElementClass.
	factory
		handleElements: #('table-name' 'not-table-name')
		withClass: self tableElementClass.

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory: factory;
			parseDocument.
	root := document root.
	self
		assert: ((root elementAt: 'date-registered') isMemberOf: self dateElementClass);
		assert: ((root elementAt: 'date-last-active') isMemberOf: self dateElementClass);
		assert: ((root elementAt: 'table-name') isMemberOf: self tableElementClass).! !

!XMLPluggableElementFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 22:38'!
userElementClass
	^ XMLFUserElement! !

!XMLPluggableElementFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 22:37'!
usernameElementClass
	^ XMLFUsernameElement! !

!XMLPluggableElementFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 22:37'!
wrongElementClass
	^ XMLFWrongElement! !

!XMLPluggableNodeFactoryTest class methodsFor: 'testing' stamp: 'monty 11/24/2014 07:39'!
shouldInheritSelectors
	^ true! !

!XMLPluggableNodeFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 22:36'!
cdataClass
	^ XMLFCData! !

!XMLPluggableNodeFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 22:36'!
commentClass
	^ XMLFComment! !

!XMLPluggableNodeFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 22:37'!
documentClass
	^ XMLFDocument! !

!XMLPluggableNodeFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 22:37'!
elementClass
	^ XMLFElement! !

!XMLPluggableNodeFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 16:51'!
factoryClass
	^ XMLPluggableNodeFactory! !

!XMLPluggableNodeFactoryTest methodsFor: 'instance creation' stamp: 'monty 11/1/2014 21:44'!
newFactory
	^ self factoryClass new
		documentClass: self documentClass;
		elementClass: self elementClass;
		stringClass: self stringClass;
		cdataClass: self cdataClass;
		commentClass: self commentClass;
		piClass: self piClass! !

!XMLPluggableNodeFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 22:38'!
piClass
	^ XMLFPI! !

!XMLPluggableNodeFactoryTest methodsFor: 'accessing' stamp: 'monty 11/1/2014 22:38'!
stringClass
	^ XMLFString! !

!XMLPluggableNodeFactoryTest methodsFor: 'tests' stamp: 'monty 4/17/2015 10:49'!
testAsPluggableNodeFactory
	| factory pluggableFactory |

	factory := self factoryClass new.
	pluggableFactory := factory asPluggableNodeFactory.
	self
		assert: (pluggableFactory isKindOf: XMLPluggableNodeFactory);
		assert: pluggableFactory == factory.! !

TestCase subclass: #XMLNodeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLNodeTest commentStamp: '<historical>' prior: 0!
This is a base class for node tests.!

XMLNodeTest subclass: #XMLAttributeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLAttributeTest methodsFor: 'accessing' stamp: 'monty 11/23/2014 00:03'!
escapedString
	^ '&lt;test name=&quot;&amp;name;&quot;>''&amp;value;''&lt;/test>'! !

!XMLAttributeTest methodsFor: 'accessing' stamp: 'monty 11/2/2014 11:17'!
nodeClass
	^ XMLAttribute! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 12/6/2014 14:24'!
testCanonicallyEquals
	super testCanonicallyEquals.
	self assert:
		((self nodeClass name: 'one' value: 'two') canonicallyEquals:
			(self nodeClass name: 'one' value: 'two')).
	self deny: ((self nodeClass name: 'one' value: 'two') canonicallyEquals: self nodeClass new).! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 11/22/2014 20:14'!
testCopy
	| node copy |

	(node := self nodeClass new)
		element: XMLElement new.
	copy := node copy.
	self
		deny: copy == node;
		assert: copy element isNil.! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:06'!
testElement
	| node element |

	node := self nodeClass new.
	element := XMLElement new.
	node element: element.
	self assert: node element == element.! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 12/30/2014 02:39'!
testExpandedName
	| element attribute |

	element := XMLElement new.
	attribute := self nodeClass name: 'one' value: 'two'.
	element attributeNodes add: attribute.
	self assert: (attribute expandedName = 'one').

	element defaultNamespace: 'defaultNS'.
	self assert: (attribute expandedName = 'one').

	element declarePrefix: 'prefix' uri: 'prefixNS'.
	attribute name: 'prefix:two'.
	self assert: (attribute expandedName = 'Q{prefixNS}two').! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 12/23/2014 16:18'!
testHasLanguage
	| attribute |

	attribute := self nodeClass new.
	attribute name: 'xml:lang'.
	self
		deny: (attribute hasLanguage: 'en-us');
		deny: (attribute hasLanguage: 'EN-US');
		deny: (attribute hasLanguage: 'En');
		deny: (attribute hasLanguage: 'de-de');
		deny: (attribute hasLanguage: 'DE-DE');
		deny: (attribute hasLanguage: 'De').

	attribute value: 'En-Us'.
	self
		assert: (attribute hasLanguage: 'en-us');
		assert: (attribute hasLanguage: 'EN-US');
		assert: (attribute hasLanguage: 'En');
		deny: (attribute hasLanguage: 'de-de');
		deny: (attribute hasLanguage: 'DE-DE');
		deny: (attribute hasLanguage: 'De').! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 2/13/2015 22:25'!
testHasNamespaceURI
	| element attribute |

	attribute := self nodeClass name: 'test' value: ''.
	self deny: attribute hasNamespaceURI.

	element := XMLElement name: 'test' namespaceURI: 'defaultNS'.
	element attributeNodes add: attribute.
	self deny: attribute hasNamespaceURI.

	element
		declarePrefix: 'prefix'
		uri: 'prefixNS'.
	attribute name: 'prefix:test'.
	self assert: attribute hasNamespaceURI.

	attribute name: 'test'.
	self deny: attribute hasNamespaceURI.! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 11/23/2014 20:28'!
testHasPrefix
	| attribute |

	attribute := self nodeClass new.
	self deny: attribute hasPrefix.

	attribute name: 'one'.
	self deny: attribute hasPrefix.

	attribute name: 'prefix:two'.
	self assert: attribute hasPrefix.

	attribute name: 'three'.
	self deny: attribute hasPrefix.! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 1/29/2015 00:31'!
testIsAttribute
	self assert: self nodeClass new isAttribute! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 12/23/2014 21:04'!
testIsInLanguage
	| element |

	super testIsInLanguage.
	element := self newElementWithAttributes.	
	element attributeNodesDo: [:each |
		self
			deny: (each isInLanguage: 'en');
			deny: (each isInLanguage: 'EN');
			deny: (each isInLanguage: 'En');
			deny: (each isInLanguage: 'en-us');
			deny: (each isInLanguage: 'EN-US');
			deny: (each isInLanguage: 'En-Us')].

	element attributeAt: 'xml:lang' put: 'En'.
	element attributeNodesDo: [:each |
		self
			assert: (each isInLanguage: 'en');
			assert: (each isInLanguage: 'EN');
			assert: (each isInLanguage: 'En');
			deny: (each isInLanguage: 'en-us');
			deny: (each isInLanguage: 'EN-US');
			deny: (each isInLanguage: 'En-Us')].! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 11/23/2014 05:01'!
testIsNamed
	| attribute |

	(attribute := self nodeClass new) name: 'one'.
	self
		assert: (attribute isNamed: 'one');
		deny: (attribute isNamed: 'two').
	attribute name: 'one:two'.
	self
		assert: (attribute isNamed: 'one:two');
		assert: (attribute isNamed: 'two');
		deny: (attribute isNamed: 'one').! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 11/23/2014 05:06'!
testIsNamedAny
	| attribute |

	(attribute := self nodeClass new) name: 'one'.
	self
		assert: (attribute isNamedAny: #('two' 'one' 'three'));
		deny: (attribute isNamedAny: #('two' 'three')).
	attribute name: 'one:two'.
	self
		assert: (attribute isNamedAny: #('one' 'one:two' 'three'));
		assert: (attribute isNamedAny: #('one' 'two' 'three'));
		deny: (attribute isNamedAny: #('one' 'three')).! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 11/23/2014 20:21'!
testLocalName
	| attribute |

	attribute := self nodeClass new.
	self assert: attribute name isEmpty.

	attribute name: 'one'.
	self assert: attribute localName = 'one'.

	attribute name: 'two'.
	self assert: attribute localName = 'two'.

	attribute name: 'prefix:three'.
	self assert: attribute localName = 'three'.

	attribute name: 'four'.
	self assert: attribute localName = 'four'.! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 11/23/2014 20:23'!
testName
	| attribute |

	attribute := self nodeClass new.
	self assert: attribute name isEmpty.

	attribute name: 'one'.
	self assert: attribute name = 'one'.

	attribute name: 'prefix:two'.
	self assert: attribute name = 'prefix:two'.! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 11/23/2014 04:55'!
testNameValue
	| attribute |

	attribute := self nodeClass name: 'one' value: 'two'.
	self
		assert: attribute name = 'one';
		assert: attribute value = 'two'! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 11/23/2014 20:33'!
testNamespaceURI
	| element attribute|

	element := XMLElement new.
	attribute := self nodeClass new.
	element attributeNodes add: attribute.
	self assert: attribute namespaceURI isEmpty.

	element defaultNamespace: 'defaultNS'.
	self assert: attribute namespaceURI isEmpty.

	element declarePrefix: 'prefix' uri: 'prefixNS'.
	attribute name: 'prefix:test'.
	self assert: attribute namespaceURI = 'prefixNS'.
	
	attribute name: 'test'.
	self assert: attribute namespaceURI isEmpty.! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 11/23/2014 20:27'!
testPrefix
	| attribute |

	attribute := self nodeClass new.
	self assert: attribute prefix isEmpty.

	attribute name: 'one'.
	self assert: attribute prefix isEmpty.

	attribute name: 'prefix:two'.
	self assert: attribute prefix = 'prefix'.

	attribute name: 'three'.
	self assert: attribute prefix isEmpty.! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 12/3/2014 01:10'!
testSortKey
	self assert: (self nodeClass new name: 'test') sortKey = 'test'! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 11/23/2014 20:16'!
testValue
	self
		assert: self nodeClass new value isEmpty;
		assert: (self nodeClass new value: 'test') value = 'test'! !

!XMLAttributeTest methodsFor: 'tests' stamp: 'monty 11/23/2014 05:17'!
testWriteXMLOn
	self
		assert: (self nodeClass name: 'name' value: '') printString = 'name=""';
		assert: (self nodeClass name: 'name' value: 'value') printString = 'name="value"';
		assert:
			(self nodeClass
				name: 'name'
				value: self markupString) printString = ('name="', self escapedString, '"')! !

XMLNodeTest subclass: #XMLCommentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLCommentTest methodsFor: 'accessing' stamp: 'monty 11/2/2014 11:17'!
nodeClass
	^ XMLComment! !

!XMLCommentTest methodsFor: 'tests' stamp: 'monty 12/6/2014 14:27'!
testCanonicallyEquals
	super testCanonicallyEquals.
	self
		assert: ((self nodeClass string: 'test') canonicallyEquals: (self nodeClass string: 'test'));
		deny: ((self nodeClass string: 'test') canonicallyEquals: self nodeClass new).! !

!XMLCommentTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:47'!
testIsComment
	self assert: self nodeClass new isComment! !

!XMLCommentTest methodsFor: 'tests' stamp: 'monty 12/3/2014 01:09'!
testSortKey
	self assert: (self nodeClass string: 'test') sortKey = 'test'! !

!XMLCommentTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:25'!
testString
	| comment |

	comment := self nodeClass new.
	self assert: comment string isEmpty.
	comment string: self markupString.
	self assert: comment string = self markupString.

	self assert: (self nodeClass string: self markupString) string = self markupString.! !

!XMLCommentTest methodsFor: 'tests' stamp: 'monty 11/22/2014 19:27'!
testWriteXMLOn
	self
		assert: self nodeClass new printString = '<!!---->';
		assert: (self nodeClass string: self markupString) printString = ('<!!--', self markupString, '-->')! !

XMLNodeTest subclass: #XMLDeclarationNodeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLDeclarationNodeTest commentStamp: 'monty 11/23/2014 18:43' prior: 0!
Abstract, do not run.!

XMLDeclarationNodeTest subclass: #XMLAttributeDeclarationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLAttributeDeclarationTest class methodsFor: 'testing' stamp: 'monty 11/23/2014 19:21'!
isAbstract
	^ false! !

!XMLAttributeDeclarationTest methodsFor: 'accessing' stamp: 'monty 11/23/2014 19:22'!
nodeClass
	^ XMLAttributeDeclaration! !

!XMLAttributeDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:24'!
testAttribute
	self
		assert: self nodeClass new attribute isEmpty;
		assert: (self nodeClass new attribute: 'test') attribute = 'test'! !

!XMLAttributeDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:34'!
testDefaultPragma
	self
		assert: self nodeClass new defaultPragma isEmpty;
		assert: (self nodeClass new defaultPragma: 'REQUIRED') defaultPragma = 'REQUIRED'! !

!XMLAttributeDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:24'!
testDefaultValue
	self
		assert: self nodeClass new defaultValue isEmpty;
		assert: (self nodeClass new defaultValue: 'test') defaultValue = 'test'! !

!XMLAttributeDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:24'!
testElement
	self
		assert: self nodeClass new element isEmpty;
		assert: (self nodeClass new element: 'test') element = 'test'! !

!XMLAttributeDeclarationTest methodsFor: 'tests' stamp: 'monty 11/24/2014 06:50'!
testElementAttributeTypeDefaultPragmaDefaultValue
	| declaration |

	declaration :=
		self nodeClass
			element: 'one'
			attribute: 'two'
			type: 'CDATA'
			defaultPragma: '#IMPLIED'
			defaultValue: 'default'.
	self
		assert: declaration element = 'one';
		assert: declaration attribute = 'two';
		assert: declaration type = 'CDATA';
		assert: declaration defaultPragma = '#IMPLIED';
		assert: declaration defaultValue = 'default'.! !

!XMLAttributeDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:34'!
testType
	self
		assert: self nodeClass new type isEmpty;
		assert: (self nodeClass new type: 'CDATA') type = 'CDATA'! !

!XMLAttributeDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:33'!
testWriteXMLOn
	self assert:
		(self nodeClass
			element: 'one'
			attribute: 'two'
			type: 'CDATA'
			defaultPragma: ''
			defaultValue: 'three') printString = '<!!ATTLIST one two CDATA "three">'.
	self assert:
		(self nodeClass
			element: 'one'
			attribute: 'two'
			type: 'ENTITY'
			defaultPragma: 'FIXED'
			defaultValue: 'three') printString = '<!!ATTLIST one two ENTITY #FIXED "three">'.
	self assert:
		(self nodeClass
			element: 'one'
			attribute: 'two'
			type: '(three|four)'
			defaultPragma: '#IMPLIED'
			defaultValue: 'four') printString = '<!!ATTLIST one two (three|four) #IMPLIED "four">'.! !

!XMLDeclarationNodeTest class methodsFor: 'testing' stamp: 'monty 11/23/2014 18:14'!
isAbstract
	^ true! !

!XMLDeclarationNodeTest methodsFor: 'accessing' stamp: 'monty 11/23/2014 18:11'!
nodeClass
	^ XMLDeclarationNode! !

!XMLDeclarationNodeTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:11'!
testIsDeclaration
	self assert: self nodeClass new isDeclaration! !

XMLDeclarationNodeTest subclass: #XMLElementDeclarationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLElementDeclarationTest class methodsFor: 'testing' stamp: 'monty 11/23/2014 18:27'!
isAbstract
	^ false! !

!XMLElementDeclarationTest methodsFor: 'accessing' stamp: 'monty 11/23/2014 18:25'!
nodeClass
	^ XMLElementDeclaration! !

!XMLElementDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:26'!
testContentModel
	self
		assert: self nodeClass new contentModel isEmpty;
		assert: (self nodeClass new contentModel: '(one|two)') contentModel = '(one|two)'! !

!XMLElementDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:25'!
testElement
	self
		assert: self nodeClass new element isEmpty;
		assert: (self nodeClass new element: 'test') element = 'test'! !

!XMLElementDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:28'!
testElementContentModel
	| declaration |

	declaration :=
		self nodeClass
			element: 'one'
			contentModel: '(two|three)'.
	self
		assert: declaration element = 'one';
		assert: declaration contentModel = '(two|three)'! !

!XMLElementDeclarationTest methodsFor: 'assertions' stamp: 'monty 11/23/2014 18:31'!
testWriteXMLOn
	self assert:
		(self nodeClass
			element: 'one'
			contentModel: '(two | three)') printString = '<!!ELEMENT one (two | three)>'.
	self assert:
		(self nodeClass
			element: 'one'
			contentModel: 'EMPTY') printString = '<!!ELEMENT one EMPTY>'.! !

XMLDeclarationNodeTest subclass: #XMLEntityDeclarationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLEntityDeclarationTest commentStamp: 'monty 11/23/2014 18:46' prior: 0!
Abstract, do not run.!

!XMLEntityDeclarationTest class methodsFor: 'testing' stamp: 'monty 11/23/2014 18:47'!
isAbstract
	^ true! !

!XMLEntityDeclarationTest methodsFor: 'accessing' stamp: 'monty 11/23/2014 18:46'!
nodeClass
	^ XMLEntityDeclaration! !

!XMLEntityDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:02'!
testIsParameter
	self deny: self nodeClass new isParameter! !

!XMLEntityDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:49'!
testName
	self
		assert: self nodeClass new name isEmpty;
		assert: (self nodeClass new name: 'test') name = 'test'! !

!XMLEntityDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:59'!
testNamePublicIDSystemID
	| declaration |

	declaration := self nodeClass name: 'one' publicID: 'two' systemID: 'three'.
	self
		assert: declaration name = 'one';
		assert: declaration publicID = 'two';
		assert: declaration systemID = 'three'.! !

!XMLEntityDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:58'!
testNameReplacement
	| declaration |

	declaration := self nodeClass name: 'one' replacement: 'two'.
	self
		assert: declaration name = 'one';
		assert: declaration replacement = 'two'.! !

!XMLEntityDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:50'!
testPublicID
	self
		assert: self nodeClass new publicID isEmpty;
		assert: (self nodeClass new publicID: 'test') publicID = 'test'! !

!XMLEntityDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:49'!
testReplacement
	self
		assert: self nodeClass new replacement isEmpty;
		assert: (self nodeClass new replacement: 'test') replacement = 'test'! !

!XMLEntityDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:50'!
testSystemID
	self
		assert: self nodeClass new systemID isEmpty;
		assert: (self nodeClass new systemID: 'test') systemID = 'test'! !

XMLEntityDeclarationTest subclass: #XMLGeneralEntityDeclarationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLGeneralEntityDeclarationTest class methodsFor: 'testing' stamp: 'monty 11/23/2014 18:47'!
isAbstract
	^ false! !

!XMLGeneralEntityDeclarationTest methodsFor: 'accessing' stamp: 'monty 11/23/2014 18:59'!
nodeClass
	^ XMLGeneralEntityDeclaration! !

!XMLGeneralEntityDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:03'!
testNamePublicIDSystemIDNdata
	| declaration |

	declaration := self nodeClass name: 'one' publicID: 'two' systemID: 'three' ndata: 'four'.
	self
		assert: declaration name = 'one';
		assert: declaration publicID = 'two';
		assert: declaration systemID = 'three';
		assert: declaration ndata = 'four'.! !

!XMLGeneralEntityDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:53'!
testNdata
	self
		assert: self nodeClass new ndata isEmpty;
		assert: (self nodeClass new ndata: 'test') ndata = 'test'! !

!XMLGeneralEntityDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:07'!
testWriteXMLOn
	self assert:
		(self nodeClass name: 'one' replacement: 'two') printString =
			'<!!ENTITY one "two">'.
	self assert:
		(self nodeClass name: 'one' publicID: 'two' systemID: 'three') printString =
			'<!!ENTITY one PUBLIC "two" "three">'.
	self assert:
		(self nodeClass name: 'one' publicID: '' systemID: 'three') printString =
			'<!!ENTITY one SYSTEM "three">'.
	self assert:
		(self nodeClass name: 'one' publicID: 'two' systemID: 'three' ndata: 'four') printString =
			'<!!ENTITY one PUBLIC "two" "three" NDATA four>'.! !

XMLEntityDeclarationTest subclass: #XMLParameterEntityDeclarationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLParameterEntityDeclarationTest class methodsFor: 'testing' stamp: 'monty 11/23/2014 19:18'!
isAbstract
	^ false! !

!XMLParameterEntityDeclarationTest methodsFor: 'accessing' stamp: 'monty 11/23/2014 19:01'!
nodeClass
	^ XMLParameterEntityDeclaration! !

!XMLParameterEntityDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:01'!
testIsParameter
	self assert: self nodeClass new isParameter! !

!XMLParameterEntityDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:07'!
testWriteXMLOn
	self assert:
		(self nodeClass name: 'one' replacement: 'two') printString =
			'<!!ENTITY % one "two">'.
	self assert:
		(self nodeClass name: 'one' publicID: 'two' systemID: 'three') printString =
			'<!!ENTITY % one PUBLIC "two" "three">'.
	self assert:
		(self nodeClass name: 'one' publicID: '' systemID: 'three') printString =
			'<!!ENTITY % one SYSTEM "three">'.! !

XMLDeclarationNodeTest subclass: #XMLNotationDeclarationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLNotationDeclarationTest class methodsFor: 'testing' stamp: 'monty 11/23/2014 19:12'!
isAbstract
	^ false! !

!XMLNotationDeclarationTest methodsFor: 'accessing' stamp: 'monty 11/23/2014 19:15'!
nodeClass
	^ XMLNotationDeclaration! !

!XMLNotationDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:12'!
testName
	self
		assert: self nodeClass new name isEmpty;
		assert: (self nodeClass new name: 'test') name = 'test'! !

!XMLNotationDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:18'!
testNamePublicIDSystemID
	| declaration |

	declaration := self nodeClass name: 'one' publicID: 'two' systemID: 'three'.
	self
		assert: declaration name = 'one';
		assert: declaration publicID = 'two';
		assert: declaration systemID = 'three'.! !

!XMLNotationDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:12'!
testPublicID
	self
		assert: self nodeClass new publicID isEmpty;
		assert: (self nodeClass new publicID: 'test') publicID = 'test'! !

!XMLNotationDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:12'!
testSystemID
	self
		assert: self nodeClass new systemID isEmpty;
		assert: (self nodeClass new systemID: 'test') systemID = 'test'! !

!XMLNotationDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 19:20'!
testWriteXMLOn
	self assert:
		(self nodeClass name: 'one' publicID: 'two' systemID: 'three') printString =
			'<!!NOTATION one PUBLIC "two" "three">'.
	self assert:
		(self nodeClass name: 'one' publicID: '' systemID: 'two') printString =
			'<!!NOTATION one SYSTEM "two">'.
	self assert:
		(self nodeClass name: 'one' publicID: 'two' systemID: '') printString =
			'<!!NOTATION one PUBLIC "two">'.! !

!XMLNodeTest class methodsFor: 'testing' stamp: 'monty 11/2/2014 10:56'!
shouldInheritSelectors
	^ true! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'monty 11/7/2014 14:19'!
assertList: aList collects: aSecondList
	self
		assert: aList = aSecondList;
		assert: aList class == self collectListClass.! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'monty 3/9/2015 19:06'!
assertList: aList selects: aSecondList
	self
		assert: aList = aSecondList;
		assert: aList class == self selectListClass.! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'monty 2/24/2016 15:26'!
attributeAssociations
	^ Array
		with: 'one' -> 'two'
		with: 'three' -> 'four'
		with: 'prefix:three' -> 'five'
		with: 'prefix:six' -> 'seven'! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'monty 2/24/2016 15:26'!
attributes
	^ OrderPreservingStringDictionary newFrom: self attributeAssociations! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'monty 11/7/2014 14:19'!
collectListClass
	^ XMLOrderedList! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'monty 12/1/2014 09:31'!
escapedString
	^ '&lt;test name="&amp;name;"&gt;''&amp;value;''&lt;/test&gt;'! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'monty 11/7/2014 13:56'!
markupString
	^ '<test name="&name;">''&value;''</test>'! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'monty 2/24/2016 16:31'!
namespaceAssociations
	^ Array
		with: '' -> 'defaultNS'
		with: 'prefix' -> 'prefixNS'
		with: 'prefixTwo' -> 'prefixTwoNS'! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'monty 2/24/2016 15:23'!
namespaces
	^ OrderPreservingStringDictionary newFrom: self namespaceAssociations! !

!XMLNodeTest methodsFor: 'instance creation' stamp: 'monty 2/24/2016 16:23'!
newElementWithAttributes
	| element |

	element :=
		XMLElement
			name: 'test'
			namespaces: self namespaceAssociations.
	self attributeAssociations do: [:each |
		element attributeNodes addLast:
			(XMLAttribute
				name: each key
				value: each value)].
	^ element.! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'monty 11/2/2014 10:59'!
nodeClass
	^ XMLNode! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'monty 3/9/2015 19:06'!
selectListClass
	^ XMLNodeList! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:25'!
should: aBlock enumerate: aCollection
	| enumerated |

	enumerated := OrderedCollection new.
	aBlock value: [:each | enumerated add: each].
	self assert: enumerated = aCollection asOrderedCollection.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/24/2014 18:19'!
testAddContent
	self should: [self nodeClass new addContent: 'test'] raise: Error! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:14'!
testAncestorElements
	self assert: self nodeClass new ancestorElements isEmpty! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:16'!
testAncestorElementsDo
	| node |

	node := self nodeClass new.
	self should: [:block |  node ancestorElementsDo: block] enumerate: #().! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:13'!
testAncestorNodes
	self assert: self nodeClass new ancestorNodes isEmpty! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:16'!
testAncestorNodesDo
	| node |

	node := self nodeClass new.
	self should: [:block |  node ancestorNodesDo: block] enumerate: #().! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/23/2014 20:54'!
testCanHaveChildren
	self deny: self nodeClass new canHaveChildren! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 12/6/2014 14:16'!
testCanonicallyEquals
	self assert: (self nodeClass new canonicallyEquals: self nodeClass new)! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/24/2014 05:05'!
testConfiguration
	self assert: self nodeClass new configuration isNil! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:07'!
testContentString
	self assert: self nodeClass new contentString isEmpty! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/22/2014 20:15'!
testCopy
	| node copy |

	(node := self nodeClass new)
		parent: self nodeClass new.
	copy := node copy.
	self
		deny: copy == node;
		deny: copy hasParent. ! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/24/2014 00:12'!
testDocument
	self assert: self nodeClass new document isNil! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:08'!
testHasChildren
	self deny: self nodeClass new hasChildren! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:12'!
testHasParent
	| node |

	node := self nodeClass new.
	self deny: node hasParent.

	node parent: self nodeClass new.
	self assert: node hasParent.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 1/29/2015 00:31'!
testIsAttribute
	self deny: self nodeClass new isAttribute! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:40'!
testIsCData
	self deny: self nodeClass new isCData! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:40'!
testIsComment
	self deny: self nodeClass new isComment! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:40'!
testIsContentNode
	self deny: self nodeClass new isContentNode! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:39'!
testIsDeclaration
	self deny: self nodeClass new isDeclaration! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:39'!
testIsDocument
	self deny: self nodeClass new isDocument! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:35'!
testIsElement
	self deny: self nodeClass new isElement! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:41'!
testIsElementNamed
	self deny: (self nodeClass new isElementNamed: '')! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:35'!
testIsElementNamedAny
	self deny: (self nodeClass new isElementNamedAny: #())! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 12/23/2014 16:23'!
testIsInLanguage
	self deny: (self nodeClass new isInLanguage: 'test')! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:38'!
testIsPI
	self deny: self nodeClass new isPI! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/20/2014 02:49'!
testIsStringNode
	self
		deny: self nodeClass new isStringNode;
		deny: (self nodeClass new isStringNode: '').! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 20:58'!
testNextNode
	self assert: self nodeClass new nextNode isNil! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:11'!
testParent
	| node parent |

	node := self nodeClass new.
	self assert: node parent isNil.

	parent := self nodeClass new.
	node parent: parent.
	self assert: node parent == parent.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 20:58'!
testPreviousNode
	self assert: self nodeClass new previousNode isNil! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 11/19/2014 23:47'!
testRawContentString
	self assert: self nodeClass new rawContentString isEmpty! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 12/2/2014 22:58'!
testSortKey
	self assert: self nodeClass new sortKey = self nodeClass name asString! !

!XMLNodeTest methodsFor: 'tests' stamp: 'monty 4/6/2015 06:41'!
testValidate
	self
		shouldnt: [self nodeClass new validate]
		raise: XMLValidationException! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'monty 2/24/2016 15:31'!
unprefixedAttributeAssociations
	^ self attributeAssociations collect: [:each |
		each key localNameAfterPrefix -> each value]! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'monty 2/24/2016 15:24'!
unprefixedAttributes
	^ OrderPreservingStringDictionary newFrom: self unprefixedAttributeAssociations! !

XMLNodeTest subclass: #XMLNodeWithChildrenTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLNodeWithChildrenTest commentStamp: 'monty 11/4/2014 19:05' prior: 0!
Abstract, do not run.!

XMLNodeWithChildrenTest subclass: #XMLDoctypeDeclarationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLDoctypeDeclarationTest class methodsFor: 'testing' stamp: 'monty 11/2/2014 11:06'!
isAbstract
	^ false! !

!XMLDoctypeDeclarationTest methodsFor: 'instance creation' stamp: 'monty 11/22/2014 20:38'!
newNodeWithNodes
	^ self nodeClass new
		addNode: (XMLComment string: 'before');
		addNode: (XMLGeneralEntityDeclaration name: 'one' replacement: 'two');
		addNode: (XMLParameterEntityDeclaration name: 'three' publicID: 'four' systemID: 'four');
		addNode: (XMLElementDeclaration element: 'five' contentModel: '(six|sevent|eight)');
		addNode:
			(XMLAttributeDeclaration
				element: 'five'
				attribute: 'nine'
				type: 'CDATA'
				defaultPragma: '#IMPLIED'
				defaultValue: 'ten');
		addNode:
			(XMLNotationDeclaration
				name: 'eleven'
				publicID: 'twelve'
				systemID: 'twelve');
		addNode: (XMLPI target: 'thirteen' data: 'fourteen');
		addNode: (XMLComment string: 'after');
		yourself! !

!XMLDoctypeDeclarationTest methodsFor: 'accessing' stamp: 'monty 11/2/2014 11:02'!
nodeClass
	^ XMLDoctypeDeclaration! !

!XMLDoctypeDeclarationTest methodsFor: 'accessing' stamp: 'monty 3/9/2015 19:06'!
selectListClass
	^ XMLInternalSubsetList! !

!XMLDoctypeDeclarationTest methodsFor: 'tests' stamp: 'monty 11/22/2014 20:40'!
testAllNodes
	| nodeWithNodes allNodes |

	nodeWithNodes := self newNodeWithNodes.
	allNodes := nodeWithNodes allNodes.
	self
		assert: allNodes size = 9;
		assert: allNodes first == nodeWithNodes;
		assert: allNodes second isComment;
		assert: allNodes second string = 'before';
		assert: allNodes third class == XMLGeneralEntityDeclaration;
		assert: allNodes fourth class == XMLParameterEntityDeclaration;
		assert: allNodes fifth class == XMLElementDeclaration;
		assert: allNodes sixth class == XMLAttributeDeclaration;
		assert: allNodes seventh class == XMLNotationDeclaration;
		assert: allNodes eighth isPI;
		assert: allNodes ninth isComment;
		assert: allNodes ninth string = 'after'.! !

!XMLDoctypeDeclarationTest methodsFor: 'tests' stamp: 'monty 12/6/2014 16:56'!
testCanonicallyEquals
	self
		assert: (self nodeClass new canonicallyEquals: self nodeClass new);
		assert: (self newNodeWithNodes canonicallyEquals: self newNodeWithNodes);
		assert: (self newNodeWithNodes canonicallyEquals: self nodeClass new)! !

!XMLDoctypeDeclarationTest methodsFor: 'tests' stamp: 'monty 11/22/2014 19:36'!
testNodes
	self assert: (self nodeClass new nodes isKindOf: XMLInternalSubsetList)! !

!XMLDoctypeDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:32'!
testPublicID
	self
		assert: self nodeClass new publicID isEmpty;
		assert: (self nodeClass new publicID: 'test') publicID = 'test'! !

!XMLDoctypeDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:32'!
testRoot
	self
		assert: self nodeClass new root isEmpty;
		assert: (self nodeClass new root: 'test') root = 'test'! !

!XMLDoctypeDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:34'!
testRootPublicIDSystemID
	| doctype |

	doctype := self nodeClass root: 'one' publicID: 'two' systemID: 'three'.
	self
		assert: doctype root = 'one';
		assert: doctype publicID = 'two';
		assert: doctype systemID = 'three'! !

!XMLDoctypeDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:32'!
testSystemID
	self
		assert: self nodeClass new systemID isEmpty;
		assert: (self nodeClass new systemID: 'test') systemID = 'test'! !

!XMLDoctypeDeclarationTest methodsFor: 'tests' stamp: 'monty 11/23/2014 18:44'!
testWriteXMLOn
	| doctype elementDeclaration |

	doctype := self nodeClass new.
	self assert: doctype printString = '<!!DOCTYPE >'.
	doctype root: 'one'.
	self assert: doctype printString = '<!!DOCTYPE one>'.
	doctype publicID: 'two'.
	self assert: doctype printString = '<!!DOCTYPE one PUBLIC "two">'.
	doctype systemID: 'three'.
	self assert: doctype printString = '<!!DOCTYPE one PUBLIC "two" "three">'.

	elementDeclaration := XMLElementDeclaration element: 'four' contentModel: '(five|six)'.
	doctype addNode: elementDeclaration.
	self assert:
		doctype printString =
			('<!!DOCTYPE one PUBLIC "two" "three" [', elementDeclaration printString, ']>').

	self assert:
		(self nodeClass root: 'one' publicID: '' systemID: 'two') printString =
			'<!!DOCTYPE one SYSTEM "two">'.! !

!XMLNodeWithChildrenTest class methodsFor: 'testing' stamp: 'monty 11/2/2014 11:01'!
isAbstract
	^ true! !

!XMLNodeWithChildrenTest methodsFor: 'accessing' stamp: 'monty 6/21/2015 02:20'!
copiedClasses
	"an object that isKindOf: any of these should be copied if it's
	reachable from a copied node directly or through another copied object"

	^ #(OrderedCollection Dictionary
		XMLNode XMLNamespaceScope XMLOrderedList
		XMLConfiguration XMLNodeFactory)! !

!XMLNodeWithChildrenTest methodsFor: 'instance creation' stamp: 'monty 11/7/2014 14:49'!
newNodeWithNodes
	^ self nodeClass new
		addNode: (XMLComment string: 'one');
		addNode: (XMLPI target: 'two' data: 'three');
		addNode: (XMLComment string: 'four');
		addNode: (XMLPI target: 'five' data: 'six');
		yourself! !

!XMLNodeWithChildrenTest methodsFor: 'accessing' stamp: 'monty 11/4/2014 19:03'!
nodeClass
	^ XMLNodeWithChildren! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 20:37'!
testAddNode
	| node nodeWithNodes added |

	node := self nodeClass new.
	nodeWithNodes := self newNodeWithNodes.
	added := OrderedCollection new.
	10 timesRepeat: [
		added add: self nodeClass new.
		self
			assert: (node addNode: added last) == added last;
			assert: (node lastNode) == added last.
		self
			should: [nodeWithNodes addNode: added last]
			raise: XMLDOMException].
	node nodes asOrderedCollection = added.
! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 07:19'!
testAddNodeAfter
	| nodeWithNodes added |

	nodeWithNodes := self newNodeWithNodes.
	added := OrderedCollection new.
	nodeWithNodes nodes copy do: [:each |
		added
			add: each;
			add: self nodeClass new.
		self
			assert: (nodeWithNodes addNode: added last after: each) == added last;
			assert: (nodeWithNodes nodeAfter: each) == added last.
		self
			should: [nodeWithNodes addNode: each after: each]
			raise: XMLDOMException].
	nodeWithNodes nodes asOrderedCollection = added.
! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 07:33'!
testAddNodeBefore
	| nodeWithNodes added |

	nodeWithNodes := self newNodeWithNodes.
	added := OrderedCollection new.
	nodeWithNodes nodes copy do: [:each | | node |
		added
			add: (node := self nodeClass new);
			add: each.
		self
			assert: (nodeWithNodes addNode: node before: each) == node;
			assert: (nodeWithNodes nodeBefore: each) == node.
		self
			should: [nodeWithNodes addNode: each before: each]
			raise: XMLDOMException].
	nodeWithNodes nodes asOrderedCollection = added.
! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 20:37'!
testAddNodeFirst
	| node nodeWithNodes added |

	node := self nodeClass new.
	nodeWithNodes := self newNodeWithNodes.
	added := OrderedCollection new.
	10 timesRepeat: [
		added addFirst: self nodeClass new.
		self
			assert: (node addNodeFirst: added first) == added first;
			assert: (node firstNode) == added first.
		self
			should: [nodeWithNodes addNodeFirst: added first]
			raise: XMLDOMException].
	node nodes asOrderedCollection = added.
! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 20:43'!
testAddNodes
	| node nodeWithNodes added |

	node := self nodeClass new.
	nodeWithNodes := self newNodeWithNodes.
	added := OrderedCollection new.
	1 to: 10 do: [:i | | nodes |
		nodes := (1 to: i) collect: [:each | self nodeClass new].
		added addAll: nodes.

		self assert: (node addNodes: nodes) == nodes.
		nodes do: [:each |
			self assert: (node includesNode: each).
			self
				should: [nodeWithNodes addNode: each]
				raise: XMLDOMException]].

	node nodes asOrderedCollection = added.
! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/7/2014 14:50'!
testAllNodes
	| nodeWithNodes allNodes |

	nodeWithNodes := self newNodeWithNodes.
	allNodes := nodeWithNodes allNodes.
	self
		assert: allNodes size = 5;
		assert: allNodes first == nodeWithNodes;
		assert: allNodes second isComment;
		assert: allNodes second string = 'one';
		assert: allNodes third isPI;
		assert: allNodes third target = 'two';
		assert: allNodes third data = 'three';
		assert: allNodes fourth isComment;
		assert: allNodes fourth string = 'four';
		assert: allNodes fifth isPI;
		assert: allNodes fifth target = 'five';
		assert: allNodes fifth data = 'six'.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/23/2014 20:57'!
testAllNodesCollect
	| nodeWithNodes allNodes |

	nodeWithNodes := self newNodeWithNodes.
	allNodes := nodeWithNodes allNodes.
	self
		assertList: (nodeWithNodes allNodesCollect: [:each | each identityHash])
		collects: (allNodes collect: [:each | each identityHash]).

	allNodes do: [:node |
		node canHaveChildren
			ifTrue: [
				self
					assertList: (node allNodesCollect: [:each | each identityHash])
					collects: (node allNodes collect: [:each | each identityHash])]].! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/22/2014 20:41'!
testAllNodesDetect
	| nodeWithNodes allNodes detectedNode |

	nodeWithNodes := self newNodeWithNodes.
	allNodes := nodeWithNodes allNodes.
	detectedNode := nodeWithNodes allNodesDetect: [:each | each isPI].
	self
		assert: detectedNode notNil;
		assert: detectedNode isPI;
		assert: detectedNode == (allNodes detect: [:each | each isPI]);
		assert: (nodeWithNodes allNodesDetect: [:each | each isElementNamed: 'absent']) isNil.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/22/2014 20:42'!
testAllNodesDetectIfNone
	| nodeWithNodes allNodes detectedNode |

	nodeWithNodes := self newNodeWithNodes.
	allNodes := nodeWithNodes allNodes.
	detectedNode :=
		nodeWithNodes
			allNodesDetect: [:each | each isPI]
			ifNone: [#absent].
	self
		deny: detectedNode = #absent;
		assert: detectedNode isPI;
		assert: detectedNode == (allNodes detect: [:each | each isPI] ifNone: [#absent]);
		assert:
			(nodeWithNodes
				allNodesDetect: [:each | each isElementNamed: 'absent']
				ifNone: [#absent]) = #absent.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/23/2014 20:56'!
testAllNodesDo
	| nodeWithNodes allNodes |

	nodeWithNodes := self newNodeWithNodes.
	allNodes := nodeWithNodes allNodes.
	self
		should: [:block | nodeWithNodes allNodesDo: block]
		enumerate: allNodes.

	allNodes do: [:node |
		node canHaveChildren
			ifTrue: [
				self
					should: [:block | node allNodesDo: block]
					enumerate: node allNodes]].! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/23/2014 20:57'!
testAllNodesSelect
	| nodeWithNodes allNodes |

	nodeWithNodes := self newNodeWithNodes.
	allNodes := nodeWithNodes allNodes.
	self
		assertList: (nodeWithNodes allNodesSelect: [:each | each isComment])
		selects: (allNodes select: [:each | each isComment]).

	allNodes do: [:node |
		node canHaveChildren
			ifTrue: [
				self
					assertList: (node allNodesSelect: [:each | each isComment])
					selects: (node allNodes select: [:each | each isComment])]].! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 3/8/2015 13:48'!
testAncestorNodes
	| nodeWithNodes ancestorStack |

	super testAncestorNodes.

	nodeWithNodes := self newNodeWithNodes.
	ancestorStack := OrderedCollection new.
	nodeWithNodes allNodesDo: [:each |
		[ancestorStack isEmpty
			or: [ancestorStack last == each parent]]
			whileFalse: [ancestorStack removeLast].
		self assert:
			each ancestorNodes asOrderedCollection =
				ancestorStack reversed.
		ancestorStack addLast: each].! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 3/8/2015 13:48'!
testAncestorNodesDo
	| nodeWithNodes |

	super testAncestorNodesDo.

	nodeWithNodes := self newNodeWithNodes.
	nodeWithNodes allNodesDo: [:each |
		self
			should: [:block | each ancestorNodesDo: block]
			enumerate: each ancestorNodes].! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/23/2014 20:54'!
testCanHaveChildren
	self assert: self nodeClass new canHaveChildren! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 12/6/2014 14:23'!
testCanonicallyEquals
	super testCanonicallyEquals.
	self
		assert: (self newNodeWithNodes canonicallyEquals: self newNodeWithNodes);
		deny: (self newNodeWithNodes canonicallyEquals: self nodeClass new).! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 6/21/2015 03:06'!
testCopy
	"this traverses the object graph of each node and compares it with
	its copy to make sure nodes and the inst vars that should be copied
	are copied. See #copiedClasses"
	| nodeWithNodes nodeWithNodesCopy unvisitedPairs visitedPairs copiedClasses |

	super testCopy.

	nodeWithNodes := self newNodeWithNodes.
	"initialize the configuration (if it can have one)"
	nodeWithNodes configuration
		ifNotNil: [:config | config nodeFactory: XMLPluggableNodeFactory new].

	nodeWithNodesCopy := nodeWithNodes copy.
	self
		assert: nodeWithNodes printString = nodeWithNodesCopy printString;
		assert: nodeWithNodes allNodes size = nodeWithNodesCopy allNodes size.

	unvisitedPairs := Set new.
	nodeWithNodes allNodes
		with: nodeWithNodesCopy allNodes
		do: [:node :nodeCopy | unvisitedPairs add: node -> nodeCopy].

	visitedPairs := Set new.
	copiedClasses := self copiedClasses collect: [:each | Smalltalk at: each].
	[unvisitedPairs isEmpty]
		whileFalse: [ | pair originalObject copyObject |
			pair := unvisitedPairs remove: unvisitedPairs anyOne.
			originalObject := pair key.
			copyObject := pair value.

			self deny: originalObject == copyObject.
			visitedPairs add: pair.
			1
				to: (originalObject class instSize min: copyObject class instSize)
				do: [:i | | newPair |
					newPair := (originalObject instVarAt: i) -> (copyObject instVarAt: i).
					((visitedPairs includes: newPair)
						or: [copiedClasses noneSatisfy: [:each | newPair key isKindOf: each]])
						ifFalse: [unvisitedPairs add: newPair]]].! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:43'!
testDescendantNodes
	| nodeWithNodes descendantNodes |

	nodeWithNodes := self newNodeWithNodes.
	descendantNodes := nodeWithNodes descendantNodes.
	self
		assertList: descendantNodes
		selects: nodeWithNodes allNodes allButFirst.

	descendantNodes do: [:node |
		node canHaveChildren
			ifTrue: [
				self
					assertList: node descendantNodes
					selects: node allNodes allButFirst]].! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:44'!
testDescendantNodesCollect
	| nodeWithNodes descendantNodes |

	nodeWithNodes := self newNodeWithNodes.
	descendantNodes := nodeWithNodes descendantNodes.
	self
		assertList: (nodeWithNodes descendantNodesCollect: [:each | each identityHash])
		collects: (descendantNodes collect: [:each | each identityHash]).

	descendantNodes do: [:node |
		node canHaveChildren
			ifTrue: [
				self
					assertList: (node descendantNodesCollect: [:each | each identityHash])
					collects: (node descendantNodes collect: [:each | each identityHash])]].! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:45'!
testDescendantNodesDetect
	| nodeWithNodes descendantNodes detectedNode |

	nodeWithNodes := self newNodeWithNodes.
	descendantNodes := nodeWithNodes descendantNodes.
	detectedNode := nodeWithNodes descendantNodesDetect: [:each | each isPI].
	self
		assert: detectedNode notNil;
		assert: detectedNode isPI;
		assert: detectedNode == (descendantNodes detect: [:each | each isPI]);
		assert: (nodeWithNodes descendantNodesDetect: [:each | each isElementNamed: 'absent']) isNil.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:45'!
testDescendantNodesDetectIfNone
	| nodeWithNodes descendantNodes detectedNode |

	nodeWithNodes := self newNodeWithNodes.
	descendantNodes := nodeWithNodes descendantNodes.
	detectedNode :=
		nodeWithNodes
			descendantNodesDetect: [:each | each isPI]
			ifNone: [#absent].
	self
		deny: detectedNode = #absent;
		assert: detectedNode isPI;
		assert: detectedNode == (descendantNodes detect: [:each | each isPI] ifNone: [#absent]);
		assert:
			(nodeWithNodes
				descendantNodesDetect: [:each | each isElementNamed: 'absent']
				ifNone: [#absent]) = #absent.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:45'!
testDescendantNodesDo
	| nodeWithNodes descendantNodes |

	nodeWithNodes := self newNodeWithNodes.
	descendantNodes := nodeWithNodes descendantNodes.
	self
		should: [:block | nodeWithNodes descendantNodesDo: block]
		enumerate: descendantNodes.

	descendantNodes do: [:node |
		node canHaveChildren
			ifTrue: [
				self
					should: [:block | node descendantNodesDo: block]
					enumerate: node descendantNodes]].! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:46'!
testDescendantNodesSelect
	| nodeWithNodes descendantNodes |

	nodeWithNodes := self newNodeWithNodes.
	descendantNodes := nodeWithNodes descendantNodes.
	self
		assertList: (nodeWithNodes descendantNodesSelect: [:each | each isComment])
		selects: (descendantNodes select: [:each | each isComment]).

	descendantNodes do: [:node |
		node canHaveChildren
			ifTrue: [
				self
					assertList: (node descendantNodesSelect: [:each | each isComment])
					selects: (node descendantNodes select: [:each | each isComment])]].! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/24/2014 00:16'!
testDocument
	| nodeWithNodes document |

	nodeWithNodes := self newNodeWithNodes.
	document := nodeWithNodes document.
	nodeWithNodes allNodesDo: [:each |
		self assert: each document == document].

	document
		ifNotNil: [ | root |
			root := document root.
			document removeNodes.
			root allNodesDo: [:each |
				self assert: each document isNil]].! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 06:58'!
testFirstNode
	| nodeWithNodes |

	nodeWithNodes := self newNodeWithNodes.
	nodeWithNodes nodes copy do: [:each |
		self  assert: nodeWithNodes firstNode == each.
		nodeWithNodes removeNode: each].
	self assert: nodeWithNodes firstNode isNil.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:09'!
testHasChildren
	super testHasChildren.

	self assert: self newNodeWithNodes hasChildren.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:50'!
testIncludesNode
	| nodeWithNodes |

	nodeWithNodes := self newNodeWithNodes.
	nodeWithNodes nodes do: [:each |
		self assert: (nodeWithNodes includesNode: each)].
	self deny: (nodeWithNodes includesNode: self nodeClass new).! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 06:55'!
testLastNode
	| nodeWithNodes |

	nodeWithNodes := self newNodeWithNodes.
	nodeWithNodes nodes reversed do: [:each |
		self  assert: nodeWithNodes lastNode == each.
		nodeWithNodes removeNode: each].

	self assert: nodeWithNodes lastNode isNil.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/24/2014 18:44'!
testNextNode
	| nodeWithNodes nextNode |

	super testNextNode.
	nodeWithNodes := self newNodeWithNodes.
	nextNode := nil.
	nodeWithNodes nodes reverseDo: [:each |
		self assert: each nextNode == nextNode.
		nextNode := each].! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/24/2014 18:43'!
testNodeAfter
	| nodeWithNodes nodeAfter |

	nodeWithNodes := self newNodeWithNodes.
	nodeAfter := nil.
	nodeWithNodes nodes reverseDo: [:each |
		self assert: (nodeWithNodes nodeAfter: each) == nodeAfter.
		nodeAfter := each].
	self assert: (nodeWithNodes nodeAfter: self nodeClass new) isNil.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 08:42'!
testNodeAt
	| nodeWithNodes |

	nodeWithNodes := self newNodeWithNodes.
	self assert: (nodeWithNodes nodeAt: 0) isNil.
	1 to: nodeWithNodes nodes size do: [:i |
		self assert: (nodeWithNodes nodeAt: i) == (nodeWithNodes nodes at: i)].
	self assert: (nodeWithNodes nodeAt: nodeWithNodes nodes size + 1) isNil.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 08:41'!
testNodeAtIfAbsent
	| nodeWithNodes |

	nodeWithNodes := self newNodeWithNodes.
	self assert: (nodeWithNodes nodeAt: 0 ifAbsent: [#absent]) = #absent.
	1 to: nodeWithNodes nodes size do: [:i |
		self assert:
			(nodeWithNodes
				nodeAt: i
				ifAbsent: [#absent]) == (nodeWithNodes nodes at: i)].
	self assert:
		(nodeWithNodes
			nodeAt: nodeWithNodes nodes size + 1
			ifAbsent: [#absent]) = #absent.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 12/16/2014 11:21'!
testNodeAtPut
	| nodeWithNodes newNodeClass newNode |

	nodeWithNodes := self newNodeWithNodes.
	newNodeClass := nodeWithNodes firstNode class.
	self
		should: [nodeWithNodes nodeAt: 0 put: newNodeClass new]
		raise: Error.
	1 to: nodeWithNodes nodes size do: [:i |
		newNode := newNodeClass new.
		self
			assert: (nodeWithNodes nodeAt: i put: newNode) == newNode;
			assert: (nodeWithNodes nodeAt: i) == newNode].

	newNode := newNodeClass new.
	self assert:
		(nodeWithNodes
			nodeAt: nodeWithNodes nodes size + 1
			put: newNode) == newNode.
	self assert: (nodeWithNodes nodeAt: nodeWithNodes nodes size) == newNode.
	self
		should: [
			nodeWithNodes
				nodeAt: nodeWithNodes nodes size + 2
				put: newNodeClass new]
		raise: Error.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/24/2014 18:43'!
testNodeBefore
	| nodeWithNodes nodeBefore |

	nodeWithNodes := self newNodeWithNodes.
	nodeBefore := nil.
	nodeWithNodes nodes do: [:each |
		self assert: (nodeWithNodes nodeBefore: each) == nodeBefore.
		nodeBefore := each].
	self assert: (nodeWithNodes nodeBefore: self nodeClass new) isNil.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/24/2014 22:14'!
testNodeListCopying
	| nodeWithNodes nodeListCopy |

	nodeWithNodes := self newNodeWithNodes.
	nodeListCopy := nodeWithNodes nodes copy.
	self
		assert: nodeListCopy = nodeWithNodes nodes;
		deny: nodeListCopy == nodeWithNodes nodes;
		assert: nodeWithNodes nodes observer == nodeWithNodes;
		assert: nodeListCopy observer isNil.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/22/2014 19:36'!
testNodes
	self assert: (self nodeClass new nodes isKindOf: XMLNodeList)! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/23/2014 21:04'!
testNodesCollect
	| node |

	node := self newNodeWithNodes.
	self
		assertList: (node nodesCollect: [:each | each identityHash])
		collects: (node nodes collect: [:each | each identityHash]).! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/23/2014 21:07'!
testNodesDetect
	| node detectedNode |

	node := self newNodeWithNodes.
	detectedNode := node nodesDetect: [:each | each isComment].
	self
		assert: detectedNode notNil;
		assert: detectedNode == (node nodes detect: [:each | each isComment]);
		assert: (node nodesDetect: [:each | each isElementNamed: 'absent']) isNil.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/24/2014 08:05'!
testNodesDetectIfNone
	| node detectedNode |

	node := self newNodeWithNodes.
	detectedNode :=
		node
			nodesDetect: [:each | each isComment]
			ifNone: [#absent].
	self deny: detectedNode = #absent.
	self assert:
		detectedNode ==
			(node nodes detect: [:each | each isComment] ifNone: [#absent]).
	self assert:
		(node
			nodesDetect: [:each | each isElementNamed: 'absent']
			ifNone: [#absent]) = #absent.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:50'!
testNodesDo
	| node |

	node := self nodeClass new.
	self should: [:block | node nodesDo: block] enumerate: #().

	node := self newNodeWithNodes.
	self should: [:block | node nodesDo: block] enumerate: node nodes.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/23/2014 21:04'!
testNodesSelect
	| node |

	node := self newNodeWithNodes.
	self
		assertList: (node nodesSelect: [:each | each hasChildren])
		selects: (node nodes select: [:each | each hasChildren]).! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/24/2014 18:44'!
testPreviousNode
	| nodeWithNodes previousNode |

	super testPreviousNode.
	nodeWithNodes := self newNodeWithNodes.
	previousNode := nil.
	nodeWithNodes nodes do: [:each |
		self assert: each previousNode == previousNode.
		previousNode := each].! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 20:28'!
testRemoveNode
	| nodeWithNodes |

	nodeWithNodes := self newNodeWithNodes.
	nodeWithNodes nodes copy do: [:each |
		self
			assert: (nodeWithNodes includesNode: each);
			assert: (nodeWithNodes removeNode: each) == each;
			deny: (nodeWithNodes includesNode: each)].
	self deny: nodeWithNodes hasChildren.
	self
		should: [nodeWithNodes removeNode: self nodeClass new]
		raise: Error.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 20:54'!
testRemoveNodeIfAbsent
	| nodeWithNodes |

	nodeWithNodes := self newNodeWithNodes.
	nodeWithNodes nodes copy do: [:each |
		self
			assert: (nodeWithNodes includesNode: each);
			assert: (nodeWithNodes removeNode: each ifAbsent: [#absent]) == each;
			deny: (nodeWithNodes includesNode: each)].
	self deny: nodeWithNodes hasChildren.
	self assert:
		(nodeWithNodes
			removeNode: self nodeClass new
			ifAbsent: [#absent]) == #absent.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 20:20'!
testRemoveNodes
	| nodeWithNodes |

	nodeWithNodes := self newNodeWithNodes.
	self
		assert: nodeWithNodes hasChildren;
		assert: nodeWithNodes removeNodes == nodeWithNodes;
		deny: nodeWithNodes hasChildren.! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 11/19/2014 20:53'!
testRemoveNodesAll
	| nodeWithNodes |

	nodeWithNodes := self newNodeWithNodes.
	1 to: nodeWithNodes nodes size do: [:i | | removed |
		removed := (nodeWithNodes nodes copyFrom: 1 to: i).
		self
			assert: (removed allSatisfy: [:each | nodeWithNodes includesNode: each]);
			assert: (nodeWithNodes removeNodes: removed) == removed;
			deny: (removed anySatisfy: [:each | nodeWithNodes includesNode: each]).
		nodeWithNodes := self newNodeWithNodes].! !

!XMLNodeWithChildrenTest methodsFor: 'tests' stamp: 'monty 2/2/2015 03:55'!
testReplaceNodeWith
	| nodeWithNodes replacements nodes firstIndex secondIndex |

	nodeWithNodes := self newNodeWithNodes.
	replacements := OrderedCollection new.
	nodeWithNodes nodes copy do: [:each |
		self assert:
			(nodeWithNodes
				replaceNode: each
				with: (replacements add: self nodeClass new)) == replacements last].
	self assert: nodeWithNodes nodes asOrderedCollection = replacements.

	nodeWithNodes := self newNodeWithNodes.
	nodes := nodeWithNodes nodes copy.
	nodes do: [:each |
		self assert:
			(nodeWithNodes
				replaceNode: each
				with: each) == each].
	self assert: nodeWithNodes nodes = nodes.

	firstIndex := 1.
	secondIndex := nodes size.
	[firstIndex < secondIndex]
		whileTrue: [
			nodeWithNodes
				replaceNode: (nodes at: firstIndex)
				with: (nodes at: secondIndex).
			firstIndex := firstIndex + 1.
			secondIndex := secondIndex - 1].
	self assert: nodeWithNodes nodes = nodes reversed.

	self
		should: [
			nodeWithNodes
				replaceNode: nodes anyOne copy
				with: self nodeClass new]
		raise: Error.! !

XMLNodeWithChildrenTest subclass: #XMLNodeWithElementsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLNodeWithElementsTest commentStamp: 'monty 11/4/2014 19:05' prior: 0!
Abstract, do not run.!

XMLNodeWithElementsTest subclass: #XMLDocumentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLDocumentTest class methodsFor: 'testing' stamp: 'monty 11/2/2014 11:08'!
isAbstract
	^ false! !

!XMLDocumentTest methodsFor: 'instance creation' stamp: 'monty 11/14/2015 05:20'!
newNodeWithElements
	^ super newNodeWithElements
		doctypeDeclaration: (XMLDoctypeDeclaration new root: 'zero');
		yourself! !

!XMLDocumentTest methodsFor: 'accessing' stamp: 'monty 11/2/2014 11:08'!
nodeClass
	^ XMLDocument! !

!XMLDocumentTest methodsFor: 'tests' stamp: 'monty 4/11/2015 01:00'!
testDoctypeDeclaration
	| document doctypeDeclaration |

	document := self nodeClass new.
	self assert: document doctypeDeclaration isNil.
	doctypeDeclaration := XMLDoctypeDeclaration new.
	self
		assert: (document doctypeDeclaration: doctypeDeclaration) == document;
		assert: document doctypeDeclaration == doctypeDeclaration;
		assert: doctypeDeclaration parent == document.
	self
		should: [self nodeClass new doctypeDeclaration: document doctypeDeclaration]
		raise: XMLDOMException.

	document doctypeDeclaration: nil.
	self
		assert: document doctypeDeclaration isNil;
		assert: doctypeDeclaration parent isNil.! !

!XMLDocumentTest methodsFor: 'tests' stamp: 'monty 3/8/2015 10:18'!
testDoctypeDefinition
	| document doctypeDefinition |

	document := self nodeClass new.
	self assert: document doctypeDefinition isNil.
	doctypeDefinition := DTDDoctypeDefinition new.
	self
		assert: (document doctypeDefinition: doctypeDefinition) == document;
		assert: document doctypeDefinition == doctypeDefinition.! !

!XMLDocumentTest methodsFor: 'tests' stamp: 'monty 11/24/2014 00:17'!
testDocument
	| document |

	super testDocument.

	document := self nodeClass new.
	self assert: document document == document.

	document := self newNodeWithElements.
	self assert: document document == document.! !

!XMLDocumentTest methodsFor: 'tests' stamp: 'monty 3/24/2015 18:37'!
testEncoding
	self
		assert: self nodeClass new encoding isEmpty;
		assert: ((self nodeClass new encoding: 'UTF-8') encoding) = 'UTF-8'.! !

!XMLDocumentTest methodsFor: 'tests' stamp: 'monty 5/9/2015 03:12'!
testHasDoctypeDeclaration
	| document |

	document := self nodeClass new.
	self deny: document hasDoctypeDeclaration.
	document doctypeDeclaration: XMLDoctypeDeclaration new.	
	self assert: document hasDoctypeDeclaration.
	document doctypeDeclaration: nil.
	self deny: document hasDoctypeDeclaration.! !

!XMLDocumentTest methodsFor: 'tests' stamp: 'monty 5/9/2015 03:14'!
testHasDoctypeDefinition
	| document |

	document := self nodeClass new.
	self deny: document hasDoctypeDefinition.
	document doctypeDefinition: DTDDoctypeDefinition new.
	self assert: document hasDoctypeDefinition.
	document doctypeDefinition: nil.
	self deny: document hasDoctypeDefinition.! !

!XMLDocumentTest methodsFor: 'tests' stamp: 'monty 3/6/2016 09:14'!
testHasEncoding
	| document |

	document := self nodeClass new.
	self deny: document hasEncoding.
	document encoding: 'UTF-8'.
	self assert: document hasEncoding.
	document encoding: ''.
	self deny: document hasEncoding.! !

!XMLDocumentTest methodsFor: 'tests' stamp: 'monty 3/6/2016 10:04'!
testHasRoot
	| nodeWithElements |

	self deny: self nodeClass new hasRoot.
	nodeWithElements := self newNodeWithElements.
	self assert: nodeWithElements hasRoot.
	nodeWithElements removeNode: nodeWithElements root.
	self deny: nodeWithElements hasRoot.! !

!XMLDocumentTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:45'!
testIsDocument
	self assert: self nodeClass new isDocument! !

!XMLDocumentTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:18'!
testIsStandalone
	self
		deny: self nodeClass new isStandalone;
		assert: (self nodeClass new isStandalone: true) isStandalone! !

!XMLDocumentTest methodsFor: 'tests' stamp: 'monty 3/6/2016 09:32'!
testRoot
	| nodeWithElements documentNodes newRoot |

	self assert: self nodeClass new root isNil.

	nodeWithElements := self newNodeWithElements.
	documentNodes := nodeWithElements nodes.
	self
		assert: documentNodes size = 3;
		assert: documentNodes first isComment;
		assert: (documentNodes second isElementNamed: 'zero');
		assert: documentNodes third isComment;
		assert: nodeWithElements root == documentNodes second.

	newRoot := XMLElement named: 'new-root'.
	documentNodes at: 2 put: newRoot.
	self
		assert: (nodeWithElements root: newRoot) == nodeWithElements;
		assert: nodeWithElements root == newRoot;
		assert: nodeWithElements nodes = documentNodes.

	nodeWithElements removeNodes.
	self assert: nodeWithElements root isNil.

	self
		should: [nodeWithElements root: 'test' asXMLString]
		raise: XMLDOMException.! !

!XMLDocumentTest methodsFor: 'tests' stamp: 'monty 4/7/2015 00:06'!
testValidate
	| xml document |

	super testValidate.

	xml := '
		<!!DOCTYPE one [
			<!!ELEMENT one (two+)>
			<!!ATTLIST one idref IDREF #REQUIRED>
			<!!ELEMENT two (#PCDATA)*>
			<!!ATTLIST two id ID #REQUIRED>]>
		<one idref="a">
			<two id="a">three</two><two id="b"><!!--comment--></two>
		</one>'.

	document :=
		(XMLDOMParser on: xml)
			preservesCommentNodes: true;
			parseDocument.
	self
		shouldnt: [document validate]
		raise: XMLValidationException.
	self assert:
		(document canonicallyEquals: 
			((XMLDOMParser on: xml)
				preservesCommentNodes: true;
				parseDocument)).

	document
		allElementsNamed: 'two'
		do: [:each | each attributeAt: 'id' put: 'a'].
	self
		should: [document validate]
		raise: XMLValidationException.

	document := XMLDOMParser parse: xml.
	document root name: 'zero'.
	self
		should: [document validate]
		raise: XMLValidationException.

	document := XMLDOMParser parse: xml.
	document root attributeAt: 'idref' put: 'undeclared'.
	self
		should: [document validate]
		raise: XMLValidationException.

	document := XMLDOMParser parse: xml.
	document root addElementNamed: 'three'.
	self
		should: [document validate]
		raise: XMLValidationException.

	(document := XMLDOMParser parse: xml) doctypeDefinition: nil.
	document root
		name: 'zero';
		attributeAt: 'idref' put: 'undeclared';
		addElementNamed: 'three';
		allElementsNamed: 'two' do: [:each | each attributeAt: 'id' put: 'same'].
	self
		shouldnt: [document validate]
		raise: XMLValidationException.! !

!XMLDocumentTest methodsFor: 'tests' stamp: 'monty 4/7/2015 00:10'!
testValidateNoSubset
	| xml document |

	xml := '<!!DOCTYPE one>
		<one idref="a">
			<two id="a">three</two><two id="b"><!!--comment--></two>
		</one>'.

	document :=
		(XMLDOMParser on: xml)
			preservesCommentNodes: true;
			parseDocument.
	self
		shouldnt: [document validate]
		raise: XMLValidationException.
	self assert:
		(document canonicallyEquals: 
			((XMLDOMParser on: xml)
				preservesCommentNodes: true;
				parseDocument)).

	document root name: 'five'.
	self
		should: [document validate]
		raise: XMLValidationException.

	document doctypeDefinition: nil.
	self
		shouldnt: [document validate]
		raise: XMLValidationException.! !

!XMLDocumentTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:19'!
testVersion
	self
		assert: self nodeClass new version isNil;
		assert: (self nodeClass new version: 1.0) version = 1.0! !

XMLNodeWithElementsTest subclass: #XMLElementTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLElementTest class methodsFor: 'testing' stamp: 'monty 11/2/2014 11:09'!
isAbstract
	^ false! !

!XMLElementTest methodsFor: 'assertions' stamp: 'monty 11/23/2014 04:37'!
assertElement: anElement hasAttributes: anAssociationCollection
	| attributeNodes |

	self assert: anElement attributeNodes size = anAssociationCollection size.
	attributeNodes := anElement attributeNodes copy.
	anAssociationCollection associationsDo: [:each | | attributeNode |
		attributeNode := attributeNodes removeFirst.
		self
			assert: attributeNode name = each key;
			assert: attributeNode value = each value.]! !

!XMLElementTest methodsFor: 'instance creation' stamp: 'monty 11/14/2015 05:20'!
newNodeWithElements
	^ super newNodeWithElements
		name: 'root';
		yourself! !

!XMLElementTest methodsFor: 'accessing' stamp: 'monty 11/2/2014 11:09'!
nodeClass
	^ XMLElement! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:23'!
testAttributeAssociations
	| element |

	element := self newElementWithAttributes.
	self
		assertList: element attributeAssociations
		collects: (element attributeNodes collect: [:each | each name -> each value]).! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:13'!
testAttributeAt
	| element |

	element := self newElementWithAttributes.
	element attributeNodes do: [:each |
		self assert: (element attributeAt: each name) == each value].
	self assert: (element attributeAt: 'absent') isEmpty.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:20'!
testAttributeAtIfAbsent
	| element |

	element := self newElementWithAttributes.
	element attributeNodes do: [:each |
		self assert:
			(element
				attributeAt: each name
				ifAbsent: [#absent]) == each value].
	self assert: (element attributeAt: 'absent' ifAbsent: [#absent]) = #absent.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:21'!
testAttributeAtIfAbsentPut
	| element |

	element := self newElementWithAttributes.
	element attributeNodes do: [:each |
		self assert:
			(element
				attributeAt: each name
				ifAbsentPut: ['newValue']) == each value].
	self
		assert: (element attributeAt: 'newName' ifAbsentPut: ['newValue']) = 'newValue';
		assert: (element attributeAt: 'newName') = 'newValue'.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:13'!
testAttributeAtPut
	| element |

	element := self newElementWithAttributes.
	element attributeNodes do: [:each | | newValue |
		newValue := 'new', each value.
		self
			assert: (element attributeAt: each name put: newValue) = newValue;
			assert: (element attributeAt: each name) = newValue].
	self
		assert: (element attributeAt: 'newName' put: 'newValue') = 'newValue';
		assert: (element attributeAt: 'newName') = 'newValue'.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/24/2014 22:15'!
testAttributeListCopying
	| element attributeListCopy |

	element := self newElementWithAttributes.
	attributeListCopy := element attributeNodes copy.
	self
		assert: attributeListCopy = element attributeNodes;
		deny: attributeListCopy == element attributeNodes;
		assert: attributeListCopy attributesByName = element attributeNodes attributesByName;
		deny: attributeListCopy attributesByName == element attributeNodes attributesByName;
		assert: element attributeNodes observer == element;
		assert: attributeListCopy observer isNil.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:13'!
testAttributeNames
	| element |

	element := self newElementWithAttributes.
	self
		assertList: element attributeNames
		collects: (element attributeNodes collect: [:each | each name])! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:13'!
testAttributeNamesAndValuesDo
	| element pairs |

	pairs := OrderedCollection new.
	XMLElement new attributeNamesAndValuesDo: [:attributeName :attributeValue|
		pairs add: attributeName -> attributeValue].
	self assert: pairs isEmpty.

	element := self newElementWithAttributes.
	element attributeNamesAndValuesDo: [:attributeName :attributeValue|
		pairs add: attributeName -> attributeValue].
	self assert:
		pairs =
			((element attributeNodes collect: [:each |
				each name -> each value]) asOrderedCollection).! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:13'!
testAttributeNamesDo
	| element |

	element := XMLElement new.
	self
		should: [:block | element attributeNamesDo: block]
		enumerate: #().

	element := self newElementWithAttributes.
	self
		should: [:block | element attributeNamesDo: block]
		enumerate: element attributeNames.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 3/22/2015 03:42'!
testAttributeNodeAt
	| element |

	element := self newElementWithAttributes.
	element attributeNodes do: [:each |
		self assert: (element attributeNodeAt: each name) == each].
	self assert: (element attributeNodeAt: 'absent') isNil.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:19'!
testAttributeNodeAtIfAbsent
	| element  |

	element := self newElementWithAttributes.
	element attributeNodes do: [:each |
		self assert:
			(element
				attributeNodeAt: each name
				ifAbsent: [#absent]) == each].
	self assert: (element attributeNodeAt: 'absent' ifAbsent: [#absent]) = #absent.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 2/24/2016 16:32'!
testAttributeNodeAtNamespaceURI
	| element |

	element := self newElementWithAttributes.
	self assert:
		(element attributeNodeAt: 'prefix:three' namespaceURI: 'absent') isNil.
	self assert:
		(element attributeNodeAt: 'prefix:three' namespaceURI: 'prefixNS') ==
			(element attributeNodes third).! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 2/24/2016 16:32'!
testAttributeNodeAtNamespaceURIIfAbsent
	| element |

	element := self newElementWithAttributes.
	self assert:
		(element
			attributeNodeAt: 'prefix:three'
			namespaceURI: 'absent'
			ifAbsent: [#absent]) = #absent.
	self assert:
		(element
			attributeNodeAt: 'prefix:three'
			namespaceURI: 'prefixNS'
			ifAbsent: [#absent]) ==
				(element attributeNodes third).! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:16'!
testAttributeNodes
	| element nodes |

	element := self newElementWithAttributes.
	nodes := element attributeNodes.
	self
		assert: (nodes isKindOf: XMLAttributeList);
		assert: nodes size = 4;
		assert: (nodes collect: [:each | each name]) asArray = #('one' 'three' 'prefix:three' 'prefix:six');
		assert: (nodes collect: [:each | each value]) asArray = #('two' 'four' 'five' 'seven').! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/26/2014 19:24'!
testAttributeNodesDo
	| element |

	element := self nodeClass new.
	self should: [:block | element attributeNodesDo: block] enumerate: #().

	element := self newElementWithAttributes.
	self
		should: [:block | element attributeNodesDo: block]
		enumerate: element attributeNodes.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 08:06'!
testAttributeRenaming
	| elementWithAttributes attributeNodes attributeNode oldSize oldNames oldName |

	elementWithAttributes := self newElementWithAttributes.
	attributeNodes := elementWithAttributes attributeNodes.

	attributeNode := attributeNodes first.
	oldSize := attributeNodes size.
	oldNames := elementWithAttributes attributeNames.
	oldName := attributeNode name.
	attributeNode name: 'newName'.
	self
		assert: attributeNodes size = oldSize;
		assert: attributeNodes first == attributeNode;
		assert: elementWithAttributes attributeNames = (oldNames allButFirst copyWithFirst: 'newName');
		deny: (elementWithAttributes includesAttribute: oldName);
		assert: (elementWithAttributes attributeNodeAt: 'newName') == attributeNode.

	oldNames := elementWithAttributes attributeNames.
	oldName := attributeNode name.
	attributeNode name: oldNames last.
	self
		assert: attributeNodes size = (oldSize - 1);
		assert: attributeNodes first == attributeNode;
		assert:
			elementWithAttributes attributeNames =
				((oldNames copyFrom: 2 to: oldNames size - 1) copyWithFirst: oldNames last);
		deny: (elementWithAttributes includesAttribute: oldName);
		assert: (elementWithAttributes attributeNodeAt: attributeNode name) == attributeNode.

	self should: [attributeNode name: 'undeclared:newName'] raise: XMLNamespaceException.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 1/6/2015 20:07'!
testAttributeSorting
	| element |

	element :=
		XMLElement
			name: 'sorted'
			attributes: (Array with: 'c' -> '3' with: 'b' -> '2' with: 'a' -> '1').
	self assert:
		(element attributeNodes sorted
			collect: [:each | each name]) asArray = #('a' 'b' 'c').
	self assert:
		((element attributeNodes sorted: [:a :b | a name <= b name])
			collect: [:each | each name]) asArray = #('a' 'b' 'c').
	"sorted/sorted: should sort copies"
	self assert: element attributeNames asArray = #('c' 'b' 'a').

	element attributeNodes sort.
	self assert: element attributeNames asArray = #('a' 'b' 'c').
	element attributeNodes sort: [:a :b | a name > b name].
	self assert: element attributeNames asArray = #('c' 'b' 'a').! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 2/29/2016 08:12'!
testAttributes
	| element attributeDictionary |

	element := self newElementWithAttributes.
	attributeDictionary := element attributes.
	self
		assert: attributeDictionary isDictionary;
		assert: attributeDictionary size = element attributeNodes size.
	self assert:
		(element attributeNodes allSatisfy: [:each |
			(attributeDictionary at: each name) = each value]).! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 10/12/2014 01:06'!
testCanonicalPrinting
	| element |

	element :=
		XMLElement
			name: 'test'
			namespaces: (Array with: 'b' -> 'ns2' with: 'a' -> 'ns1' with: '' -> 'default')
			attributes: (Array with: 'c' -> '3' with: 'b' -> '2' with: 'a' -> '1').

	self assert:
		element printString =
			'<test xmlns:b="ns2" xmlns:a="ns1" xmlns="default" c="3" b="2" a="1" />'.
	self assert:
		element canonicallyPrinted =
			'<test xmlns="default" xmlns:a="ns1" xmlns:b="ns2" a="1" b="2" c="3"></test>'.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/24/2014 06:34'!
testDeclareNamespaces
	| element |

	element := self nodeClass new.
	element declareNamespaces: self namespaces.
	self namespaces keysAndValuesDo: [:prefix :uri |
		self assert: (element isDeclaredPrefix: prefix uri: uri)].

	element := self nodeClass new.
	element declareNamespaces: self namespaces associations.
	self namespaces keysAndValuesDo: [:prefix :uri |
		self assert: (element isDeclaredPrefix: prefix uri: uri)]! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 3/6/2016 17:55'!
testDeclarePrefixURI
	| element elementsWithDeclaredPrefix |

	(element := self newNodeWithElements)
		declarePrefix: 'newPrefix' uri: 'prefixNS';
		allElementsDo: [:each | self assert: (each isDeclaredPrefix: 'newPrefix' uri: 'prefixNS')];
		declarePrefix: 'newPrefix' uri: 'newPrefixNS';
		allElementsDo: [:each | self assert: (each isDeclaredPrefix: 'newPrefix' uri: 'newPrefixNS')].

	elementsWithDeclaredPrefix :=
		element allElementsSelect: [:each | each prefix = 'prefix'].
	element
		declarePrefix: 'prefix' uri: 'newPrefixNS';
		allElementsDo: [:each |
			(elementsWithDeclaredPrefix includes: each)
				ifTrue: [
					self
						assert: (each isDeclaredPrefix: 'prefix');
						deny: (each isDeclaredPrefix: 'prefix' uri: 'newPrefixNS')]
				ifFalse: [self assert: (each isDeclaredPrefix: 'prefix' uri: 'newPrefixNS')]].

	self
		should: [element declarePrefix: 'nonDefault' uri: '']
		raise: XMLNamespaceException.
	"cannot rebind the xml prefix to a new NS"
	self 
		should: [element declarePrefix: 'xml' uri: 'newNS']
		raise: XMLNamespaceException.
	"cannot rebind the xmlns prefix, even to the same NS"
	self
		should: [element declarePrefix: 'xmlns' uri: XMLNamespaceScope xmlnsNamespaceURI]
		raise: XMLNamespaceException.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 2/24/2016 06:13'!
testDeclaredPrefixes
	| element prefixes |

	element := self nodeClass new.
	prefixes := OrderedCollection new.
	self namespaces keysAndValuesDo: [:prefix :uri |
		element
			declarePrefix: prefix
			uri: uri.
		prefixes addLast: prefix.
		self assert: element declaredPrefixes = prefixes asArray].! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/24/2014 07:00'!
testDeclaresDefaultNamespace
	| element |

	element := self newNodeWithElements.
	self deny: element declaresDefaultNamespace.

	element defaultNamespace: 'defaultNS'.
	self assert: element declaresDefaultNamespace.
	element elementsDo: [:each |
		self deny: each declaresDefaultNamespace].

	element elementsDo: [:each |
		each defaultNamespace: 'newDefaultNS'.
		self assert: each declaresDefaultNamespace].

	self assert: element declaresDefaultNamespace.	
	element undeclarePrefix: ''.
	self deny: element declaresDefaultNamespace.
	element elementsDo: [:each |
		self assert: each declaresDefaultNamespace].! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/24/2014 07:00'!
testDeclaresPrefixURI
	| element |

	element := self newNodeWithElements.
	self deny: element declaresDefaultNamespace.

	element declarePrefix: 'prefix' uri: 'prefixNS'.
	self assert: (element declaresPrefix: 'prefix' uri: 'prefixNS').
	element elementsDo: [:each |
		self deny: (each declaresPrefix: 'prefix' uri: 'prefixNS')].

	element elementsDo: [:each |
		each declarePrefix: 'prefix' uri: 'newPrefixNS'.
		self assert: (each declaresPrefix: 'prefix' uri: 'newPrefixNS')].

	self assert: (element declaresPrefix: 'prefix' uri: 'prefixNS').
	element undeclarePrefix: 'prefix'.
	self deny: (element declaresPrefix: 'prefix' uri: 'prefixNS').
	element elementsDo: [:each |
		self assert: (each declaresPrefix: 'prefix' uri: 'newPrefixNS')].! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/24/2014 07:00'!
testDefaultNamespace
	| element elementsWithDefaultNamespace |

	element := self newNodeWithElements.
	elementsWithDefaultNamespace :=
		element allElementsSelect: [:each |
			each defaultNamespace notEmpty].

	element
		defaultNamespace: 'newDefaultNS';
		allElementsDo: [:each |
			(elementsWithDefaultNamespace includes: each)
				ifTrue: [self deny: (each defaultNamespace = 'newDefaultNS')]
				ifFalse: [self assert: (each defaultNamespace = 'newDefaultNS')]].

	element
		defaultNamespace: '';
		allElementsDo: [:each |
			(elementsWithDefaultNamespace includes: each)
				ifTrue: [self assert: each defaultNamespace notEmpty]
				ifFalse: [self assert: each defaultNamespace isEmpty]].! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/1/2014 21:02'!
testElementSorting
	| element |

	(element := XMLElement name: 'sorted')
		addElementNamed: 'c';
		addElementNamed: 'b';
		addElementNamed: 'a'.

	self assert: (element nodes sorted collect: [:each | each name]) asArray = #('a' 'b' 'c').
	self assert:
		((element nodes sorted: [:a :b | a name <= b name])
			collect: [:each | each name]) asArray = #('a' 'b' 'c').

	self assert: (element nodesCollect: [:each | each name]) asArray = #('c' 'b' 'a').
	element nodes sort.
	self assert: (element nodesCollect: [:each | each name]) asArray = #('a' 'b' 'c').
	element nodes sort: [:a :b | a name > b name].
	self assert: (element nodesCollect: [:each | each name]) asArray = #('c' 'b' 'a').! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 12/30/2014 02:40'!
testExpandedName
	| element |

	element := self nodeClass named: 'test'.
	self assert: (element expandedName = 'test').

	element defaultNamespace: 'defaultNS'.
	self assert: (element expandedName = 'Q{defaultNS}test').

	element name: 'prefix:test' namespaceURI: 'prefixNS'.
	self assert: (element expandedName = 'Q{prefixNS}test').

	element
		defaultNamespace: '';
		name: 'test'.
	self assert: (element expandedName = 'test').! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:27'!
testHasAttributes
	self
		deny: self nodeClass new hasAttributes;
		assert: self newElementWithAttributes hasAttributes;
		deny: self newElementWithAttributes removeAttributes hasAttributes! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 3/22/2015 03:42'!
testHasNamespaceURI
	| element |

	element := self nodeClass named: 'test'.
	self deny: element hasNamespaceURI.

	element defaultNamespace: 'defaultNS'.
	self assert: element hasNamespaceURI.

	element defaultNamespace: ''.
	self deny: element hasNamespaceURI.

	element name: 'prefix:test' namespaceURI: 'prefixNS'.
	self assert: element hasNamespaceURI.

	element name: 'test'.
	self deny: element hasNamespaceURI.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 02:06'!
testHasNamespaces
	| element |

	element := self nodeClass new.
	self deny: element hasNamespaces.

	element defaultNamespace: 'defaultNS'.
	self assert: element hasNamespaces.

	element undeclarePrefix: ''.
	self deny: element hasNamespaces.

	element declarePrefix: 'prefix' uri: 'prefixNS'.
	self assert: element hasNamespaces.

	element undeclarePrefix: 'prefix'.
	self deny: element hasNamespaces.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 01:45'!
testHasPrefix
	| element |

	element := self nodeClass new.
	self deny: element hasPrefix.

	element name: 'one'.
	self deny: element hasPrefix.

	element declarePrefix: 'prefix' uri: 'prefixNS'.
	element name: 'prefix:two'.
	self assert: element hasPrefix.

	element name: 'three'.
	self deny: element hasPrefix.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:31'!
testIncludesAttribute
	| element |

	element := self newElementWithAttributes.
	element attributeNames do: [:each |
		self assert: (element includesAttribute: each).
		element removeAttribute: each.
		self deny: (element includesAttribute: each)].! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/24/2014 08:11'!
testIsDeclaredPrefix
	| element elementsWithDeclaredPrefix |

	element := self newNodeWithElements.
	elementsWithDeclaredPrefix :=
		element allElementsSelect: [:each | each isDeclaredPrefix: 'prefix'].
	self assert: elementsWithDeclaredPrefix size = 2.

	element allElementsDo: [:each |
		(elementsWithDeclaredPrefix includes: each )
			ifFalse: [self deny: (each isDeclaredPrefix: 'prefix')]].

	element declarePrefix: 'prefix' uri: 'newPrefixNS'.
	element allElementsDo: [:each | self assert: (each isDeclaredPrefix: 'prefix')].

	element undeclarePrefix: 'prefix'.
	element allElementsDo: [:each |
		(elementsWithDeclaredPrefix includes: each )
			ifTrue: [self assert: (each isDeclaredPrefix: 'prefix')]
			ifFalse: [self deny: (each isDeclaredPrefix: 'prefix')]].! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 3/22/2015 03:43'!
testIsDeclaredPrefixURI
	| element elementsWithDeclaredPrefix oldNamespace |

	element := self newNodeWithElements.
	elementsWithDeclaredPrefix :=
		element allElementsSelect: [:each | each isDeclaredPrefix: 'prefix'].
	oldNamespace := elementsWithDeclaredPrefix anyOne resolvePrefix: 'prefix'.
	self assert: elementsWithDeclaredPrefix size = 2.

	element allElementsDo: [:each |
		self deny: (each isDeclaredPrefix: 'prefix' uri: 'newPrefixNS')].

	element declarePrefix: 'prefix' uri: 'newPrefixNS'.
	element allElementsDo: [:each |
		(elementsWithDeclaredPrefix includes: each )
			ifTrue: [self assert: (each isDeclaredPrefix: 'prefix' uri: oldNamespace)]
			ifFalse: [self assert: (each isDeclaredPrefix: 'prefix' uri: 'newPrefixNS')]].

	element undeclarePrefix: 'prefix'.
	element allElementsDo: [:each |
		(elementsWithDeclaredPrefix includes: each )
			ifTrue: [self assert: (each isDeclaredPrefix: 'prefix' uri: oldNamespace)]
			ifFalse: [self deny: (each isDeclaredPrefix: 'prefix')]].! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:41'!
testIsElement
	self assert: self nodeClass new isElement! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 05:24'!
testIsElementNamed
	| element |

	element := self nodeClass named: 'one'.
	self
		assert: (element isElementNamed: 'one');
		deny: (element isElementNamed: 'two').
	element name: 'one:two' namespaceURI: 'testNS'.
	self
		assert: (element isElementNamed: 'one:two');
		assert: (element isElementNamed: 'two');
		deny: (element isElementNamed: 'one').! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 05:23'!
testIsElementNamedAny
	| element |

	element := self nodeClass named: 'one'.
	self
		assert: (element isElementNamedAny: #('two' 'one' 'three'));
		deny: (element isElementNamedAny: #('two' 'three')).
	element name: 'one:two' namespaceURI: 'testNS'.
	self
		assert: (element isElementNamedAny: #('one' 'one:two' 'three'));
		assert: (element isElementNamedAny: #('one' 'two' 'three'));
		deny: (element isElementNamedAny: #('one' 'three')).! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 12/23/2014 16:19'!
testIsInLanguage
	| nodeWithElements |

	super testIsInLanguage.
	nodeWithElements := self newNodeWithElements.
	nodeWithElements allNodesDo: [:each |
		self
			deny: (each isInLanguage: 'en-us');
			deny: (each isInLanguage: 'EN-US');
			deny: (each isInLanguage: 'En');
			deny: (each isInLanguage: 'de-de');
			deny: (each isInLanguage: 'DE-DE');
			deny: (each isInLanguage: 'De')].

	nodeWithElements attributeAt: 'xml:lang' put: 'En-Us'.
	nodeWithElements allNodesDo: [:each |
		self
			assert: (each isInLanguage: 'en-us');
			assert: (each isInLanguage: 'EN-US');
			assert: (each isInLanguage: 'En');
			deny: (each isInLanguage: 'de-de');
			deny: (each isInLanguage: 'DE-DE');
			deny: (each isInLanguage: 'De')].! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 05:00'!
testIsNamed
	| element |

	element := self nodeClass named: 'one'.
	self
		assert: (element isNamed: 'one');
		deny: (element isNamed: 'two').
	element name: 'one:two' namespaceURI: 'testNS'.
	self
		assert: (element isNamed: 'one:two');
		assert: (element isNamed: 'two');
		deny: (element isNamed: 'one').! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 05:02'!
testIsNamedAny
	| element |

	element := self nodeClass named: 'one'.
	self
		assert: (element isNamedAny: #('two' 'one' 'three'));
		deny: (element isNamedAny: #('two' 'three')).
	element name: 'one:two' namespaceURI: 'testNS'.
	self
		assert: (element isNamedAny: #('one' 'one:two' 'three'));
		assert: (element isNamedAny: #('one' 'two' 'three'));
		deny: (element isNamedAny: #('one' 'three')).! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:40'!
testIsRoot
	| element |

	element := self newNodeWithElements.
	self assert: element allElements first isRoot.
	element descendantElementsDo: [:each |
		self deny: each isRoot].
	(element removeNodes: element elements copy) do: [:each |
		self assert: each isRoot].! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 20:20'!
testLocalName
	| element |

	element := self nodeClass new.
	self assert: element name isEmpty.

	element name: 'one'.
	self assert: element localName = 'one'.

	element name: 'two'.
	self assert: element localName = 'two'.

	element declarePrefix: 'prefix' uri: 'prefixNS'.
	element name: 'prefix:three'.
	self assert: element localName = 'three'.

	element name: 'four'.
	self assert: element localName = 'four'.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 04:18'!
testName
	| element |

	self
		assert: (self nodeClass name: 'one') name = 'one';
		should: [self nodeClass name: 'unmapped:two'] raise: XMLNamespaceException.
	
	element := self nodeClass new.
	self assert: element name isEmpty.

	element name: 'one'.
	self assert: element name = 'one'.

	element name: 'two'.
	self assert: element name = 'two'.
	self should: [element name: 'unmapped:two'] raise: XMLNamespaceException.

	element usesNamespaces: false.
	self shouldnt: [element name: 'unmapped:three'] raise: XMLNamespaceException.
	self assert: element name = 'unmapped:three'.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 2/24/2016 15:26'!
testNameAttributes
	| attributes unprefixedAttributes element |

	attributes := self attributes.
	unprefixedAttributes := self unprefixedAttributes.
	element :=
		self nodeClass
			name: 'one'
			attributes: self unprefixedAttributeAssociations.
	self
		assert: element name = 'one';
		assertElement: element hasAttributes: unprefixedAttributes.

	element := self nodeClass name: 'two' attributes: unprefixedAttributes associations.
	self
		assert: element name = 'two';
		assertElement: element hasAttributes: unprefixedAttributes.

	self
		should: [self nodeClass name: 'undeclared:three' attributes: unprefixedAttributes]
		raise: XMLNamespaceException.
	self
		should: [self nodeClass name: 'four' attributes: attributes]
		raise: XMLNamespaceException.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 01:13'!
testNameNamespaceURI
	| element |

	element := self nodeClass new.
	element name: 'one' namespaceURI: 'defaultNS'.
	self
		assert: element name = 'one';
		assert: element namespaceURI = 'defaultNS'.

	element := self nodeClass name: 'prefix:two' namespaceURI: 'prefixNS'.
	self
		assert: element name = 'prefix:two';
		assert: element namespaceURI = 'prefixNS'.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 2/24/2016 15:26'!
testNameNamespaceURIAttributes
	| attributes unprefixedAttributes element |

	attributes := self attributes.
	unprefixedAttributes := self unprefixedAttributes.
	element :=
		self nodeClass
			name: 'prefix:one'
			namespaceURI: 'prefixNS'
			attributes: self attributeAssociations.
	self
		assert: element name = 'prefix:one';
		assert: element namespaceURI = 'prefixNS';
		assertElement: element hasAttributes: attributes.

	element :=
		self nodeClass
			name: 'two'
			namespaceURI: 'prefixNS'
			attributes: unprefixedAttributes.
	self
		assert: element name = 'two';
		assert: element namespaceURI = 'prefixNS';
		assertElement: element hasAttributes: unprefixedAttributes.

	self
		should: [self nodeClass name: 'three' namespaceURI: '' attributes: attributes]
		raise: XMLNamespaceException.
	self
		should: [self nodeClass name: 'four' namespaceURI: 'prefixNS' attributes: attributes]
		raise: XMLNamespaceException.
	self
		should: [self nodeClass name: 'prefix:five' namespaceURI: '' attributes: unprefixedAttributes]
		raise: XMLNamespaceException.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/24/2014 00:33'!
testNameNamespaces
	| element |
	
	element := self nodeClass name: 'one' namespaces: self namespaces.
	self assert: element namespaceURI = 'defaultNS'.
	self namespaces keysAndValuesDo: [:prefix :uri |
		self assert: (element isDeclaredPrefix: prefix uri: uri)].

	element := self nodeClass name: 'two' namespaces: self namespaces associations.
	self assert: element namespaceURI = 'defaultNS'.
	self namespaces keysAndValuesDo: [:prefix :uri |
		self assert: (element isDeclaredPrefix: prefix uri: uri)].

	self
		should: [
			self nodeClass
				name: 'undeclared:three'
				namespaces: self namespaces]
		raise: XMLNamespaceException.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/24/2014 00:33'!
testNameNamespacesAttributes
	| element |

	element :=
		self nodeClass
			name: 'one'
			namespaces: self namespaces
			attributes: self attributes.
	self
		assert: element namespaceURI = 'defaultNS';
		assertElement: element hasAttributes: self attributes.
	self namespaces keysAndValuesDo: [:prefix :uri |
		self assert: (element isDeclaredPrefix: prefix uri: uri)].

	element :=
		self nodeClass
			name: 'two'
			namespaces: self namespaces associations
			attributes: self attributes associations.
	self
		assert: element namespaceURI = 'defaultNS';
		assertElement: element hasAttributes: self attributes.
	self namespaces keysAndValuesDo: [:prefix :uri |
		self assert: (element isDeclaredPrefix: prefix uri: uri)].

	self
		should: [
			self nodeClass
				name: 'undeclared:three'
				namespaces: self namespaces
				attributes: self attributes]
		raise: XMLNamespaceException.
	self
		should: [
			self nodeClass
				name: 'four'
				namespaces: #()
				attributes: self attributes]
		raise: XMLNamespaceException.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 04:15'!
testNamed
	self
		assert: (self nodeClass named: 'test') name = 'test';
		should: [self nodeClass named: 'undeclared:test'] raise: XMLNamespaceException! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 20:32'!
testNamespaceURI
	| element |

	element := self nodeClass new.
	self assert: element namespaceURI isEmpty.

	element defaultNamespace: 'defaultNS'.
	self assert: element namespaceURI = 'defaultNS'.

	element name: 'prefix:test' namespaceURI: 'prefixNS'.
	self assert: element namespaceURI = 'prefixNS'.
	
	element
		defaultNamespace: '';
		name: 'test'.
	self assert: element namespaceURI isEmpty.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/24/2014 18:40'!
testNextElement 
	| root nextElement |

	root := self newNodeWithElements firstElement.
	nextElement := nil.
	root elements reverseDo: [:each |
		self assert: each nextElement == nextElement.
		nextElement := each].! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 01:46'!
testPrefix
	| element |

	element := self nodeClass new.
	self assert: element prefix isEmpty.

	element name: 'one'.
	self assert: element prefix isEmpty.

	element declarePrefix: 'prefix' uri: 'prefixNS'.
	element name: 'prefix:two'.
	self assert: element prefix = 'prefix'.

	element name: 'three'.
	self assert: element prefix isEmpty.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/24/2014 18:40'!
testPreviousElement 
	| root previousElement |

	root := self newNodeWithElements firstElement.
	previousElement := nil.
	root elements do: [:each |
		self assert: each previousElement == previousElement.
		previousElement := each].! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/1/2014 21:02'!
testPrintingDefaultNamespaces
	| element |

	element :=
		XMLElement
			name: 'element'
			namespaceURI: 'default'.
	element addNode:
		((XMLElement
			name: 'element'
			namespaceURI: '')
			addElementNamed: 'empty';
			yourself).

	self assert:
		element printString =
			('<element xmlns="default">',
			'<element xmlns=""><empty /></element>',
			'</element>').! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 10/25/2014 21:33'!
testPrintingNonRootNamespaces
	| element |

	element :=
		XMLElement
			name: 'element'.
	element addNode:
		(XMLElement
			name: 'used:empty'
			namespaces: (Array with: '' -> 'defaultNS' with: 'used' -> 'usedNS')).

	self assert:
		element printString =
			('<element>',
			'<used:empty xmlns="defaultNS" xmlns:used="usedNS" />',
			'</element>').! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/1/2014 21:02'!
testPrintingRootNamespaces
	| element |

	element :=
		XMLElement
			name: 'used:element'
			namespaces: (Array with: '' -> 'defaultNS' with: 'used' -> 'usedNS').
	element addElementNamed: 'empty'.

	self assert:
		element printString =
			('<used:element xmlns="defaultNS" xmlns:used="usedNS">',
			'<empty />',
			'</used:element>').! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:52'!
testRemoveAttribute
	| element |

	element := self newElementWithAttributes.
	element attributeNodes copy do: [:each |
		self
			assert: (element removeAttribute: each name) = each value;
			should: [element removeAttribute: each name] raise: Error;
			deny: (element includesAttribute: each name)].
	self assert: element attributeNodes isEmpty.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/24/2014 22:05'!
testRemoveAttributeIfAbsent
	| element |

	element := self newElementWithAttributes.
	element attributeNodes copy do: [:each |
		self
			assert: (element removeAttribute: each name ifAbsent: [#absent]) = each value;
			assert: (element removeAttribute: each name ifAbsent: [#absent]) = #absent;
			deny: (element includesAttribute: each name)].
	self assert: element attributeNodes isEmpty.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:58'!
testRemoveAttributeNode
	| element |

	element := self newElementWithAttributes.
	element attributeNodes copy do: [:each |
		self
			assert: (element removeAttributeNode: each) == each ;
			should: [element removeAttribute: each] raise: Error;
			deny: (element attributeNodes includes: each)].
	self assert: element attributeNodes isEmpty.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:58'!
testRemoveAttributeNodeIfAbsent
	| element |

	element := self newElementWithAttributes.
	element attributeNodes copy do: [:each |
		self
			assert: (element removeAttributeNode: each ifAbsent: [#absent]) == each ;
			assert: (element removeAttributeNode: each ifAbsent: [#absent]) = #absent;
			deny: (element attributeNodes includes: each)].
	self assert: element attributeNodes isEmpty.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:40'!
testRemoveAttributes
	| element |

	element := self newElementWithAttributes.
	self
		assert: element removeAttributes == element;
		assert: element attributeNodes isEmpty;
		deny: element hasAttributes.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 3/6/2016 18:08'!
testResolvePrefix
	| element |

	element := self nodeClass new.
	self namespaces keysAndValuesDo: [:prefix :uri |
		self assert: (element resolvePrefix: prefix) = ''.
		element declarePrefix: prefix uri: uri.
		self assert: (element resolvePrefix: prefix) = uri].

	self assert:
		(element resolvePrefix: 'xml') = XMLNamespaceScope xmlNamespaceURI.

	self assert:
		(element resolvePrefix: 'xmlns') = XMLNamespaceScope xmlnsNamespaceURI.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 3/6/2016 18:07'!
testResolvePrefixIfUnresolvable
	| element |

	element := self nodeClass new.
	self namespaces keysAndValuesDo: [:prefix :uri |
		self assert:
			(element
				resolvePrefix: prefix
				ifUnresolvable: [#absent]) = #absent.
		element declarePrefix: prefix uri: uri.
		self assert:
			(element
				resolvePrefix: prefix
				ifUnresolvable: [#absent]) = uri].

	self assert:
		(element
			resolvePrefix: 'xml'
			ifUnresolvable: [#absent]) = XMLNamespaceScope xmlNamespaceURI.
	
	self assert:
		(element
			resolvePrefix: 'xmlns'
			ifUnresolvable: [#absent]) = XMLNamespaceScope xmlnsNamespaceURI.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 3/6/2016 18:05'!
testResolvePrefixesOfNamespaceURI
	| element |

	element := self nodeClass new.
	self namespaces keysAndValuesDo: [:prefix :uri |
		self assert: (element resolvePrefixesOfNamespaceURI: uri) isEmpty.

		element
			declarePrefix: prefix
			uri: uri.
		self assert:
			(element resolvePrefixesOfNamespaceURI: uri) =
				(Set with: prefix).

		element
			declarePrefix: 'newPrefix'
			uri: uri.
		self assert:
			(element resolvePrefixesOfNamespaceURI: uri) =
				(Set with: prefix with: 'newPrefix')].

	self assert:
		(element resolvePrefixesOfNamespaceURI:
			XMLNamespaceScope xmlNamespaceURI) =
				(Set with: 'xml').

	self assert:
		(element resolvePrefixesOfNamespaceURI:
			XMLNamespaceScope xmlnsNamespaceURI) =
				(Set with: 'xmlns').! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 3/2/2016 15:54'!
testSetAttributes
	| element newAttributeAssociations |

	element := self newElementWithAttributes.
	newAttributeAssociations :=
		Array
			with: 'newName' -> 'newValue'
			with: 'newName2' -> 'newValue2'.
	element setAttributes: newAttributeAssociations.

	self assert:
		(element attributeNodes collect: [:each |
			each name -> each value]) asArray =
				newAttributeAssociations.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 12/2/2014 22:59'!
testSortKey
	self assert: (self nodeClass named: 'test') sortKey = 'test'! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/24/2014 07:08'!
testUndeclareDefaultNamespace
	| element elementsWithDefaultNamespace |

	element := self newNodeWithElements.
	elementsWithDefaultNamespace :=
		element allElementsSelect: [:each | each defaultNamespace notEmpty].

	element
		defaultNamespace: 'defaultNS';
		undeclareDefaultNamespace;
		allElementsDo: [:each |
			(elementsWithDefaultNamespace includes: each)
				ifTrue: [self assert: each defaultNamespace notEmpty]
				ifFalse: [self assert: each defaultNamespace isEmpty]].! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 3/2/2016 19:14'!
testUndeclarePrefix
	| element |

	element := self newElementWithAttributes.
	element addElementNamed: 'prefix:inner'.

	self
		assert: (element undeclarePrefix: '') == element;
		assert: (element undeclarePrefix: 'prefixTwo') == element.
	element allElementsDo: [:each |
		self
			deny: (each isDeclaredPrefix: '');
			deny: (each isDeclaredPrefix: 'prefixTwo')].
	self
		should: [element undeclarePrefix: 'prefix']
		raise: XMLNamespaceException.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 2/29/2016 08:46'!
testUndeclarePrefixes
	| element innerElement |

	element :=
		self nodeClass
			name: 'outer'
			namespaces: self namespaces
			attributes: self unprefixedAttributes.
	(innerElement := element addElementNamed: 'prefixTwo:inner')
		declarePrefix: 'prefixTwo'
		uri: 'newNS'.

	self
		assert: (element undeclarePrefixes) == element;
		deny: element hasNamespaces;
		assert: innerElement hasNamespaces;
		assert: innerElement declaredPrefixes asArray = #('prefixTwo');
		assert: (innerElement resolvePrefix: 'prefixTwo') = 'newNS'.

	self
		should: [innerElement undeclarePrefixes]
		raise: XMLNamespaceException.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 2/24/2016 16:26'!
testUndeclaringAttributePrefixes
	| element |

	element := self newElementWithAttributes.
	self
		shouldnt: [element undeclarePrefix: '']
		raise: XMLNamespaceException.
	self
		shouldnt: [element undeclarePrefix: 'prefixTwo']
		raise: XMLNamespaceException.
	self
		should: [element undeclarePrefix: 'prefix']
		raise: XMLNamespaceException.! !

!XMLElementTest methodsFor: 'tests' stamp: 'monty 11/24/2014 08:50'!
testWriteXMLOn
	| element attributes |

	element := XMLElement named: 'element'.
	attributes := OrderPreservingDictionary with: 'b'->'two' with: 'a'->'one'.
	self
		assert: element printString = '<element />';
		assert: (element contentString: 'test') printString = '<element>test</element>';
		assert: (element setAttributes: attributes) printString = '<element b="two" a="one">test</element>'! !

!XMLNodeWithElementsTest class methodsFor: 'testing' stamp: 'monty 11/2/2014 11:13'!
isAbstract
	^ true! !

!XMLNodeWithElementsTest methodsFor: 'instance creation' stamp: 'monty 12/9/2015 09:36'!
newNodeWithElements
	^ self nodeClass new
		addNode: (XMLComment string: 'before');
		addNode:
			((XMLElement name: 'zero')
				addNode: (XMLComment string: 'during');
				addNode:
					((XMLElement named: 'one')
						attributeAt: 'one' put: '1';
						addNode: (XMLString string: 'testing one');
						addNode: (XMLElement named: 'two');
						addNode: (XMLPI target: 'target' data: 'data');
						yourself);
				addNode: (XMLString string: 'testing two');
				addNode:
					((XMLElement name: 'three' namespaceURI: 'testNS')
						addNode: (XMLElement name: 'four');
						addNode: (XMLCData string: 'testing three');
						yourself);
				addNode:
					((XMLElement name: 'prefix:four' namespaceURI: 'testNS')
						attributeAt: 'four' put: '4';
						addNode: (XMLElement name: 'prefix:five' namespaceURI: 'testNS');
						yourself);
				addNode: (XMLElement name: 'four');
				yourself);
		addNode: (XMLComment string: 'after');
		yourself.! !

!XMLNodeWithElementsTest methodsFor: 'instance creation' stamp: 'monty 11/7/2014 14:18'!
newNodeWithNodes
	^ self newNodeWithElements! !

!XMLNodeWithElementsTest methodsFor: 'accessing' stamp: 'monty 11/4/2014 19:02'!
nodeClass
	^ XMLNodeWithElements! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 00:04'!
testAddContent
	| node |

	node := self nodeClass new.
	node addContent: 'one'.
	self
		assert: node nodes size = 1;
		assert: (node firstNode isStringNode: 'one').
	node addContent: 'two' asXMLString.
	self
		assert: node nodes size = 1;
		assert: (node firstNode isStringNode: 'onetwo').

	node addNode: (XMLElement named: 'three').
	self
		assert: node nodes size = 2;
		assert: (node firstNode isStringNode: 'onetwo');
		assert: (node nodes second isElementNamed: 'three').

	node
		addContent: (XMLCData string: 'four');
		addContent: (XMLString string: 'five').
	self
		assert: node nodes size = 4;
		assert: (node firstNode isStringNode: 'onetwo');
		assert: (node nodes second isElementNamed: 'three');
		assert: (node nodes third isCData and: [node nodes third string = 'four']);
		assert: (node nodes fourth isStringNode: 'five').! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/7/2014 15:41'!
testAddElementNamed
	| node element |

	node := self nodeClass new.
	(element := node addElementNamed: 'test')
		addElementNamed: 'first';
		declarePrefix: 'prefix' uri: 'ns';
		addElementNamed: 'prefix:second'.
	self assert: element elementNames asArray = #('first' 'prefix:second').
	self
		should: [element addElementNamed: 'undeclared:fourth']
		raise: XMLNamespaceException.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/7/2014 15:45'!
testAddElementNamedNamespaceURI
	| node element |

	node := self nodeClass new.
	(element := node addElementNamed: 'test' namespaceURI: 'defaultNS')
		addElementNamed: 'prefix:first' namespaceURI: 'firstNS';
		addElementNamed: 'second' namespaceURI: 'secondNS'.
	self
		assert: element namespaceURI = 'defaultNS';
		assert: element nodes first name = 'prefix:first';
		assert: element nodes first namespaceURI = 'firstNS';
		assert: element nodes second name = 'second';
		assert: element nodes second namespaceURI = 'secondNS'.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/14/2015 05:18'!
testAllElements
	| nodeWithElements allElements |

	nodeWithElements := self newNodeWithElements.
	allElements := nodeWithElements allElements.
	nodeWithElements isElement
		ifTrue: [
			self assert: allElements first == nodeWithElements.
			allElements removeFirst].
	self assert:
		(allElements collect: [:each | each name]) asArray =
			#('zero' 'one' 'two' 'three' 'four' 'prefix:four' 'prefix:five' 'four').
	self
		assertList: allElements first allElements selects: (allElements copyFrom: 1 to: 8);
		assertList: allElements second allElements selects: (allElements copyFrom: 2 to: 3);
		assertList: allElements third allElements selects: (allElements copyFrom: 3 to: 3);
		assertList: allElements fourth allElements selects: (allElements copyFrom: 4 to: 5);
		assertList: allElements fifth allElements selects: (allElements copyFrom: 5 to: 5);
		assertList: allElements sixth allElements selects: (allElements copyFrom: 6 to: 7);
		assertList: allElements seventh allElements selects: (allElements copyFrom: 7 to: 7);
		assertList: allElements eighth allElements selects: (allElements copyFrom: 8 to: 8).! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/7/2014 14:07'!
testAllElementsCollect
	| nodeWithElements allElements |

	nodeWithElements := self newNodeWithElements.
	allElements := nodeWithElements allElements.
	self
		assertList: (nodeWithElements allElementsCollect: [:each | each identityHash])
		collects: (allElements collect: [:each | each identityHash]).

	allElements do: [:node |
		self
			assertList: (node allElementsCollect: [:each | each identityHash])
			collects: (node allElements collect: [:each | each identityHash])].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/7/2014 14:37'!
testAllElementsDetect
	| nodeWithElements allElements detectedElement |

	nodeWithElements := self newNodeWithElements.
	allElements := nodeWithElements allElements.
	detectedElement := nodeWithElements allElementsDetect: [:each | each isNamed: 'prefix:four'].
	self
		assert: detectedElement notNil;
		assert: (detectedElement isNamed: 'prefix:four');
		assert: detectedElement == (allElements detect: [:each | each isNamed: 'prefix:four']);
		assert: (nodeWithElements allElementsDetect: [:each | each isNamed: 'absent']) isNil.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/7/2014 14:46'!
testAllElementsDetectIfNone
	| nodeWithElements allElements detectedElement |

	nodeWithElements := self newNodeWithElements.
	allElements := nodeWithElements allElements.
	detectedElement :=
		nodeWithElements
			allElementsDetect: [:each | each isNamed: 'prefix:four']
			ifNone: [#absent].
	self
		deny: detectedElement = #absent;
		assert: (detectedElement isNamed: 'prefix:four').
	self assert:	
		detectedElement ==
			(allElements
				detect: [:each | each isNamed: 'prefix:four']
				ifNone: [#absent]).
	self assert:
		(nodeWithElements
			allElementsDetect: [:each | each isNamed: 'absent']
			ifNone: [#absent]) = #absent.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/7/2014 14:07'!
testAllElementsDo
	| nodeWithElements allElements |

	nodeWithElements := self newNodeWithElements.
	allElements := nodeWithElements allElements.
	self
		should: [:block | nodeWithElements allElementsDo: block]
		enumerate: allElements.

	allElements do: [:element |
		self
			should: [:block | element allElementsDo: block]
			enumerate: element allElements].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 21:22'!
testAllElementsNamed
	| nodeWithElements allElements namedElements |

	nodeWithElements := self newNodeWithElements.
	self assert: (nodeWithElements allElementsNamed: 'absent') isEmpty.

	allElements := nodeWithElements allElements.
	namedElements := nodeWithElements allElementsNamed: 'four'. 
	self
		assert: namedElements size = 3;
		assertList: namedElements selects: (allElements select: [:each | each isNamed: 'four']).! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 21:25'!
testAllElementsNamedDo
	| nodeWithElements allElements namedElements |

	nodeWithElements := self newNodeWithElements.
	self
		should: [:block | nodeWithElements allElementsNamed: 'absent' do: block]
		enumerate: #().

	allElements := nodeWithElements allElements.
	namedElements := nodeWithElements allElementsNamed: 'four'. 
	self
		should: [:block | nodeWithElements allElementsNamed: 'four' do: block]
		enumerate: namedElements.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 21:22'!
testAllElementsNamedNamespaceURI
	| nodeWithElements allElements namedElements |

	nodeWithElements := self newNodeWithElements.
	self assert: (nodeWithElements allElementsNamed: 'four' namespaceURI: 'absent') isEmpty.

	allElements := nodeWithElements allElements.
	namedElements := nodeWithElements allElementsNamed: 'four' namespaceURI: 'testNS'. 
	self
		assert: namedElements size = 2;
		assertList: namedElements
		selects:
			(allElements select: [:each |
				(each isNamed: 'four')
					and: [each namespaceURI = 'testNS']]).! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/7/2014 14:07'!
testAllElementsSelect
	| nodeWithElements allElements |

	nodeWithElements := self newNodeWithElements.
	allElements := nodeWithElements allElements.
	self
		assertList: (nodeWithElements allElementsSelect: [:each | each hasChildren])
		selects: (allElements select: [:each | each hasChildren]).

	allElements do: [:node |
		self
			assertList: (node allElementsSelect: [:each | each hasChildren])
			selects: (node allElements select: [:each | each hasChildren])].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/14/2015 05:18'!
testAllNodes
	| nodeWithElements allNodes |

	nodeWithElements := self newNodeWithElements.
	allNodes := nodeWithElements allNodes.
	self
		assert: allNodes size = 16;
		assert: (allNodes at: 1) == nodeWithElements;
		assert: (allNodes at: 2) isComment;
		assert: ((allNodes at: 3) isElementNamed: 'zero');
		assert: (allNodes at: 4) isComment;
		assert: ((allNodes at: 5) isElementNamed: 'one');
		assert: ((allNodes at: 6) isStringNode: 'testing one');
		assert: ((allNodes at: 7) isElementNamed: 'two');
		assert: (allNodes at: 8) isPI;
		assert: ((allNodes at: 9) isStringNode: 'testing two');
		assert: ((allNodes at: 10) isElementNamed: 'three');
		assert: ((allNodes at: 11) isElementNamed: 'four');
		assert: ((allNodes at: 12) isStringNode: 'testing three');
		assert: ((allNodes at: 13) isElementNamed: 'prefix:four');
		assert: ((allNodes at: 14) isElementNamed: 'prefix:five');
		assert: ((allNodes at: 15) isElementNamed: 'four');
		assert: (allNodes at: 16) isComment.
	self
		assertList: (allNodes at: 3) allNodes selects: (allNodes copyFrom: 3 to: 15);
		assertList: (allNodes at: 5) allNodes selects: (allNodes copyFrom: 5 to: 8);
		assertList: (allNodes at: 7) allNodes selects: (allNodes copyFrom: 7 to: 7);
		assertList: (allNodes at: 10) allNodes selects: (allNodes copyFrom: 10 to: 12);
		assertList: (allNodes at: 11) allNodes selects: (allNodes copyFrom: 11 to: 11);
		assertList: (allNodes at: 13) allNodes selects: (allNodes copyFrom: 13 to: 14);
		assertList: (allNodes at: 14) allNodes selects: (allNodes copyFrom: 14 to: 14);
		assertList: (allNodes at: 15) allNodes selects: (allNodes copyFrom: 15 to: 15).! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 3/8/2015 17:07'!
testAncestorElements
	| nodeWithElements ancestorStack |

	super testAncestorElements.

	nodeWithElements := self newNodeWithElements.
	ancestorStack := OrderedCollection new.
	nodeWithElements allNodesDo: [:each |
		[ancestorStack isEmpty
			or: [ancestorStack last == each parent]]
			whileFalse: [ancestorStack removeLast].
		self assert:
			each ancestorElements asOrderedCollection =
				ancestorStack reversed.
		each isElement
			ifTrue: [ancestorStack addLast: each]].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 3/8/2015 17:07'!
testAncestorElementsDo
	| nodeWithElements |

	super testAncestorElementsDo.

	nodeWithElements := self newNodeWithElements.
	nodeWithElements allNodesDo: [:each |
		self
			should: [:block | each ancestorElementsDo: block]
			enumerate: each ancestorElements].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 05:08'!
testConfiguration
	| nodeWithElements configuration newConfiguration |

	nodeWithElements := self newNodeWithElements.
	configuration := nodeWithElements configuration.
	self assert: configuration class == XMLDOMConfiguration.
	nodeWithElements allElementsDo: [:element |
		self assert: element configuration == configuration.
		element attributeNodes do: [:each |
			self assert: each configuration == configuration]].

	newConfiguration := XMLDOMConfiguration new.
	nodeWithElements configuration: newConfiguration.
	self assert: nodeWithElements configuration == newConfiguration.
	nodeWithElements allElementsDo: [:element |
		self assert: element configuration == newConfiguration.
		element attributeNodes do: [:each |
			self assert: each configuration == newConfiguration]].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 05:43'!
testContentNodes
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements.
	self
		assertList: nodeWithElements contentNodes
		selects: (nodeWithElements nodes select: [:each | each isContentNode]).
	nodeWithElements allElementsDo: [:element |
		self
			assertList: element contentNodes
			selects: (element nodes select: [:each | each isContentNode])].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 05:44'!
testContentNodesDo
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements.
	self
		should: [:block | nodeWithElements contentNodesDo: block]
		enumerate: nodeWithElements contentNodes.
	nodeWithElements allElementsDo: [:each |
		self
			should: [:block | each contentNodesDo: block]
			enumerate: each contentNodes].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 2/2/2015 16:35'!
testContentString
	self
		assert: self newNodeWithElements contentString = 'testing one testing two testing three';
		assert: (self newNodeWithElements contentString: 'new') contentString = 'new'! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 2/2/2015 16:35'!
testContentStringAt
	| root |

	root := self newNodeWithElements firstElement.
	root elementsDo: [:each |
		self assert: (root contentStringAt: each name) = each contentString].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:50'!
testDescendantElements
	| nodeWithElements descendantElements |

	nodeWithElements := self newNodeWithElements.
	descendantElements := nodeWithElements descendantElements.
	self assert:
		(descendantElements collect: [:each | each name]) asArray =
			#('zero' 'one' 'two' 'three' 'four' 'prefix:four' 'prefix:five' 'four').
	self
		assertList: descendantElements first descendantElements selects: (descendantElements copyFrom: 2 to: 8);
		assertList: descendantElements second descendantElements selects: (descendantElements copyFrom: 3 to: 3);
		assertList: descendantElements third descendantElements selects: descendantElements copyEmpty;
		assertList: descendantElements fourth descendantElements selects: (descendantElements copyFrom: 5 to: 5);
		assertList: descendantElements fifth descendantElements selects: descendantElements copyEmpty;
		assertList: descendantElements sixth descendantElements selects: (descendantElements copyFrom: 7 to: 7);
		assertList: descendantElements seventh descendantElements selects: descendantElements copyEmpty;
		assertList: descendantElements eighth descendantElements selects: descendantElements copyEmpty.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:49'!
testDescendantElementsCollect
	| nodeWithElements descendantElements |

	nodeWithElements := self newNodeWithElements.
	descendantElements := nodeWithElements descendantElements.
	self
		assertList: (nodeWithElements descendantElementsCollect: [:each | each identityHash])
		collects: (descendantElements collect: [:each | each identityHash]).

	descendantElements do: [:node |
		self
			assertList: (node descendantElementsCollect: [:each | each identityHash])
			collects: (node descendantElements collect: [:each | each identityHash])].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:49'!
testDescendantElementsDetect
	| nodeWithElements descendantElements detectedElement |

	nodeWithElements := self newNodeWithElements.
	descendantElements := nodeWithElements descendantElements.
	detectedElement := nodeWithElements descendantElementsDetect: [:each | each isNamed: 'prefix:four'].
	self
		assert: detectedElement notNil;
		assert: (detectedElement isNamed: 'prefix:four');
		assert: detectedElement == (descendantElements detect: [:each | each isNamed: 'prefix:four']);
		assert: (detectedElement descendantElementsDetect: [:each | each isNamed: 'prefix:four']) isNil.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:49'!
testDescendantElementsDetectIfNone
	| nodeWithElements descendantElements detectedElement |

	nodeWithElements := self newNodeWithElements.
	descendantElements := nodeWithElements descendantElements.
	detectedElement :=
		nodeWithElements
			descendantElementsDetect: [:each | each isNamed: 'prefix:four']
			ifNone: [#absent].
	self
		deny: detectedElement = #absent;
		assert: (detectedElement isNamed: 'prefix:four').
	self assert:	
		detectedElement ==
			(descendantElements
				detect: [:each | each isNamed: 'prefix:four'] 
				ifNone: [#absent]).
	self assert:
		(detectedElement
			descendantElementsDetect: [:each | each isNamed: 'prefix:four']
			ifNone: [#absent]) = #absent.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:49'!
testDescendantElementsDo
	| nodeWithElements descendantElements |

	nodeWithElements := self newNodeWithElements.
	descendantElements := nodeWithElements descendantElements.
	self
		should: [:block | nodeWithElements descendantElementsDo: block]
		enumerate: descendantElements.

	descendantElements do: [:element |
		self
			should: [:block | element descendantElementsDo: block]
			enumerate: element descendantElements].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:49'!
testDescendantElementsNamed
	| nodeWithElements descendantElements namedElements |

	nodeWithElements := self newNodeWithElements.
	self assert: (nodeWithElements descendantElementsNamed: 'absent') isEmpty.

	descendantElements := nodeWithElements descendantElements.
	namedElements := nodeWithElements descendantElementsNamed: 'four'. 
	self assert: namedElements size = 3.
	self
		assertList: namedElements
		selects: (descendantElements select: [:each | each isNamed: 'four']).

	namedElements do: [:each |
		self assert: (each descendantElementsNamed: 'four') isEmpty]! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:49'!
testDescendantElementsNamedDo
	| nodeWithElements namedElements |

	nodeWithElements := self newNodeWithElements.
	self
		should: [:block | nodeWithElements descendantElementsNamed: 'absent' do: block]
		enumerate: #().

	namedElements := nodeWithElements descendantElementsNamed: 'four'. 
	self
		should: [:block | nodeWithElements descendantElementsNamed: 'four' do: block]
		enumerate: namedElements.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:49'!
testDescendantElementsNamedNamespaceURI
	| nodeWithElements descendantElements namedElements |

	nodeWithElements := self newNodeWithElements.
	self assert:
		(nodeWithElements
			descendantElementsNamed: 'four'
			namespaceURI: 'absent') isEmpty.

	descendantElements := nodeWithElements descendantElements.
	namedElements :=
		nodeWithElements
			descendantElementsNamed: 'four'
			namespaceURI: 'testNS'. 
	self
		assert: namedElements size = 2;
		assertList: namedElements
		selects:
			(descendantElements select: [:each |
				(each isNamed: 'four')
					and: [each namespaceURI = 'testNS']]).! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 2/27/2016 13:48'!
testDescendantElementsSelect
	| nodeWithElements descendantElements |

	nodeWithElements := self newNodeWithElements.
	descendantElements := nodeWithElements descendantElements.
	self
		assertList: (nodeWithElements descendantElementsSelect: [:each | each hasChildren])
		selects: (descendantElements select: [:each | each hasChildren]).

	descendantElements do: [:node |
		self
			assertList: (node descendantElementsSelect: [:each | each hasChildren])
			selects: (node descendantElements select: [:each | each hasChildren])].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 18:42'!
testElementAfter
	| root elementAfter |

	root := self newNodeWithElements firstElement.
	elementAfter := nil.
	root elements reverseDo: [:each |
		self assert: (root elementAfter: each) == elementAfter.
		elementAfter := each].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 21:34'!
testElementAt
	| root  |

	root := self newNodeWithElements firstElement.
	self assert: (root elementAt: 'absent') isNil.
	root elements do: [:each |
		self assert:
			(root elementAt: each name) ==
				(root elements detect: [:element | element isNamed: each name])].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 21:35'!
testElementAtIfAbsent
	| root  |

	root := self newNodeWithElements firstElement.
	self assert: (root elementAt: 'absent' ifAbsent: [#absent]) = #absent.
	root elements do: [:each |
		self assert:
			(root elementAt: each name ifAbsent: [#absent]) ==
				(root elements detect: [:element | element isNamed: each name])].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/23/2014 03:57'!
testElementAtNamespaceURI
	| root element |

	root := self newNodeWithElements firstElement.
	self assert: (root elementAt: 'four' namespaceURI: 'absent') isNil.

	element := root elementAt: 'four' namespaceURI: 'testNS'.
	self deny: element isNil.
	self assert:
		element ==
			(root elements detect: [:each |
				(each isNamed: 'four') and: [each namespaceURI = 'testNS']]).

	root removeNode: element.
	self deny: (root elementAt: 'four' namespaceURI: 'testNS') == element.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/25/2014 00:04'!
testElementAtNamespaceURIIfAbsent
	| root element |

	root := self newNodeWithElements firstElement.
	self assert:
		(root
			elementAt: 'four'
			namespaceURI: 'absent'
			ifAbsent: [#absent]) = #absent.

	element := root elementAt: 'four' namespaceURI: 'testNS' ifAbsent: [#absent].
	self deny: element = #absent.
	self assert:
		element ==
			(root elements
				detect: [:each |
					(each isNamed: 'four')
						and: [each namespaceURI = 'testNS']]
				ifNone: [#absent]).

	root removeNode: element.
	self assert:
		(root
			elementAt: 'four'
			namespaceURI: 'testNS'
			ifAbsent: [#absent]) = #absent.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 18:41'!
testElementBefore
	| root elementBefore |

	root := self newNodeWithElements firstElement.
	elementBefore := nil.
	root elements do: [:each |
		self assert: (root elementBefore: each) == elementBefore.
		elementBefore := each].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 22:09'!
testElementNames
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements.
	self
		assertList: nodeWithElements elementNames
		collects: (nodeWithElements elements collect: [:each | each name]).
	nodeWithElements allElementsDo: [:element |
		self
			assertList: element elementNames
			collects: (element elements collect: [:each | each name])].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 01:42'!
testElementRenamingToNext
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements
		usesNamespaces: false;
		allElementsDo: [:each |
		| parent oldName oldLocalName nextElement |

		((parent := each parent) notNil
			and: [(nextElement := each nextElement) notNil])
			ifTrue: [
				oldName := each name.
				oldLocalName := each localName.
				
				each name: nextElement name.
				each name = oldName
					ifFalse: [
						self
							deny: ((parent elementAt: oldName) == each);
							deny: ((parent elementsAt: oldName) includes: each)].
				each localName = oldLocalName
					ifFalse: [
						self
							deny: ((parent elementAt: oldLocalName) == each);
							deny: ((parent elementsAt: oldLocalName) includes: each)].
				self
					assert: ((parent elementsAt: each name) after: each) == nextElement;
					assert: ((parent elementsAt: each localName) after: each) == nextElement]].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 01:44'!
testElementRenamingToPrevious
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements
		usesNamespaces: false;
		allElementsDo: [:each |
		| parent oldName oldLocalName previousElement |

		((parent := each parent) notNil
			and: [(previousElement := each previousElement) notNil])
			ifTrue: [
				oldName := each name.
				oldLocalName := each localName.
				
				each name: previousElement name.
				each name = oldName
					ifFalse: [
						self
							deny: ((parent elementAt: oldName) == each);
							deny: ((parent elementsAt: oldName) includes: each)].
				each localName = oldLocalName
					ifFalse: [
						self
							deny: ((parent elementAt: oldLocalName) == each);
							deny: ((parent elementsAt: oldLocalName) includes: each)].
				self
					assert: ((parent elementsAt: each name) before: each) == previousElement;
					assert: ((parent elementsAt: each localName) before: each) == previousElement]].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/23/2014 04:05'!
testElements
	| nodeWithElmeents |

	nodeWithElmeents := self newNodeWithElements.
	self
		assertList: nodeWithElmeents elements 
		selects: (nodeWithElmeents nodes select: [:each | each isElement]).
	nodeWithElmeents allElementsDo: [:element |
		self
			assertList: element elements 
			selects: (element nodes select: [:each | each isElement])].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 23:58'!
testElementsAt
	| root namedElements |

	root := self newNodeWithElements firstElement.
	self assert: (root elementsAt: 'absent') isEmpty.

	namedElements := root elementsAt: 'four'.
	self assert: namedElements size = 2.
	self
		assertList: namedElements
		selects: (root elements select: [:each | each isNamed: 'four']).

	(namedElements := namedElements copy) do: [:each |
		root removeNode: each.
		self
			assertList: (root elementsAt: 'four')
			selects: (namedElements copyAfter: each)].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 23:58'!
testElementsAtDo
	| root namedElements |

	root := self newNodeWithElements firstElement.
	self
		should: [:block | root elementsAt: 'absent' do: block]
		enumerate: #().
	namedElements := root elementsAt: 'four'.
	self
		should: [:block | root elementsAt: 'four' do: block]
		enumerate: namedElements.

	(namedElements := namedElements copy) do: [:each |
		root removeNode: each.
		self
			should: [:block | root elementsAt: 'four' do: block]
			enumerate: (namedElements copyAfter: each)].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 23:58'!
testElementsAtNamespaceURI
	| root namedElements |

	root := self newNodeWithElements firstElement.
	self assert: (root elementsAt: 'four' namespaceURI: 'absent') isEmpty.

	namedElements := root elementsAt: 'four' namespaceURI: 'testNS'.
	self assert: namedElements size = 1.
	self
		assertList: namedElements
		selects:
			(root elements select: [:each |
				(each isNamed: 'four') and: [each namespaceURI = 'testNS']]).

	root removeNode: namedElements first.
	self assert: (root elementsAt: 'four' namespaceURI: 'testNS') isEmpty.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 02:30'!
testElementsCollect
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements.
	self
		assertList: (nodeWithElements elementsCollect: [:each | each name])
		collects: (nodeWithElements elements collect: [:each | each name]).
	nodeWithElements allElementsDo: [:element |
		self
			assertList: (element elementsCollect: [:each | each name])
			collects: (element elements collect: [:each | each name])].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 05:18'!
testElementsDetect
	| root element |

	root := self newNodeWithElements firstElement.
	element :=
		root elementsDetect: [:each |
			(each isNamed: 'four') and: [each namespaceURI = 'testNS']].
	self deny: element isNil.
	self assert:
		element == 
			(root elements detect: [:each |
				(each isNamed: 'four') and: [each namespaceURI = 'testNS']]).
	self assert:
		(root elementsDetect: [:each |
			(each isNamed: 'four') and: [each namespaceURI = 'absentNS']]) isNil.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 05:20'!
testElementsDetectIfNone
	| root element |

	root := self newNodeWithElements firstElement.
	element :=
		root
			elementsDetect: [:each |
				(each isNamed: 'four') and: [each namespaceURI = 'testNS']]
			ifNone: [#absent].
	self deny: element = #absent.
	self assert:
		element == 
			(root elements
				detect: [:each |
					(each isNamed: 'four') and: [each namespaceURI = 'testNS']]
				ifNone: [#absent]).
	self assert:
		(root
			elementsDetect: [:each |
				(each isNamed: 'four') and: [each namespaceURI = 'absentNS']]
			ifNone: [#absent]) = #absent.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 02:33'!
testElementsDo
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements.
	self
		should: [:block | nodeWithElements elementsDo: block]
		enumerate: nodeWithElements elements.

	nodeWithElements allElementsDo: [:each |
		self
			should: [:block | each elementsDo: block]
			enumerate: each elements].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 02:32'!
testElementsSelect
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements.
	self
		assertList: (nodeWithElements elementsSelect: [:each | each hasChildren])
		selects: (nodeWithElements elements select: [:each | each hasChildren]).
	nodeWithElements allElementsDo: [:element |
		self
			assertList: (element elementsSelect: [:each | each hasChildren])
			selects: (element elements select: [:each | each hasChildren])].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 21:57'!
testFindElementNamed
	| nodeWithElements allElements foundElement |

	nodeWithElements := self newNodeWithElements.
	allElements := nodeWithElements allElements.
	foundElement := nodeWithElements findElementNamed: 'four'.
	self
		assert: foundElement notNil;
		assert: (foundElement isNamed: 'four');
		assert: foundElement == (allElements detect: [:each | each isNamed: 'four']);
		assert: (foundElement findElementNamed: 'four') == foundElement;
		assert: (nodeWithElements findElementNamed: 'absent') isNil.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 22:02'!
testFindElementNamedNamespaceURI
	| nodeWithElements allElements foundElement |

	nodeWithElements := self newNodeWithElements.
	allElements := nodeWithElements allElements.
	foundElement := nodeWithElements findElementNamed: 'four' namespaceURI: 'testNS'.
	self
		assert: foundElement notNil;
		assert: (foundElement isNamed: 'four').
	self assert:
		foundElement ==
			(allElements detect: [:each |
				(each isNamed: 'four') and: [each namespaceURI = 'testNS']]).
	self
		assert: (foundElement findElementNamed: 'four' namespaceURI: 'testNS') == foundElement;
		assert: (nodeWithElements findElementNamed: 'four' namespaceURI: 'absentNS') isNil.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 09:09'!
testFindElementNamedNamespaceURIWith
	| nodeWithElements allElements foundElement |

	nodeWithElements := self newNodeWithElements.
	allElements := nodeWithElements allElements.
	foundElement :=
		nodeWithElements
			findElementNamed: 'four'
			namespaceURI: 'testNS'
			with: [:each | each hasChildren].
	self
		assert: foundElement notNil;
		assert: (foundElement isNamed: 'prefix:four').
	self assert:
		foundElement ==
			(allElements detect: [:each |
				(each isNamed: 'four')
					and: [(each namespaceURI = 'testNS')
						and: [each hasChildren]]]).
	self assert:
		(foundElement
			findElementNamed: 'four'
			namespaceURI: 'testNS'
			with: [:each | each hasChildren]) == foundElement.
	self assert:
		(nodeWithElements 
			findElementNamed: 'four'
			namespaceURI: 'testNS'
			with: [:each | false]) isNil.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 22:04'!
testFindElementNamedWith
	| nodeWithElements allElements foundElement |

	nodeWithElements := self newNodeWithElements.
	allElements := nodeWithElements allElements.
	foundElement := nodeWithElements findElementNamed: 'four' with: [:each | each hasChildren].
	self
		assert: foundElement notNil;
		assert: (foundElement isNamed: 'four').
	self assert:
		foundElement ==
			(allElements detect: [:each | (each isNamed: 'four') and: [each hasChildren]]).
	self
		assert: (foundElement findElementNamed: 'four' with: [:each | each hasChildren]) == foundElement;
		assert: (nodeWithElements findElementNamed: 'four' with: [:each | false]) isNil.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 12/9/2015 11:02'!
testFindElementWithID
	| xml document idElement xmlIDElement |

	xml := '
		<!!DOCTYPE a [
			<!!ELEMENT a (
				b,
				c,
				d,
				e
			)>
			<!!ELEMENT b EMPTY>
			<!!ELEMENT c EMPTY>
			<!!ELEMENT d EMPTY>
			<!!ELEMENT e EMPTY>
			<!!ATTLIST b id CDATA #REQUIRED>
			<!!ATTLIST c id ID #REQUIRED>
			<!!ATTLIST d xml:id ID #REQUIRED> ]>
		<a>
			<b id="one"/>
			<c id="one"/>
			<d xml:id="two"/>
			<e/>
		</a>'.

	document := XMLDOMParser parse: xml.
	idElement := document findElementNamed: 'c'.
	xmlIDElement := document findElementNamed: 'd'.
	self
		assert: (document findElementWithID: 'one') = idElement;
		assert: (idElement findElementWithID: 'one') = idElement;
		assert: (idElement nextElement findElementWithID: 'one') isNil;
		assert: (document findElementWithID: 'two') = xmlIDElement;
		assert: (xmlIDElement findElementWithID: 'two') = xmlIDElement;
		assert: (xmlIDElement nextElement findElementWithID: 'two') isNil;
		assert: (document findElementWithID: 'three') isNil.

	document root
		removeNode: idElement;
		removeNode: xmlIDElement.
	self
		assert: (document findElementWithID: 'one') isNil;
		assert: (idElement findElementWithID: 'one') isNil;
		assert: (document findElementWithID: 'two') isNil;
		assert: (xmlIDElement findElementWithID: 'two') isNil.

	document :=
		XMLDOMParser
			parse: xml
			usingNamespaces: true
			validation: false.
	self
		assert: (document findElementWithID: 'one') isNil;
		assert: (document findElementWithID: 'two') isNil.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/19/2014 22:19'!
testFirstElement
	| root |

	root := self newNodeWithElements nodeAt: 2.
	root elements copy do: [:each |
		self  assert: root firstElement == each.
		root removeNode: each].
	self assert: root firstElement isNil.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 05:42'!
testHasContentNodes
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements.
	self assert: nodeWithElements hasContentNodes.
	nodeWithElements allElementsDo: [:element |
		self assert: element hasContentNodes = element contentNodes notEmpty].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 00:21'!
testHasElements
	self
		deny: self nodeClass new hasElements;
		assert: self newNodeWithElements hasElements;
		deny: self newNodeWithElements removeNodes hasElements! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 05:41'!
testHasStringNodes
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements.
	self deny: nodeWithElements hasStringNodes.
	nodeWithElements allElementsDo: [:element |
		self assert: element hasStringNodes = element stringNodes notEmpty].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/25/2014 00:02'!
testIncludesElement
	| root |
	
	root := self newNodeWithElements firstElement.
	self deny: (root includesElement: 'absent').
	root elements copy do: [:each |
		self assert: (root includesElement: each name).
		root removeNode: each.
		self deny: (root includesElement: each name)].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/7/2014 15:10'!
testInnerXML
	| node element |

	node := self nodeClass new.
	element :=
		node addNode:
			(XMLElement name: 'test' namespaces: (Array with: 'prefix'-> 'ns')).
	element innerXML: '<prefix:inner></prefix:inner>'.
	element firstNode innerXML: 'test<!!--comment-->'.
	self
		assert: element nodes size = 1;
		assert: (element firstNode isNamed: 'prefix:inner');
		assert: (element firstNode contentString = 'test');
		assert: node innerXML = '<test xmlns:prefix="ns"><prefix:inner>test</prefix:inner></test>';
		assert: element innerXML = '<prefix:inner xmlns:prefix="ns">test</prefix:inner>';
		assert: element firstNode innerXML = 'test'.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 12/9/2015 09:43'!
testInnerXMLPrettyPrinted
	| nodeWithElements printString prettyPrintString prettyNodeWithElements |

	nodeWithElements := self newNodeWithElements.
	printString := nodeWithElements innerXML.
	prettyPrintString := nodeWithElements innerXMLPrettyPrinted.
	self deny: printString = prettyPrintString.

	prettyNodeWithElements :=
		(XMLDOMParser on: prettyPrintString)
			preservesCommentNodes: true;
			preservesIgnorableWhitespace: true;
			parseDocument.
	(nodeWithElements allElements copyWithout: nodeWithElements)
		with: (prettyNodeWithElements allElements copyWithout: prettyNodeWithElements)
		do: [:element :prettyElement |
			element hasChildren
				ifTrue: [
					element firstNode isStringNode
						ifFalse: [
							self
								assert: prettyElement firstNode isStringNode;
								assert: prettyElement firstNode isWhitespace].
					element hasStringNodes
						ifFalse: [
							self
								assert: prettyElement lastNode isStringNode;
								assert: prettyElement lastNode isWhitespace]]].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 5/7/2015 23:10'!
testInnerXMLResetting
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements.
	self
		should: [nodeWithElements firstElement innerXML: '<one><two></one></two>']
		raise: XMLWellFormednessException.
	self assert: nodeWithElements printString = self newNodeWithNodes printString.

	self
		should: [nodeWithElements firstElement innerXML: '<one>&invalid;</one>']
		raise: XMLValidationException.
	self assert: nodeWithElements printString = self newNodeWithNodes printString.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:46'!
testIsContentNode
	self assert: self nodeClass new isContentNode! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/19/2014 22:19'!
testLastElement
	| root |

	root := self newNodeWithElements nodeAt: 2.
	root elements reversed do: [:each |
		self  assert: root lastElement == each.
		root removeNode: each].
	self assert: root lastElement isNil.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:01'!
testNewCData
	| cdata |

	cdata := self nodeClass new newCData: 'test'.
	self
		assert: cdata isCData;
		assert: cdata string = 'test'.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:12'!
testNewComment
	| comment |

	comment := self nodeClass new newComment: 'test'.
	self
		assert: comment isComment;
		assert: comment string = 'test'.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:09'!
testNewDocument
	| node document |

	node := self nodeClass new.
	document := node newDocument.
	self
		assert: document isDocument;
		assert: document configuration == node configuration;
		assert: document nodeFactory == node nodeFactory.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:09'!
testNewElement
	| node element |

	node := self nodeClass new.
	element := node newElement.
	self
		assert: element isElement;
		assert: element configuration == node configuration;
		assert: element nodeFactory == node nodeFactory.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:09'!
testNewElementNamed
	| node element |

	node := self nodeClass new.
	element := node newElementNamed: 'test'.
	self
		assert: element isElement;
		assert: element configuration == node configuration;
		assert: element nodeFactory == node nodeFactory;
		assert: element name = 'test'.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:29'!
testNewElementNamedNamespaceURI
	| node element |

	node := self nodeClass new.
	element := node newElementNamed: 'prefix:test' namespaceURI: 'prefixNS'.
	self
		assert: element isElement;
		assert: element configuration == node configuration;
		assert: element nodeFactory == node nodeFactory;
		assert: element name = 'prefix:test';
		assert: element namespaceURI = 'prefixNS'.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:11'!
testNewPI
	self assert: self nodeClass new newPI isPI! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:06'!
testNewString
	| string |

	string := self nodeClass new newString: 'test'.
	self
		assert: string isStringNode;
		assert: string string = 'test'.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/23/2014 00:44'!
testNodeFactory
	| nodeWithElements newFactory |

	nodeWithElements := self newNodeWithElements.
	newFactory := XMLPluggableNodeFactory new.
	self
		assert: (nodeWithElements nodeFactory isKindOf: XMLNodeFactory);
		assert: (nodeWithElements nodeFactory: newFactory) nodeFactory == newFactory.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 12/9/2015 09:22'!
testPrettyPrinted
	| nodeWithElements printString prettyPrintString prettyNodeWithElements |

	nodeWithElements := self newNodeWithElements.
	printString := nodeWithElements printString.
	prettyPrintString := nodeWithElements prettyPrinted.
	self deny: printString = prettyPrintString.

	prettyNodeWithElements :=
		(XMLDOMParser on: prettyPrintString)
			preservesCommentNodes: true;
			preservesIgnorableWhitespace: true;
			parseDocument.
	nodeWithElements allElements
		with: prettyNodeWithElements allElements
		do: [:element :prettyElement |
			element hasChildren
				ifTrue: [
					element firstNode isStringNode
						ifFalse: [
							self
								assert: prettyElement firstNode isStringNode;
								assert: prettyElement firstNode isWhitespace].
					element hasStringNodes
						ifFalse: [
							self
								assert: prettyElement lastNode isStringNode;
								assert: prettyElement lastNode isWhitespace]]].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/14/2015 05:21'!
testPrintedWithoutSelfClosingTags
	| nodeWithElements printString printStringWithoutSelfClosingTags |

	nodeWithElements := self newNodeWithElements.
	printString := nodeWithElements printString.
	printStringWithoutSelfClosingTags := nodeWithElements printedWithoutSelfClosingTags.
	self deny: printString = printStringWithoutSelfClosingTags.

	nodeWithElements allElementsDo: [:each |
		each hasChildren
			ifFalse: [
				printString :=
					printString
						copyReplaceAll: '<', each name, ' />'
						with: '<', each name, '></', each name, '>']].
	self assert: printString = printStringWithoutSelfClosingTags.! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 2/2/2015 16:35'!
testRawContentString
	self assert: self newNodeWithElements rawContentString = 'testing onetesting twotesting three'! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 05:24'!
testStringNodes
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements.
	self
		assertList: nodeWithElements stringNodes
		selects: (nodeWithElements nodes select: [:each | each isStringNode]).
	nodeWithElements allElementsDo: [:element |
		self
			assertList: element stringNodes
			selects: (element nodes select: [:each | each isStringNode])].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 05:23'!
testStringNodesDo
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements.
	self
		should: [:block | nodeWithElements stringNodesDo: block]
		enumerate: nodeWithElements stringNodes.
	nodeWithElements allElementsDo: [:each |
		self
			should: [:block | each stringNodesDo: block]
			enumerate: each stringNodes].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 05:26'!
testStrings
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements.
	self
		assertList: nodeWithElements strings
		collects: (nodeWithElements stringNodes collect: [:each | each string]).
	nodeWithElements allElementsDo: [:element |
		self
			assertList: element strings
			collects: (element stringNodes collect: [:each | each string]).].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 05:27'!
testStringsDo
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements.
	self
		should: [:block | nodeWithElements stringsDo: block]
		enumerate: nodeWithElements strings.
	nodeWithElements allElementsDo: [:each |
		self
			should: [:block | each stringsDo: block]
			enumerate: each strings].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 11/24/2014 00:27'!
testUsesNamespaces
	| nodeWithElements |

	nodeWithElements := self newNodeWithElements.
	self assert: nodeWithElements usesNamespaces.
	nodeWithElements allElementsDo: [:each |
		self assert: each usesNamespaces].

	nodeWithElements usesNamespaces: false.
	self deny: nodeWithElements usesNamespaces.
	nodeWithElements allElementsDo: [:each |
		self deny: each usesNamespaces.
		self
			shouldnt: [each prefix: 'undeclared']
			raise: XMLNamespaceException].! !

!XMLNodeWithElementsTest methodsFor: 'tests' stamp: 'monty 4/6/2015 06:54'!
testValidate
	super testValidate.
	self
		shouldnt: [self newNodeWithNodes validate]
		raise: XMLValidationException.! !

XMLNodeTest subclass: #XMLPITest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLPITest methodsFor: 'accessing' stamp: 'monty 11/2/2014 11:18'!
nodeClass
	^ XMLPI! !

!XMLPITest methodsFor: 'tests' stamp: 'monty 12/6/2014 14:23'!
testCanonicallyEquals
	super testCanonicallyEquals.
	self assert:
		((self nodeClass target: 'one' data: 'two')
			canonicallyEquals: (self nodeClass target: 'one' data: 'two')).
	self deny: ((self nodeClass target: 'one' data: 'two') canonicallyEquals: self nodeClass new).! !

!XMLPITest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:24'!
testData
	| pi |

	pi := self nodeClass new.
	self assert: pi data isEmpty.
	pi data: 'test'.
	self assert: pi data = 'test'.! !

!XMLPITest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:46'!
testIsPI
	self assert: self nodeClass new isPI! !

!XMLPITest methodsFor: 'tests' stamp: 'monty 12/3/2014 01:09'!
testSortKey
	self assert: (self nodeClass new target: 'test') sortKey = 'test'! !

!XMLPITest methodsFor: 'tests' stamp: 'monty 11/22/2014 23:24'!
testTarget
	| pi |

	pi := self nodeClass new.
	self assert: pi target isEmpty.
	pi target: 'test'.
	self assert: pi target = 'test'.! !

!XMLPITest methodsFor: 'tests' stamp: 'monty 11/22/2014 19:31'!
testTargetData
	| pi |

	pi := self nodeClass target: 'target' data: 'data'.
	self
		assert: pi target = 'target';
		assert: pi data = 'data'.! !

!XMLPITest methodsFor: 'tests' stamp: 'monty 11/24/2014 07:35'!
testWriteXMLOn
	self
		assert: (self nodeClass target: 'target' data: 'data') printString = '<?target data?>';
		assert: (self nodeClass target: 'target ' data: ' data') printString = '<?target   data?>'.! !

XMLNodeTest subclass: #XMLStringTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

XMLStringTest subclass: #XMLCDataTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLCDataTest methodsFor: 'accessing' stamp: 'monty 11/2/2014 11:17'!
nodeClass
	^ XMLCData! !

!XMLCDataTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:44'!
testIsCData
	self assert: self nodeClass new isCData! !

!XMLCDataTest methodsFor: 'tests' stamp: 'monty 11/22/2014 19:37'!
testWriteXMLOn
	self
		assert: self nodeClass new printString = '<!![CDATA[]]>';
		assert: (self nodeClass string: self markupString) printString = ('<!![CDATA[', self markupString, ']]>').! !

!XMLStringTest methodsFor: 'accessing' stamp: 'monty 11/2/2014 11:18'!
nodeClass
	^ XMLString! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/4/2014 17:08'!
testAddContent
	| node |

	node := self nodeClass new.
	node addContent: 'one'.
	self assert: node string = 'one'.

	node addContent: (self nodeClass string: 'two').
	self assert: node string = 'onetwo'.

	node addContent: 'three'.
	self assert: node string = 'onetwothree'.! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/4/2014 12:45'!
testAppend
	| node |

	node := self nodeClass new.
	node append: 'one'.
	self assert: node string = 'one'.

	node append: (self nodeClass string: 'two').
	self assert: node string = 'onetwo'.

	node append: 'three'.
	self assert: node string = 'onetwothree'.! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/20/2014 00:46'!
testAsString
	self
		assert: self nodeClass new asString isEmpty;
		assert: (self nodeClass string: 'test') asString = 'test'! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/4/2014 14:56'!
testAsXMLEscapedString
	| node |

	node := self nodeClass string: self markupString.
	self	
		assert: node string = self markupString;
		deny: node string = self escapedString;
		assert: node asXMLEscapedString = self escapedString.! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/4/2014 17:05'!
testAsXMLString
	| node |

	node := self nodeClass new.
	self assert: node asXMLString == node.! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/24/2014 00:39'!
testBeginsWith
	| node |

	node := self nodeClass string: 'test'.
	0 to: node string size - 1 do: [:i | | prefix |
		prefix := node string allButLast: i.
		self
			assert: (node beginsWith: prefix);
			assert: (node beginsWith: (self nodeClass string: prefix));
			deny: (node beginsWith: (prefix copyWith: $.));
			deny: (node beginsWith: (self nodeClass string: (prefix copyWith: $.)))].! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 12/6/2014 14:23'!
testCanonicallyEquals
	super testCanonicallyEquals.
	self
		assert: ((self nodeClass string: 'test') canonicallyEquals: (self nodeClass string: 'test'));
		deny: ((self nodeClass string: 'test') canonicallyEquals: self nodeClass new).! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/20/2014 00:48'!
testContentString
	super testContentString.
	self
		assert: (self nodeClass string: 'test') contentString = 'test';
		assert: ((self nodeClass new) contentString: 'test') contentString = 'test'.! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/4/2014 12:12'!
testDeleteFromTo
	| node |

	node := self nodeClass new.
	node deleteFrom: 1 to: 0.
	self assert: node string isEmpty.

	node append: 'onetwothree'.
	node deleteFrom: 0 to: -1.
	node deleteFrom: 12 to: 13.
	self assert: node string = 'onetwothree'.

	node deleteFrom: 4 to: 6.
	self assert: node string = 'onethree'.

	node deleteFrom: 4 to: 8.
	self assert: node string = 'one'.

	node deleteFrom: 1 to: 3.
	self assert: node string isEmpty.! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/24/2014 00:41'!
testEndsWith
	| node |

	node := self nodeClass string: 'test'.
	0 to: node string size - 1 do: [:i | | suffix |
		suffix := node string allButFirst: i.
		self
			assert: (node endsWith: suffix);
			assert: (node endsWith: (self nodeClass string: suffix));
			deny: (node endsWith: (suffix copyWithFirst: $.));
			deny: (node endsWith: (self nodeClass string: (suffix copyWithFirst: $.)))].! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 12/9/2015 08:59'!
testIncludesSubstring
	| node |

	node := self nodeClass new.
	self
		assert: (node includesSubstring: '');
		assert: (node includesSubstring: (self nodeClass string: ''));
		deny: (node includesSubstring: 'one');
		deny: (node includesSubstring: (self nodeClass string: 'one')).
	node prepend: 'one'.
	self
		assert: (node includesSubstring: 'one');
		assert: (node includesSubstring: (self nodeClass string: 'one')).

	self
		deny: (node includesSubstring: 'three');
		deny: (node includesSubstring: (self nodeClass string: 'three')).
	node append: 'three'.
	self
		assert: (node includesSubstring: 'three');
		assert: (node includesSubstring: (self nodeClass string: 'three')).

	self
		deny: (node includesSubstring: 'two');
		deny: (node includesSubstring: (self nodeClass string: 'two')).
	node replaceFrom: 4 to: 3 with: 'two'.
	self
		assert: (node includesSubstring: 'two');
		assert: (node includesSubstring: (self nodeClass string: 'two')).

	self
		assert: (node includesSubstring: '');
		assert: (node includesSubstring: (self nodeClass string: '')).! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/6/2014 20:15'!
testIndexOfSubstring
	| node |

	node := self nodeClass new.
	self
		assert: (node indexOfSubstring: 'one') isZero;
		assert: (node indexOfSubstring: (self nodeClass string: 'one')) isZero.
	node prepend: 'one'.
	self
		assert: (node indexOfSubstring: 'one') = 1;
		assert: (node indexOfSubstring: (self nodeClass string: 'one')) = 1.

	self
		assert: (node indexOfSubstring: 'two') isZero;
		assert: (node indexOfSubstring: (self nodeClass string: 'two')) isZero.
	node append: 'two'.
	self
		assert: (node indexOfSubstring: 'two') = 4;
		assert: (node indexOfSubstring: (self nodeClass string: 'two')) = 4.! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/19/2014 21:46'!
testIsContentNode
	self assert: self nodeClass new isContentNode! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/4/2014 12:23'!
testIsEmpty
	self
		assert: self nodeClass new isEmpty;
		deny: (self nodeClass string: ' ') isEmpty! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/6/2014 02:20'!
testIsString
	| node |

	node := self nodeClass new.
	self
		assert: (node isString: '');
		assert: (node isString: self nodeClass new);
		deny: (node isString: 'test');
		deny: (node isString: (self nodeClass string: 'test')).
	node string: 'test'.
	self
		assert: (node isString: 'test');
		assert: (node isString: (self nodeClass string: 'test'));
		deny: (node isString: '');
		deny: (node isString: self nodeClass new).! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/20/2014 02:48'!
testIsStringNode
	| node |

	self assert: self nodeClass new isStringNode.

	node := self nodeClass new.
	self
		assert: (node isStringNode: '');
		assert: (node isStringNode: self nodeClass new);
		deny: (node isStringNode: 'test');
		deny: (node isStringNode: (self nodeClass string: 'test')).
	node string: 'test'.
	self
		assert: (node isStringNode: 'test');
		assert: (node isStringNode: (self nodeClass string: 'test'));
		deny: (node isStringNode: '');
		deny: (node isStringNode: self nodeClass new).! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 3/12/2016 19:10'!
testIsWhitespace
	| node |

	node := self nodeClass new.
	self deny: node isWhitespace.

	node string: Character xmlWhitespace.
	self assert: node isWhitespace.

	node append: 'non-space'.
	self deny: node isWhitespace.! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/4/2014 20:59'!
testMergeAdjacent
	| element nodes |

	(element := XMLElement new)
		addNode: (self nodeClass string: 'one');
		addNode: (self nodeClass string: 'two');
		addNode: (self nodeClass string: 'three');
		addNode: XMLComment new;
		addNode: (self nodeClass string: 'four');
		addNode: (self nodeClass string: 'five');
		addNode: XMLComment new;
		addNode: (self nodeClass string: 'six').

	element nodes second mergeAdjacent.
	nodes := element nodes.
	self
		assert: nodes size = 6;
		assert: (nodes first isStringNode and: [nodes first string = 'onetwothree']);
		assert: nodes second isComment;
		assert: (nodes third isStringNode and: [nodes third string = 'four']);
		assert: (nodes fourth isStringNode and: [nodes fourth string = 'five']);
		assert: nodes fifth isComment;
		assert: (nodes sixth isStringNode and: [nodes sixth string = 'six']).

	nodes fourth mergeAdjacent.
	nodes fifth mergeAdjacent.
	self
		assert: nodes size = 5;
		assert: (nodes first isStringNode and: [nodes first string = 'onetwothree']);
		assert: nodes second isComment;
		assert: (nodes third isStringNode and: [nodes third string = 'fourfive']);
		assert: nodes fourth isComment;
		assert: (nodes fifth isStringNode and: [nodes fifth string = 'six']).! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/14/2015 20:07'!
testNotEmpty
	self
		deny: self nodeClass new notEmpty;
		assert: (self nodeClass string: ' ') notEmpty! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/4/2014 12:46'!
testPrepend
	| node |

	node := self nodeClass new.
	node prepend: 'three'.
	self assert: node string = 'three'.

	node prepend: (self nodeClass string: 'two').
	self assert: node string = 'twothree'.

	node prepend: 'one'.
	self assert: node string = 'onetwothree'.! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/4/2014 12:46'!
testReplaceAllWith
	| node |

	(node := self nodeClass new) replaceAll: 'old' with: 'new'.
	self assert: node string isEmpty.

	node string: 'one old two old three old'.
	node replaceAll: 'old' with: (self nodeClass string: 'new').
	self assert: node string = 'one new two new three new'.

	node replaceAll: 'new' with: ''.
	self assert: node string = 'one  two  three '.! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 1/16/2015 08:46'!
testReplaceFromToWith
	| node |

	(node := self nodeClass new) replaceFrom: -1 to: -3 with: 'one'.
	self assert: node string = 'one'.

	node replaceFrom: 1 to: 0 with: 'zero'.
	self assert: node string = 'zeroone'.

	node replaceFrom: 9 to: 7 with: (self nodeClass string: 'fourthree').
	self assert: node string = 'zeroonefourthree'.

	node replaceFrom: 8 to: 11 with: ' two '.
	self assert: node string = 'zeroone two three'.

	node replaceFrom: 0 to: 4 with: ''.
	self assert: node string = 'one two three'.

	node replaceFrom: 14 to: 13 with: (self nodeClass string: ' four').
	self assert: node string = 'one two three four'.! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/4/2014 00:44'!
testSize
	self
		assert: self nodeClass new size = 0;
		assert: (self nodeClass string: 'test') size = 4! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 12/2/2014 22:59'!
testSortKey
	self assert: (self nodeClass string: 'test') sortKey = 'test'! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/4/2014 17:16'!
testSplitAt
	| node newNode element |

	node := self nodeClass string: 'onetwo'.
	newNode := node splitAt: 0.
	self
		deny: node == newNode;
		assert: node string isEmpty;
		assert: newNode string = 'onetwo'.

	node := self nodeClass string: 'onetwo'.
	newNode := node splitAt: 7.
	self
		deny: node == newNode;
		assert: node string = 'onetwo';
		assert: newNode string isEmpty.

	(element := XMLElement new) addNode: node.
	newNode := node splitAt: 4.
	self
		deny: node == newNode;
		assert: node string = 'one';
		assert: newNode string = 'two';
		assert: (element nodeAfter: node) == newNode.! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/4/2014 17:13'!
testString
	| node |

	node := self nodeClass new.
	self assert: node string isEmpty.

	node string: self markupString.
	self assert: node string = self markupString.

	node string: ''.
	self assert: node string isEmpty.! !

!XMLStringTest methodsFor: 'tests' stamp: 'monty 11/22/2014 19:29'!
testWriteXMLOn
	self
		assert: self nodeClass new printString isEmpty;
		assert: (self nodeClass string: self markupString) printString = self escapedString.! !

TestCase subclass: #XMLOrderedListTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

XMLOrderedListTest subclass: #XMLObservableListTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

XMLObservableListTest subclass: #XMLAttributeListTest
	instanceVariableNames: 'attributes newAttribute absentAttribute changedAttributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLAttributeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 11:09'!
absentElement
	^ absentAttribute
		ifNil: [absentAttribute := XMLAttribute name: 'absentName' value: 'absentValue']! !

!XMLAttributeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 11:06'!
afterAddingToNotifications
	^ #(addedAttribute:)! !

!XMLAttributeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 11:06'!
afterRemovingFromNotifications
	^ #(removedAttribute:)! !

!XMLAttributeListTest methodsFor: 'testing' stamp: 'monty 1/18/2015 11:26'!
allowsDuplicates
	^ false! !

!XMLAttributeListTest methodsFor: 'assertions' stamp: 'monty 1/17/2015 14:57'!
assertList: aList copiedFrom: aSecondList equals: aThirdList
	super
		assertList: aList
		copiedFrom: aSecondList
		equals: aThirdList.
	self
		assert: aList attributesByName size = aList size;
		assert: aList attributesByName = aThirdList attributesByName;
		deny: aList attributesByName == aSecondList attributesByName.! !

!XMLAttributeListTest methodsFor: 'assertions' stamp: 'monty 1/17/2015 14:57'!
assertList: aList isCopyOf: aSecondList
	super assertList: aList isCopyOf: aSecondList.
	self
		assert: aList attributesByName size = aList size;
		assert: aList attributesByName = aSecondList attributesByName;
		deny: aList attributesByName == aSecondList attributesByName.! !

!XMLAttributeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 11:06'!
beforeAddingToNotifications
	^ #(addingAttribute:)! !

!XMLAttributeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 11:24'!
changeElement: anAttribute
	^ (changedAttributes ifNil: [changedAttributes := IdentityDictionary new])
		at: anAttribute
		ifAbsentPut: [
			XMLAttribute
				name: 'new', anAttribute name
				value: 'new', anAttribute value]! !

!XMLAttributeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 11:07'!
listClass
	^ XMLAttributeList! !

!XMLAttributeListTest methodsFor: 'accessing' stamp: 'monty 1/18/2015 18:09'!
listElements
	attributes ifNil: [
		attributes :=
			super listElements collect: [:each |
				XMLAttribute name: each value: each]].
	^ attributes copy.! !

!XMLAttributeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 11:10'!
newElement
	^ newAttribute
		ifNil: [newAttribute := XMLAttribute name: 'newName' value: 'newValue']! !

!XMLAttributeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 11:12'!
observerClass
	^ XMLRecordingAttributeListObserver! !

!XMLAttributeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 11:25'!
reverseSortBlock
	^ [:a :b | a sortKey > b sortKey]! !

!XMLAttributeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 11:25'!
sortBlock
	^ [:a :b | a sortKey <= b sortKey]! !

XMLObservableListTest subclass: #XMLNodeListTest
	instanceVariableNames: 'nodes newNode absentNode changedNodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

XMLNodeListTest subclass: #XMLCachingNodeListTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLCachingNodeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 11:48'!
listClass
	^ XMLCachingNodeList! !

!XMLCachingNodeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 11:50'!
listCopyClass
	^ XMLNodeList! !

!XMLCachingNodeListTest methodsFor: 'tests' stamp: 'monty 2/8/2015 23:08'!
testSort
	| list cachedElements |

	super testSort.

	list := self listWithElements.
	cachedElements := list elements.
	list sort: self reverseSortBlock.
	self assert: list elements = (cachedElements sort: self reverseSortBlock).! !

!XMLCachingNodeListTest methodsFor: 'tests' stamp: 'monty 2/2/2015 00:01'!
testSwapWith
	| list cachedElements start end |

	super testSwapWith.

	list := self listWithElements.
	cachedElements := list elements.
	start := 1.
	end := list size.
	[start < end]
		 whileTrue: [
			list swap: start with: end.
			start := start + 1.
			end := end - 1].
	self assert: list elements = cachedElements reversed.! !

!XMLNodeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 10:26'!
absentElement
	^ absentNode ifNil: [absentNode := 'absentElement' asXMLString]! !

!XMLNodeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 09:52'!
afterAddingToNotifications
	^ #(addedNode: addedElement:)! !

!XMLNodeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 09:53'!
afterRemovingFromNotifications
	^ #(removedNode: removedElement:)! !

!XMLNodeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 09:54'!
beforeAddingToNotifications
	^ #(addingNode:)! !

!XMLNodeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 10:28'!
changeElement: anElement
	^ (changedNodes ifNil: [changedNodes := IdentityDictionary new])
		at: anElement
		ifAbsentPut: [anElement copy]! !

!XMLNodeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 09:54'!
listClass
	^ XMLNodeList! !

!XMLNodeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 10:03'!
listElements
	nodes ifNil: [
		nodes := OrderedCollection new.
		super listElements withIndexDo: [:each :i |
			i even
				ifTrue: [nodes addLast: (XMLElement named: each)]
				ifFalse: [nodes addLast: each asXMLString]].
		nodes := nodes asArray].
	^ nodes copy.! !

!XMLNodeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 10:08'!
newElement
	^ newNode ifNil: [newNode := 'newElement' asXMLString]! !

!XMLNodeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 09:53'!
observerClass
	^ XMLRecordingNodeListObserver! !

!XMLNodeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 10:35'!
reverseSortBlock
	^ [:a :b | a sortKey > b sortKey]! !

!XMLNodeListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 10:35'!
sortBlock
	^ [:a :b | a sortKey <= b sortKey]! !

!XMLObservableListTest class methodsFor: 'testing' stamp: 'JAA 3/26/2011 19:00'!
shouldInheritSelectors
	^ true! !

!XMLObservableListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 09:45'!
afterAddingToNotifications
	^ #(added:to:)! !

!XMLObservableListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 09:45'!
afterRemovingFromNotifications
	^ #(removed:from:)! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/17/2015 11:20'!
assertList: aList copiedFrom: aSecondList equals: aThirdList
	super
		assertList: aList
		copiedFrom: aSecondList
		equals: aThirdList.
	self denyList: aList sharesObserverWith: aSecondList.! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/17/2015 11:20'!
assertList: aList isCopyOf: aSecondList
	super assertList: aList isCopyOf: aSecondList.
	self denyList: aList sharesObserverWith: aSecondList.! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/21/2015 11:58'!
assertList: aList receivedNotificationOfAdding: anElement
	self
		assertList: aList
		receivedNotificationOfBeforeAdding: anElement;
		assertList: aList
		receivedNotificationOfAfterAdding: anElement! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/21/2015 11:57'!
assertList: aList receivedNotificationOfAfterAdding: anElement
	self
		assertNotification: aList observer nextNotification
		forElement: anElement
		andList: aList
		hasType: self afterAddingToNotifications! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/21/2015 11:57'!
assertList: aList receivedNotificationOfBeforeAdding: anElement
	self
		assertNotification: aList observer nextNotification
		forElement: anElement
		andList: aList
		hasType: self beforeAddingToNotifications! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/18/2015 08:19'!
assertList: aList receivedNotificationOfRemoving: anElement
	self
		assertNotification: aList observer nextNotification
		forElement: anElement
		andList: aList
		hasType: self afterRemovingFromNotifications! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/21/2015 12:02'!
assertList: aList receivedNotificationOfReplacing: anOldElement with: aNewElement
	self
		assertList: aList
		receivedNotificationOfBeforeAdding: aNewElement.
	self
		assertList: aList
		receivedNotificationOfRemoving: anOldElement.
	self
		assertList: aList
		receivedNotificationOfAfterAdding: aNewElement.! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/18/2015 18:40'!
assertNotification: aNotification forElement: anElement andList: aList hasType: aSelectorColleciton
	self assert: (aSelectorColleciton includes: aNotification selector).
	self assert:
		aNotification arguments =
			((Array with: anElement with: aList)
				copyFrom: 1
				to: (aNotification selector asString occurrencesOf: $:)).! !

!XMLObservableListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 09:42'!
beforeAddingToNotifications
	^ #(adding:to:)! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/17/2015 11:19'!
denyList: aList sharesObserverWith: aSecondList
	self deny:
		(aList observer notNil
			and: [aSecondList observer notNil
				and: [aList observer == aSecondList observer]])! !

!XMLObservableListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 10:41'!
emptyList
	| list |

	list := super emptyList.
	self observerOnList: list.
	^ list.! !

!XMLObservableListTest methodsFor: 'accessing' stamp: 'JAA 3/26/2011 19:01'!
listClass
	^ XMLObservableList! !

!XMLObservableListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 10:41'!
listWithElements
	| list |

	list := super listWithElements.
	self observerOnList: list.
	^ list.! !

!XMLObservableListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 10:42'!
listWithSingleElement
	| list |

	list := super listWithSingleElement.
	self observerOnList: list.
	^ list.! !

!XMLObservableListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 09:52'!
observerClass
	^ XMLRecordingListObserver! !

!XMLObservableListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 09:53'!
observerOnList: aList
	| observer |

	observer := self observerClass new.
	aList observer: observer.
	^ observer.! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/18/2015 07:23'!
should: aBlock add: anElement toAndNotify: aList
	self
		should: aBlock
		addAll: (Array with: anElement)
		toAndNotify: aList! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/18/2015 15:50'!
should: aBlock addAll: anElementCollection toAndNotify: aList
	aList observer clearNotifications.
	aBlock value.
	anElementCollection do: [:each |
		self assert: (aList includes: each).
		self 
			assertList: aList
			receivedNotificationOfAdding: each].! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/18/2015 07:23'!
should: aBlock remove: anElement fromAndNotify: aList
	self
		should: aBlock
		removeAll: (Array with: anElement)
		fromAndNotify: aList! !

!XMLObservableListTest methodsFor: 'assertions' stamp: 'monty 1/18/2015 15:50'!
should: aBlock removeAll: anElementCollection fromAndNotify: aList
	aList observer clearNotifications.
	aBlock value.
	anElementCollection do: [:each |
		self deny: (aList includes: each).
		self
			assertList: aList
			receivedNotificationOfRemoving: each].! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:38'!
testAdd
	| list |

	super testAdd.

	list := self emptyList.
	self listElements do: [:each |
		self
			should: [list add: each]
			add: each
			toAndNotify: list].! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:38'!
testAddAfter
	| list |

	super testAddAfter.

	list := self listWithSingleElement.
	self listElements allButFirst reverseDo: [:each |
		self
			should: [list add: each after: self singleElement]
			add: each
			toAndNotify: list].! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:38'!
testAddAfterIndex
	| list |

	super testAddAfterIndex.

	list := self listWithSingleElement.
	self listElements allButFirst reverseDo: [:each |
		self
			should: [list add: each afterIndex: 1]
			add: each
			toAndNotify: list].! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:38'!
testAddAll
	| list |

	super testAddAll.

	list := self emptyList.
	self
		should: [list addAll: self listElements]
		addAll: self listElements
		toAndNotify: list.
	self
		should: [list addAll: self changedListElements]
		addAll: self changedListElements
		toAndNotify: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:38'!
testAddAllFirst
	| list |

	super testAddAllFirst.

	list := self emptyList.
	self
		should: [list addAllFirst: self listElements]
		addAll: self listElements reversed
		toAndNotify: list.
	self
		should: [list addAllFirst: self changedListElements]
		addAll: self changedListElements reversed
		toAndNotify: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:38'!
testAddAllLast
	| list |

	super testAddAllLast.

	list := self emptyList.
	self
		should: [list addAllLast: self listElements]
		addAll: self listElements
		toAndNotify: list.
	self
		should: [list addAllLast: self changedListElements]
		addAll: self changedListElements
		toAndNotify: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:38'!
testAddBefore
	| list last |

	super testAddBefore.

	last := self listElements last.
	(list := self emptyList) add: last.
	"allButLastDo: is not portable"
	self listElements allButLast do: [:each |
		self
			should: [list add: each before: last]
			add: each
			toAndNotify: list].! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:42'!
testAddBeforeIndex
	| list |

	super testAddBeforeIndex.

	(list := self emptyList) add: self listElements last.
	self listElements allButLast withIndexDo: [:each :i |
		self
			should: [list add: each beforeIndex: i]
			add: each
			toAndNotify: list].! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:44'!
testAddFirst
	| list |

	super testAddFirst.

	list := self emptyList.
	self listElements reverseDo: [:each |
		self
			should: [list addFirst: each]
			add: each
			toAndNotify: list].! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:45'!
testAddLast
	| list |

	super testAddLast.

	list := self emptyList.
	self listElements do: [:each |
		self
			should: [list addLast: each]
			add: each
			toAndNotify: list].! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/21/2015 12:01'!
testAtPut
	| list |

	super testAtPut.

	list := self listWithElements.
	1 to: list size do: [:i | | old new |
		old := list at: i.
		new := self changedListElements at: i.
		list at: i put: new.
		self
			assertList: list
			receivedNotificationOfReplacing: old
			with: new].
	self
		should: [list at: list size + 2 put: self newElement]
		raise: Error.
	self
		assertList: list
		receivedNotificationOfBeforeAdding: self newElement.
	self deny: list observer hasNotifications.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/17/2015 11:52'!
testOfSize
	self
		should: [self listClass ofSize: 10]
		raise: Error! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:37'!
testRemoveAll
	| list |

	super testRemoveAll.

	list := self listWithElements.
	self
		should: [list removeAll]
		removeAll: self listElements reversed
		fromAndNotify: list.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 08:00'!
testRemoveAt
	super testRemoveAt.

	self listElements withIndexDo: [:each :i | | list |
		list := self listWithElements.
		self
			should: [list removeAt: i]
			remove: each
			fromAndNotify: list.
		self
			should: [list removeAt: 0] raise: Error;
			should: [list removeAt: list size + 1] raise: Error;
			deny: list observer hasNotifications].! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:52'!
testRemoveFirst
	| list |

	super testRemoveFirst.

	list := self listWithElements.
	self listElements do: [:each |
		self
			should: [list removeFirst]
			remove: each
			fromAndNotify: list].
	self
		should: [list removeFirst] raise: Error;
		deny: list observer hasNotifications.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 08:07'!
testRemoveFirstFew
	| list |

	super testRemoveFirstFew.

	1 to: self listElements size do: [:i |
		list := self listWithElements.
		self
			should: [list removeFirst: i]
			removeAll: (self listElementsFirst: i)
			fromAndNotify: list].
	list := self emptyList.
	self
		should: [list removeFirst: list size + 1] raise: Error;
		deny: list observer hasNotifications.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 08:09'!
testRemoveIfAbsent
	| list |

	super testRemoveIfAbsent.

	list := self listWithElements.
	self
		should: [list remove: self singleElement ifAbsent: [nil]]
		remove: self singleElement
		fromAndNotify: list.
	self
		assert: (list remove: self singleElement ifAbsent: [nil]) isNil;
		deny: list observer hasNotifications.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 08:08'!
testRemoveLast
	| list |

	super testRemoveLast.

	list := self listWithElements.
	self listElements reverseDo: [:each |
		self
			should: [list removeLast]
			remove: each
			fromAndNotify: list].
	self
		should: [list removeFirst] raise: Error;
		deny: list observer hasNotifications.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 08:08'!
testRemoveLastFew
	| list |
	
	super testRemoveLastFew.

	1 to: self listElements size do: [:i |
		list := self listWithElements.
		self
			should: [list removeLast: i]
			removeAll: (self listElementsLast: i) reversed
			fromAndNotify: list].
	list := self emptyList.
	self
		should: [list removeFirst: list size + 1] raise: Error;
		deny: list observer hasNotifications.! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/21/2015 12:00'!
testReplaceFromToWith
	super testReplaceFromToWith.

	1 to: self listElements size do: [:start |
		start to: self listElements size do: [:stop | | list old new |
			list := self listWithElements.
			old := self listElements copyFrom: start to: stop.
			new := self changedListElements copyFrom: 1 to: stop - start + 1.
			list
				replaceFrom: start
				to: stop
				with: self changedListElements.
			old with: new do: [:oldElement :newElement |
				self
					assertList: list
					receivedNotificationOfReplacing: oldElement
					with: newElement].
			self deny: list observer hasNotifications]].! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/21/2015 12:03'!
testReplaceFromToWithStartingAt
	super testReplaceFromToWithStartingAt.

	1 to: self listElements size do: [:start |
		start to: self listElements size do: [:stop | | list old new |
			list := self listWithElements.
			old := self listElements copyFrom: start to: stop.
			new := self changedListElements copyFrom: start to: stop.
			list
				replaceFrom: start
				to: stop
				with: self changedListElements
				startingAt: start.
			old with: new do: [:oldElement :newElement |
				self
					assertList: list
					receivedNotificationOfReplacing: oldElement
					with: newElement].
			self deny: list observer hasNotifications]].! !

!XMLObservableListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:23'!
testReset
	| list |

	super testReset.
	list := self listWithElements.
	self
		should: [list reset]
		removeAll: self listElements
		fromAndNotify: list.! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:15'!
absentElement
	^ 'absent'! !

!XMLOrderedListTest methodsFor: 'testing' stamp: 'monty 1/18/2015 11:26'!
allowsDuplicates
	^ true! !

!XMLOrderedListTest methodsFor: 'assertions' stamp: 'monty 10/28/2014 19:09'!
assertList: aList collectedFrom: aSecondList equals: aThirdList
	self
		assert: aList = aThirdList;
		assert: aList class = aSecondList species;
		deny: aList == aSecondList;
		deny: aList collection == aSecondList collection! !

!XMLOrderedListTest methodsFor: 'assertions' stamp: 'monty 4/12/2015 11:29'!
assertList: aList copiedFrom: aSecondList equals: aThirdList
	self
		assert: aList = aThirdList;
		assert: aList class == self listCopyClass;
		assert: aList collection class == aSecondList collection class;
		deny: aList == aSecondList;
		deny: aList collection == aSecondList collection! !

!XMLOrderedListTest methodsFor: 'assertions' stamp: 'monty 4/11/2015 07:39'!
assertList: aFirstList equals: aSecondList
	self
		assert: aFirstList = aFirstList;
		assert: aSecondList = aSecondList;
		assert: aFirstList = aSecondList;
		assert: aSecondList = aFirstList! !

!XMLOrderedListTest methodsFor: 'assertions' stamp: 'monty 4/12/2015 11:29'!
assertList: aList isCopyOf: aSecondList
	self
		assert: aList = aSecondList;
		assert: aList class == self listCopyClass;
		assert: aList collection class == aSecondList collection class;
		assert: aList collection = aSecondList collection;
		deny: aList == aSecondList;
		deny: aList collection == aSecondList collection! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 09:25'!
changeElement: anObject
	^ anObject asUppercase! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 09:26'!
changedListElements
	^ self listElements collect: [:each | self changeElement: each]! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 09:31'!
changedSingleElement
	^ self changeElement: self singleElement! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:46'!
defaultValue
	^ nil! !

!XMLOrderedListTest methodsFor: 'assertions' stamp: 'monty 4/11/2015 07:39'!
denyList: aFirstList equals: aSecondList
	self
		deny: aFirstList = aSecondList;
		deny: aSecondList = aFirstList! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:19'!
emptyList
	^ self listClass new! !

!XMLOrderedListTest methodsFor: 'testing' stamp: 'monty 12/14/2014 13:59'!
hasCapacity
	"for portability"
	^ self listClass collectionClass includesSelector: #capacity! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:15'!
listClass
	^ XMLOrderedList! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 11:47'!
listCopyClass
	^ self listClass! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'monty 10/23/2014 08:14'!
listElements
	^ #('one' 'two' 'three' 'four' 'five' 'six' 'seven' 'eight' 'nine') copy! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'monty 1/16/2015 08:49'!
listElementsFirst: anInteger
	"first: is not portable"
	^ self listElements
		copyFrom: 1
		to: ((anInteger max: 0) min: self listElements size)! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 08:54'!
listElementsLast: anInteger
	"last: is not portable"
	anInteger > 0
		ifFalse: [^ #()].
	^ self listElements
		copyFrom: (((self listElements size - anInteger) + 1) max: 1)
		to: self listElements size.! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/26/2011 18:15'!
listSpecies
	^ XMLOrderedList! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:24'!
listWithElements
	^ self listClass new
		addAll: self listElements;
		yourself! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:26'!
listWithSingleElement
	^ self listClass new
		add: self singleElement;
		yourself! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'monty 10/22/2014 17:01'!
newElement
	^ 'newElement'! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 10:34'!
reverseSortBlock
	^ [:a :b | a > b]! !

!XMLOrderedListTest methodsFor: 'assertions' stamp: 'JAA 3/26/2011 17:37'!
should: aBlock enumerate: aCollection
	| enumerated |

	enumerated := OrderedCollection new.
	aBlock value: [:each | enumerated add: each].
	self assert: enumerated = aCollection asOrderedCollection.! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'JAA 3/24/2011 20:15'!
singleElement
	^ self listElements first! !

!XMLOrderedListTest methodsFor: 'accessing' stamp: 'monty 1/17/2015 10:34'!
sortBlock
	^ [:a :b | a <= b]! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 19:19'!
testAdd
	| list |

	list := self emptyList.
	self
		deny: list = self listWithElements;
		assert: (self listElements allSatisfy: [:each | (list add: each) == each]);
		assert: (list = self listWithElements).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 19:26'!
testAddAfter
	| list |

	list := self listWithSingleElement.
	self
		assert:
			(self listElements allButFirst reversed
				allSatisfy: [:each | (list add: each after: self singleElement) = each]);
		assert: list = self listWithElements;
		should: [list add: self newElement after: self absentElement] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 19:27'!
testAddAfterIndex
	| list |

	list := self listWithSingleElement.
	self
		assert:
			(self listElements allButFirst reversed
				allSatisfy: [:each | (list add: each afterIndex: 1) = each]);
		assert: list = self listWithElements;
		should: [list add: self newElement afterIndex: self listElements size + 1] raise: Error;
		should: [list add: self newElement afterIndex: -1] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/17/2015 09:35'!
testAddAll
	| list |

	list := self emptyList.
	self
		deny: list = self listWithElements;
		assert: (list addAll: self listElements) = self listElements;
		assert: list = self listWithElements;
		assert: (list addAll: self changedListElements) = self changedListElements;
		assert: list = (self listClass newFrom: (self listElements, self changedListElements)).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/17/2015 09:35'!
testAddAllFirst
	| list |

	list := self emptyList.
	self
		assert: (list addAllFirst: self listElements) = self listElements;
		assert: list = self listWithElements;
		assert: (list addAllFirst: self changedListElements) = self changedListElements;
		assert: list = (self listClass newFrom: self changedListElements, self listElements).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/17/2015 09:35'!
testAddAllLast
	| list |

	list := self emptyList.
	self
		assert: (list addAllLast: self listElements) = self listElements;
		assert: list = self listWithElements;
		assert: (list addAllLast: self changedListElements) = self changedListElements;
		assert: list = (self listClass newFrom: self listElements, self changedListElements).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:36'!
testAddBefore
	| list last |

	last := self listElements last.
	(list := self emptyList) add: last.
	self
		assert:
			(self listElements allButLast
				allSatisfy: [:each | (list add: each before: last) = each]);
		assert: list = self listWithElements;
		should: [list add: self newElement before: self absentElement] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:42'!
testAddBeforeIndex
	| list |

	(list := self emptyList) add: self listElements last.
	self listElements allButLast withIndexDo: [:each :i |
		self assert: (list add: each beforeIndex: i) = each].
	self
		assert: list = self listWithElements;
		should: [list add: self newElement beforeIndex: self listElements size + 2] raise: Error;
		should: [list add: self newElement beforeIndex: 0] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 17:17'!
testAddFirst
	| list |

	list := self emptyList.
	self
		assert: (self listElements reversed allSatisfy: [:each | (list addFirst: each) = each]);
		assert: list = self listWithElements.

	self assert: (self listElements allSatisfy: [:each | (list addFirst: each) = each]).
	self assert:
		list =
			(self listClass newFrom: 
				(self allowsDuplicates
					ifTrue: [self listElements reversed, self listElements]
					ifFalse: [self listElements reversed])).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 17:17'!
testAddLast
	| list |

	list := self emptyList.
	self
		assert: (self listElements allSatisfy: [:each | (list addLast: each) = each]);
		assert: list = self listWithElements.

	self assert: (self listElements allSatisfy: [:each | (list addLast: each) = each]).
	self assert:
		list =
			(self listClass newFrom: 
				(self allowsDuplicates
					ifTrue: [self listElements, self listElements]
					ifFalse: [self listElements])).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 07:01'!
testAfter
	| list |

	list := self listWithElements.
	self assert:
		(self listElements allSatisfy: [:each |
			(list after: each) = (self listElements after: each ifAbsent: [self defaultValue])]).
	self assert: (list after: self absentElement) = self defaultValue.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 07:02'!
testAfterIfAbsent
	| list |

	list := self listWithElements.
	self assert:
		(self listElements allSatisfy: [:each |
			(list after: each ifAbsent: [self newElement]) =
				(self listElements after: each ifAbsent: [self newElement])]).
	self assert: (list after: self absentElement ifAbsent: [self newElement]) = self newElement.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 18:07'!
testAllButFirst
	| list |

	self assert: self emptyList allButFirst = self emptyList.

	list := self listWithElements.
	self
		assertList: list allButFirst
		copiedFrom: list
		equals: (self listClass newFrom: self listElements allButFirst).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 05:19'!
testAllButFirstDo
	self
		should: [:block | self emptyList allButFirstDo: block]
		enumerate: #().
	self
		should: [:block | self listWithElements allButFirstDo: block]
		enumerate: self listElements allButFirst.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 18:06'!
testAllButFirstFew
	| list |

	list := self listWithElements.
	1 to: list size do: [:i |
		self
			assertList: (list allButFirst: i)
			copiedFrom: list
			equals: (self listClass newFrom: (self listElements allButFirst: i)).
		self
			assertList: (list allButFirst: i + list size)
			copiedFrom: list
			equals: self emptyList.
		self
			assertList: (list allButFirst: i * -1)
			copiedFrom: list
			equals: list.
		self assert: (self emptyList allButFirst: i) = self emptyList].
	self
		assertList: (list allButFirst: 0)
		copiedFrom: list
		equals: list.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 18:08'!
testAllButLast
	| list |

	self assert: self emptyList allButLast = self emptyList.

	list := self listWithElements.
	self
		assertList: list allButLast
		copiedFrom: list
		equals: (self listClass newFrom: self listElements allButLast).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 05:19'!
testAllButLastDo
	self
		should: [:block | self emptyList allButLastDo: block]
		enumerate: #().
	self
		should: [:block | self listWithElements allButLastDo: block]
		enumerate: self listElements allButLast.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 18:08'!
testAllButLastFew
	| list |

	list := self listWithElements.
	1 to: list size do: [:i |
		self
			assertList: (list allButLast: i)
			copiedFrom: list
			equals: (self listClass newFrom: (self listElements allButLast: i)).
		self
			assertList: (list allButLast: i + list size)
			copiedFrom: list
			equals: self emptyList.
		self
			assertList: (list allButLast: i * -1)
			copiedFrom: list
			equals: list.
		self assert: (self emptyList allButLast: i) = self emptyList].
	self
		assertList: (list allButLast: 0)
		copiedFrom: list
		equals: list.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 04:52'!
testAnyOne
	self
		assert: self emptyList anyOne = self defaultValue;
		assert: self listWithElements anyOne = self listElements first.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/24/2011 20:48'!
testAsArray
	self
		assert: self emptyList asArray = #();
		assert: self listWithElements asArray = self listElements asArray! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 16:38'!
testAsOrderedCollection
	| list |

	list := self listWithElements.
	self
		assert: self emptyList asOrderedCollection = OrderedCollection new;
		assert: list asOrderedCollection = self listElements asOrderedCollection;
		deny: list asOrderedCollection == list collection.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/25/2011 00:47'!
testAt
	| list |

	list := self listWithElements.
	self
		assert: ((1 to: list size) allSatisfy: [:i | (list at: i) = (self listElements at: i)]);
		assert: (list at: 0) = self defaultValue;
		assert: (list at: list size + 1) = self defaultValue.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 19:32'!
testAtIfAbsent
	| list |

	list := self listWithElements.
	self
		assert: ((1 to: list size) allSatisfy: [:i | (list at: i ifAbsent: [self newElement]) = (self listElements at: i)]);
		assert: (list at: 0 ifAbsent: [self newElement]) = self newElement;
		assert: (list at: list size + 1 ifAbsent: [self newElement]) = self newElement.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 06:56'!
testAtLast
	| list |

	list := self listWithElements.
	1 to: list size do: [:i | self assert: (list atLast: i) = (self listElements atLast: i)].
	self
		assert: (list atLast: 0) = self defaultValue;
		assert: (list atLast: list size + 1) = self defaultValue.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 06:57'!
testAtLastIfAbsent
	| list |

	list := self listWithElements.
	1 to: list size do: [:i |
		self assert: (list atLast: i ifAbsent: [self newElement]) = (self listElements atLast: i)].
	self
		assert: (list atLast: 0 ifAbsent: [self newElement]) = self newElement;
		assert: (list atLast: list size + 1 ifAbsent: [self newElement]) = self newElement.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/21/2015 14:30'!
testAtPut
	| list |

	list := self listWithElements.
	1 to: list size do: [:i | | replacement |
		replacement := list at: i.
		self
			assert: (list at: i put: replacement) == replacement;
			assert: (list at: i) == replacement].
	self assert: list = self listWithElements.

	self assert: (list at: list size + 1 put: self newElement) = self newElement.
	self assert:
		list =
			(self listClass newFrom: (self listElements copyWith: self newElement)).

	list := self listWithElements.
	1 to: list size do: [:i | | replacement |
		replacement := self changedListElements at: i.
		self
			assert: (list at: i put: replacement) == replacement;
			assert: (list at: i) == replacement].
	self assert: list = (self listClass newFrom: self changedListElements).

	list := self listWithElements.
	self listElements do: [:each |
		self assert: (list at: list size + 1 put: each) == each].
	self assert:
		list =
			(self allowsDuplicates
				ifTrue: [self listClass newFrom: self listElements, self listElements]
				ifFalse: [self listWithElements]).
	self
		should: [list at: 0 put: self newElement]
		raise: Error.
	self
		should: [list at: list size + 2 put: self newElement]
		raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 07:03'!
testBefore
	| list |

	list := self listWithElements.
	self assert:
		(self listElements allSatisfy: [:each |
			(list before: each) = (self listElements before: each ifAbsent: [self defaultValue])]).
	self assert: (list before: self absentElement) = self defaultValue.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 07:03'!
testBeforeIfAbsent
	| list |

	list := self listWithElements.
	self assert:
		(self listElements allSatisfy: [:each |
			(list before: each ifAbsent: [self newElement]) =
				(self listElements before: each ifAbsent: [self newElement])]).
	self assert: (list before: self absentElement ifAbsent: [self newElement]) = self newElement.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 16:41'!
testBeginsWith
	| list |

	self deny: (self emptyList beginsWith: self listElements).
	list := self listWithElements.
	1 to: list size do: [:i |
		self
			assert: (list beginsWith: (self listElements copyFrom: 1 to: i));
			deny: (list beginsWith: ((self listElements copyFrom: 1 to: i) copyWith: self absentElement));
			deny: (list beginsWith: ((self listElements copyFrom: 1 to: i) copyWithFirst: self absentElement))].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 13:56'!
testCapacity
	self hasCapacity
		ifTrue: [
			self
				assert: self emptyList capacity >= 0;
				assert: self listWithSingleElement capacity >= 1;
				assert: self listWithElements capacity >= self listElements size]! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/17/2015 09:26'!
testCollect
	| list |

	list := self listWithElements.
	self
		assertList: (list collect: [:each | self changeElement: each])
		collectedFrom: list
		equals: (self listSpecies newFrom: self changedListElements).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 18:48'!
testCollectThenSelect
	| list |

	list := self listWithElements.
	self
		assertList:
			(list
				collect: [:each | self changeElement: each]
				thenSelect: [:each | each ~= self changedSingleElement])
		collectedFrom: list
		equals:
			(self listSpecies newFrom:
				(self changedListElements copyWithout: self changedSingleElement)).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/24/2014 04:38'!
testConcatenation
	| list |

	list := self emptyList.
	self
		assertList: (list, self emptyList) copiedFrom: list equals: self emptyList;
		assertList: (list, self listWithElements) copiedFrom: list equals: self listWithElements.

	list := self listWithElements.
	self assertList: (list, self emptyList) copiedFrom: list equals: self listWithElements.
	self
		assertList: (list, self listWithElements)
		copiedFrom: list
		equals: (self listClass newFrom: (self listElements, self listElements)).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/25/2011 15:38'!
testCopy
	| list |

	list := self emptyList.
	self assertList: list copy isCopyOf: list.

	list := self listWithSingleElement.
	self assertList: list copy isCopyOf: list.

	list := self listWithElements.
	self assertList: list copy isCopyOf: list.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/24/2014 04:33'!
testCopyAfter
	| list |

	list := self listWithElements.
	list do: [:each |
		self
			assertList: (list copyAfter: each)
			copiedFrom: list
			equals: (self listClass newFrom: (self listElements copyAfter: each)).
		self assert: (self emptyList copyAfter: each) = self emptyList].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/24/2014 04:33'!
testCopyAfterLast
	| list |

	list := self listWithElements.
	list do: [:each |
		self
			assertList: (list copyAfterLast: each)
			copiedFrom: list
			equals: (self listClass newFrom: (self listElements copyAfterLast: each)).
		self assert: (self emptyList copyAfterLast: each) = self emptyList].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 9/28/2014 23:33'!
testCopyEmpty
	| list |

	list := self emptyList.
	self assertList: list copyEmpty copiedFrom: list equals: self emptyList.

	list := self listWithSingleElement.
	self assertList: list copyEmpty copiedFrom: list equals: self emptyList.

	list := self listWithElements.
	self assertList: list copyEmpty copiedFrom: list equals: self emptyList.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/26/2014 12:06'!
testCopyFromTo
	| list emptyList |

	list := self listWithElements.
	emptyList  := self emptyList.
	-1 to: list size + 1 do: [:start |
		start to: list size + 1 do: [:stop |
			self
				assertList: (list copyFrom: start to: stop)
				copiedFrom: list
				equals: (self listWithElements copyFrom: (start max: 1) to: (stop min: list size)).
			self
				assertList: (list copyFrom: start to: start - 1)
				copiedFrom: list
				equals: self emptyList.
			self
				assertList: (list copyFrom: start to: start - 2)
				copiedFrom: list
				equals: self emptyList.
			self
				assertList: (emptyList copyFrom: start to: stop)
				copiedFrom: list
				equals: self emptyList]].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/17/2015 09:29'!
testCopyReplaceAllWith
	| list |

	list := self listClass newFrom: self listElements, self listElements.
	1 to: self listElements size do: [:i | | replacedElements |
		replacedElements := self listElementsLast: i.
		self
			assertList:
				(list
					copyReplaceAll: replacedElements
					with: (replacedElements collect: [:each | self changeElement: each]))
			copiedFrom: list
			equals: 
				(self listClass newFrom:
					((self listElements, self listElements) 
						copyReplaceAll: replacedElements
						with: (replacedElements collect: [:each | self changeElement: each]))).
		self
			assertList:
				(list
					copyReplaceAll: (Array with: self absentElement)
					with: (Array with: self newElement))
			copiedFrom: list
			equals: list.	
		self
			assertList: (list copyReplaceAll: #() with: (Array with: self newElement))
			copiedFrom: list
			equals: list.
		self
			assertList: (list copyReplaceAll: (self listElements copyWith: self absentElement) with: #())
			copiedFrom: list
			equals: list].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/17/2015 09:29'!
testCopyReplaceFromToWith
	| list emptyList |

	list := self listWithElements.
	emptyList := self emptyList.
	-1 to: list size + 1 do: [:start |
		start to: list size + 1 do: [:stop |
			self
				assertList: (list copyReplaceFrom: start to: stop with: self changedListElements)
				copiedFrom: list
				equals:
					(self listClass newFrom:
						(self listElementsFirst: start - 1),
						(self changedListElements),
						(self listElementsLast: list size - stop)).
			self
				assertList: (emptyList copyReplaceFrom: start to: stop with: self listElements)
				copiedFrom: emptyList
				equals: self listWithElements]].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/25/2014 10:54'!
testCopyUpTo
	| list |

	list := self listWithElements.
	list do: [:each |
		self
			assertList: (list copyUpTo: each)
			copiedFrom: list
			equals: (self listClass newFrom: (self listElements copyUpTo: each))].
	self
		assertList: (list copyUpTo: self absentElement)
		copiedFrom: list
		equals: self listWithElements.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 11:29'!
testCopyUpToLast
	| list |

	list := self listClass newFrom: self listElements, self listElements.
	list do: [:each |
		self
			assertList: (list copyUpToLast: each)
			copiedFrom: list
			equals:
				(self listClass newFrom:
					(self allowsDuplicates
						ifTrue: [self listElements, self listElements copyUpToLast: each]
						ifFalse: [self listElements copyUpToLast: each]))].
	self
		assertList: (list copyUpToLast: self absentElement)
		copiedFrom: list
		equals:
			(self listClass newFrom:
				(self allowsDuplicates
					ifTrue: [self listElements, self listElements]
					ifFalse: [self listElements])).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 07:14'!
testCopyWith
	| list |

	list := self emptyList.
	self
		assertList: (list copyWith: self singleElement)
		copiedFrom: list
		equals: self listWithSingleElement.

	list := self listWithElements.
	self
		assertList: (list copyWith: self newElement)
		copiedFrom: list
		equals: (self listClass newFrom: (self listElements copyWith: self newElement)).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/24/2014 14:47'!
testCopyWithFirst
	| list |

	list := self emptyList.
	self
		assertList: (list copyWithFirst: self singleElement)
		copiedFrom: list
		equals: self listWithSingleElement.

	list := self listWithElements.
	self
		assertList: (list copyWithFirst: self newElement)
		copiedFrom: list
		equals:
			(self listClass newFrom:
				(self listElements copyWithFirst: self newElement)).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/16/2015 14:11'!
testCopyWithout
	| list |

	list := self emptyList.
	self
		assertList: (list copyWithout: self singleElement)
		copiedFrom: list
		equals: self emptyList.

	list := self listWithElements.
	self
		assertList: (list copyWithout: self singleElement)
		copiedFrom: list
		equals: (self listClass newFrom: (self listElements copyWithout: self singleElement)).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/16/2015 14:14'!
testCopyWithoutAll
	| list |

	list := self emptyList.
	self
		assertList: (list copyWithoutAll: self listElements)
		copiedFrom: list
		equals: self emptyList.

	list := self listWithElements.
	self
		assertList: (list copyWithoutAll: (Array with: self absentElement))
		copiedFrom: list
		equals: self listWithElements.
	self
		assertList: (list copyWithoutAll: self listElements)
		copiedFrom: list
		equals: self emptyList.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/25/2011 16:02'!
testDefaultValue
	self assert: self emptyList defaultValue = self defaultValue! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 11/7/2014 00:25'!
testDetect
	| list |

	list := self listWithElements.
	self listElements do: [:element |
		self assert: (list detect: [:each | each = element]) = element].
	self assert: (list detect: [:each | each = self absentElement]) = self defaultValue.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 11/7/2014 00:25'!
testDetectIfNone
	| list |

	list := self listWithElements.
	self listElements do: [:element |
		self assert:
			(list
				detect: [:each | each = element]
				ifNone: [self newElement]) = element].
	self assert:
		(list
			detect: [:each | each = self absentElement]
			ifNone: [self newElement]) = self newElement.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 17:32'!
testDo
	self
		should: [:block | self emptyList do: block] enumerate: #();
		should: [:block | self listWithElements do: block] enumerate: self listElements! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 07:09'!
testDoWithIndex
	| list enumerated |

	list := self listWithElements.
	enumerated := OrderedCollection new.
	list withIndexDo: [:each :i | enumerated addLast: each -> i].
	self assert:
		enumerated =
			(self listElements withIndexCollect: [:each : i | each -> i ])
				asOrderedCollection.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 06:52'!
testEighth
	self
		assert: self emptyList eighth = self defaultValue;
		assert: self listWithElements eighth = (self listElements at: 8).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 16:43'!
testEndsWith
	| list |

	self deny: (self emptyList endsWith: self listElements).
	list := self listWithElements.
	1 to: list size do: [:i |
		self
			assert: (list endsWith: (self listElements copyFrom: i to: list size));
			deny: (list endsWith: ((self listElements copyFrom: i to: list size) copyWith: self absentElement));
			deny: (list endsWith: ((self listElements copyFrom: i to: list size) copyWithFirst: self absentElement))].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 4/11/2015 07:40'!
testEquals
	self
		assertList: self emptyList equals: self emptyList;
		assertList: self listWithSingleElement equals: self listWithSingleElement;
		assertList: self listWithElements equals: self listWithElements;
		denyList: self emptyList equals: self listWithSingleElement;
		denyList: self emptyList equals: self listWithElements;
		denyList: self listWithSingleElement equals: self listWithElements! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 4/19/2015 15:21'!
testEqualsInternalCollections
	| emptyArrayList emptyOCList arrayListWithSingleElement ocListWithSingleElement arrayListWithElements ocListWithElements |

	emptyArrayList := self listClass collection: #().
	emptyOCList := self listClass collection: OrderedCollection new.
	arrayListWithSingleElement := self listClass collection: (Array with: self singleElement).
	ocListWithSingleElement := self listClass collection: (OrderedCollection with: self singleElement).
	"use withAll: for portability to GS"
	arrayListWithElements := self listClass collection: (Array withAll: self listElements).
	ocListWithElements := self listClass collection: (OrderedCollection withAll: self listElements).

	self
		assertList: emptyArrayList equals: emptyOCList;
		assertList: arrayListWithSingleElement equals: ocListWithSingleElement;
		assertList: arrayListWithElements equals: ocListWithElements;
		denyList: emptyArrayList equals: ocListWithSingleElement;
		denyList: emptyArrayList equals: ocListWithElements;
		denyList: arrayListWithSingleElement equals: emptyOCList;
		denyList: arrayListWithSingleElement equals: ocListWithElements;
		denyList: arrayListWithElements equals: emptyOCList;
		denyList: arrayListWithElements equals: ocListWithSingleElement.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 4/11/2015 07:44'!
testEqualsSpecies
	| emptySpeciesList speciesListWithSingleElement speciesListWithElements |

	emptySpeciesList := self listSpecies new.
	speciesListWithSingleElement := self listSpecies with: self singleElement.
	speciesListWithElements := self listSpecies newFrom: self listElements.
	self
		assertList: self emptyList equals: emptySpeciesList;
		assertList: self listWithSingleElement equals: speciesListWithSingleElement;
		assertList: self listWithElements equals: speciesListWithElements;
		denyList: self emptyList equals: speciesListWithSingleElement;
		denyList: self emptyList equals: speciesListWithElements;
		denyList: self listWithSingleElement equals: emptySpeciesList;
		denyList: self listWithSingleElement equals: speciesListWithElements;
		denyList: self listWithElements equals: emptySpeciesList;
		denyList: self listWithElements equals: speciesListWithSingleElement.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 06:52'!
testFifth
	self
		assert: self emptyList fifth = self defaultValue;
		assert: self listWithElements fifth = (self listElements at: 5).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 06:51'!
testFirst
	self
		assert: self emptyList first = self defaultValue;
		assert: self listWithElements first = (self listElements at: 1).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 11/24/2014 08:34'!
testFirstFew
	| list |

	list := self listWithElements.
	1 to: list size do: [:i |
		self
			assertList: (list first: i)
			copiedFrom: list
			equals: (self listClass newFrom: (self listElementsFirst: i)).
		self
			assertList: (list first: list size + i)
			copiedFrom: list
			equals: self listWithElements.
		self
			assertList: (list first: (i * -1))
			copiedFrom: list
			equals: self emptyList].
	self
		assertList: (list first: 0)
		copiedFrom: list
		equals: self emptyList.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 06:51'!
testFourth
	self
		assert: self emptyList fourth = self defaultValue;
		assert: self listWithElements fourth = (self listElements at: 4).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 05:27'!
testIdentityIndexOf
	| list |

	list := self listWithElements.
	1 to: list size do: [:i |
		self
			assert: (list identityIndexOf: (list at: i)) = i;
			assert: (list identityIndexOf: (list at: i) copy) isZero].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 05:33'!
testIdentityIndexOfIfAbsent
	| list |

	list := self listWithElements.
	1 to: list size do: [:i |
		self
			assert: (list identityIndexOf: (list at: i) ifAbsent: [-1]) = i;
			assert: (list identityIndexOf: (list at: i) copy ifAbsent: [-1]) = -1].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/17/2015 10:22'!
testIndexOf
	| list |

	list := self listWithElements.
	1 to: list size do: [:i |
		self
			assert: (list indexOf: (list at: i)) = i;
			assert: (list indexOf: (self listElements at: i)) = i].
	self assert: (list indexOf: self absentElement) isZero.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/17/2015 10:23'!
testIndexOfIfAbsent
	| list |

	list := self listWithElements.
	1 to: list size do: [:i |
		self
			assert: (list indexOf: (list at: i) ifAbsent: [-1]) = i;
			assert: (list indexOf: (self listElements at: i) ifAbsent: [-1]) = i].
	self assert: (list indexOf: self absentElement ifAbsent: [-1]) = -1.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 05:44'!
testIndexOfStartingAt
	| list |

	list := self listWithElements.
	1 to: list size do: [:i |
		self
			assert: (list indexOf: (list at: i) startingAt: i) = i;
			assert: (list indexOf: (list at: i) startingAt: i + 1) isZero].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 05:46'!
testIndexOfStartingAtIfAbsent
	| list |

	list := self listWithElements.
	1 to: list size do: [:i |
		self
			assert: (list indexOf: (list at: i) startingAt: i ifAbsent: [-1]) = i;
			assert: (list indexOf: (list at: i) startingAt: i + 1 ifAbsent: [-1]) = -1].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 08:00'!
testLast
	self
		assert: self emptyList last = self defaultValue;
		assert: self listWithElements last = self listElements last.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 11/24/2014 08:35'!
testLastFew
	| list |

	list := self listWithElements.
	1 to: list size do: [:i |
		self
			assertList: (list last: i)
			copiedFrom: list
			equals: (self listClass newFrom: (self listElementsLast: i)).
		self
			assertList: (list last: list size + i)
			copiedFrom: list
			equals: self listWithElements.
		self
			assertList: (list last: (i * -1))
			copiedFrom: list
			equals: self emptyList].
	self
		assertList: (list last: 0)
		copiedFrom: list
		equals: self emptyList.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 11:31'!
testLastIndexOf
	| list |

	list := self listClass newFrom: self listElements, self listElements.
	list do: [:each |
		self assert:
			(list lastIndexOf: each) =
				(self allowsDuplicates
					ifTrue: [self listElements, self listElements lastIndexOf: each]
					ifFalse: [self listElements lastIndexOf: each])].
	self assert: (list lastIndexOf: self newElement) isZero.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 11:32'!
testLastIndexOfIfAbsent
	| list |

	list := self listClass newFrom: self listElements, self listElements.
	list do: [:each |
		self assert:
			(list lastIndexOf: each ifAbsent: [-1]) =
				(self allowsDuplicates
					ifTrue: [self listElements, self listElements lastIndexOf: each]
					ifFalse: [self listElements lastIndexOf: each])].
	self assert: (list lastIndexOf: self newElement ifAbsent: [-1])  = -1.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/3/2015 02:30'!
testLastOrNil
	self
		assert: self emptyList lastOrNil isNil;
		assert: self listWithSingleElement lastOrNil = self singleElement;
		assert: self listWithElements lastOrNil = self listElements last! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 13:58'!
testNew
	| list |

	list := self listClass new: 20.
	self assert: list = self emptyList.
	self hasCapacity
		ifTrue: [self assert: list capacity >= 20].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:21'!
testNewFrom
	self assert: (self listClass newFrom: self listElements) = self listWithElements! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 11:26'!
testNewWithAll
	| list |

	list := self listClass new: 10 withAll: self singleElement.
	self allowsDuplicates
		ifTrue: [self assert: list size = 10]
		ifFalse: [self assert: list size = 1].
	self assert: (list allSatisfy: [:each | each = self singleElement]).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 06:49'!
testNinth
	self
		assert: self emptyList ninth = self defaultValue;
		assert: self listWithElements ninth = (self listElements at: 9).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:32'!
testOfSize
	self assert: (self listClass ofSize: 10) size = 10! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/28/2014 15:54'!
testPairsDo
	| pairs |

	0 to: self listElements size do: [:i |
		pairs := OrderedCollection new.
		(self listClass newFrom: (self listElements allButLast: i))
			pairsDo: [:a :b | pairs addLast: a -> b].
		
		(self listElements allButLast: i) pairsDo: [:a :b |
			self assert: pairs removeFirst = (a -> b)].
		self assert: pairs isEmpty].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 16:53'!
testReadStream
	| list stream elements |

	list := self listWithElements.
	stream := list readStream.
	self assert: (stream isKindOf: ReadStream).

	elements := self listElements asOrderedCollection.
	[stream atEnd]
		whileFalse: [self assert: stream next = elements removeFirst].
	self assert: elements isEmpty.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/24/2014 14:45'!
testReject
	| list |

	list := self listWithElements.
	self
		assertList: (list reject: [:each | each = self singleElement])
		copiedFrom: list
		equals:
			(self listClass newFrom:
				(self listWithElements copyWithout: self singleElement)).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/17/2015 09:36'!
testRejectThenCollect
	| list |

	list := self listWithElements.
	self
		assertList:
			(list
				reject: [:each | each = self singleElement]
				thenCollect: [:each | self changeElement: each])
		collectedFrom: list
		equals:
			(self listSpecies newFrom:
				(self changedListElements copyWithout: self changedSingleElement)).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/12/2014 21:16'!
testRemoveAll
	| list |

	list := self listWithElements.
	self
		assert: (list removeAll) == list;
		assert: list isEmpty.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:51'!
testRemoveAt
	self listElements withIndexDo: [:each :i | | list |
		list := self listWithElements.
		self
			assert: (list removeAt: i) = each;
			assert: (list = (self listClass newFrom: (self listElements copyWithout: each)))].
	self
		should: [self listWithElements removeAt: 0] raise: Error;
		should: [self listWithElements removeAt: self listElements size + 1] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 17:30'!
testRemoveFirst
	| list |

	list := self listWithElements.
	self listElements do: [:each | 
	self
		assert: list removeFirst = each;
		assert: list = (self listClass newFrom: (self listElements copyAfter: each))].
	self should: [list removeFirst] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 07:57'!
testRemoveFirstFew
	1 to: self listElements size do: [:i | | list removedElements |
		list := self listWithElements.
		removedElements := list removeFirst: i.
		self
			assert: removedElements asArray = (self listElementsFirst: i);
			assert: list = (self listClass newFrom: (self listElements allButFirst: i));
			assert: (list removeFirst: 0) isEmpty;
			should: [self emptyList removeFirst: i] raise: Error].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 17:40'!
testRemoveIfAbsent
	| list |

	list := self listWithElements.
	self
		assert: (list remove: self singleElement ifAbsent: [nil]) = self singleElement;
		deny: (list includes: self singleElement);
		assert: (list remove: self absentElement ifAbsent: [nil]) isNil.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 17:30'!
testRemoveLast
	| list |

	list := self listWithElements.
	self listElements reverseDo: [:each |
		self
			assert: list removeLast = each;
			assert: list = (self listClass newFrom: (self listElements copyUpTo: each))].
	self should: [list removeLast] raise: Error.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 11/24/2014 08:37'!
testRemoveLastFew
	| list removedElements |

	1 to: self listElements size do: [:i |
		list := self listWithElements.
		removedElements := list removeLast: i.
		self
			assert: removedElements asArray = (self listElementsLast: i) asArray;
			assert: list = (self listClass newFrom: (self listElements allButLast: i));
			assert: (list removeLast: 0) isEmpty;
			should: [self emptyList removeLast: i] raise: Error].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/17/2015 11:28'!
testReplaceFromToWith
	| list |

	1 to: self listElements size do: [:start |
		start to: self listElements size do: [:stop |
			list := self listWithElements.
			self assert:
				(list
					replaceFrom: start
					to: stop
					with: (self changedListElements allButFirst: start - 1)) == list.

			1 to: start - 1 do: [:i | self assert: (list at: i) = (self listElements at: i)].
			start to: stop do: [:i | self assert: (list at: i) = (self changedListElements at: i)].
			stop + 1 to: list size do: [:i | self assert: (list at: i) = (self listElements at: i)].

			start > 1
				ifTrue: [
					self
						should: [
							self emptyList
								replaceFrom: start
								to: stop
								with: self listElements]
						raise: Error]
				ifFalse: [
					self assert:
						(self emptyList
							replaceFrom: start
							to: stop
							with: self listElements) =
								(self listClass newFrom:
									(self listElementsFirst: (stop - start) + 1))]]].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/18/2015 08:10'!
testReplaceFromToWithStartingAt
	1 to: self listElements size do: [:start |
		start to: self listElements size do: [:stop | | list |
			list := self listWithElements.
			self assert:
				(list
					replaceFrom: start
					to: stop
					with: self changedListElements
					startingAt: start) == list.

			1 to: start - 1 do: [:i | self assert: (list at: i) = (self listElements at: i)].
			start to: stop do: [:i | self assert: (list at: i) = (self changedListElements at: i)].
			stop + 1 to: list size do: [:i | self assert: (list at: i) = (self listElements at: i)].

			start > 1
				ifTrue: [
					self
						should: [
							self emptyList
								replaceFrom: start
								to: stop
								with: self listElements
								startingAt: start]
						raise: Error]
				ifFalse: [
					self assert:
						(self emptyList
							replaceFrom: start
							to: stop
							with: self listElements
							startingAt: start) =
								(self listClass newFrom:
									(self listElementsFirst: (stop - start) + 1))]]].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 13:58'!
testReset
	| list oldCapacity |

	list := self listWithElements.
	self hasCapacity
		ifTrue: [oldCapacity := list capacity].
	list reset.
	self assert: list isEmpty.
	oldCapacity
		ifNotNil: [self assert: list capacity >= oldCapacity].! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 10:47'!
testReverse
	| list |

	list := self listWithElements.
	self
		assertList: list reverse
		copiedFrom: list
		equals: (self listClass newFrom: self listElements reverse).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:08'!
testReverseDo
	self
		should: [:block | self emptyList reverseDo: block] enumerate: #();
		should: [:block | self listWithElements reverseDo: block] enumerate: self listElements reversed.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/23/2014 10:47'!
testReversed
	| list |

	list := self listWithElements.
	self
		assertList: list reversed
		copiedFrom: list
		equals: (self listClass newFrom: self listElements reversed).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 06:51'!
testSecond
	self
		assert: self emptyList second = self defaultValue;
		assert: self listWithElements second = (self listElements at: 2).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/24/2014 14:39'!
testSelect
	| list |

	list := self listWithElements.
	self
		assertList: (list select: [:each | each ~= self singleElement])
		copiedFrom: list
		equals:
			(self listClass newFrom:
				(self listWithElements copyWithout: self singleElement)).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/17/2015 09:31'!
testSelectThenCollect
	| list |

	list := self listWithElements.
	self
		assertList:
			(list
				select: [:each | each ~= self singleElement]
				thenCollect: [:each | self changeElement: each])
		collectedFrom: list
		equals:
			(self listSpecies newFrom:
				(self changedListElements copyWithout: self changedSingleElement)).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 06:51'!
testSeventh
	self
		assert: self emptyList seventh = self defaultValue;
		assert: self listWithElements seventh = (self listElements at: 7).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 06:51'!
testSixth
	self
		assert: self emptyList sixth = self defaultValue;
		assert: self listWithElements sixth = (self listElements at: 6).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'JAA 3/26/2011 18:13'!
testSize
	self
		assert: self emptyList size isZero;
		assert: self listWithSingleElement size = 1;
		assert: self listWithElements size = self listElements size! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/17/2015 10:34'!
testSort
	| list |

	list := self listWithElements.
	self
		assert: list sort == list;
		assert: list = (self listClass newFrom: (self listElements copy sort: self sortBlock));
		assert: (list sort: self reverseSortBlock) == list;
		assert: list = (self listClass newFrom: (self listElements copy sort: self reverseSortBlock)).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 1/17/2015 10:35'!
testSorted
	| list |

	list := self listWithElements.
	self
		assertList: list sorted
		copiedFrom: list
		equals: (self listClass newFrom: (self listElements copy sorted: self sortBlock)).
	self
		assertList: (list sorted: self reverseSortBlock)
		copiedFrom: list
		equals: (self listClass newFrom: (self listElements copy sorted: self reverseSortBlock)).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/19/2014 06:27'!
testSpecies
	self
		assert: self emptyList species = self listSpecies! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 2/2/2015 02:20'!
testSwapWith
	| list start end |

	list := self listWithElements.

	start := 1.
	end := list size.
	[start < end]
		whileTrue: [
			list swap: start with: end.
			start := start + 1.
			end := end - 1].
	self assert: list = (self listClass newFrom: self listElements reversed).

	start := 1.
	end := list size.
	[start < end]
		whileTrue: [
			list swap: end with: start.
			start := start + 1.
			end := end - 1].
	self assert: list = self listWithElements.

	1 to: list size do: [:i | list swap: i with: i].
	self assert: list = self listWithElements.

	self
		should: [list swap: 0 with: 1]
		raise: Error.
	self
		should: [list swap: list size + 1 with: list size]
		raise: Error.
	self assert: list = self listWithElements.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 06:51'!
testThird
	self
		assert: self emptyList third = self defaultValue;
		assert: self listWithElements third = (self listElements at: 3).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 07:10'!
testWithCollect
	| list |

	list := self listWithElements.
	self
		assertList: (list with: (1 to: list size + 1) collect: [:each :i | each -> i])
		collectedFrom: list
		equals:
			(self listSpecies newFrom:
				(self listElements withIndexCollect: [:each :i | each -> i])).
	self
		assertList: (list with: (1 to: list size - 1) collect: [:each :i | each -> i])
		collectedFrom: list
		equals:
			(self listSpecies newFrom:
				(self listElements allButLast
					withIndexCollect: [:each :i | each -> i])).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 07:10'!
testWithDo
	| list enumerated |

	list := self listWithElements.
	enumerated := OrderedCollection new.
	list with: (1 to: list size + 1) do: [:each :i | enumerated addLast: each -> i].
	self assert:
		enumerated =
			(self listElements withIndexCollect: [:each : i | each -> i ])
				asOrderedCollection.

	enumerated removeAll.
	list with: (1 to: list size - 1) do: [:each :i | enumerated addLast: each -> i].
	self assert:
		enumerated =
			(self listElements allButLast
				withIndexCollect: [:each : i | each -> i ])
					asOrderedCollection.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 07:11'!
testWithIndexCollect
	| list |

	list := self listWithElements.
	self
		assertList: (list withIndexCollect: [:each :i | each -> i])
		collectedFrom: list
		equals:
			(self listSpecies newFrom:
				(self listElements withIndexCollect: [:each :i | each -> i])).! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 12/14/2014 07:11'!
testWithIndexDo
	| enumerated |

	enumerated := OrderedCollection new.
	self listWithElements withIndexDo: [:each :i | enumerated addLast: each -> i].
	self assert:
		enumerated =
			(self listElements withIndexCollect: [:each : i | each -> i ]) asOrderedCollection.! !

!XMLOrderedListTest methodsFor: 'tests' stamp: 'monty 10/24/2014 14:40'!
testWriteStream
	| list stream |

	list := self emptyList.
	stream := list writeStream.
	self assert: (stream isKindOf: WriteStream).

	self listElements do: [:each | stream nextPut: each].
	self assert:
		(stream contents) asOrderedCollection =
			self listElements asOrderedCollection.! !

TestCase subclass: #XMLParserLimitsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLParserLimitsTest methodsFor: 'source' stamp: 'monty 1/19/2014 04:10'!
recursiveEntitiesNamed: aName depth: aLimit isParameter: aBoolean
	| xml |

	xml := String new writeStream.
	1 to: aLimit do: [:i |
		xml
			cr;
			nextPutAll: '<!!ENTITY '.
		aBoolean
			ifTrue: [xml nextPutAll: '% '].
		xml
			nextPutAll: aName, i printString;
			nextPutAll: ' "'.
			(i = aLimit)
				ifTrue: [xml nextPutAll: '<!!--last-->']
				ifFalse: [
					aBoolean
						ifTrue: [xml nextPutAll: '&#37;']
						ifFalse: [xml nextPut: $&].
					xml
						nextPutAll: aName, (i + 1) printString;
						nextPut: $;].
		xml nextPutAll: '">'].
	^ xml contents.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 1/27/2015 22:49'!
testDefaultLimits
	| parser |

	parser := SAXHandler on: ''.
	self
		assert: parser documentReadLimit = XMLParserLimits defaultDocumentReadLimit;
		assert: parser maxEntityReplacementDepth > 0.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 11/16/2014 00:24'!
testDocumentReadLimit
	| xml replacement |

	replacement := 'entity replacement'.
	xml := '<!!DOCTYPE test [<!!ENTITY entity "', replacement, '">]><test>&entity;</test>'.
	self
		shouldnt: [
			(SAXHandler on: xml)
				documentReadLimit: xml size + replacement size;
				parseDocument]
		raise: XMLLimitException.
	self
		should: [
			(SAXHandler on: xml)
				documentReadLimit: xml size + replacement size - 1;
				parseDocument]
		raise: XMLLimitException.
	self
		shouldnt: [
			(SAXHandler on: xml)
				documentReadLimit: nil;
				parseDocument]
		raise: XMLLimitException.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 1/19/2014 03:54'!
testRecursiveGeneralEntityReplacement
	| limit xml |

	limit := 3.
	xml :=
		self
			xmlWithRecursiveEntitiesNamed: 'entity'
			depth: limit
			isParameter: false.
	self
		shouldnt: [
			(SAXHandler on: xml)
				maxEntityReplacementDepth: limit;
				parseDocument]
		raise: XMLLimitException.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 1/19/2014 22:50'!
testRecursiveGeneralEntityReplacementPastLimit
	| limit xml |

	limit := 3.
	xml :=
		self
			xmlWithRecursiveEntitiesNamed: 'entity'
			depth: limit + 1
			isParameter: false.
	self
		should: [
			(SAXHandler on: xml)
				maxEntityReplacementDepth: limit;
				parseDocument]
		raise: XMLLimitException.
	self
		shouldnt: [
			(SAXHandler on: xml)
				maxEntityReplacementDepth: nil;
				parseDocument]
		raise: XMLLimitException.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 1/19/2014 03:56'!
testRecursiveParameterEntityReplacement
	| limit xml |

	limit := 3.
	xml :=
		self
			xmlWithRecursiveEntitiesNamed: 'entity'
			depth: limit
			isParameter: true.
	self
		shouldnt: [
			(SAXHandler on: xml)
				maxEntityReplacementDepth: limit;
				parseDocument]
		raise: XMLLimitException.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 1/19/2014 22:49'!
testRecursiveParameterEntityReplacementPastLimit
	| limit xml |

	limit := 3.
	xml :=
		self
			xmlWithRecursiveEntitiesNamed: 'entity'
			depth: limit + 1
			isParameter: true.
	self
		should: [
			(SAXHandler on: xml)
				maxEntityReplacementDepth: limit;
				parseDocument]
		raise: XMLLimitException.
	self
		shouldnt: [
			(SAXHandler on: xml)
				maxEntityReplacementDepth: nil;
				parseDocument]
		raise: XMLLimitException.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 1/27/2015 23:08'!
testRemoveLimits
	| parser readLimit depthLimit |

	readLimit := 100.
	depthLimit := 5.
	(parser := SAXHandler on: '')
		documentReadLimit: readLimit;
		maxEntityReplacementDepth: depthLimit.
	self
		assert: parser documentReadLimit = readLimit;
		assert: parser maxEntityReplacementDepth = depthLimit.

	parser removeLimits.
	self
		assert: parser documentReadLimit isNil;
		assert: parser maxEntityReplacementDepth isNil.! !

!XMLParserLimitsTest methodsFor: 'tests' stamp: 'monty 2/10/2014 02:43'!
testTokenizerMaxPushBack
	| tokenizer |

	tokenizer :=
		XMLWellFormedParserTokenizer
			driver: nil
			on: '' readStream.
	self
		shouldnt: [
			tokenizer streamReader maxNestedStreams
				timesRepeat: [tokenizer streamReader pushBack: 'test']]
		raise: XMLLimitException.
	self
		should: [tokenizer streamReader pushBack: 'test']
		raise: XMLLimitException.! !

!XMLParserLimitsTest methodsFor: 'source' stamp: 'monty 3/7/2016 20:08'!
xmlWithRecursiveEntitiesNamed: aName depth: aLimit isParameter: aBoolean
	| xml |

	(xml := String new writeStream)
		nextPutAll: '<!!DOCTYPE test ['.
	xml nextPutAll:
		(self
			recursiveEntitiesNamed: aName
			depth: aLimit
			isParameter: aBoolean).
	aBoolean
		ifTrue: [xml nextPutAll: '%', aName, '1;]><test/>']
		ifFalse: [xml nextPutAll: ']><test>&', aName, '1;</test>'].

	^ xml contents.! !

TestCase subclass: #XMLParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

XMLParserTest subclass: #SAXHandlerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!SAXHandlerTest methodsFor: 'accessing' stamp: 'monty 4/12/2015 09:28'!
prefixedXMLAttributes
	^ OrderPreservingDictionary with: 'name' -> 'value'! !

!SAXHandlerTest methodsFor: 'accessing' stamp: 'monty 4/12/2015 09:28'!
prefixedXMLAttributesNamespacesDisabled
	^ OrderPreservingStringDictionary
		with: 'xmlns:prefix' -> 'prefixNS'
		with: 'xmlns' -> 'defaultNS'
		with: 'name' -> 'value'! !

!SAXHandlerTest methodsFor: 'source' stamp: 'monty 1/31/2014 21:05'!
prefixedXMLSource
	^ '<prefix:test xmlns:prefix="prefixNS" xmlns="defaultNS" name="value">test</prefix:test>'! !

!SAXHandlerTest methodsFor: 'accessing' stamp: 'monty 10/24/2014 07:06'!
prefixedXMLSourceSelectors
	^ #(startDocument
		startPrefixMapping:uri: startPrefixMapping:uri:
		startElement:prefix:uri:localName:attributes:
		characters:
		endElement:prefix:uri:localName:
		endPrefixMapping: endPrefixMapping:
		endDocument)! !

!SAXHandlerTest methodsFor: 'accessing' stamp: 'monty 10/24/2014 07:16'!
prefixedXMLSourceSelectorsNamespacesDisabled
	^ self prefixedXMLSourceSelectors reject: [:each |
		#(startPrefixMapping:uri: endPrefixMapping:) includes: each].
! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/12/2015 09:34'!
testSAXEndElementForwarding
	| messageRecorder message |

	messageRecorder :=
		(SAXElementEventRecordingHandler on: self prefixedXMLSource)
			parseDocument;
			messageRecorder.

	message :=
		messageRecorder
			nextMessage;
			nextMessage.
	self
		assert: message selector == #endElement:;
		assert: message arguments = #('prefix:test').! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/12/2015 10:11'!
testSAXEndElementForwardingNamespacesDisabled
	| messageRecorder message |

	messageRecorder :=
		(SAXElementEventRecordingHandler on: self prefixedXMLSource)
			usesNamespaces: false;
			parseDocument;
			messageRecorder.

	message :=
		messageRecorder
			nextMessage;
			nextMessage.
	self
		assert: message selector == #endElement:;
		assert: message arguments = #('prefix:test').! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/12/2015 09:42'!
testSAXEndElementPrefixURIForwarding
	| messageRecorder message |

	messageRecorder :=
		(SAXElementPrefixURIEventRecordingHandler on: self prefixedXMLSource)
			parseDocument;
			messageRecorder.

	message :=
		messageRecorder
			nextMessage;
			nextMessage.
	self
		assert: message selector == #endElement:prefix:uri:;
		assert: message arguments = #('prefix:test' 'prefix' 'prefixNS').! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/12/2015 09:42'!
testSAXEndElementPrefixURIForwardingNamespacesDisabled
	| messageRecorder message |

	messageRecorder :=
		(SAXElementPrefixURIEventRecordingHandler on: self prefixedXMLSource)
			usesNamespaces: false;
			parseDocument;
			messageRecorder.

	message :=
		messageRecorder
			nextMessage;
			nextMessage.
	self
		assert: message selector == #endElement:prefix:uri:;
		assert: message arguments = #('prefix:test' 'prefix' '').! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/12/2015 10:09'!
testSAXEndElementPrefixURILocalName
	| messageRecorder message |

	messageRecorder :=
		(SAXElementPrefixURILocalNameEventRecordingHandler on: self prefixedXMLSource)
			parseDocument;
			messageRecorder.

	message :=
		messageRecorder
			nextMessage;
			nextMessage.
	self
		assert: message selector == #endElement:prefix:uri:localName:;
		assert: message arguments = #('prefix:test' 'prefix' 'prefixNS' 'test').! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/12/2015 10:10'!
testSAXEndElementPrefixURILocalNameNamespacesDisabled
	| messageRecorder message |

	messageRecorder :=
		(SAXElementPrefixURILocalNameEventRecordingHandler on: self prefixedXMLSource)
			usesNamespaces: false;
			parseDocument;
			messageRecorder.

	message :=
		messageRecorder
			nextMessage;
			nextMessage.
	self
		assert: message selector == #endElement:prefix:uri:localName:;
		assert: message arguments = #('prefix:test' 'prefix' '' 'test').! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 3/13/2016 19:40'!
testSAXEndPrefixMapping
	| parser messages |

	(parser := SAXHandlerProxy on: self prefixedXMLSource)
		parseDocument.
	self assert: (parser wasSentSelectors: self prefixedXMLSourceSelectors).
	messages := parser allSentMessagesNamed: #endPrefixMapping:.
	self
		assert: messages size = 2;
		assert: messages first arguments = #('prefix');
		assert: messages second arguments = #('').

	(parser :=
		SAXHandlerProxy on:
			'<one xmlns="defaultNS"><two xmlns="defaultNS"/></one>')
		parseDocument.
	messages := parser allSentMessagesNamed: #endPrefixMapping:.
	self
		assert: messages size = 1;
		assert: messages first arguments = #('').

	(parser :=
		SAXHandlerProxy on:
			'<one xmlns="defaultNS"><two xmlns="newDefaultNS"/></one>')
		parseDocument.
	messages := parser allSentMessagesNamed: #endPrefixMapping:.
	self
		assert: messages size = 2;
		assert: messages first arguments = #('');
		assert: messages second arguments = #('').

	(parser :=
		SAXHandlerProxy on:
			'<one xmlns:xml="', XMLNamespaceScope xmlNamespaceURI, '"/>')
		parseDocument.
	self assert:
		(parser allSentMessagesNamed: #endPrefixMapping:) isEmpty.! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 11/26/2014 06:28'!
testSAXEntityHandlers
	| xml parser messages |

	xml := '<!!DOCTYPE root [
		<!!ENTITY % firstDeclaration "<!!ENTITY first &#39; &second; &#39;>">
		%firstDeclaration;
		<!!ENTITY second " replacement ">]>
	<root name="&second;">&first;</root>'.
	(parser :=SAXHandlerProxy on: xml) parseDocument.
	self assert:
		(parser wasSentSelectors: 
			#(startDocument
				startDTD:publicID:systemID:
				parameterEntityDeclaration:replacement:
				generalEntityDeclaration:replacement: generalEntityDeclaration:replacement:
				endDTD
				startElement:prefix:uri:localName:attributes:
				startEntity: startEntity: endEntity: endEntity: characters:
				endElement:prefix:uri:localName:
				endDocument)).
	messages := parser allSentMessagesNamed: #startEntity:.
	self
		assert: messages first arguments = #('first');
		assert: messages second arguments = #('second').
	messages := parser allSentMessagesNamed: #endEntity:.
	self
		assert: messages first arguments = #('second');
		assert: messages second arguments = #('first').! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 3/12/2016 19:09'!
testSAXIgnorableWhitespace
	| xml parser |

	xml := '
	<one>
		<two>
			test
		</two>
	</one>
	'.
	(parser := SAXHandlerProxy on: xml) parseDocument.
	self assert:
		(parser wasSentSelectors: 
			#(startDocument
				startElement:prefix:uri:localName:attributes:
				ignorableWhitespace:
				startElement:prefix:uri:localName:attributes: characters: endElement:prefix:uri:localName:
				ignorableWhitespace:
				endElement:prefix:uri:localName:
				ignorableWhitespace:
				endDocument)).
	(parser allSentMessagesNamed: #ignorableWhitespace:) do: [:each |
		self assert:
			((each arguments first) notEmpty
				and: [each arguments first allSatisfy: [:char | char isXMLSeparator]])].! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/5/2015 01:45'!
testSAXParsingInterrupt
	| uniqueSelectors parser |

	uniqueSelectors := OrderedCollection new.
	self prefixedXMLSourceSelectors do: [:each |
		(uniqueSelectors includes: each)
			ifFalse: [uniqueSelectors addLast: each]].
	uniqueSelectors do: [:each |
		(parser := SAXHandlerPluggableProxy on: self prefixedXMLSource)
			handle: each with: [:message | parser interruptParsing];
			parseDocument.
		self assert:
			(parser wasSentSelectors:
				((self prefixedXMLSourceSelectors copyUpTo: each) copyWith: each))].! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/13/2015 02:35'!
testSAXSkippedEntityUndeclared
	| xml parser |

	xml := '<root>&undeclared;</root>'.
	(parser := SAXHandlerProxy on: xml)
		isValidating: false;
		parseDocument.
	self assert:
		(parser wasSentSelectors: 
			#(startDocument
				startElement:prefix:uri:localName:attributes:
				skippedEntity:
				endElement:prefix:uri:localName:
				endDocument)).
	self assert: (parser argumentsSentWith: #skippedEntity:) = #('undeclared').! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/13/2015 02:36'!
testSAXSkippedEntityUndeclaredPreserved
	| xml parser |

	xml := '<root>&undeclared;</root>'.
	(parser := SAXHandlerProxy on: xml)
		isValidating: false;
		preservesUndeclaredEntityReferences: true;
		parseDocument.
	self assert:
		(parser wasSentSelectors:
			#(startDocument
				startElement:prefix:uri:localName:attributes:
				skippedEntity:
				characters:
				endElement:prefix:uri:localName:
				endDocument)).
	self
		assert: (parser argumentsSentWith: #skippedEntity:) = #('undeclared');
		assert: (parser argumentsSentWith: #characters:) = #('&undeclared;').! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/13/2015 02:37'!
testSAXSkippedEntityUnreplaced
	| xml parser |

	xml := '<!!DOCTYPE root [<!!ENTITY declared "replacement">]><root>&declared;</root>'.
	(parser := SAXHandlerProxy on: xml)
		isValidating: true;
		replacesContentEntityReferences: false;
		parseDocument.
	self assert:
		(parser wasSentSelectors:
			#(startDocument
				startDTD:publicID:systemID:
				generalEntityDeclaration:replacement:
				endDTD
				startElement:prefix:uri:localName:attributes:
				skippedEntity:
				characters:
				endElement:prefix:uri:localName:
				endDocument)).
	self
		assert: (parser argumentsSentWith: #skippedEntity:) = #('declared');
		assert: (parser argumentsSentWith: #characters:) = #('&declared;').! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/12/2015 09:31'!
testSAXStartElementForwarding
	| messageRecorder message arguments |

	messageRecorder :=
		(SAXElementEventRecordingHandler on: self prefixedXMLSource)
			parseDocument;
			messageRecorder.

	message := messageRecorder nextMessage.
	self assert: message selector == #startElement:attributes:.

	arguments := message arguments.
	self
		assert: arguments first = 'prefix:test';
		assert: arguments second = self prefixedXMLAttributes.! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/12/2015 09:32'!
testSAXStartElementForwardingNamespacesDisabled
	| messageRecorder message arguments |

	messageRecorder :=
		(SAXElementEventRecordingHandler on: self prefixedXMLSource)
			usesNamespaces: false;
			parseDocument;
			messageRecorder.
	
	message := messageRecorder nextMessage.
	self assert: message selector == #startElement:attributes:.

	arguments := message arguments.
	self
		assert: arguments first = 'prefix:test';
		assert: arguments second = self prefixedXMLAttributesNamespacesDisabled.! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/12/2015 09:40'!
testSAXStartElementPrefixURIForwarding
	| messageRecorder message arguments |

	messageRecorder :=
		(SAXElementPrefixURIEventRecordingHandler on: self prefixedXMLSource)
			parseDocument;
			messageRecorder.

	message := messageRecorder nextMessage.
	self assert: message selector == #startElement:prefix:uri:attributes:.

	arguments := message arguments.
	self
		assert: arguments first = 'prefix:test';
		assert: arguments second = 'prefix';
		assert: arguments third = 'prefixNS';
		assert: arguments fourth = self prefixedXMLAttributes.! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/12/2015 09:41'!
testSAXStartElementPrefixURIForwardingNamespacesDisabled
	| messageRecorder message arguments |

	messageRecorder :=
		(SAXElementPrefixURIEventRecordingHandler on: self prefixedXMLSource)
			usesNamespaces: false;
			parseDocument;
			messageRecorder.

	message := messageRecorder nextMessage.
	self assert: message selector == #startElement:prefix:uri:attributes:.

	arguments := message arguments.
	self
		assert: arguments first = 'prefix:test';
		assert: arguments second = 'prefix';
		assert: arguments third isEmpty;
		assert: arguments fourth = self prefixedXMLAttributesNamespacesDisabled.! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/12/2015 10:04'!
testSAXStartElementPrefixURILocalName
	| messageRecorder message arguments |

	messageRecorder :=
		(SAXElementPrefixURILocalNameEventRecordingHandler on: self prefixedXMLSource)
			parseDocument;
			messageRecorder.

	message := messageRecorder nextMessage.
	self assert: message selector == #startElement:prefix:uri:localName:attributes:.

	arguments := message arguments.
	self
		assert: arguments first = 'prefix:test';
		assert: arguments second = 'prefix';
		assert: arguments third = 'prefixNS';
		assert: arguments fourth = 'test';
		assert: arguments fifth = self prefixedXMLAttributes.! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 4/12/2015 10:07'!
testSAXStartElementPrefixURILocalNameNamespacesDisabled
	| messageRecorder message arguments |

	messageRecorder :=
		(SAXElementPrefixURILocalNameEventRecordingHandler on: self prefixedXMLSource)
			usesNamespaces: false;
			parseDocument;
			messageRecorder.

	message := messageRecorder nextMessage.
	self assert: message selector == #startElement:prefix:uri:localName:attributes:.

	arguments := message arguments.
	self
		assert: arguments first = 'prefix:test';
		assert: arguments second = 'prefix';
		assert: arguments third = '';
		assert: arguments fourth = 'test';
		assert: arguments fifth = self prefixedXMLAttributesNamespacesDisabled.! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 3/13/2016 19:39'!
testSAXStartPrefixMapping
	| parser messages |

	(parser := SAXHandlerProxy on: self prefixedXMLSource)
		parseDocument.
	self assert: (parser wasSentSelectors: self prefixedXMLSourceSelectors).
	messages := parser allSentMessagesNamed: #startPrefixMapping:uri:.
	self
		assert: messages size = 2;
		assert: messages first arguments = #('prefix' 'prefixNS');
		assert: messages second arguments = #('' 'defaultNS').

	(parser :=
		SAXHandlerProxy on:
			'<one xmlns="defaultNS"><two xmlns="defaultNS"/></one>')
		parseDocument.
	messages := parser allSentMessagesNamed: #startPrefixMapping:uri:.
	self
		assert: messages size = 1;
		assert: messages first arguments = #('' 'defaultNS').

	(parser :=
		SAXHandlerProxy on:
			'<one xmlns="defaultNS"><two xmlns="newDefaultNS"/></one>')
		parseDocument.
	messages := parser allSentMessagesNamed: #startPrefixMapping:uri:.
	self
		assert: messages size = 2;
		assert: messages first arguments = #('' 'defaultNS');
		assert: messages second arguments = #('' 'newDefaultNS').

	(parser :=
		SAXHandlerProxy on:
			'<one xmlns:xml="', XMLNamespaceScope xmlNamespaceURI, '"/>')
		parseDocument.
	self assert:
		(parser allSentMessagesNamed: #startPrefixMapping:uri:) isEmpty.! !

!SAXHandlerTest methodsFor: 'tests' stamp: 'monty 3/7/2016 20:07'!
testSAXXMLDeclaration
	| parser selectors arguments |

	(parser :=
		SAXHandlerProxy on:
			'<?xml version="1.0" encoding="UTF-8"?><test/>')
		parseDocument.
	selectors :=
		#(startDocument
			xmlVersion:encoding:standalone:
			startElement:prefix:uri:localName:attributes:
			endElement:prefix:uri:localName:
			endDocument).
	self assert: (parser wasSentSelectors: selectors).

	arguments := parser argumentsSentWith: #xmlVersion:encoding:standalone:.
	self
		assert: arguments first = 1.0;
		assert: arguments second = 'UTF-8';
		deny: arguments third.

	(parser :=
		SAXHandlerProxy on:
			'<?xml version="1.1" encoding="utf-8" standalone="yes"?><test/>')
		parseDocument.
	self assert: (parser wasSentSelectors: selectors).

	arguments := (parser argumentsSentWith: #xmlVersion:encoding:standalone:).
	self
		assert: arguments first = 1.1;
		assert: arguments second = 'utf-8';
		assert: arguments third.! !

XMLParserTest subclass: #XMLDOMParserTest
	instanceVariableNames: 'addressBookNamespacePrefixedXML addressBookNamespacedXML'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLDOMParserTest class methodsFor: 'source' stamp: 'monty 10/24/2014 11:32'!
addressBookNamespacePrefixedXML
	"
	self addressBookNamespacePrefixedXML
	"
	| xmlWithPrefixes |

	xmlWithPrefixes := self addressBookNamespacedXML.
	#('<addressbook' '</addressbook' '<person' '</person' 'family-name=' 'first-name=')
		with: #('<book:addressbook' '</book:addressbook' '<ps:person' '</ps:person' 'ps:family-name=' 'ps:first-name=')
		do: [:source :replacement |
			xmlWithPrefixes := xmlWithPrefixes copyReplaceAll: source with: replacement].
	^ xmlWithPrefixes.! !

!XMLDOMParserTest class methodsFor: 'source' stamp: 'monty 10/24/2014 11:30'!
addressBookNamespacedXML
	"
	self addressBookNamespacedXML
	"
	^ '<addressbook xmlns="defaultNS" xmlns:book="bookNS" xmlns:ps="personNS">',
		(self addressBookXML copyAfter:  $>)! !

!XMLDOMParserTest methodsFor: 'source' stamp: 'monty 3/22/2015 03:27'!
addressBookNamespacePrefixedXML
	^ addressBookNamespacePrefixedXML
		ifNil: [addressBookNamespacePrefixedXML := self class addressBookNamespacePrefixedXML]! !

!XMLDOMParserTest methodsFor: 'source' stamp: 'monty 3/22/2015 03:28'!
addressBookNamespacedXML
	^ addressBookNamespacedXML
		ifNil: [addressBookNamespacedXML := self class addressBookNamespacedXML]! !

!XMLDOMParserTest methodsFor: 'source' stamp: 'monty 3/7/2016 20:12'!
internalSubsetWithDeclarations
	^ '<!!DOCTYPE root PUBLIC "pubid" "external.dtd" [
			<!!ELEMENT root ( #PCDATA | one | two)* >
			<!!ATTLIST root
						name CDATA #IMPLIED
						number ( one | two | three ) "three">
			<!!ENTITY general "replacement">
			<!!ENTITY externalGeneral PUBLIC "pubid2" "external1.dtd">
			<!!ENTITY % parameter "replacement">
			<!!ENTITY % externalParameter SYSTEM "external2.dtd">
			<!!NOTATION notation SYSTEM "note.doc">
		]>
		<root>test</root>'! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testAllElementsNamespaceURIs
	| document people |

	document := XMLDOMParser parse: self addressBookNamespacedXML.

	people :=
		document
			allElementsNamed: 'person'
			namespaceURI: 'defaultNS'.
	self
		assert: people size = 4;
		assert:
			(document
				allElementsNamed: 'person'
				namespaceURI: 'undeclaredNS') isEmpty.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 10/24/2014 12:06'!
testAttributeNamespaces
	| document  people |

	document := XMLDOMParser parse: self addressBookNamespacePrefixedXML.
	people := document allElementsNamed: 'person' namespaceURI: 'personNS'.
	self assert: people notEmpty.
	
	people do: [:each |
		self
			assert: each hasAttributes;
			deny: each attributeNames isEmpty;
			assert:
				(each attributeNames allSatisfy: [:name |
					(each attributeNodeAt: name namespaceURI: 'defaultNS') isNil]);
			assert: (each attributeNodeAt: 'ps:family-name' namespaceURI: 'personNS') notNil;
			assert: (each attributeNodeAt: 'ps:first-name' namespaceURI: 'personNS') notNil].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 10/24/2014 12:06'!
testAttributeNamespacesDisabled
	| document  people |

	document :=
		XMLDOMParser
			parse: self addressBookNamespacePrefixedXML
			usingNamespaces: false.
	people := document allElementsNamed: 'person'.
	self assert: people notEmpty.
	people do: [:each |
		self
			assert: each hasAttributes;
			deny: each attributeNames isEmpty;
			assert: (each attributeNodeAt: 'ps:family-name' namespaceURI: 'personNS') isNil;
			assert: (each attributeNodeAt: 'ps:first-name' namespaceURI: 'personNS') isNil].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testAttributesExcludingDeclarations
	| document   |

	document := XMLDOMParser parse: self addressBookNamespacedXML.
	self deny: document root hasAttributes.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testAttributesIncludingDeclarations
	| document  root |

	document :=
		XMLDOMParser
			parse: self addressBookNamespacedXML
			usingNamespaces: false.
	root := document root.
	self
		assert: root hasAttributes;
		assert: (root attributeAt: 'xmlns') = 'defaultNS';
		assert: (root attributeAt: 'xmlns:book') = 'bookNS';
		assert: (root attributeAt: 'xmlns:ps') = 'personNS'.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:55'!
testCDataSection
	| document element unescaped escaped xml |

	unescaped := 'this is CDATA test content 1 < 2 & 5 > 3'.
	escaped := 'this is CDATA test content 1 &lt; 2 &amp; 5 &gt; 3'.
	xml := 
		'<?xml version="1.0" encoding="UTF-8"?>
		<test-data>test<!![CDATA[', unescaped, ']]>test</test-data>'.
	document := XMLDOMParser parse: xml.
	element := document findElementNamed: 'test-data'.
	self
		assert: element nodes size = 1;
		assert: element firstNode isStringNode;
		assert: element firstNode printString = ('test', escaped, 'test').

	document :=
		(XMLDOMParser on: xml)
			preservesCDataNodes: true;
			parseDocument.
	element := document findElementNamed: 'test-data'.
	self
		assert: element nodes size = 3;
		assert: element nodes second isCData;
		assert: element nodes second printString = ('<!![CDATA[', unescaped, ']]>');
		assert: element strings asArray = (Array with: 'test' with: unescaped with: 'test').! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 10/24/2014 12:07'!
testChangingNamespaces
	| document people person |

	document := XMLDOMParser parse: self addressBookNamespacedXML.

	people := document allElementsNamed: 'person' namespaceURI: 'defaultNS'.
	(person := people third) prefix: 'ps'.
	self assert: person == (document findElementNamed: 'person' namespaceURI: 'personNS').
	self assert:
		(people copyWithout: person) =
			((document findElementNamed: 'addressbook') nodesSelect: [:each |
				each isElement and: [each namespaceURI = 'defaultNS']]).! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/21/2013 06:10'!
testCollectingParsedNodes
	| document firstPerson allNames |
	"self debug: #testParsing"

	document := XMLDOMParser parse: self addressBookXML.
	firstPerson := document findElementNamed: 'person'.

	allNames := #('person' 'contact-info' 'address' 'job-info' 'manager').
	self
		assert: (firstPerson allElementsCollect: [:each | each name]) asArray = allNames;
		assert: (firstPerson allElements collect: [:each | each name]) asArray = allNames;
		assert: (firstPerson nodesCollect: [:each | each name]) asArray = allNames allButFirst;
		assert: (firstPerson nodes collect: [:each | each name]) asArray = allNames allButFirst.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 3/12/2015 18:19'!
testDeclaringXMLNamespace
	| xmlNamespace elements |

	xmlNamespace :=XMLNamespaceScope xmlNamespaceURI.
	self deny:
		((XMLDOMParser parse: self addressBookNamespacedXML) root
			declaresPrefix: 'xml'
			uri: xmlNamespace).

	elements :=
		(XMLDOMParser parse:
			'<one><two xmlns:xml="', xmlNamespace, '"/><xml:three/></one>') allElements.
	self
		deny: (elements first declaresPrefix: 'xml' uri: xmlNamespace);
		assert: (elements second declaresPrefix: 'xml' uri: xmlNamespace);
		deny: (elements third declaresPrefix: 'xml' uri: xmlNamespace).! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/30/2014 21:42'!
testDisabledNamespacesAttributes
	| document |

	document :=
		XMLDOMParser
			parse: '<root xmlns:prefix="prefixNS" xmlns="defaultNS" name="value"/>'
			usingNamespaces: false.
	self
		assert: document root attributeNames asArray = #('xmlns:prefix' 'xmlns' 'name');
		assert: (document root attributeAt: 'xmlns:prefix') = 'prefixNS';
		assert: (document root attributeAt: 'xmlns') = 'defaultNS';
		assert: (document root attributeAt: 'name') = 'value'.! !

!XMLDOMParserTest methodsFor: 'tests - internal subset' stamp: 'monty 12/22/2013 06:22'!
testDoctypeDeclaration
	| doctype |

	doctype :=
		(XMLDOMParser
			parse: self internalSubsetWithDeclarations
			usingNamespaces: true
			validation: true
			externalEntities: false) doctypeDeclaration.
	self
		assert: doctype root = 'root';
		assert: doctype publicID = 'pubid';
		assert: doctype systemID = 'external.dtd';
		assert: doctype nodes notEmpty;
		assert: (doctype nodes allSatisfy: [:each | each isDeclaration]).! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 11/7/2014 14:53'!
testDocument
	| document root |

	document := XMLDOMParser parse: self addressBookXML.
	document allNodesDo: [:each | self assert: each document == document].

	root := document root.
	root innerXML: self addressBookXML.
	document allNodesDo: [:each | self assert: each document == document].

	document removeNode: root.
	root allNodesDo: [:each | self assert: each document isNil].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 4/11/2015 01:06'!
testDocumentInternalSubset
	| document doctypeDeclaration |

	document := XMLDOMParser parse: self internalSubsetWithDeclarations.
	doctypeDeclaration := document doctypeDeclaration.
	doctypeDeclaration allNodesDo: [:each |
		self assert: each document == document].

	document doctypeDeclaration: nil.
	doctypeDeclaration allNodesDo: [:each |
		self assert: each document isNil].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 10/29/2014 10:04'!
testElementNames
	| document root  persons |

	document := XMLDOMParser parse: self addressBookXML.
	root := document root.

	persons := 	root allNodesSelect: [:each |
			each isElement
				and: [(each isNamedAny: #('notperson' 'person'))
					and: [(each isNamed: 'person')
						and: [(each name = 'person') and: [each localName = 'person']]]]].
	self assert: persons size = 4.

	root allElementsNamed: 'person' do: [:p |
		self assert: (persons includes: p).
		persons remove: p].
	self assert: persons isEmpty.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:44'!
testElementPrefixes
	| document  persons |

	document := XMLDOMParser parse: self addressBookNamespacePrefixedXML.
	persons := 	document allElementsSelect: [:each |
		(each isNamedAny: #('notperson' 'ps:person'))
			and: [(each isNamedAny: #('notperson' 'person'))
				and: [(each isNamed: 'person')
					and: [(each isNamed: 'ps:person')
						and: [(each name = 'ps:person') and: [each localName = 'person']]]]]].
	self assert: persons size = 4.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:44'!
testElementPrefixesNamespacesDisabled
	| document  persons |

	document :=
		XMLDOMParser
			parse: self addressBookNamespacePrefixedXML
			usingNamespaces: false.
	persons := 	document allElementsSelect: [:each |
		(each isNamedAny: #('notperson' 'ps:person'))
			and: [(each isNamedAny: #('notperson' 'person'))
				and: [(each isNamed: 'person')
					and: [(each isNamed: 'ps:person')
						and: [(each name = 'ps:person') and: [each localName = 'person']]]]]].
	self assert: persons size = 4.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testFindElementNamespaceURI
	| document people |

	document := XMLDOMParser parse: self addressBookNamespacedXML.

	people := document allElementsNamed: 'person' namespaceURI: 'defaultNS'.
	self assert:
		(document
			findElementNamed: 'person'
			namespaceURI: 'defaultNS'
			with: [:each | (each attributeAt: 'employee-number') = 'A7987']) == people last.
	self assert:
		(document
			findElementNamed: 'person'
			namespaceURI: 'defaultNS'
			with: [:each | (each attributeAt: 'family-name') = 'Brown']) == people second.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/3/2015 01:09'!
testIgnorableWhitespacePreservation
	| xml document elements |

	xml :=
		'<one>
			<two>
					<three>
					test
					</three>
			</two>
		</one>'.
	document :=
		(XMLDOMParser on: xml)
			preservesIgnorableWhitespace: true;
			parseDocument.
	elements := document allElements.
	self
		assert: document allNodes size = 9;
		assert: elements size = 3.
	"avoid allButLastDo: for portability"
	elements allButLast do: [:each |
		self
			assert: each nodes size = 3;
			assert: (each firstNode isStringNode and: [each firstNode isWhitespace]);
			assert: (each lastNode isStringNode and: [each lastNode isWhitespace])].
	self
		assert: elements last nodes size = 1;
		assert: elements last firstNode isStringNode;
		deny: elements last firstNode isWhitespace.

	document :=
		(XMLDOMParser on: xml)
			preservesIgnorableWhitespace: false;
			parseDocument.
	elements := document allElements.
	self
		assert: document allNodes size = 5;
		assert: elements size = 3.
	elements do: [:each |
		self
			assert: each nodes size = 1;
			deny: (each firstNode isStringNode and: [each firstNode isWhitespace])].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/13/2014 19:32'!
testInnerXML
	| document |

	document := XMLDOMParser parse: self addressBookXML.
	document allElementsNamed: 'person' do: [:person | | contactInfo address1 address2 |
		#('address' 'job-info' 'manager') do: [:each |
			self assert: (person elementAt: each) innerXML isEmpty].

		contactInfo := person elementAt: 'contact-info'.
		address1 := (person attributeAt: 'first-name'), '@example.com'.
		address2 := (person attributeAt: 'first-name'), '@users.example.com'.
		self
			assert: (contactInfo innerXML: '<email></email><email2/>') innerXML = '<email /><email2 />';
			assert: contactInfo elements size = 2;
			assert: (contactInfo elements first isNamed: 'email');
			assert: (contactInfo elements second isNamed: 'email2');
			assert: (contactInfo firstElement innerXML: address1, '<!!--ignored-->') innerXML = address1;
			assert: (contactInfo elements second innerXML: '<!![CDATA[', address2, ']]>') innerXML = address2.
		self assert:
			contactInfo innerXML = 
				('<email>', address1, '</email><email2>', address2, '</email2>')].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 5/9/2015 23:34'!
testInnerXMLDocument
	| document |

	document := XMLDOMParser parse: self addressBookXMLWithDTD.
	self
		assert: ((document innerXML: '<new-root/>') root isNamed: 'new-root');
		assert: document innerXML = '<new-root />';
		assert: ((document root innerXML: 'test') contentString = 'test');
		assert: document root innerXML = 'test'.
	
	document := XMLDOMParser parse: self addressBookXMLWithDTD.
	self assert:
		(document innerXML: document innerXML) printString =
			(XMLDOMParser parse: self addressBookXMLWithDTD) printString.
	self
		should: [document innerXML: self addressBookXMLWithDTD]
		raise: XMLWellFormednessException.

	document := XMLDOMParser parse: '<root/>'.
	self
		should: [document innerXML: '<root1/><root2/>']
		raise: XMLWellFormednessException.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 5/10/2015 06:33'!
testInnerXMLEntities
	| document |

	document :=
		XMLDOMParser parse:
			'<!!DOCTYPE test [
					<!!ENTITY one "one">
					<!!ENTITY two "two">
				]>
			<test name="&one;">&one;</test>'.
	self
		assert: (document root attributeAt: 'name') = 'one';
		assert: document root contentString = 'one'.

	document innerXML: '<test name="&two;">&two;</test>'.
	self
		assert: (document root attributeAt: 'name') = 'two';
		assert: document root contentString = 'two'.

	self
		should: [document innerXML: '<test name="&three;"></test>']
		raise: XMLValidationException.
	self
		should: [document innerXML: '<test name="">&four;</test>']
		raise: XMLValidationException.! !

!XMLDOMParserTest methodsFor: 'tests - internal subset' stamp: 'monty 12/8/2014 11:05'!
testInternalAttributeDeclarations
	| doctype firstAttribute secondAttribute |

	doctype :=
		(XMLDOMParser
			parse: self internalSubsetWithDeclarations
			usingNamespaces: true
			validation: true
			externalEntities: false) doctypeDeclaration.

	firstAttribute := doctype nodes second.
	self
		assert: firstAttribute element = 'root';
		assert: firstAttribute attribute = 'name';
		assert: firstAttribute type = 'CDATA';
		assert: firstAttribute defaultPragma = 'IMPLIED';
		assert: firstAttribute defaultValue isEmpty.

	secondAttribute := doctype nodes third.
	self
		assert: secondAttribute element = 'root';
		assert: secondAttribute attribute = 'number';
		assert: secondAttribute type = '(one|two|three)';
		assert: secondAttribute defaultPragma isEmpty;
		assert: secondAttribute defaultValue = 'three'.! !

!XMLDOMParserTest methodsFor: 'tests - internal subset' stamp: 'monty 12/22/2013 06:22'!
testInternalElementDeclaration
	| doctype elementDeclaration |

	doctype :=
		(XMLDOMParser
			parse: self internalSubsetWithDeclarations
			usingNamespaces: true
			validation: true
			externalEntities: false) doctypeDeclaration.
	elementDeclaration := doctype nodes first.
	self
		assert: elementDeclaration element = 'root';
		assert: elementDeclaration contentModel = '(#PCDATA|one|two)*'.! !

!XMLDOMParserTest methodsFor: 'tests - internal subset' stamp: 'monty 12/21/2013 06:10'!
testInternalGeneralEntityDeclarations
	| doctype firstEntity secondEntity |

	doctype :=
		(XMLDOMParser
			parse: self internalSubsetWithDeclarations
			usingNamespaces: true
			validation: true
			externalEntities: false) doctypeDeclaration.

	firstEntity := doctype nodes fourth.
	self
		deny: firstEntity isParameter;
		assert: firstEntity name = 'general';
		assert: firstEntity replacement = 'replacement';
		assert: firstEntity publicID isEmpty;
		assert: firstEntity systemID isEmpty;
		assert: firstEntity ndata isEmpty.

	secondEntity := doctype nodes fifth.
	self
		deny: secondEntity isParameter;
		assert: secondEntity name = 'externalGeneral';
		assert: secondEntity replacement isEmpty;
		assert: secondEntity publicID = 'pubid2';
		assert: secondEntity systemID = 'external1.dtd';
		assert: secondEntity ndata isEmpty.! !

!XMLDOMParserTest methodsFor: 'tests - internal subset' stamp: 'monty 12/21/2013 06:09'!
testInternalNotationDeclarations
	| doctype notation |

	doctype :=
		(XMLDOMParser
			parse: self internalSubsetWithDeclarations
			usingNamespaces: true
			validation: true
			externalEntities: false) doctypeDeclaration.

	notation := doctype nodes eighth.
	self
		assert: notation name = 'notation';
		assert: notation publicID isEmpty;
		assert: notation systemID = 'note.doc'.! !

!XMLDOMParserTest methodsFor: 'tests - internal subset' stamp: 'monty 12/21/2013 06:09'!
testInternalParameterEntityDeclarations
	| doctype firstEntity secondEntity |

	doctype :=
		(XMLDOMParser
			parse: self internalSubsetWithDeclarations
			usingNamespaces: true
			validation: true
			externalEntities: false) doctypeDeclaration.

	firstEntity := doctype nodes sixth.
	self
		assert: firstEntity isParameter;
		assert: firstEntity name = 'parameter';
		assert: firstEntity replacement = 'replacement';
		assert: firstEntity publicID isEmpty;
		assert: firstEntity systemID isEmpty.

	secondEntity := doctype nodes seventh.
	self
		assert: secondEntity isParameter;
		assert: secondEntity name = 'externalParameter';
		assert: secondEntity replacement isEmpty;
		assert: secondEntity publicID isEmpty;
		assert: secondEntity systemID = 'external2.dtd'.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:44'!
testNamespaces
	| document  |

	document := XMLDOMParser parse: self addressBookNamespacePrefixedXML.
	document allElementsDo: [:each |
		(each == document root or: [each isNamed: 'person'])
			ifTrue: [
				self assert: each hasPrefix.
				each == document root
					ifTrue: [
						self
							assert: each prefix = 'book';
							assert: each namespaceURI = 'bookNS']
					ifFalse: [
						self
							assert: each prefix = 'ps';
							assert: each namespaceURI = 'personNS']]
			ifFalse: [
				self
					assert: each namespaceURI = 'defaultNS';
					deny: each hasPrefix;
					assert: each prefix isEmpty]].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/28/2014 23:42'!
testNamespacesCopyOnWrite
	| xml elements |

	xml := 
		'<one xmlns:prefix1="ns" xmlns="defualt">
			<two xmlns:prefix2="ns2"><three /></two>
			<four><five xmlns:prefix1="ns3" xmlns="defualt2" /></four>
		</one>'.
	elements := (XMLDOMParser parse: xml) allElements.
	self assert:
		(IdentitySet withAll:
			(elements collect: [:each |
				each namespaceScope prefixMappings])) size = 3.

	elements := (XMLDOMParser parse: xml usingNamespaces: false) allElements.
	self assert:
		(IdentitySet withAll:
			(elements collect: [:each |
				each namespaceScope prefixMappings])) size = elements size.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testNamespacesDeclaring
	| document |

	document := XMLDOMParser parse: self addressBookNamespacedXML.
	document allElementsDo: [:each |
		self assert: each hasNamespaces.
		each == document root
			ifTrue: [
				self
					assert: each declaresDefaultNamespace;
					assert: (each declaresPrefix: 'ps' uri: 'personNS')]
			ifFalse: [ 
				self
					deny: each declaresDefaultNamespace;
					deny: (each declaresPrefix: 'ps' uri: 'personNS')].
		self
			deny: (each declaresPrefix: 'undeclared' uri: 'undeclaredNS');
			deny: (each isDeclaredPrefix: 'undeclared');
			deny: (each isDeclaredPrefix: 'undeclared' uri: 'undeclaredNS');
			assert: (each isDeclaredPrefix: 'ps');
			assert: (each isDeclaredPrefix: 'ps' uri: 'personNS')].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testNamespacesDeclaringDisabled
	| document |

	document := XMLDOMParser parse: self addressBookNamespacedXML usingNamespaces: false.
	document allElementsDo: [:each |
		self
			deny: each hasNamespaces;
			deny: each declaresDefaultNamespace;
			deny: (each declaresPrefix: 'ps' uri: 'personNS');
			deny: (each isDeclaredPrefix: 'ps');
			deny: (each isDeclaredPrefix: 'ps' uri: 'personNS')].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:44'!
testNamespacesDisabled
	| document |

	document :=
		XMLDOMParser
			parse: self addressBookNamespacePrefixedXML
			usingNamespaces: false.
	document allElementsDo: [:each |
		self assert: each namespaceURI isEmpty.
		(each == document root or: [each isNamed: 'person'])
			ifTrue: [
				self
					assert: each hasPrefix;
					assert:
						each prefix =
							((each == document root) ifTrue: ['book'] ifFalse: ['ps'])]
			ifFalse: [
				self
					deny: each hasPrefix;
					assert: each prefix isEmpty]].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testNamespacesResolving
	| document |

	document := XMLDOMParser parse: self addressBookNamespacedXML.
	document allElementsDo: [:each |
		self
			assert: each defaultNamespace = 'defaultNS';
			assert: (each resolvePrefix: '') = 'defaultNS';
			assert: (each resolvePrefix: 'book') = 'bookNS';
			assert: (each resolvePrefix: 'ps') = 'personNS';
			assert:
				each namespaceURI =
					(each hasPrefix
						ifTrue: [each resolvePrefix: each prefix]
						ifFalse: [each defaultNamespace])]! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/25/2014 04:46'!
testNamespacesResolvingDisabled
	| document |

	document :=
		XMLDOMParser
			parse: self addressBookNamespacedXML
			usingNamespaces: false.
	document allElementsDo: [:each |
		self
			assert: each defaultNamespace isEmpty;
			assert: (each resolvePrefix: '') isEmpty;
			assert: (each resolvePrefix: 'book') isEmpty;
			assert: (each resolvePrefix: 'ps') isEmpty;
			assert:
				each namespaceURI =
					(each hasPrefix
						ifTrue: [each resolvePrefix: each prefix]
						ifFalse: [each defaultNamespace])]! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/21/2013 06:21'!
testNodeDetecting
	| document people |

	document := XMLDOMParser parse: self addressBookXML.
	people := document root elementsAt: 'person'.
	self assert:
		((document findElementNamed: 'addressbook') nodesDetect: [:each |
			each isElement and: [(each attributeAt: 'family-name') = 'Brown']]) == people second.
	self assert:
		((document findElementNamed: 'addressbook')
			nodesDetect: [:each | each isElement and: [(each attributeAt: 'unknown') = 'Brown']]
			ifNone: ['not found']) = 'not found'.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 2/27/2016 12:41'!
testNodeListClass
	| document parser listClass |

	listClass := XMLCachingNodeList.
	parser := XMLDOMParser on: self addressBookXML.
	parser configuration nodeListClass: listClass.
	document := parser parseDocument.

	self
		assert: parser configuration nodeListClass == listClass;
		assert: document nodes class == listClass;
		assert: (document allElements allSatisfy: [:each | each nodes class == listClass]).! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 10/13/2014 01:37'!
testOverridingDefaultNamespace
	| document |

	document := XMLDOMParser parse: '<outer xmlns="default"><inner xmlns=""/></outer>'.
	self
		assert: document root namespaceURI = 'default';
		assert: document root defaultNamespace = 'default';
		assert: document root firstNode namespaceURI isEmpty;
		assert: document root firstNode defaultNamespace isEmpty.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 11/26/2014 21:28'!
testParseDocumentUntil
	| i document |

	self
		shouldnt: [
			i := 0.
			document :=
				(XMLDOMParser on: '<root><one>two<#malformed')
					parseDocumentUntil: [(i := i + 1) > 4]]
		raise: XMLWellFormednessException.
	self
		assert: document allNodes size = 4;
		assert: (document findElementNamed: 'one') nodes size = 1;
		assert: (document findElementNamed: 'one') contentString = 'two'.
	self
		should: [
			i := 0.
			(XMLDOMParser on: '<root><one>two<#malformed')
				parseDocumentUntil: [(i := i + 1) > 5]]
		raise: XMLWellFormednessException.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 10/14/2014 23:11'!
testParseDocumentWhile
	| i document |

	self
		shouldnt: [
			i := 0.
			document :=
				(XMLDOMParser on: '<root><one>two<#malformed')
					parseDocumentWhile: [(i := i + 1) <= 4]]
		raise: XMLWellFormednessException.
	self
		assert: document allNodes size = 4;
		assert: (document findElementNamed: 'one') nodes size = 1;
		assert: (document findElementNamed: 'one') contentString = 'two'.
	self
		should: [
			i := 0.
			(XMLDOMParser on: '<root><one>two<#malformed')
				parseDocumentWhile: [(i := i + 1) <= 5]]
		raise: XMLWellFormednessException.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 10/29/2014 13:54'!
testParsedElementNames
	| document |

	document := XMLDOMParser parse: self addressBookNamespacePrefixedXML.
	self assert:
		document elementNames =
			(document elements collect: [:each | each name ]).

	document allElementsDo: [:element |
		self assert:
			element elementNames =
				(element elements collect: [:each | each name])].! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/12/2013 00:41'!
testParsingAttributes
	| document firstPerson  |

	document := XMLDOMParser parse: self addressBookXML.
	firstPerson := document findElementNamed: 'person'.
	self
		assert: (firstPerson attributeAt: 'employee-number') = 'A0000';
		assert: (firstPerson attributeAt: 'family-name') = 'Gates';
		assert: (firstPerson attributeAt: 'first-name') = 'Bob';
		assert:
			firstPerson attributeNames asArray =
				#('employee-number' 'family-name' 'first-name');
		assert: firstPerson attributeAssociations asArray =
			{('employee-number'->'A0000'). ('family-name'->'Gates'). ('first-name'->'Bob').}.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 12/12/2013 00:42'!
testParsingRoot
	| document root  |

	document := XMLDOMParser parse: self addressBookXML.
	self assert: (document isKindOf: XMLDocument).
	root := document root.
	self assert: (root class == XMLElement).

	self assert:
		(root allNodesDetect: [:each |
			each isElement
				and: [each isNamedAny: #('AddressBook' 'addressbook')]]) == root.
	self assert: (root findElementNamed: 'addressbook') == root.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 3/22/2015 17:36'!
testPreservingCommentNodes
	| document comment|

	document := XMLDOMParser parse: self addressBookXML.
	self deny: (document findElementNamed: 'contact-info') hasChildren.

	document :=
		(XMLDOMParser on: self addressBookXML)
			preservesCommentNodes: true;
			parseDocument.
	comment := (document findElementNamed: 'contact-info') firstNode.
	self
		assert: comment notNil;
		assert: comment string = 'Confidential'.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 1/6/2015 05:00'!
testUnreplacedEntityRererencePrinting
	| xml |

	xml :=
		'<one two="&quot;&three;&quot;">',
		'&lt;four five=&#34;&amp;six;&#34;/&gt;',
		'</one>'.
	self assert:
		((XMLDOMParser on: xml)
			isValidating: false;
			replacesContentEntityReferences: false;
			parseDocument) printString = xml.! !

!XMLDOMParserTest methodsFor: 'tests' stamp: 'monty 3/9/2016 20:49'!
testXMLDeclaration
	| document |

	document :=
		XMLDOMParser parse:
			'<?xml version="1.0" encoding="UTF-8"?><test/>'.
	self
		assert: document version = 1.0;
		assert: document encoding = 'UTF-8';
		deny: document isStandalone.

	document :=
		XMLDOMParser parse:
			'<?xml version="1.1" encoding="utf-8" standalone="yes"?><test/>'.
	self
		assert: document version = 1.1;
		assert: document encoding = 'utf-8';
		assert: document isStandalone.! !

!XMLParserTest class methodsFor: 'source' stamp: 'monty 1/12/2014 23:36'!
addressBookXML
	^'<addressbook>
  <person employee-number="A0000" family-name="Gates" first-name="Bob">
    <contact-info><!!--Confidential--></contact-info>
    <address city="Los Angeles" number="1239" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Manager"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7000" family-name="Brown"
    first-name="Robert" middle-initial="L.">
    <contact-info>
      <email address="robb@iro.ibm.com"/>
      <home-phone number="03-3987873"/>
    </contact-info>
    <address city="New York" number="344" state="NY" street="118 St."/>
    <job-info employee-type="Full-Time" is-manager="yes" job-description="Group Leader"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7890" family-name="DePaiva"
    first-name="Kassie" middle-initial="W.">
    <contact-info><!!-- Kassie''s agent phone: 03-987654 --></contact-info>
    <address city="Los Angeles" number="1234" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Actor"/>
    <manager employee-number="A0000"/>
    <misc-info>One of the most talented actresses on Daytime. Kassie
      plays the devious and beautiful Blair Cramer on ABC&apos;s
      &quot;One Life To Live.&quot;</misc-info>
  </person>
  <person employee-number="A7987" family-name="Smith" first-name="Joe">
    <contact-info>
      <email address="joes@iro.ibm.com"/>
      <mobile-phone number="888-7657765"/>
      <home-phone number="03-8767898"/>
      <home-phone number="03-8767871"/>
    </contact-info>
    <address city="New York" number="12789" state="NY" street="W. 15th Ave."/>
    <job-info employee-type="Part-Time" is-manager="no" job-description="Hacker"/>
    <manager employee-number="A7000"/>
  </person>
</addressbook>
'! !

!XMLParserTest class methodsFor: 'source' stamp: 'monty 1/12/2014 23:36'!
addressBookXMLWithDTD
	^'<?xml version="1.0" encoding="UTF-8"?>
<!!DOCTYPE addressbook SYSTEM "addressbook.dtd">
<?xml-stylesheet type="text/xsl" href="demo.xsl"?>
<addressbook>
  <person employee-number="A0000" family-name="Gates" first-name="Bob">
    <contact-info><!!--Confidential--></contact-info>
    <address city="Los Angeles" number="1239" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Manager"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7000" family-name="Brown"
    first-name="Robert" middle-initial="L.">
    <contact-info>
      <email address="robb@iro.ibm.com"/>
      <home-phone number="03-3987873"/>
    </contact-info>
    <address city="New York" number="344" state="NY" street="118 St."/>
    <job-info employee-type="Full-Time" is-manager="yes" job-description="Group Leader"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7890" family-name="DePaiva"
    first-name="Kassie" middle-initial="W.">
    <contact-info><!!-- Kassie''s agent phone: 03-987654 --></contact-info>
    <address city="Los Angeles" number="1234" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Actor"/>
    <manager employee-number="A0000"/>
    <misc-info>One of the most talented actresses on Daytime. Kassie
      plays the devious and beautiful Blair Cramer on ABC&apos;s
      &quot;One Life To Live.&quot;</misc-info>
  </person>
  <person employee-number="A7987" family-name="Smith" first-name="Joe">
    <contact-info>
      <email address="joes@iro.ibm.com"/>
      <mobile-phone number="888-7657765"/>
      <home-phone number="03-8767898"/>
      <home-phone number="03-8767871"/>
    </contact-info>
    <address city="New York" number="12789" state="NY" street="W. 15th Ave."/>
    <job-info employee-type="Part-Time" is-manager="no" job-description="Hacker"/>
    <manager employee-number="A7000"/>
  </person>
</addressbook>
'! !

!XMLParserTest methodsFor: 'source' stamp: 'monty 1/12/2014 23:36'!
addressBookXML
	^ self class addressBookXML! !

!XMLParserTest methodsFor: 'source' stamp: 'monty 1/12/2014 23:37'!
addressBookXMLWithDTD
	^ self class addressBookXMLWithDTD! !

!XMLParserTest methodsFor: 'benchmark' stamp: 'JAA 3/24/2011 18:26'!
benchmark1
	"
	XMLParserTest new benchmark1
	600 timesRepeat => 2131 (Alexandre Bergel 3/29/2010 09:09)
	"
	Smalltalk garbageCollect.
	^ [4000 timesRepeat: [SAXHandler parse: self addressBookXML]] timeToRun! !

!XMLParserTest methodsFor: 'benchmark' stamp: 'JAA 3/24/2011 18:26'!
benchmark2
	"
	XMLParserTest new benchmark2
	"
	Smalltalk garbageCollect.
	^ [4000 timesRepeat: [XMLDOMParser parse: self addressBookXML]] timeToRun! !

!XMLParserTest methodsFor: 'benchmark' stamp: 'monty 2/6/2014 19:17'!
benchmark3
	"
	XMLParserTest new benchmark3
	"
	Smalltalk garbageCollect.
	^ [4000 timesRepeat: [XMLDOMParser parse: self addressBookXMLWithDTD]] timeToRun! !

!XMLParserTest methodsFor: 'benchmark' stamp: 'monty 11/16/2013 07:34'!
benchmark4
	| doc |
	"
	XMLParserTest new benchmark4
	"
	doc := XMLDOMParser
		parse: self addressBookXML readStream
		usingNamespaces: false.
	^ [500000 timesRepeat: [
		doc root firstElement
			elementAt: 'contact-info';
			elementAt: 'address';
			elementAt: 'job-info';
			elementAt: 'manager']] timeToRun.! !

!XMLParserTest methodsFor: 'source' stamp: 'monty 12/19/2013 02:04'!
elementDeclarationsNamed: aNameCollection
	| stream |

	stream := String new writeStream.
	aNameCollection do: [:each |
		stream
			nextPutAll: '<!!ELEMENT ';
			nextPutAll: each;
			nextPutAll: ' EMPTY>';
			cr].
	^ stream contents.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:54'!
testAttributeDeclarationPrefixed
	| dtd |

	dtd := '<!!DOCTYPE root [
		<!!ELEMENT root EMPTY>
		<!!ATTLIST root
			prefix:name CDATA #IMPLIED
			unprefixed CDATA #IMPLIED>]>'.
	self
		shouldnt: [
			SAXHandler parse:
				dtd, '<root xmlns:prefix="ns" prefix:name="value" unprefixed="value"/>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root name="value" unprefixed="value" />']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				dtd, '<root xmlns:prefix="ns" prefix:name="value" prefix:unprefixed="value"/>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 20:23'!
testAttributeDeclarationWhitespace
	#('<!!DOCTYPE test [
			<!!ELEMENT test ANY>
			<!!ATTLISTtest name (value) #FIXED "value">]>'
		'<!!DOCTYPE test [
			<!!ELEMENT test ANY>
			<!!ATTLIST test name(value) #FIXED "value">]>'
		'<!!DOCTYPE test [
			<!!ELEMENT test ANY>
			<!!ATTLIST test name (value)#FIXED "value">]>'
		'<!!DOCTYPE test [
			<!!ELEMENT test ANY>
			<!!ATTLIST test name (value) #FIXED"value">]>'
		'<!!DOCTYPE test [
			<!!ELEMENT test ANY>
			<!!ATTLIST test name (value) #FIXED "value"name2
			(value) #FIXED "value">]>')
		do: [:each |
			self
				should: [SAXHandler parse: each, '<test/>']
				raise: XMLWellFormednessException].
	#('<!!DOCTYPE test [
			<!!ELEMENT test ANY>
			<!!ATTLIST   test   name  ( value )  #FIXED  "value">]>'
		'<!!DOCTYPE test [
			<!!ELEMENT test ANY>
			<!!ATTLIST test
				name (value) #FIXED "value"
				name2 (value) #FIXED "value">]>'
		'<!!DOCTYPE test [
			<!!ELEMENT test ANY>
			<!!ATTLIST   test   >]>')
		do: [:each |
			self
				shouldnt: [SAXHandler parse: each, '<test/>']
				raise: XMLWellFormednessException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 2/20/2016 08:56'!
testAttributeDefaultFixedValue
	| dtd document root |

	dtd := '
		<!!DOCTYPE root [
			<!!ELEMENT root EMPTY>
			<!!ATTLIST root
				one CDATA #FIXED "fixedDefault"
				one CDATA #FIXED "redundant"
				two CDATA #FIXED "fixedDefaultTwo"
				three CDATA #FIXED "" >
		]>'.

	document :=
		XMLDOMParser parse:
			dtd, '<root one=''fixedDefault'' two="fixedDefaultTwo" three=""/>'.
	root := document root.
	self
		assert: (root attributeAt: 'one') = 'fixedDefault';
		assert: (root attributeAt: 'two') = 'fixedDefaultTwo';
		assert: (root includesAttribute: 'three');
		assert: (root attributeAt: 'three') isEmpty.

	"attributes with fixed defaults are set automatically if not present"
	document := XMLDOMParser parse: dtd, '<root />'.
	root := document root.
	self
		assert: (root attributeAt: 'one') = 'fixedDefault';
		assert: (root attributeAt: 'two') = 'fixedDefaultTwo';
		assert: (root includesAttribute: 'three');
		assert: (root attributeAt: 'three') isEmpty.

	"but if present cannot have non-default values"
	#('<root one="nonDefault" />'
		'<root two="nonDefault"/>'
		'<root three="nonDefault"/>') do: [:each |
		self
			should: [SAXHandler parse: dtd, each]
			raise: XMLValidationException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 2/20/2016 08:53'!
testAttributeDefaultImplied
	| dtd document root |

	dtd := '
		<!!DOCTYPE root [
			<!!ELEMENT root EMPTY>
			<!!ATTLIST root
				one CDATA #IMPLIED
				one CDATA #REQUIRED
				two CDATA #IMPLIED >
		]>'.

	document :=
		XMLDOMParser parse:
			dtd, '<root one=''nonDefault'' two="nonDefaultTwo" />'.
	root := document root.
	self
		assert: (root attributeAt: 'one') = 'nonDefault';
		assert: (root attributeAt: 'two') = 'nonDefaultTwo'.

	document := XMLDOMParser parse: dtd, '<root one='''' two="" />'.
	root := document root.
	self
		assert: (root includesAttribute: 'one');
		assert: (root attributeAt: 'one') isEmpty;
		assert: (root includesAttribute: 'two');
		assert: (root attributeAt: 'two') isEmpty.

	"implied makes them optional"
	document := XMLDOMParser parse: dtd, '<root />'.
	root := document root.
	self
		deny: (root includesAttribute: 'one');
		deny: (root includesAttribute: 'two').! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 2/20/2016 09:02'!
testAttributeDefaultRequired
	| dtd document root |

	dtd := '
		<!!DOCTYPE root [
			<!!ELEMENT root EMPTY>
			<!!ATTLIST root
				one CDATA #REQUIRED
				one CDATA #IMPLIED
				two CDATA #REQUIRED >
		]>'.

	document :=
		XMLDOMParser parse:
			dtd, '<root one=''nonDefault'' two="nonDefaultTwo" />'.
	root := document root.
	self
		assert: (root attributeAt: 'one') = 'nonDefault';
		assert: (root attributeAt: 'two') = 'nonDefaultTwo'.

	"empty values still count as being present"
	document := XMLDOMParser parse: dtd, '<root one='''' two="" />'.
	root := document root.
	self
		assert: (root includesAttribute: 'one');
		assert: (root attributeAt: 'one') isEmpty;
		assert: (root includesAttribute: 'two');
		assert: (root attributeAt: 'two') isEmpty.

	"all required attributes must be present"
	#('<root one="nonDefault" />'
		'<root two="nonDefault" />') do: [:each |
		self
			should: [SAXHandler parse: dtd, each]
			raise: XMLValidationException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 2/20/2016 08:56'!
testAttributeDefaultValue
	| dtd document root |

	dtd := '
		<!!DOCTYPE root [
			<!!ELEMENT root EMPTY>
			<!!ATTLIST root
				one CDATA "default"
				one CDATA "redundant"
				two CDATA "" >
		]>'.

	document :=
		XMLDOMParser parse:
			dtd, '<root one=''nonDefault'' two="nonDefaultTwo" />'.
	root := document root.
	self
		assert: (root attributeAt: 'one') = 'nonDefault';
		assert: (root attributeAt: 'two') = 'nonDefaultTwo'.

	document := XMLDOMParser parse: dtd, '<root one='''' two="" />'.
	root := document root.
	self
		assert: (root includesAttribute: 'one');
		assert: (root attributeAt: 'one') isEmpty;
		assert: (root includesAttribute: 'two');
		assert: (root attributeAt: 'two') isEmpty.

	"attributes with defaults are set automatically if not present"
	document := XMLDOMParser parse: dtd, '<root />'.
	root := document root.
	self
		assert: (root attributeAt: 'one') = 'default';
		assert: (root includesAttribute: 'two');
		assert: (root attributeAt: 'two') isEmpty.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/3/2015 02:58'!
testAttributeEntityDeclaration
	| dtd |

	dtd := '<!!DOCTYPE root [
		<!!NOTATION first SYSTEM "first.txt">
		<!!NOTATION second SYSTEM "second.txt">
		<!!ENTITY firstEntity SYSTEM "first.txt" NDATA first>
		<!!ENTITY secondEntity SYSTEM "second.txt" NDATA second>
		<!!ELEMENT root EMPTY>
		<!!ATTLIST root
			unparsedEntity ENTITY #IMPLIED
			unparsedEntities ENTITIES #IMPLIED>]>'.
	self
		shouldnt: [
			SAXHandler parse:
				dtd, '<root unparsedEntity="firstEntity"  unparsedEntities="firstEntity  secondEntity"/> ']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root unparsedEntity="thirdEntity"/>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root unparsedEntity="secondEntity thirdEntity"/>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/18/2013 19:18'!
testAttributeEnumerationDeclaration
	| dtd |

	dtd := '<!!DOCTYPE root [
		<!!NOTATION txtFile PUBLIC "//file.txt//">
		<!!NOTATION confFile PUBLIC "//file.conf//">
		<!!ELEMENT root EMPTY>
		<!!ATTLIST root
			number ( one | two | three) #IMPLIED
			note NOTATION (txtFile | confFile ) #IMPLIED>]>'.
	self
		shouldnt: [SAXHandler parse: dtd, '<root number="two" note="txtFile"/>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root number="four"/>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root note="htmlFile" />']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/18/2013 19:19'!
testAttributeIDDeclaration
	| dtd |

	dtd := '<!!DOCTYPE root [
		<!!ELEMENT root (unique|unique2)+>
		<!!ELEMENT unique EMPTY>
		<!!ELEMENT unique2 EMPTY>
		<!!ATTLIST unique id ID #REQUIRED>
		<!!ATTLIST unique2 id ID #REQUIRED>]>'.
	self
		shouldnt: [
			SAXHandler parse:
				dtd, '<root><unique id=" one "/><unique id="two"/><unique2 id="three"/></root> ']
		raise:XMLValidationException.
	self
		should: [
			SAXHandler parse:
				dtd, '<root><unique id="one"/><unique id=" one "/></root> ']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [
					<!!ATTLIST unique
						id ID #REQUIRED
						id2 ID #REQUIRED>]>
					<root/>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 22:36'!
testAttributeIDRefDeclaration
	| dtd |

	dtd := '<!!DOCTYPE root [
		<!!ELEMENT root (unique|ref|refs)+>
		<!!ELEMENT unique EMPTY>
		<!!ELEMENT ref EMPTY>
		<!!ELEMENT refs EMPTY>
		<!!ATTLIST unique id ID #REQUIRED>
		<!!ATTLIST ref idref IDREF #REQUIRED>
		<!!ATTLIST refs idrefs IDREFS #REQUIRED>]>'.
	SAXHandler parse:
		dtd, '<root>
		<unique id=" one "/><unique id=" two "/>
		<ref idref="one"/><refs idrefs="  one  two"/>
		</root> '.
	self
		should: [
			SAXHandler parse:
				dtd, '<root><unique id="one"/><ref idref="two"/></root> ']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				dtd, '<root><unique id="one"/><refs idrefs="one two"/></root> ']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/29/2013 00:58'!
testAttributeNmtokenDeclaration
	| dtd |

	dtd := '<!!DOCTYPE root [
		<!!ELEMENT root ANY>
		<!!ATTLIST root name NMTOKEN #IMPLIED>
		<!!ATTLIST root names NMTOKENS #IMPLIED>]>'.
	self
		shouldnt: [SAXHandler parse: dtd, '<root name=" one " names="  one  two three "/> ']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root name="#" names="one two"/>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root name="one" names="# #" />']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/10/2016 14:50'!
testAttributeRedundance
	#('<test name="value" newName="newValue" name="value" />'
		'<test name="value" newName="newValue" name="newValue" />'
		'<test xml:lang="en" name="value" xml:lang="de" />')
		do: [:each |
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/10/2016 15:00'!
testAttributeReserved
	| xml document |

	xml := '<a xml="one" xmll="two" xml:="three" xmlnss="four"></a>'.
	document := XMLDOMParser parse: xml usingNamespaces: true.

	self
		assert: (document root attributeAt: 'xml') = 'one';
		assert: (document root attributeAt: 'xmll') = 'two';
		assert: (document root attributeAt: 'xml:') = 'three';
		assert: (document root attributeAt: 'xmlnss') = 'four'.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/9/2016 20:56'!
testAttributeUndeclared
	self
		shouldnt: [
			SAXHandler
				parse: '<!!DOCTYPE root [<!!ENTITY ignored "ignored">]><root name="value"/>'
				usingNamespaces:true
				validation: true]
		raise: XMLValidationException.
	self
		shouldnt: [
			SAXHandler parse:
				'<!!DOCTYPE root [
					<!!ELEMENT root ANY>
					<!!ATTLIST root name CDATA #IMPLIED>]><root name="value"></root>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [<!!ELEMENT root ANY>]><root name="value"></root>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [
					<!!ELEMENT root ANY>
					<!!ATTLIST root name CDATA #IMPLIED>]><root newName="value"></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 11/19/2013 05:55'!
testAttributeValueNormalization
	| separators unnormalized normalized document |

	separators :=
		String
			with: Character space
			with: Character tab
			with: Character lf.
	unnormalized := separators, 'nonspace', separators.
	normalized := '   nonspace   '.
	document :=
		(XMLDOMParser parse:
			'<root name="', unnormalized,'">', unnormalized, '</root>').
	self
		assert: (document root attributeAt: 'name') = normalized;
		assert: (document root contentString) = unnormalized.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 4/22/2015 15:11'!
testAttributeValueWithAngleBrackets
	self
		should: [SAXHandler parse: '<test name="<test name=''value''/>"/>']
		raise: XMLWellFormednessException.
	self
		shouldnt: [SAXHandler parse: '<test name="&lt;test name=''value''/>"/>']
		raise: XMLWellFormednessException.
	self
		shouldnt: [SAXHandler parse: '<test name=''&lt;test name="value"/>''/>']
		raise: XMLWellFormednessException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:31'!
testAttributeXMLIDDTDValidation
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE one [
					<!!ELEMENT one ANY>
					<!!ATTLIST one xml:id CDATA #IMPLIED>]>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE one [
					<!!ELEMENT one ANY>
					<!!ATTLIST one xml:id ID #IMPLIED>]>
				<one xml:id="invalid:id"/>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE one [
					<!!ELEMENT one ANY>
					<!!ELEMENT two ANY>
					<!!ATTLIST one xml:id ID #IMPLIED>
					<!!ATTLIST two xml:id ID #IMPLIED>]>
				<one xml:id="valid"><two xml:id="valid"/></one>']
		raise: XMLValidationException.
	self
		shouldnt: [
			SAXHandler parse:
				'<!!DOCTYPE one [
					<!!ELEMENT one ANY>
					<!!ELEMENT two ANY>
					<!!ATTLIST one xml:id ID #IMPLIED>
					<!!ATTLIST two xml:id ID #IMPLIED>]>
				<one xml:id="valid1"><two xml:id="valid2"/></one>']
		raise: XMLValidationException.
	self
		shouldnt: [
			SAXHandler
				parse:
					'<!!DOCTYPE one [
						<!!ELEMENT one ANY>
						<!!ELEMENT two ANY>
						<!!ATTLIST one xml:id CDATA #IMPLIED>
						<!!ATTLIST two xml:id ID #IMPLIED>]>
					<one xml:id="invalid:id"><two xml:id="invalid:id"/></one>'
				usingNamespaces: true
				validation: false]
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:28'!
testAttributeXMLIDValidation
	| xml xmlWithDuplicateID |

	xml := '<a xml:id="one"><b xml:id="two"/></a>'.
	xmlWithDuplicateID := '<a xml:id="one"><b xml:id="one"/></a>'.
	self
		shouldnt: [SAXHandler parse: xml]
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: xmlWithDuplicateID]
		raise: XMLValidationException.
	self
		shouldnt: [
			(SAXHandler on: xmlWithDuplicateID)
				isValidating: false;
				parseDocument]
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/11/2016 11:25'!
testCDataOutsideElement
	#('<!!DOCTYPE test [ <!![CDATA[test]]> ]><test/>'
		'<!![CDATA[test]]><test/>'
		'	<!![CDATA[test]]><test/>'
		'<test/><!![CDATA[test]]>'
		'<test/>	<!![CDATA[test]]>')
		do: [:each |
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/10/2016 15:15'!
testCDataSectionUnterminated
	#('<test><!!['
		'<test><!![CDATA['
		'<test><!![CDATA[ test'
		'<test><!![CDATA[ test ]'
		'<test><!![CDATA[ test ]]')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/8/2016 09:53'!
testCharacterReferenceUnterminated
	#('<test>&#</test>'
		'<test>&#34</test>'
		'<test>&#x22</test>'
		'<test>&#;</test>'
		'<test>&#x;</test>')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException.
			self
				should: [
					(SAXHandler on: each)
						replacesContentEntityReferences: false;
						parseDocument]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:52'!
testComment
	self
		shouldnt: [SAXHandler parse: '<test><!!-- <comment/> - - --></test>']
		raise: XMLWellFormednessException.
	"can't contain --"
	self
		should: [SAXHandler parse: '<test><!!-- <comment/> -- --></test>']
		raise: XMLWellFormednessException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:52'!
testCommentUnterminated
	#('<!!--' '<!!-- comment' '<!!-- comment -' '<!!-- comment --')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 4/23/2015 18:46'!
testConditionalSectionInInternalSubset
	#('<!![INCLUDE[ <!!-- comment --> ]]>'
		'<!![IGNORE[ <!!-- comment --> ]]>'
		'<!!ENTITY % include "INCLUDE"> <!![%include;[ <!!-- comment --> ]]>'
		'<!!ENTITY % ignore "IGNORE"> <!![%ignore;[ <!!-- comment --> ]]>'
		) do: [:each |
			self
				should: [SAXHandler parse:'<!!DOCTYPE test [', each, ']>']
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 5/3/2015 16:47'!
testConditionalSectionLabelReplacementIgnore
	#('<!!ENTITY % label "{1}">
		<!![%label;[ <malformed-xml/> ]]>'
		'<!!ENTITY % label "{1}">
		<!![ %label; [ <malformed-xml/> ]]>'
		) do: [:each |
		self
			should: [
				(SAXHandler on: (each format: (Array with: 'INCLUDE')))
					parser
						context: XMLExternalSubsetContext new;
						parseDocument]
			raise: XMLWellFormednessException.
		self
			shouldnt: [
				(SAXHandler on: (each format: (Array with: 'IGNORE')))
					parser
						context: XMLExternalSubsetContext new;
						parseDocument]
			raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 5/3/2015 16:47'!
testConditionalSectionLabelReplacementInclude
	#('<!!ENTITY % label "{1}">
		<!![%label;[
			<!!ENTITY % comment "<!!--comment-->">
		]]> %comment;'
		'<!!ENTITY % label "{1}">
		<!![ %label; [
			<!!ENTITY % comment "<!!--comment-->">
		]]> %comment;'
	) do: [:each |
		self
			shouldnt: [
				(SAXHandler on: (each format: (Array with: 'INCLUDE')))
					parser
						context: XMLExternalSubsetContext new;
						parseDocument]
			raise: XMLValidationException.
		self
			should: [
				(SAXHandler on: (each format: (Array with: 'IGNORE')))
					parser
						context: XMLExternalSubsetContext new;
						parseDocument]
			raise: XMLValidationException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 5/3/2015 16:47'!
testConditionalSectionMixedIncludeIgnore
	| xml |

	xml := '
		<!![INCLUDE[
			<!![IGNORE[ %comment; ]]>
			<!!ENTITY % comment "<!!--comment]]>-->">
			<!![INCLUDE[
				<!![IGNORE[ %commentTwo; ]]>
				<!!ENTITY % commentTwo "<!!--comment]]>-->">
				%commentTwo;
			]]>
			%comment;
			%commentTwo;
		]]>
		%comment;
		%commentTwo;'.
	self
		shouldnt: [
			(SAXHandler on: xml) parser
				context: XMLExternalSubsetContext new;
				parseDocument]
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:43'!
testConditionalSectionNestedIgnore
	| xml |

	xml := '
		<!![ IGNORE [
			<!!--ignored terminator {1} -->
			<!![ INCLUDE [
				<!!ENTITY % comment "<!!--ignored terminator {1} -->">
				%comment;
			]]>
			%comment;
		]]>'.
	self
		shouldnt: [
			(SAXHandler on: (xml format: (Array with: ']]')))
				parser
					context: XMLExternalSubsetContext new;
					parseDocument]
		raise: XMLWellFormednessException.
	self
		should: [
			(SAXHandler on: (xml format: (Array with: ']]>')))
				parser
					context: XMLExternalSubsetContext new;
					parseDocument]
		raise: XMLWellFormednessException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:43'!
testConditionalSectionNestedInclude
	| xml |

	xml := '
		<!![ INCLUDE [
			<!!--ignored terminator ]]> -->
			<!![ INCLUDE [
				<!!ENTITY % comment "<!!--ignored terminator ]]> -->">
				%comment;
			]]>
			%comment;
		]]>
		%comment;'.
	self
		shouldnt: [
			(SAXHandler on: xml) parser
				context: XMLExternalSubsetContext new;
				parseDocument]
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/10/2016 15:05'!
testConditionalSectionUnterminatedIgnore
	#('<!![IGNORE['
		'<!![IGNORE[<!!--comment-->'
		'<!![IGNORE[<!!--comment--><!![IGNORE['
		'<!![IGNORE[<!!--comment--><!![IGNORE[ ]]>'
	) do: [:each |
		self
			should: [
				(SAXHandler on: each) parser
					context: XMLExternalSubsetContext new;
					parseDocument]
			raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:41'!
testConditionalSectionUnterminatedInclude
	#('<!![INCLUDE['
		'<!![INCLUDE[<!!--comment-->'
		'<!![INCLUDE[<!!--comment--><!![INCLUDE['
		'<!![INCLUDE[<!!--comment--><!![INCLUDE[ ]]>'
	) do: [:each |
		self
			should: [
				(SAXHandler on: each) parser
					context: XMLExternalSubsetContext new;
					parseDocument]
			raise: XMLWellFormednessException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/11/2016 11:11'!
testContentlessDocument
	#(''
		' 	'
		'<?xml version="1.0"?>'
		'	<!!DOCTYPE test>'
		'	<!!--comment-->'
		'	<?target data?>'
		'<?xml version="1.0"?><!!DOCTYPE test>'
		'<?xml version="1.0"?><!!DOCTYPE test><!!--comment-->'
		'<?xml version="1.0"?><!!DOCTYPE test><!!--comment--><?pi data?>')
		do: [:each |
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/10/2016 15:05'!
testContentlessSubset
	#(''
		'<?xml encoding="UTF-8"?>'
		'<!!--comment-->'
		'<?target data?>'
		'<?xml encoding="UTF-8"?><!!--comment-->'
		'<?xml encoding="UTF-8"?><!!--comment--><?target data?>')
		do: [:each |
			self
				shouldnt: [
					((SAXHandler on: '') parser driver
						externalSubsetParserOn: each
						documentURI: 'extern.dtd') parseDocument]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 4/7/2015 05:17'!
testDefaultDocumentURI
	| cwd documentURI |

	documentURI := (SAXHandler on: '' documentURI: 'test/test.xml') documentURI.
	self assert: documentURI = 'test/test.xml'.

	cwd := XMLFileHandle currentWorkingDirectory.
	documentURI := (SAXHandler on: '') documentURI.
	self
		assert: documentURI size > (cwd size + 1);
		assert: (documentURI copyUpToLast: $/) = cwd.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/12/2016 19:10'!
testDoctypeDeclarationBadPublicIDChar
	| validChars |

	validChars :=
		'!!#$%''()*+,-./0123456789:;=?@ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz',
		(Character xmlSeparators copyWithout: Character tab).
	0 to: 255 do: [:each | | char charString |
		char := each asCharacter.
		(validChars includes: char)
			ifFalse: [
				charString := char asString.
				self
					should: [
						SAXHandler parse:
							'<!!DOCTYPE test PUBLIC "//public//', charString, '" "external.dtd"><test/>']
					raise: XMLWellFormednessException]].
	self
		shouldnt: [
			SAXHandler parse:
				'<!!DOCTYPE test PUBLIC "//public//', validChars, '" "external.dtd"><test/>']
		raise: XMLWellFormednessException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 4/6/2015 05:22'!
testDoctypeDeclarationInContent
	self
		should: [SAXHandler parse: '<test><!!DOCTYPE test></test>']
		raise: XMLWellFormednessException.
	self
		shouldnt: [SAXHandler parse: '<!!DOCTYPE test><test></test>']
		raise: XMLWellFormednessException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/8/2016 08:49'!
testDoctypeDeclarationRedundance
	#('<!!DOCTYPE test><!!DOCTYPE test>'
		'<!!DOCTYPE test><!!--comment--><!!DOCTYPE test>'
		'<!!DOCTYPE test><!!pi data?><!!DOCTYPE test>'
		'<!!DOCTYPE test [<!!DOCTYPE test>]>'
		'<test><!!DOCTYPE test></test>')
		do: [:each |
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 4/6/2015 05:24'!
testDoctypeDeclarationRoot
	| xmlWithInvalidRoot xmlWithValidRoot |

	xmlWithInvalidRoot := '<!!DOCTYPE root><invalidRoot/>'.
	xmlWithValidRoot := '<!!DOCTYPE root><root/>'.
	self
		should: [SAXHandler parse: xmlWithInvalidRoot]
		raise: XMLValidationException.
	self
		shouldnt: [
			SAXHandler
				parse: xmlWithInvalidRoot
				usingNamespaces: false
				validation: false]
		raise: XMLValidationException.
	self
		shouldnt: [SAXHandler parse: xmlWithValidRoot]
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:48'!
testDoctypeDeclarationUnterminated
	#('<!!DOCTYPE' '<!!DOCTYPE >'
		'<!!DOCTYPE test' '<!!DOCTYPE test PUBLIC' '<!!DOCTYPE test PUBLIC "pubid'
		'<!!DOCTYPE test PUBLIC "pubid" "sysid' '<!!DOCTYPE test PUBLIC "pubid" "sysid"'
		'<!!DOCTYPE test SYSTEM' '<!!DOCTYPE test SYSTEM "sysid' '<!!DOCTYPE test SYSTEM "sysid"'
		'<!!DOCTYPE test [' '<!!DOCTYPE test []')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 20:25'!
testDoctypeDeclarationWhitespace
	#('<!!DOCTYPEtest>'
		'<!!DOCTYPE test PUBLIC"publicid">'
		'<!!DOCTYPE test PUBLIC "publicid""systemid">'
		'<!!DOCTYPE test SYSTEM"systemid">')
		do: [:each |
			self
				should: [SAXHandler parse: each, '<test/>']
				raise: XMLWellFormednessException].
	#('<!!DOCTYPE test PUBLIC "publicid" "systemid"	 >'
		'<!!DOCTYPE test SYSTEM "systemid"	>')
		do: [:each |
			self
				shouldnt: [SAXHandler parse: each, '<test/>']
				raise: XMLWellFormednessException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:56'!
testElementAnyDeclaration
	self
		shouldnt: [
			SAXHandler parse: 
				'<!!DOCTYPE one [
					<!!ELEMENT one ANY>
					<!!ELEMENT two ANY>]>
				<one>one<two><!![CDATA[two]]><two /></two>three</one>']
		raise: XMLValidationException.
	self
		shouldnt: [
			SAXHandler parse: 
				'<!!DOCTYPE root [
					<!!ELEMENT root ANY>
					<!!ELEMENT two ANY>]>
				<root><two></two></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/14/2016 13:33'!
testElementBadNames
	XMLTokenizerTest badNames do: [:each |
		self
			should: [SAXHandler parse: '<', each, '/>']
			raise: XMLWellFormednessException.
		self
			should: [SAXHandler parse: '<test ', each, '="value"/>']
			raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:58'!
testElementContentModelBad
	#('<!!DOCTYPE root [<!!ELEMENT root ()>]>'
		'<!!DOCTYPE root [<!!ELEMENT root (one,)>]>'
		'<!!DOCTYPE root [<!!ELEMENT root (one|)>]>'
		'<!!DOCTYPE root [<!!ELEMENT root (one)(two)>]>'
		'<!!DOCTYPE root [<!!ELEMENT root (one|,two)>]>'
	) do: [:each |
		self
			should: [SAXHandler parse: each]
			raise: XMLWellFormednessException].

	'|,?*+' do: [:each |
		self
			should: [SAXHandler parse: '<!!DOCTYPE root [<!!ELEMENT root (', each asString, 'one)>]>']
			raise: XMLWellFormednessException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:40'!
testElementContentModelBranching
	| dtd |

	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: #('one' 'two' 'three' 'four' 'five')),
		'<!!ELEMENT root ((one|two),(three|four|five))>]>'.

	self
		shouldnt: [
			SAXHandler
				parse: dtd, '<root><one/><three/></root>';
				parse: dtd, '<root><one/><four/></root>';
				parse: dtd, '<root><one/><five/></root>';
				parse: dtd, '<root><two/><three/></root>';
				parse: dtd, '<root><two/><four/></root>';
				parse: dtd, '<root><two/><five/></root>']
		raise: XMLValidationException.

	self
		should: [SAXHandler parse: dtd, '<root><one/><two/><three/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><three/><four/><five/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><four/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:40'!
testElementContentModelManyBranches
	| dtd names |

	names := (1 to: 50) collect: [:each | 'element', each printString].
	dtd := String streamContents: [:stream |
		stream
			nextPutAll: '<!!DOCTYPE root [';
			nextPutAll: (self elementDeclarationsNamed: names);
			nextPutAll: '<!!ELEMENT root ('.
		names
			do: [:each | stream nextPutAll: each]
			separatedBy: [stream nextPut: $|].
		stream nextPutAll: ')>]>'].

	names do: [:each |
		self
			shouldnt: [SAXHandler parse: dtd, '<root><', each, '/></root>']
			raise: XMLValidationException].
	self
		should: [
			SAXHandler parse:
				dtd, '<root><element', (names size + 1) printString, '/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:58'!
testElementContentModelNonDeterministic
	| template |
	
	template :=
		'<!!DOCTYPE a [
			<!!ELEMENT a {1}>
			<!!ELEMENT b EMPTY>
			<!!ELEMENT c EMPTY>
			<!!ELEMENT d EMPTY>
		]>{2}'.

	#(#('((b,c)|(b,d))' '(b,(c|d))' '<a><b/><c/></a>')
		#('(b?,b*)' '(b,b*)' '<a><b/></a>')
		#('(b+,b?)' '(b,b+)' '<a><b/><b/></a>')
		) do: [:each | | nonDeterministic deterministic source |
		nonDeterministic := each first.
		deterministic := each second.
		source := each third.
		self
			should: [
				SAXHandler parse:
					(template format: (Array with: nonDeterministic with: source))]
			raise: XMLValidationException.
		self
			shouldnt: [
				SAXHandler parse:
					(template format: (Array with: deterministic with: source))]
			raise: XMLValidationException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:40'!
testElementContentModelOptional
	| dtd |

	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: #('one' 'two' 'three' 'four' 'five')),
		'<!!ELEMENT root (one?,two,three?,(four?, five)?)>]>'.

	self
		shouldnt: [
			SAXHandler
				parse: dtd, '<root><one/><two/><three/><four/><five/></root>';
				parse: dtd, '<root><two/><three/></root>';
				parse: dtd, '<root><one/><two/><three/><five/></root>';
				parse: dtd, '<root><two/></root>';
				parse: dtd, '<root><one/><two/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><three/><four/><five/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><two/><three/><four/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><two/><three/><five/><four/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:40'!
testElementContentModelOptionalBranching
	| dtd |

	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: #('one' 'two' 'three' 'four' 'five' 'six')),
		'<!!ELEMENT root (((one,two)|(three,four)),five?)>]>'.

	self
		shouldnt: [
			SAXHandler
				parse: dtd, '<root><one/><two/><five/></root>';
				parse: dtd, '<root><three/><four/><five/></root>';
				parse: dtd, '<root><one/><two/></root>';
				parse: dtd, '<root><three/><four/></root>']
		raise: XMLValidationException.

	self
		should: [SAXHandler parse: dtd, '<root><three/><five/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><five/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><five/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:40'!
testElementContentModelOptionalEmpty
	| dtd |

	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: #('one' 'two' 'three')),
		'<!!ELEMENT root (one?,two)?>]>'.

	self
		shouldnt: [
			SAXHandler
				parse: dtd, '<root><one/><two/></root>';
				parse: dtd, '<root><two/></root>';
				parse: dtd, '<root></root>']
		raise: XMLValidationException.

	self
		should: [SAXHandler parse: dtd, '<root><one/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><three/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><one/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:40'!
testElementContentModelPlus
	| dtd |

	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: #('one' 'two' 'three' 'four' 'five')),
		'<!!ELEMENT root ((one?,two)+,three,four+,five)>]>'.

	self
		shouldnt: [
			SAXHandler
				parse: dtd, '<root><one/><two/><three/><four/><five/></root>';
				parse: dtd, '<root><two/><one/><two/><two/><three/><four/><five/></root>';
				parse: dtd, '<root><two/><three/><four/><four/><five/></root>';
				parse: dtd, '<root><two/><three/><four/><five/></root>']
		raise: XMLValidationException.

	self
		should: [SAXHandler parse: dtd, '<root><three/><four/><five/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><two/><four/><four/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><one/><two/><three/><five/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:41'!
testElementContentModelStar
	| dtd |

	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: #('one' 'two' 'three' 'four' 'five')),
		'<!!ELEMENT root ((one?,two)*,(three)*,four,five*)>]>'.

	self
		shouldnt: [
			SAXHandler
				parse: dtd, '<root><one/><two/><three/><four/><five/></root>';
				parse: dtd, '<root><two/><one/><two/><three/><four/><five/></root>';
				parse: dtd, '<root><three/><three/><four/><five/></root>';
				parse: dtd, '<root><four/><five/><five/></root>';
				parse: dtd, '<root><four/></root>']
		raise: XMLValidationException.

	self
		should: [SAXHandler parse: dtd, '<root><three/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:41'!
testElementContentModelStarEmpty
	| dtd |

	dtd := '<!!DOCTYPE root [',
		(self elementDeclarationsNamed: #('one' 'two' 'three')),
		'<!!ELEMENT root (one*,two)*>]>'.

	self
		shouldnt: [
			SAXHandler
				parse: dtd, '<root><one/><one/><two/><two/></root>';
				parse: dtd, '<root><two/><one/><two/></root>';
				parse: dtd, '<root><two/><two/></root>';
				parse: dtd, '<root></root>']
		raise: XMLValidationException.

	self
		should: [SAXHandler parse: dtd, '<root><one/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><three/></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:54'!
testElementDeclarationPrefixed
	| dtd |

	dtd := '<!!DOCTYPE prefix:root [
		<!!ELEMENT prefix:root ANY>
		<!!ELEMENT unprefixed EMPTY>]>'.
	self
		shouldnt: [SAXHandler parse: dtd, '<prefix:root xmlns:prefix="ns"><unprefixed/></prefix:root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><unprefixed/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<prefix:root xmlns:prefix="ns"><prefix:unprefixed/></prefix:root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 20:21'!
testElementDeclarationWhitespace
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE test [<!!ELEMENTtest (#PCDATA)>]><test/>']
		raise: XMLWellFormednessException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE test [<!!ELEMENT test(#PCDATA)>]><test/>']
		raise: XMLWellFormednessException.
	self
		shouldnt: [
			SAXHandler parse:
				'<!!DOCTYPE test [<!!ELEMENT test (	#PCDATA	)   >]><test/>']
		raise: XMLWellFormednessException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:56'!
testElementEmptyDeclaration
	self
		shouldnt: [
			SAXHandler parse: 
				'<!!DOCTYPE root [<!!ELEMENT root EMPTY>]>',
				'<root> <!!--empty--> <?pi data?> </root>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse: 
				'<!!DOCTYPE root [
					<!!ELEMENT root EMPTY>
					<!!ELEMENT non-empty ANY>]><root><non-empty/></root>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse: 
				'<!!DOCTYPE root [<!!ELEMENT root EMPTY>]><root>non-empty</root>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse: 
				'<!!DOCTYPE root [<!!ELEMENT root EMPTY>]><root>&#32;</root>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse: 
				'<!!DOCTYPE root [<!!ELEMENT root EMPTY>]><root><!![CDATA[non-empty]]></root>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 22:38'!
testElementMixedContentDeclaration
	| dtd |

	dtd := '
		<!!DOCTYPE root [
			<!!ELEMENT root (#PCDATA | one | two | three)*>
			<!!ELEMENT one EMPTY>
			<!!ELEMENT two (#PCDATA)>
			<!!ELEMENT three (#PCDATA)*>
			<!!ELEMENT four EMPTY>]>'.
	self
		shouldnt: [
			SAXHandler parse:
				dtd, '<root>text<one/><two><!![CDATA[text]]></two></root>']
		raise: XMLValidationException.
	self
		shouldnt: [
			SAXHandler parse:
				dtd, '<root><three>text<!!--comment--></three></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><four/></root>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root><two><three/></two></root>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse: "missing *"
				'<!!DOCTYPE root [<!!ELEMENT root (#PCDATA|one)>]><root/>']
		raise: XMLWellFormednessException.
	'+*?,' do: [:each |
		self
			should: [
				SAXHandler parse:
					'<!!DOCTYPE root [<!!ELEMENT two (#PCDATA', each asString, ')>]><root/>']
			raise: XMLWellFormednessException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/10/2016 15:02'!
testElementNesting
	| error |

	error := XMLWellFormednessException.
	self
		should: [SAXHandler parse: '<unclosed>'] raise: error;
		should: [SAXHandler parse: '</unopened>'] raise: error;
		should: [SAXHandler parse: '<unclosed1/><unclosed2/>'] raise: error;
		should: [SAXHandler parse: '<root></unopened></root>'] raise: error;
		should: [SAXHandler parse: '<root><unclosed></root>'] raise: error;
		should: [SAXHandler parse: '<root><first><second></first></second></root>'] raise: error.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:48'!
testElementUnterminatedEndTag
	#('<test></' '<test></test' '<test></ test' '<test></ test>')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:49'!
testElementUnterminatedStartTag
	#('<' '<test' '<test name'
		'<test name=' '<test name="value' '<test name="value"'
		'<test name="value" /')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 4/4/2015 06:56'!
testElementWhitespace
	self
		should: [SAXHandler parse: '<root name1="value1"name2="value2"/>']
		raise: XMLWellFormednessException.
	self
		should: [SAXHandler parse: '<root name1=''value1''name2="value2"/>']
		raise: XMLWellFormednessException.
	self
		shouldnt: [SAXHandler parse: '<root name1="value1" name2="value2"/>']
		raise: XMLWellFormednessException.
	self
		shouldnt: [SAXHandler parse: '<root name1   =   "value1"    name2   =   "value2"    />']
		raise: XMLWellFormednessException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:33'!
testElementsUndeclared
	self
		shouldnt: [
			SAXHandler
				parse: '<!!DOCTYPE root [<!!ENTITY ignored "">]><root/>'
				usingNamespaces:true
				validation: true]
		raise:XMLValidationException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [<!!ELEMENT root ANY>]><root><undeclared/></root>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [<!!ATTLIST root name CDATA #IMPLIED>]><root/>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 20:26'!
testEntityDeclarationWhitespace
	#('<!!DOCTYPE test [<!!ENTITYname "value">]>'
		'<!!DOCTYPE test [<!!ENTITY name"value">]>'
		'<!!DOCTYPE test [<!!ENTITY name PUBLIC"publicid" "systemid">]>'
		'<!!DOCTYPE test [<!!ENTITY name PUBLIC "publicid""systemid">]>'
		'<!!DOCTYPE test [<!!ENTITY name SYSTEM"systemid">]>'
		'<!!DOCTYPE test [<!!ENTITY name PUBLIC "publicid" "systemid"NDATA GIF>]>'
		'<!!DOCTYPE test [<!!ENTITY name SYSTEM "systemid"NDATA GIF>]>'
		'<!!DOCTYPE test [<!!ENTITY% name "value">]>'
		'<!!DOCTYPE test [<!!ENTITY %name "value">]>')
		do: [:each |
			self
				should: [SAXHandler parse: each, '<test/>']
				raise: XMLWellFormednessException].
	#('<!!DOCTYPE test [<!!ENTITY  name 	"value"   >]>'
		'<!!DOCTYPE test [<!!ENTITY   % 	 name "value"	>]>'
		'<!!DOCTYPE test [<!!ENTITY   name   PUBLIC   "publicid"    "systemid"   >]>'
		'<!!DOCTYPE test [<!!ENTITY   name   SYSTEM    "systemid"   >]>')
		do: [:each |
			self
				shouldnt: [SAXHandler parse: each, '<test/>']
				raise: XMLWellFormednessException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 2/20/2016 14:14'!
testEntityLiteralContextNesting
	| xml inner outer document |

	xml := '<?xml version="1.0"?>
	<!!DOCTYPE entities [
		<!!ENTITY % inner " '' inner &#34; ">
		<!!ENTITY % outer '' "%inner;&#39; ''>
		<!!ENTITY inner  "%outer;"> 
		<!!ENTITY outer '' &#39; " &inner; &#39; " ''>]>
	<entities one="&outer;" two="&inner;" three=''&outer;'' four=''&inner;''>&outer;&inner;</entities>'.
	inner := ' " '' inner " '' '.
	outer := ' '' " ', inner, ' '' " '.

	self
		shouldnt: [document := XMLDOMParser parse: xml]
		raise: XMLWellFormednessException.
	self
		assert: (document root attributeAt: 'one') = outer;
		assert: (document root attributeAt: 'two') = inner;
		assert: (document root attributeAt: 'three') = outer;
		assert: (document root attributeAt: 'four') = inner;
		assert: (document root contentString) = (outer, inner).! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:52'!
testEntityReferenceUnterminated
	#('<!!DOCTYPE test [
			<!!ENTITY % entity "<!!ELEMENT test EMPTY>">
			%]><test/>'
		'<!!DOCTYPE test [
			<!!ENTITY % entity "<!!ELEMENT test EMPTY>">
			%;]><test/>'
		'<!!DOCTYPE test [
			<!!ENTITY % entity "<!!ELEMENT test EMPTY>">
			%entity]><test/>'
		'<test>&</test>'
		'<test>&test</test>'
		'<test>&;</test>')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException.
			self
				should: [
					(SAXHandler on: each)
						replacesContentEntityReferences: false;
						parseDocument]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/8/2013 01:54'!
testEntityReplacement
	| document |

	document := XMLDOMParser parse: '
		<!!DOCTYPE root [
			<!!ENTITY % partialDeclaration "name1 &#39;replacement1&#x27;">
			<!!ENTITY % name1Declaration "<!!ENTITY %partialDeclaration;>">
			%name1Declaration;
			<!!ENTITY name2 "replacement2 &name1;" >
			<!!ENTITY name3 "replacement3 &name2;" >
			<!!ENTITY quotes "&#x27;quoted &#x22;value&#34;&#39;">
		]>
		<root name="&quotes;">&name1; &name2; &name3; ',
		'%name1Declaration;</root>'.
	self assert:
		document root contentString =
			('replacement1 ',
			'replacement2 replacement1 ',
			'replacement3 replacement2 replacement1 ',
			'%name1Declaration;').
	self assert:
		(document root attributeAt: 'name') =
			(('quoted "value"' copyWithFirst: $') copyWith: $').! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/9/2016 20:59'!
testEntityReplacementEmpty
	| xml document |

	xml := '<?xml version="1.0"?>
	<!!DOCTYPE entities [
		<!!ENTITY % inner "">
		%inner;
		<!!ENTITY % outer ''%inner;''>
		%outer;
		<!!ENTITY inner  "%outer;"> 
		<!!ENTITY outer ''&inner;''>]>
	<entities one="&outer;" two="&inner;" three=''&outer;'' four=''&inner;''>&outer;&inner;</entities>'.

	self
		shouldnt: [document := XMLDOMParser parse: xml]
		raise: XMLWellFormednessException.
	self
		assert: (document root attributeAt: 'one') isEmpty;
		assert: (document root attributeAt: 'two') isEmpty;
		assert: (document root attributeAt: 'three') isEmpty;
		assert: (document root attributeAt: 'four') isEmpty;
		assert: document root contentString isEmpty.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:54'!
testEntityReplacementPredefined
	| document references replacements |

	references := String new writeStream.
	replacements := String new writeStream.
	XMLWellFormedParserTokenizer predefinedEntities keysAndValuesDo: [:name :char |
		references
			nextPut: $&;
			nextPutAll: name;
			nextPut: $;.

		replacements nextPut: char].

	document :=
		XMLDOMParser parse:
			('<!!DOCTYPE root [
				<!!ENTITY nonRecursive "&amp;nonRecursive;">]>
			<root name=''{1}''>
				<one>{1}</one>
				<two>&nonRecursive;</two>
			</root>' format: (Array with: references contents)).
	self
		assert: (document root attributeAt: 'name') = replacements contents;
		assert: (document root contentStringAt: 'one') = replacements contents;
		assert: (document root contentStringAt: 'two') = '&nonRecursive;'.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:51'!
testEntityReplacementSelfReferential
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [<!!ENTITY first "&first;">]><root>&first;</root>']
		raise: XMLWellFormednessException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [<!!ENTITY % first "&#37;first;">%first;]><root/>']
		raise: XMLWellFormednessException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [
					<!!ENTITY first "&second;">
					<!!ENTITY second "&first;">]><root>&first;</root>']
		raise: XMLWellFormednessException.
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [
					<!!ENTITY % first "&#37;second;">
					<!!ENTITY % second "&#37;first;">
					%first;]><root/>']
		raise: XMLWellFormednessException.
	self
		shouldnt: [
			SAXHandler parse:
				'<!!DOCTYPE root [
					<!!ENTITY first "&second;">
					<!!ENTITY second "<!!--replacement-->">]><root>&first;</root>']
		raise: XMLWellFormednessException.
	self
		shouldnt: [
			SAXHandler parse:
				'<!!DOCTYPE root [
					<!!ENTITY % first "&#37;second;">
					<!!ENTITY % second "<!!--replacement-->">
					%first;]><root/>']
		raise: XMLWellFormednessException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/2/2015 11:14'!
testEntityURIInheritance
	| xml parser driver |

	xml := '<!!DOCTYPE root [%first;]><root xml:base="/new/xml">&second;</root>'.
	parser :=
		(SAXHandler
			on: xml
			documentURI: 'test.xml') parser.
	(driver := parser driver)
		doctypeDefinition parameterEntities
			at: 'first'
			put:
				(DTDExternalParameterEntity
					name: 'first'
					replacement: '<!!ENTITY second "<!!--comment-->">'
					uri: 'test.ent').

	self assert: driver currentURI = 'test.xml'.

	3 timesRepeat: [parser parseToken].
	self assert: driver currentURI = 'test.ent'.

	2 timesRepeat: [parser parseToken].
	self assert: driver currentURI = 'test.xml'.

	2 timesRepeat: [parser parseToken].
	self assert: driver currentURI = '/new/xml'.

	3 timesRepeat: [parser parseToken].
	self
		assert: driver currentURI = 'test.xml';
		assert: parser atEnd.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:34'!
testEntityUndeclared
	| document dtd content |

	dtd := '<!!DOCTYPE root [<!!ENTITY declared "%undeclared;">]>'.
	content := '<root>&undeclared;test&undeclared;</root>'.
	self should: [XMLDOMParser parse: content] raise: XMLValidationException.
	self should: [XMLDOMParser parse: dtd, '<root/>'] raise: XMLValidationException.
	self
		shouldnt: [
			document :=
				XMLDOMParser
					parse: dtd, content
					usingNamespaces: false
					validation: false]
		raise: XMLValidationException.
	self assert: document allNodes second contentString = 'test'.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:34'!
testEntityUndeclaredAndPreserved
	| xml parser document |

	xml := '<!!DOCTYPE root [
		<!!ENTITY unreferenced "%undeclaredOne;">
		%undeclaredOne;]>
		<root name="&#34;&undeclaredTwo;&#34;">&lt;&undeclaredThree;&gt;</root>'.
	(parser := XMLDOMParser on: xml)
		isValidating: false;
		preservesUndeclaredEntityReferences: true.
	document := parser parseDocument.

	self
		assert: document doctypeDeclaration nodes size = 1;
		assert: document doctypeDeclaration firstNode replacement = '%undeclaredOne;';
		assert: (document root attributeAt: 'name') = '"&undeclaredTwo;"';
		assert: (document root contentString) = '<&undeclaredThree;>'.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:50'!
testEntityUnreplaced
	| beforeSubset subset afterSubset xml document |

	beforeSubset := '<?xml version="1.1"?> <!!DOCTYPE entity ['.
	subset :=
		'<!!ENTITY % first "<!!ENTITY second &#34;replacement&#34;>">
		%first;'.
	afterSubset := ']><entity>&second;</entity>'.
	xml := beforeSubset, subset, afterSubset.
	self assert: (XMLDOMParser parse: xml) root contentString = 'replacement'.
	
	document :=
		(XMLDOMParser on: xml)
			replacesContentEntityReferences: false;
			parseDocument.
	self assert: document root contentString = '&second;'.
	self
		should: [
			(SAXHandler on: (beforeSubset, '%first;]>'))
				replacesContentEntityReferences: false;
				parseDocument]
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:50'!
testEntityUnreplacedInContent
	| xml document |

	xml := '
		<!!DOCTYPE one [<!!ENTITY declared "replacement">]>
		<one two="&quot;&declared;&quot;">&#60;&declared;&#x2F;&#x3e;</one>'.
	document :=
		(XMLDOMParser on: xml)
			replacesContentEntityReferences: false;
			parseDocument.
	self
		assert: (document root attributeAt: 'two') = '&quot;&declared;&quot;';
		assert: document root contentString = '&#60;&declared;&#x2F;&#x3e;'.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:50'!
testEntityUnreplacedInContentAndUndeclared
	| xml document |

	xml := '<one two="&quot;&undeclared;&quot;">&#60;&undeclared;&#62;</one>'.
	self
		should: [
			(SAXHandler on: xml)
				replacesContentEntityReferences: false;
				parseDocument]
		raise: XMLValidationException.
	self
		shouldnt: [
			document :=
				(XMLDOMParser on: xml)
					isValidating: false;
					replacesContentEntityReferences: false;
					parseDocument]
		raise: XMLValidationException.
	self
		assert: (document root attributeAt: 'two') = '&quot;&undeclared;&quot;';
		assert: document root contentString = '&#60;&undeclared;&#62;'.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 4/4/2015 21:06'!
testInternalDeclarationParameterEntityReference
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [
					<!!ENTITY % name "root">
					<!!ENTITY % contentModel "EMPTY">
					<!!ELEMENT %name; %contentModel;>
				]><root/>']
		raise: XMLWellFormednessException.
	self
		shouldnt: [
			SAXHandler parse:
				'<!!DOCTYPE root [
					<!!ENTITY % element "<!!ELEMENT root EMPTY>">
					%element;
				]><root/>']
		raise: XMLWellFormednessException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/24/2015 20:33'!
testIsInContent
	| xml handler parser |

	xml := '<?xml version="1.0"?>
			<!!--comment-->
			<?target data?>
			<!!DOCTYPE root [<!!ELEMENT root EMPTY>]>
			<!!--comment-->
			<?target data?>
			<root/>'.
	handler := SAXHandler on: xml.
	parser := handler parser.
	9 timesRepeat: [
		parser parseToken.
		self deny: handler isInContent].
	parser parseToken.
	self assert: handler isInContent.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/22/2015 03:34'!
testLineEndingsDoNotMatter
	| source docFromSourceWithCRs docFromSourceWithLFs docFromSourceWithCRLFs |

	source := '<root>{1}one{1}two{1}</root>'.
	docFromSourceWithCRs := XMLDOMParser parse: (source format: (Array with: String cr)).
	docFromSourceWithLFs := XMLDOMParser parse: (source format: (Array with: String lf)).
	docFromSourceWithCRLFs := XMLDOMParser parse: (source format: (Array with: String crlf)).
	self
		assert: (docFromSourceWithCRs root contentString) = (docFromSourceWithLFs root contentString);
		assert: (docFromSourceWithLFs root contentString) = (docFromSourceWithCRLFs root contentString).! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:45'!
testNamespaceDeclaration
	self
		shouldnt: [SAXHandler parse: '<prefix:root prefix:name="test" xmlns:prefix="NS"/>']
		raise: XMLNamespaceException.
	self
		shouldnt: [SAXHandler parse: '<root xmlns="defaultNS"/>']
		raise: XMLNamespaceException.
	self
		shouldnt: [SAXHandler parse: '<root xmlns=""/>' usingNamespaces: true]
		raise: XMLNamespaceException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:46'!
testNamespaceDeclarationBadPrefix
	self
		should: [SAXHandler parse: '<root xmlns:invalid:prefix="invalidNS"/>']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<root xmlns::invalidprefix="invalidNS"/>']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<root xmlns:invalidprefix:="invalidNS"/>']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<:invalidname />']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<invalidname: />']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<root :invalidname="test" />']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<root invalidname:="test" />']
		raise: XMLNamespaceException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:47'!
testNamespaceDeclarationBadURI
	self
		should: [SAXHandler parse: '<root xmlns:empty=""/>' usingNamespaces: true]
		raise: XMLNamespaceException.

	"Cannot change xmlns or xml prefixes"
	self
		should: [SAXHandler parse: '<root xmlns:xmlns="newNS"/>' usingNamespaces: true]
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<root xmlns:xml="newNS" />' usingNamespaces: true]
		raise: XMLNamespaceException.

	"Redundant xml prefix mapping is ok, but xmlns is not"
	self
		shouldnt: [
			SAXHandler
				parse: '<root xmlns:xml="', XMLNamespaceScope xmlNamespaceURI, '"/>'
				usingNamespaces: true]
		raise: XMLNamespaceException.
	self
		should: [
			SAXHandler
				parse: '<root xmlns:xmlns="', XMLNamespaceScope xmlnsNamespaceURI, '"/>'
				usingNamespaces: true]
		raise: XMLNamespaceException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/8/2016 09:04'!
testNamespaceDeclarationRedundance
	#('<test xmlns:prefix="prefixNS" xmlns:prefixTwo="prefixTwoNS" xmlns:prefix="prefixNS" />'
		'<test xmlns:prefix="prefixNS" xmlns:prefixTwo="prefixTwoNS" xmlns:prefix="prefixTwoNS" />'
		'<test xmlns="defaultNS" xmlns:prefix="prefixNS" xmlns="defaultNS" />'
		'<test xmlns="defaultNS" xmlns:prefix="prefixNS" xmlns="defaultTwoNS" />')
		do: [:each |
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/31/2014 00:15'!
testNamespacePrefixAliasing
	| declarations |

	declarations := '<test xmlns:alias1="ns" xmlns:alias2="ns" '.
	self
		shouldnt: [
			SAXHandler
				parse: declarations, ' alias1:name="value" />'
				usingNamespaces: true]
		raise: XMLNamespaceException.
	self
		should: [
			SAXHandler
				parse: declarations, ' alias1:name="value" alias2:name="value"/>'
				usingNamespaces: true]
		raise: XMLNamespaceException.
	self
		shouldnt: [
			SAXHandler
				parse: declarations, ' alias1:name="value" alias2:name2="value"/>'
				usingNamespaces: true]
		raise: XMLNamespaceException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/28/2014 19:50'!
testNamespacePrefixInheritance
	self
		shouldnt: [
			SAXHandler
				parse:
					'<one xmlns:prefix1="ns">
						<prefix1:two xmlns:prefix2="ns2">
							<prefix2:three><prefix1:four /></prefix2:three>
						</prefix1:two>
					</one>'
				usingNamespaces: true]
		raise: XMLNamespaceException.
	self
		should: [
			SAXHandler
				parse:
					'<one xmlns:prefix1="ns">
						<prefix1:two xmlns:prefix2="ns2" />
						<prefix2:three />
					</one>'
				usingNamespaces: true]
		raise: XMLNamespaceException.
	self
		should: [
			SAXHandler
				parse:
					'<one xmlns:prefix1="ns">
						<prefix1:two xmlns:prefix2="ns2">
							<prefix2:three xmlns:prefix3="ns3" />
						</prefix1:two>
						<prefix3:four />
					</one>'
				usingNamespaces: true]
		raise: XMLNamespaceException.
	self
		shouldnt: [
			SAXHandler
				parse:
					'<one xmlns:prefix1="ns">
						<prefix1:two xmlns:prefix2="ns2">
							<prefix2:three xmlns:prefix3="ns3" />
						</prefix1:two>
						<prefix2:four />
						<prefix3:five />
					</one>'
				usingNamespaces: false]
		raise: XMLNamespaceException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:46'!
testNamespaceUndeclared
	self
		shouldnt: [SAXHandler parse: '<prefix:root prefix:name="test"/>' usingNamespaces: false]
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<prefix:root/>']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<root prefix:name="test"/>']
		raise: XMLNamespaceException.

	"xml is OK as a prefix, xmlns is not"
	self
		shouldnt: [SAXHandler parse: '<xml:root xml:name="test"/>']
		raise: XMLNamespaceException.
	self
		should: [SAXHandler parse: '<xmlns:root />']
		raise: XMLNamespaceException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 20:19'!
testNotationDeclarationWhitespace
	#('<!!DOCTYPE test [<!!NOTATIONname PUBLIC "publicid">]>'
		'<!!DOCTYPE test [<!!NOTATION name PUBLIC"publicid">]>'
		'<!!DOCTYPE test [<!!NOTATION name PUBLIC "publicid""systemid">]>'
		'<!!DOCTYPE test [<!!NOTATION name SYSTEM"systemid">]>')
		do: [:each |
			self
				should: [SAXHandler parse: each, '<test/>']
				raise: XMLWellFormednessException].

	#('<!!DOCTYPE test [<!!NOTATION     name  PUBLIC  "publicid"   >]>'
		'<!!DOCTYPE test [<!!NOTATION name PUBLIC        "publicid"   "systemid"   >]>'
		'<!!DOCTYPE test [<!!NOTATION name SYSTEM        "systemid"		>]>')
		do: [:each |
			self
				shouldnt: [SAXHandler parse: each, '<test/>']
				raise: XMLWellFormednessException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 11/14/2015 22:46'!
testOptimizeForLargeDocuments
	| parser |

	parser := SAXHandler on: ''.
	self
		assert: parser isValidating;
		assert: parser usesNamespaces;
		assert: parser documentReadLimit = XMLParserLimits defaultDocumentReadLimit.

	parser optimizeForLargeDocuments.
	self
		deny: parser isValidating;
		deny: parser usesNamespaces;
		assert: parser documentReadLimit isNil.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/11/2016 11:15'!
testPCDataOutsideElement
	#('<!!DOCTYPE test [ test ]><test/>'
		'test<test/>'
		'	test<test/>'
		'<test/>test'
		'<test/>	test')
		do: [:each |
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/11/2016 11:16'!
testPCDataWithCDataTerminator
	self
		should: [SAXHandler parse: '<test>test]]>test</test>']
		raise: XMLWellFormednessException.

	#('<test>test]] >test</test>'
		'<test>test] ]>test</test>'
		'<test>test ] ] > test</test>'
		'<test name="test]]>test]] >test] ]>test ] ] > test"/>')
		do: [:each | 
			self
				shouldnt: [SAXHandler parse: each]
				raise: XMLWellFormednessException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'StephaneDucasse 12/27/2013 16:52'!
testParserLittleDocument

	| doc |
	doc := XMLDOMParser parse: 
'<?xml version="1.0" encoding="UTF-8"?>
<!!--an example-->
<example:user xmlns:example="urn://example">
    <username>jsmith</username>
    <real-name>John Smith</real-name>
    <age>25</age>
    <id>101</id>
</example:user>' readStream.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:53'!
testProcessingInstruction
	| document |

	document :=
		XMLDOMParser parse:
			'<?xml version="1.1" standalone="yes"?><?firstTarget first>Data?>',
			'<root><?secondTarget second?Data?></root> '.

	self
		assert: document version = 1.1;
		assert: document nodes size = 2;
		assert: document firstNode isPI;
		assert: document firstNode target = 'firstTarget';
		assert: document firstNode data = 'first>Data';
		assert: document root firstNode target = 'secondTarget';
		assert: document root firstNode data = 'second?Data'.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/14/2016 00:17'!
testProcessingInstructionBadTarget
	#('<test><?xml version="1.0"?></test>'
		'<test><?XML version="1.0"?></test>'
		'<test><?XmL version="1.0"?></test>'
		'<test><?xMl version="1.0"?></test>')
		do: [:each |
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:53'!
testProcessingInstructionUnterminated
	#('<?' '<??' '<??>' '<?name' '<?name value' '<?name value?' '<?name value>')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:53'!
testProcessingInstructionWhitespace
	self
		should: [SAXHandler parse: '<pi><?target?data?></pi>']
		raise: XMLWellFormednessException.
	self
		shouldnt: [SAXHandler parse: '<pi><?target ?data  ?></pi>']
		raise: XMLWellFormednessException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/10/2016 15:04'!
testRequiresDocumentConstraints
	self
		shouldnt: [
			(SAXHandler on: '<!!DOCTYPE root [<!!ELEMENT root EMPTY>]><root/>')
				requiresDocumentConstraints: true;
				parseDocument]
		raise: XMLValidationException.
	self
		should: [
			(SAXHandler on: '<root/>')
				requiresDocumentConstraints: true;
				parseDocument]
		raise: XMLValidationException.
	self
		should: [
			(SAXHandler on: '
				<!!DOCTYPE root [
					<!!ENTITY one "two">
					<!!ENTITY % three "four">
					<!!NOTATION five PUBLIC "six">
					<!!--seven-->]>
				<root/>')
				requiresDocumentConstraints: true;
				parseDocument]
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 6/21/2015 03:04'!
testReusingParser
	| parser oldConfiguraiton |

	parser :=
		SAXHandler
			on: self addressBookXML
			documentURI: 'oldURI'.
	oldConfiguraiton := parser configuration.
	parser parseDocument.
	parser on: self addressBookXMLWithDTD.
	self
		assert: parser input = self addressBookXMLWithDTD;
		deny: parser documentURI = 'oldURI';
		deny: parser configuration == oldConfiguraiton.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/22/2015 03:35'!
testStandalone
	| dtdXML standaloneDeclXML |

	dtdXML := self addressBookXMLWithDTD.
	self
		shouldnt: [
			SAXHandler
				parse: dtdXML
				usingNamespaces: true
				validation: true
				externalEntities: false]
		raise: XMLFileException.

	standaloneDeclXML :=
		(dtdXML copyUpTo: $>) allButLast, ' standalone="yes" ?>', (dtdXML copyAfter: $>).
	self
		shouldnt: [
			SAXHandler
				parse: standaloneDeclXML
				usingNamespaces: true
				validation: false
				externalEntities: true]
		raise: XMLFileException.
	self
		shouldnt: [
			SAXHandler
				parse: standaloneDeclXML
				usingNamespaces: true
				validation: true
				externalEntities: false]
		raise: XMLFileException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/12/2014 23:20'!
testSubsetEndingEntityReplacement
	self
		should: [
			SAXHandler parse:
				'<!!DOCTYPE root [
					<!!ENTITY % endOfSubset "]>">
					%endOfSubset;
					<root/>']
		raise: XMLWellFormednessException! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/22/2015 03:35'!
testUnparsedEntities
	| dtd |

	dtd := '
		<!!DOCTYPE root [
			<!!NOTATION note PUBLIC "-//ext//note//EN">
			<!!ENTITY unparsedEntity SYSTEM "note.txt" NDATA note>
			<!!ELEMENT root EMPTY>
			<!!ATTLIST root notation ENTITY #REQUIRED>
		]>'.
	self
		shouldnt: [SAXHandler parse: dtd, '<root notation="unparsedEntity" />']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<root notation="undeclaredUnparsedEntity" />']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 6/17/2015 22:17'!
testUnparsedEntitiesUndeclaredNotations
	| beforeNotation afterNotation |

	beforeNotation := '
		<!!DOCTYPE entity [
			<!!ELEMENT entity EMPTY>
			<!!ATTLIST entity name ENTITY #REQUIRED>
			<!!ENTITY notationEntity SYSTEM "note.txt" NDATA note>'.
	afterNotation := ']><entity name="notationEntity"/>'.
	self
		should: [SAXHandler parse: beforeNotation, afterNotation]
		raise: XMLValidationException.
	self
		shouldnt: [
			SAXHandler
				parse: beforeNotation, afterNotation
				usingNamespaces: true
				validation: false]
		raise: XMLValidationException.
	self
		shouldnt: [
			SAXHandler parse:
				beforeNotation, '<!!NOTATION note SYSTEM "note.txt">', afterNotation]
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 12/29/2013 02:49'!
testValidationAndConstraintsToggle
	self assert:
		((SAXHandler on: '')
			isValidating: false;
			requiresDocumentConstraints: true;
			isValidating).
	self deny:
		((SAXHandler on: '')
			requiresDocumentConstraints: true;
			isValidating: false;
			requiresDocumentConstraints).! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/6/2015 06:11'!
testValidationAndEntityReferenceToggle
	self deny:
		((SAXHandler on: '')
			isValidating: true;
			preservesUndeclaredEntityReferences: true;
			isValidating).
	self deny:
		((SAXHandler on: '')
			preservesUndeclaredEntityReferences: true;
			isValidating: true;
			preservesUndeclaredEntityReferences).! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 1/3/2015 00:48'!
testWhitespacePreservation
	| xml root notPreserved preserved |

	xml := '
		<root xml:space="preserve">
			<not-preserved xml:space="default">
				<preserved xml:space="preserve">
				</preserved>
			</not-preserved>
		</root>'.
	root := (XMLDOMParser parse: xml) root.
	self
		assert: root nodes size = 3;
		assert: root stringNodes size = 2.
	root stringNodesDo: [:each | self assert: each isWhitespace].

	notPreserved := root nodes second.
	self assert: notPreserved nodes size = 1.

	preserved := notPreserved firstElement.
	self
		assert: preserved firstNode isStringNode;
		assert: preserved firstNode isWhitespace.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 20:15'!
testWhitespacePreservationValidation
	| dtd |

	dtd := '
		<!!DOCTYPE whitespace [
			<!!ELEMENT whitespace EMPTY>
			<!!ATTLIST whitespace xml:space (default|preserve) ''preserve''>
		]>'.
	self
		shouldnt: [SAXHandler parse: dtd, '<whitespace></whitespace>']
		raise: XMLValidationException.
	self
		shouldnt: [SAXHandler parse: dtd, '<whitespace xml:space="default"> </whitespace>']
		raise: XMLValidationException.
	self
		shouldnt: [
			SAXHandler parse:
				(dtd copyReplaceAll: '(default|preserve)' with: '(preserve|default)'),
				'<whitespace></whitespace>']
		raise: XMLValidationException.
	self
		shouldnt: [
			SAXHandler parse:
				(dtd copyReplaceAll: '(default|preserve)' with: '(preserve)'),
				'<whitespace></whitespace>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				(dtd copyReplaceAll: '(default|preserve)' with: '(default|preserve|other)'),
				'<whitespace></whitespace>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				(dtd copyReplaceAll: '(default|preserve)' with: 'CDATA'),
				'<whitespace></whitespace>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<whitespace> </whitespace>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<whitespace><!![CDATA[ ]]></whitespace>']
		raise: XMLValidationException.
	self
		should: [SAXHandler parse: dtd, '<whitespace xml:space="default"><!![CDATA[ ]]></whitespace>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				(dtd copyReplaceAll: '(default|preserve)' with: '(not|valid)'),
				'<whitespace></whitespace>']
		raise: XMLValidationException.
	self
		should: [
			SAXHandler parse:
				(dtd copyReplaceAll: '(default|preserve)' with: 'CDATA'),
				'<whitespace></whitespace>']
		raise: XMLValidationException.! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 19:29'!
testXMLDeclarationEncodingNameChar
	| validChars |

	validChars := '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_.-'.
	0 to: 255 do: [:each | | char charString |
		char := each asCharacter.
		(validChars includes: char)
			ifFalse: [
				charString := char asString.
				self
					should: [SAXHandler parse: '<?xml version="1.0" encoding="ISO', charString, '"?>']
					raise: XMLWellFormednessException]].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/14/2016 01:17'!
testXMLDeclarationRedundance
	#('
<?xml version="1.0"?><test/>'
		' <?xml version="1.0"?><test/>'
		'	<?xml version="1.0"?><test/>'
		'<?xml version="1.0"?><?xml version="1.0"?><test/>'
		'<?xml version="1.0"?><!!--comment--><?xml version="1.0"?><test/>'
		'<?xml version="1.0"?><!!DOCTYPE test [<?xml version="1.0"?>]><test/>'
		'<?xml version="1.0"?><test><?xml version="1.0"?></test>'
		'<test><?xml version="1.0"?></test>')
		do: [:each |
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 20:16'!
testXMLDeclarationStandaloneAttribute
	#('<?xml version="1.0" standalone=""?>'
		'<?xml version="1.0" standalone="YES"?>'
		'<?xml version="1.0" standalone="Yes"?>'
		'<?xml version="1.0" standalone=" yes "?>'
		'<?xml version="1.0" standalone="NO"?>'
		'<?xml version="1.0" standalone="No"?>'
		'<?xml version="1.0" standalone=" no "?>')
		do: [:each |
			self
				should: [SAXHandler parse: each, '<test/>']
				raise: XMLWellFormednessException].

	#('<?xml version="1.0" standalone="no"?>'
		'<?xml version="1.0" standalone="yes"?>')
		do: [:each |
			self
				shouldnt: [SAXHandler parse: each, '<test/>']
				raise: XMLWellFormednessException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/8/2016 09:53'!
testXMLDeclarationUnterminated
	#('<?xml'
		'<?xml ?'
		'<?xml ?>'
		'<?xml version'
		'<?xml version='
		'<?xml version="1.0'
		'<?xml version="1.0" encoding'
		'<?xml version="1.0" encoding="UTF-8'
		'<?xml version="1.0" encoding="UTF-8" standalone'
		'<?xml version="1.0" encoding="UTF-8" standalone="yes'
		'<?xml version="1.0" encoding="UTF-8" standalone="yes"'
		'<?xml version="1.0" encoding="UTF-8" standalone="yes"?')
		do: [:each | 
			self
				should: [SAXHandler parse: each]
				raise: XMLWellFormednessException]! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/7/2016 20:17'!
testXMLDeclarationVersionAttribute
	#('<?xml version=""?>'
		'<?xml version="1"?>'
		'<?xml version="1."?>'
		'<?xml version="1.0."?>'
		'<?xml version="1.0.1"?>'
		'<?xml version="10"?>'
		'<?xml version="10."?>'
		'<?xml version="10.00."?>'
		'<?xml version="10.00.10"?>'
		'<?xml version="one"?>'
		'<?xml version="one."?>'
		'<?xml version="one.zero."?>'
		'<?xml version="one.zero.one"?>')
		do: [:each |
			self
				should: [SAXHandler parse: each, '<test/>']
				raise: XMLWellFormednessException].

	#('<?xml version="1.0"?>'
		'<?xml version="1.1"?>'
		'<?xml version="1.00"?>'
		'<?xml version="1.10"?>')
		do: [:each |
			self
				shouldnt: [SAXHandler parse: each, '<test/>']
				raise: XMLWellFormednessException].! !

!XMLParserTest methodsFor: 'tests' stamp: 'monty 3/8/2016 09:50'!
testXMLDeclarationWhitespace
	#('<? xml version="1.0" encoding="UTF-8" standalone="yes"?>'
		'<?xml version="1.0"standalone="yes"?>'
		'<?xml version="1.0"encoding="UTF-8" standalone="yes"?>'
		'<?xml version="1.0" encoding="UTF-8"standalone="yes"?>')
		do: [:each |
			self
				should: [SAXHandler parse: each, '<test/>']
				raise: XMLWellFormednessException].
	self
		shouldnt: [
			SAXHandler parse:
				'<?xml
					version
					 =
					 "1.0"
					encoding
					 =
					 "UTF-8"	
					standalone
					 =
					 "yes"
					?>
				<test/>']
		raise: XMLWellFormednessException.! !

TestCase subclass: #XMLStreamAdapterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

XMLStreamAdapterTest subclass: #XMLReadStreamAdapterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

XMLReadStreamAdapterTest subclass: #XMLBinaryReadStreamAdapterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLBinaryReadStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:17'!
inputCollection
	^ super inputCollection asArray collect: [:each | each asciiValue]! !

!XMLBinaryReadStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:38'!
streamAdapterClass
	^ XMLBinaryReadStreamAdapter! !

XMLReadStreamAdapterTest subclass: #XMLDecodingReadStreamAdapterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

XMLDecodingReadStreamAdapterTest subclass: #XMLDecodingReadStreamAdapterByteTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLDecodingReadStreamAdapterByteTest methodsFor: 'accessing' stamp: 'monty 3/14/2016 02:06'!
encodedInvalidCharacters
	^ self stringFrom:
		#(16r0100
			16rD800
			16rDBFF
			16rDC00
			16rD8FF)
	"16r11FFFF is ommitted because GS disallows chars with >= that value"! !

!XMLDecodingReadStreamAdapterByteTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:24'!
streamConverterClass
	^ XMLByteStreamConverter! !

!XMLDecodingReadStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 3/9/2016 10:32'!
encodedInvalidCharacters
	^ ''! !

!XMLDecodingReadStreamAdapterTest methodsFor: 'instance creation' stamp: 'monty 3/5/2016 12:39'!
newStreamAdapterOn: aStream
	^ (super newStreamAdapterOn: aStream)
		streamConverter: self streamConverterClass new! !

!XMLDecodingReadStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:38'!
streamAdapterClass
	^ XMLDecodingReadStreamAdapter! !

!XMLDecodingReadStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:24'!
streamConverterClass
	^ XMLNullStreamConverter! !

!XMLDecodingReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 16:09'!
testHasDefaultStreamConverter
	| adapter |

	adapter := self newStreamAdapter.
	self assert:
		adapter hasDefaultStreamConverter = adapter streamConverter isDefault.! !

!XMLDecodingReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 16:09'!
testHasNullStreamConverter
	| adapter |

	adapter := self newStreamAdapter.
	self assert:
		adapter hasNullStreamConverter = adapter streamConverter isNull.! !

!XMLDecodingReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/9/2016 10:35'!
testInvalidCharacters
	| streamAdapter |

	streamAdapter :=
		self newStreamAdapterOn:
			self encodedInvalidCharacters readStream.
	[streamAdapter atEnd]
		whileFalse: [
			self
				should: [streamAdapter next]
				raise: XMLEncodingException].
	self assert: streamAdapter atEnd.! !

!XMLDecodingReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testIsConvertingStreamAdapter
	self assert: self newStreamAdapter isConvertingStreamAdapter! !

!XMLDecodingReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/6/2016 08:29'!
testOn
	| stream adapter |

	super testOn.

	stream := #(1) readStream.
	adapter := self newStreamAdapterOn: stream.
	self
		assert: adapter class == self streamAdapterClass;
		assert: adapter stream class == XMLBinaryReadStreamAdapter;
		assert: adapter stream stream == stream.! !

!XMLDecodingReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testStreamConverter
	| adapter newConverter |

	adapter := self newStreamAdapter.
	self
		deny: adapter streamConverter isNil;
		assert: adapter streamConverter class == self streamConverterClass.

	newConverter := self streamConverterClass new.
	self
		assert: (adapter streamConverter: newConverter) == adapter;
		assert: adapter streamConverter == newConverter.

	self
		assert: (adapter streamConverter: nil) == adapter;
		assert: adapter streamConverter class == XMLNullStreamConverter.! !

XMLDecodingReadStreamAdapterTest subclass: #XMLDecodingReadStreamAdapterUTF16BETest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLDecodingReadStreamAdapterUTF16BETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:17'!
inputCollection
	^ self encodedUTF16BEString! !

!XMLDecodingReadStreamAdapterUTF16BETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 04:07'!
outputCollection
	^ self decodedUTF16String! !

!XMLDecodingReadStreamAdapterUTF16BETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 04:09'!
streamConverterClass
	^ XMLUTF16BigEndianStreamConverter! !

XMLDecodingReadStreamAdapterTest subclass: #XMLDecodingReadStreamAdapterUTF16LETest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLDecodingReadStreamAdapterUTF16LETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:17'!
inputCollection
	^ self encodedUTF16LEString! !

!XMLDecodingReadStreamAdapterUTF16LETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 04:10'!
outputCollection
	^ self decodedUTF16String! !

!XMLDecodingReadStreamAdapterUTF16LETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 04:10'!
streamConverterClass
	^ XMLUTF16LittleEndianStreamConverter! !

XMLDecodingReadStreamAdapterTest subclass: #XMLDecodingReadStreamAdapterUTF32BETest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLDecodingReadStreamAdapterUTF32BETest methodsFor: 'accessing' stamp: 'monty 3/9/2016 10:32'!
encodedInvalidCharacters
	^ self stringFrom:
		#(16r00 16r00 16rD8 16r00
			16r00 16r00 16rDB 16rFF
			16r00 16r00 16rDC 16r00
			16r00 16r00 16rD8 16rFF
			16r00 16r11 16rFF 16rFF)! !

!XMLDecodingReadStreamAdapterUTF32BETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:17'!
inputCollection
	^ self encodedUTF32BEString! !

!XMLDecodingReadStreamAdapterUTF32BETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 04:19'!
outputCollection
	^ self decodedUTF32String! !

!XMLDecodingReadStreamAdapterUTF32BETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 04:20'!
streamConverterClass
	^ XMLUTF32BigEndianStreamConverter! !

XMLDecodingReadStreamAdapterTest subclass: #XMLDecodingReadStreamAdapterUTF32LETest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLDecodingReadStreamAdapterUTF32LETest methodsFor: 'accessing' stamp: 'monty 3/9/2016 10:33'!
encodedInvalidCharacters
	^ self stringFrom:
		#(16r00 16rD8 16r00 16r00
			16rFF 16rDB 16r00 16r00
			16r00 16rDC 16r00 16r00
			16rFF 16rD8 16r00 16r00
			16rFF 16rFF 16r11 16r00)! !

!XMLDecodingReadStreamAdapterUTF32LETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:17'!
inputCollection
	^ self encodedUTF32LEString! !

!XMLDecodingReadStreamAdapterUTF32LETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 04:12'!
outputCollection
	^ self decodedUTF32String! !

!XMLDecodingReadStreamAdapterUTF32LETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 04:12'!
streamConverterClass
	^ XMLUTF32LittleEndianStreamConverter! !

XMLDecodingReadStreamAdapterTest subclass: #XMLDecodingReadStreamAdapterUTF8Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLDecodingReadStreamAdapterUTF8Test methodsFor: 'accessing' stamp: 'monty 3/9/2016 10:33'!
encodedInvalidCharacters
	^ self stringFrom:
		#(16rED 16rA0 16r80
			16rED 16rAF 16rBF
			16rED 16rB0 16r80
			16rED 16rA3 16rBF
			16rF4 16r9F 16rBF 16rBF)! !

!XMLDecodingReadStreamAdapterUTF8Test methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:17'!
inputCollection
	^ self encodedUTF8String! !

!XMLDecodingReadStreamAdapterUTF8Test methodsFor: 'accessing' stamp: 'monty 2/11/2016 04:07'!
outputCollection
	^ self decodedUTF8String! !

!XMLDecodingReadStreamAdapterUTF8Test methodsFor: 'accessing' stamp: 'monty 2/11/2016 04:06'!
streamConverterClass
	^ XMLUTF8StreamConverter! !

XMLDecodingReadStreamAdapterTest subclass: #XMLPercentDecodingReadStreamAdapterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLPercentDecodingReadStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:17'!
inputCollection
	^ self percentEncodedUTF8String! !

!XMLPercentDecodingReadStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 08:41'!
outputCollection
	^ self decodedUTF8String! !

!XMLPercentDecodingReadStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:38'!
streamAdapterClass
	^ XMLPercentDecodingReadStreamAdapter! !

!XMLPercentDecodingReadStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 08:41'!
streamConverterClass
	^ XMLUTF8StreamConverter! !

!XMLReadStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:17'!
inputCollection
	^ 'test'! !

!XMLReadStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 03:41'!
outputCollection
	^ 'test'! !

!XMLReadStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:38'!
streamAdapterClass
	^ XMLReadStreamAdapter! !

!XMLReadStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:17'!
streamCollection
	^ self inputCollection! !

!XMLReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testAtEnd
	| adapter |

	adapter := self newStreamAdapter.
	self outputCollection size timesRepeat: [
		self deny: adapter atEnd.
		adapter next].
	self assert: adapter atEnd.! !

!XMLReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testNext
	| adapter |

	adapter := self newStreamAdapter.
	self outputCollection do: [:each |
		self assert: adapter next = each].
	self assert: adapter next isNil.

	0 to: self outputCollection size do: [:i |
		adapter := self newStreamAdapter.
		self assert: (adapter next: i) = (self outputCollection copyFrom: 1 to: i)].

	adapter := self newStreamAdapter.
	self
		assert: (adapter next: self outputCollection size + 1) = self outputCollection;
		assert: (adapter next: 1) isEmpty.! !

!XMLReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:38'!
testOnStreamIfBinary
	| stream adapter |

	stream := '1' readStream.
	self assert: (self streamAdapterClass onStreamIfBinary: stream) == stream.

	stream upToEnd.
	self assert: (self streamAdapterClass onStreamIfBinary: stream) == stream.

	stream := #(1) readStream.
	adapter := self streamAdapterClass onStreamIfBinary: stream.
	self assert: adapter class == self streamAdapterClass.! !

!XMLReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testPeek
	| adapter |

	adapter := self newStreamAdapter.
	self outputCollection do: [:each |
		self assert: adapter peek = each.
		adapter next].
	self assert: adapter peek isNil.! !

!XMLReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testPeekFor
	| adapter |

	adapter := self newStreamAdapter.
	self outputCollection do: [:each |
		self
			deny: (adapter peekFor: #absent);
			assert: (adapter peekFor: each);
			deny: (adapter peekFor: #absent)].
	self deny: (adapter peekFor: #absent).! !

!XMLReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testSkip
	| adapter |

	0 to: self outputCollection size do: [:i |
		adapter := self newStreamAdapter.
		self
			assert: (adapter skip: i) == adapter;
			assert: adapter upToEnd = (self outputCollection allButFirst: i)].

	adapter := self newStreamAdapter.
	self
		assert: (adapter skip: self outputCollection size + 1) == adapter;
		assert: adapter upToEnd isEmpty.! !

!XMLReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testSkipTo
	| adapter |

	self outputCollection do: [:each |
		adapter := self newStreamAdapter.
		self
			assert: (adapter skipTo: each);
			assert: adapter upToEnd = (self outputCollection copyAfter: each)].

	adapter := self newStreamAdapter.
	self
		deny: (adapter skipTo: #absent);
		assert: adapter atEnd.! !

!XMLReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testStreamReadingPortability
	| stream adapter |

	stream := XMLExceptionRaisingReadStream on: self streamCollection.
	adapter := self newStreamAdapterOn: stream.
	self outputCollection do: [:each |
		self
			deny: adapter atEnd;
			assert: adapter peek = each;
			assert: adapter next = each].
	self assert: adapter atEnd.
	self
		shouldnt: [
			adapter
				peek;
				next]
		raise: Error.
	self
		assert: adapter peek isNil;
		assert: adapter next isNil.

	self assert: stream atEnd.
	self should: [stream peek] raise: Error.
	self should: [stream next] raise: Error.! !

!XMLReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testUpTo
	| adapter |

	self outputCollection do: [:each |
		adapter := self newStreamAdapter.
		self assert:
			(adapter upTo: each) =
				(self outputCollection copyUpTo: each)].

	adapter := self newStreamAdapter.
	self assert: (adapter upTo: #absent) = self outputCollection.! !

!XMLReadStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testUpToEnd
	| adapter |

	adapter := self newStreamAdapter.
	self
		assert: adapter upToEnd = self outputCollection;
		assert: adapter upToEnd isEmpty.! !

!XMLStreamAdapterTest class methodsFor: 'converting' stamp: 'monty 2/12/2016 05:57'!
asciiToUTF16BE: anASCIIString
	^ String streamContents: [:stream |
		anASCIIString do: [:each |
			stream
				nextPut: 0 asCharacter;
				nextPut: each]]! !

!XMLStreamAdapterTest class methodsFor: 'converting' stamp: 'monty 2/12/2016 05:57'!
asciiToUTF16LE: anASCIIString
	^ String streamContents: [:stream |
		anASCIIString do: [:each |
			stream
				nextPut: each;
				nextPut: 0 asCharacter]]! !

!XMLStreamAdapterTest class methodsFor: 'converting' stamp: 'monty 2/12/2016 05:58'!
asciiToUTF32BE: anASCIIString
	^ String streamContents: [:stream |
		anASCIIString do: [:each |
			stream
				nextPut: 0 asCharacter;
				nextPut: 0 asCharacter;
				nextPut: 0 asCharacter;
				nextPut: each]]! !

!XMLStreamAdapterTest class methodsFor: 'converting' stamp: 'monty 2/12/2016 05:58'!
asciiToUTF32LE: anASCIIString
	^ String streamContents: [:stream |
		anASCIIString do: [:each |
			stream
				nextPut: each;
				nextPut: 0 asCharacter;
				nextPut: 0 asCharacter;
				nextPut: 0 asCharacter]]! !

!XMLStreamAdapterTest class methodsFor: 'testing' stamp: 'monty 2/10/2016 22:57'!
shouldInheritSelectors
	^ true! !

!XMLStreamAdapterTest class methodsFor: 'converting' stamp: 'monty 2/14/2016 04:01'!
stringFrom: aCollection
	"avoid asString for portability"
	^ String withAll: (aCollection collect: [:each | each asCharacter])! !

!XMLStreamAdapterTest methodsFor: 'converting' stamp: 'monty 2/12/2016 08:57'!
asciiToUTF16BE: anASCIIString
	^ self class asciiToUTF16BE: anASCIIString! !

!XMLStreamAdapterTest methodsFor: 'converting' stamp: 'monty 2/12/2016 08:57'!
asciiToUTF16LE: anASCIIString
	^ self class asciiToUTF16LE: anASCIIString! !

!XMLStreamAdapterTest methodsFor: 'converting' stamp: 'monty 2/12/2016 08:58'!
asciiToUTF32BE: anASCIIString
	^ self class asciiToUTF32BE: anASCIIString! !

!XMLStreamAdapterTest methodsFor: 'converting' stamp: 'monty 2/12/2016 08:58'!
asciiToUTF32LE: anASCIIString
	^ self class asciiToUTF32LE: anASCIIString! !

!XMLStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/14/2016 03:01'!
decodedUTF16String
	^ self stringFrom:
		#(16rFF 16r0100 16rD7FF 16rE000 16rFFFF 16r10000 16r10437 16r24B62 16r10FFFF)! !

!XMLStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 04:01'!
decodedUTF32String
	^ self stringFrom: #(16rFE 16rD7FE 16r10FFFF)! !

!XMLStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 03:58'!
decodedUTF8String
	^ self stringFrom:
		#(16r7F 16r80 16r7FF 16r800 16rFFFF 16r10000 16r10FFFF)! !

!XMLStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/14/2016 03:00'!
encodedUTF16BEString
	^ self stringFrom:
		#(16r00 16rFF
			16r01 16r00
			16rD7 16rFF
			16rE0 16r00
			16rFF 16rFF
			16rD8 16r00 16rDC 16r00 "first surrogate pair"
			16rD8 16r01 16rDC 16r37
			16rD8 16r52 16rDF 16r62
			16rDB 16rFF 16rDF 16rFF) "last surrogate pair"! !

!XMLStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/14/2016 03:01'!
encodedUTF16LEString
	^ self stringFrom:
		#(16rFF 16r00
			16r00 16r01
			16rFF 16rD7
			16r00 16rE0
			16rFF 16rFF
			16r00 16rD8 16r00 16rDC "first surrogate pair"
			16r01 16rD8 16r37 16rDC
			16r52 16rD8 16r62 16rDF
			16rFF 16rDB 16rFF 16rDF) "last surrogate pair"! !

!XMLStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 04:03'!
encodedUTF32BEString
	^ self stringFrom:
		#(16r00 16r00 16r00 16rFE
			16r00 16r00 16rD7 16rFE
			16r00 16r10 16rFF 16rFF)! !

!XMLStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 04:03'!
encodedUTF32LEString
	^ self stringFrom:
		#(16rFE 16r00 16r00 16r00
			16rFE 16rD7 16r00 16r00
			16rFF 16rFF 16r10 16r00)! !

!XMLStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 03:55'!
encodedUTF8String
	^ self stringFrom:
		#(16r7F
			16rC2 16r80
			16rDF 16rBF
			16rE0 16rA0 16r80
			16rEF 16rBF 16rBF
			16rF0 16r90 16r80 16r80
			16rF4 16r8F 16rBF 16rBF)! !

!XMLStreamAdapterTest methodsFor: 'instance creation' stamp: 'monty 2/15/2016 09:17'!
newStream
	^ self newStreamOn: self streamCollection! !

!XMLStreamAdapterTest methodsFor: 'instance creation' stamp: 'monty 3/5/2016 12:39'!
newStreamAdapter
	^ self newStreamAdapterOn: self newStream! !

!XMLStreamAdapterTest methodsFor: 'instance creation' stamp: 'monty 3/5/2016 12:39'!
newStreamAdapterOn: aStream
	^ self streamAdapterClass on: aStream! !

!XMLStreamAdapterTest methodsFor: 'instance creation' stamp: 'monty 2/15/2016 09:17'!
newStreamOn: aCollection
	^ self streamClass on: aCollection! !

!XMLStreamAdapterTest methodsFor: 'converting' stamp: 'monty 2/17/2016 11:58'!
percentEncodeCharacter: aCharacter
	| hexString |

	(hexString := aCharacter asciiValue printStringHex asUppercase) size > 1
		ifTrue: [^ '%', hexString]
		ifFalse: [^ '%0', hexString]! !

!XMLStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/17/2016 11:58'!
percentEncodedUTF8String
	^ String streamContents: [:stream |
		self encodedUTF8String do: [:each |
			stream nextPutAll: (self percentEncodeCharacter: each)]]! !

!XMLStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:38'!
streamAdapterClass
	^ XMLStreamAdapter! !

!XMLStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/10/2016 22:58'!
streamClass
	^ ReadStream! !

!XMLStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:19'!
streamCollection
	^ String new! !

!XMLStreamAdapterTest methodsFor: 'converting' stamp: 'monty 2/14/2016 04:02'!
stringFrom: aCollection
	^ self class stringFrom: aCollection! !

!XMLStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testClose
	self
		shouldnt: [self newStreamAdapter close]
		raise: Error! !

!XMLStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testIsBinary
	self deny: self newStreamAdapter isBinary! !

!XMLStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testIsConvertingStreamAdapter
	self deny: self newStreamAdapter isConvertingStreamAdapter! !

!XMLStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testIsStream
	self assert: self newStreamAdapter isStream! !

!XMLStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/6/2016 08:28'!
testOn
	| stream adapter |

	stream := self newStream.
	adapter := self newStreamAdapterOn: stream.
	self
		assert: adapter class == self streamAdapterClass;
		assert: adapter stream == stream.! !

XMLStreamAdapterTest subclass: #XMLWriteStreamAdapterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

XMLWriteStreamAdapterTest subclass: #XMLEncodingWriteStreamAdapterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

XMLEncodingWriteStreamAdapterTest subclass: #XMLEncodingWriteStreamAdapterByteTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLEncodingWriteStreamAdapterByteTest methodsFor: 'accessing' stamp: 'monty 3/8/2016 16:58'!
decodedInvalidCharacters
	^ self decodedUnicodeInvalidCharacters copyWithFirst: 256 asCharacter! !

!XMLEncodingWriteStreamAdapterByteTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:28'!
streamConverterClass
	^ XMLByteStreamConverter! !

!XMLEncodingWriteStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 3/8/2016 16:58'!
decodedInvalidCharacters
	^ #()! !

!XMLEncodingWriteStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 3/14/2016 02:06'!
decodedUnicodeInvalidCharacters
	"16r11FFFF is ommitted because GS disallows chars with >= that value"
	^ self stringFrom: #(16rD800 16rDBFF 16rDC00 16rD8FF)! !

!XMLEncodingWriteStreamAdapterTest methodsFor: 'instance creation' stamp: 'monty 3/5/2016 12:39'!
newStreamAdapterOn: aStream
	^ (super newStreamAdapterOn: aStream)
		streamConverter: self streamConverterClass new! !

!XMLEncodingWriteStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:38'!
streamAdapterClass
	^ XMLEncodingWriteStreamAdapter! !

!XMLEncodingWriteStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:26'!
streamConverterClass
	^ XMLNullStreamConverter! !

!XMLEncodingWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 16:09'!
testHasDefaultStreamConverter
	| adapter |

	adapter := self newStreamAdapter.
	self assert:
		adapter hasDefaultStreamConverter = adapter streamConverter isDefault.! !

!XMLEncodingWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 16:09'!
testHasNullStreamConverter
	| adapter |

	adapter := self newStreamAdapter.
	self assert:
		adapter hasNullStreamConverter = adapter streamConverter isNull.! !

!XMLEncodingWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/8/2016 16:54'!
testInvalidCharacters
	self decodedInvalidCharacters do: [:each |
		self
			should: [self newStreamAdapter nextPut: each]
			raise: XMLEncodingException]! !

!XMLEncodingWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testIsConvertingStreamAdapter
	self assert: self newStreamAdapter isConvertingStreamAdapter! !

!XMLEncodingWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testNextPutByteOrderMark
	| adapter |

	adapter := self newStreamAdapter.
	self
		assert: adapter nextPutByteOrderMark == adapter;
		assert: adapter contents = self streamConverterClass byteOrderMark.! !

!XMLEncodingWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testStreamConverter
	| adapter newConverter |

	adapter := self newStreamAdapter.
	self
		deny: adapter streamConverter isNil;
		assert: adapter streamConverter class == self streamConverterClass.

	newConverter := self streamConverterClass new.
	self
		assert: (adapter streamConverter: newConverter) == adapter;
		assert: adapter streamConverter == newConverter.

	self
		assert: (adapter streamConverter: nil) == adapter;
		assert: adapter streamConverter class == XMLNullStreamConverter.! !

XMLEncodingWriteStreamAdapterTest subclass: #XMLEncodingWriteStreamAdapterUTF16BETest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLEncodingWriteStreamAdapterUTF16BETest methodsFor: 'converting' stamp: 'monty 2/12/2016 09:01'!
contentsFromCharacter: aCharacter
	^ self asciiToUTF16BE: aCharacter asString! !

!XMLEncodingWriteStreamAdapterUTF16BETest methodsFor: 'accessing' stamp: 'monty 3/8/2016 16:58'!
decodedInvalidCharacters
	^ self decodedUnicodeInvalidCharacters! !

!XMLEncodingWriteStreamAdapterUTF16BETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 23:36'!
inputCollection
	^ self decodedUTF16String! !

!XMLEncodingWriteStreamAdapterUTF16BETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 23:36'!
outputCollection
	^ self encodedUTF16BEString! !

!XMLEncodingWriteStreamAdapterUTF16BETest methodsFor: 'accessing' stamp: 'monty 2/11/2016 23:36'!
streamConverterClass
	^ XMLUTF16BigEndianStreamConverter! !

XMLEncodingWriteStreamAdapterTest subclass: #XMLEncodingWriteStreamAdapterUTF16LETest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLEncodingWriteStreamAdapterUTF16LETest methodsFor: 'converting' stamp: 'monty 2/12/2016 09:00'!
contentsFromCharacter: aCharacter
	^ self asciiToUTF16LE: aCharacter asString! !

!XMLEncodingWriteStreamAdapterUTF16LETest methodsFor: 'accessing' stamp: 'monty 3/8/2016 16:58'!
decodedInvalidCharacters
	^ self decodedUnicodeInvalidCharacters! !

!XMLEncodingWriteStreamAdapterUTF16LETest methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:00'!
inputCollection
	^ self decodedUTF16String! !

!XMLEncodingWriteStreamAdapterUTF16LETest methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:01'!
outputCollection
	^ self encodedUTF16LEString! !

!XMLEncodingWriteStreamAdapterUTF16LETest methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:00'!
streamConverterClass
	^ XMLUTF16LittleEndianStreamConverter! !

XMLEncodingWriteStreamAdapterTest subclass: #XMLEncodingWriteStreamAdapterUTF32BETest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLEncodingWriteStreamAdapterUTF32BETest methodsFor: 'converting' stamp: 'monty 2/12/2016 09:00'!
contentsFromCharacter: aCharacter
	^ self asciiToUTF32BE: aCharacter asString! !

!XMLEncodingWriteStreamAdapterUTF32BETest methodsFor: 'accessing' stamp: 'monty 3/8/2016 16:58'!
decodedInvalidCharacters
	^ self decodedUnicodeInvalidCharacters! !

!XMLEncodingWriteStreamAdapterUTF32BETest methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:02'!
inputCollection
	^ self decodedUTF32String! !

!XMLEncodingWriteStreamAdapterUTF32BETest methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:03'!
outputCollection
	^ self encodedUTF32BEString! !

!XMLEncodingWriteStreamAdapterUTF32BETest methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:02'!
streamConverterClass
	^ XMLUTF32BigEndianStreamConverter! !

XMLEncodingWriteStreamAdapterTest subclass: #XMLEncodingWriteStreamAdapterUTF32LETest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLEncodingWriteStreamAdapterUTF32LETest methodsFor: 'converting' stamp: 'monty 2/12/2016 09:00'!
contentsFromCharacter: aCharacter
	^ self asciiToUTF32LE: aCharacter asString! !

!XMLEncodingWriteStreamAdapterUTF32LETest methodsFor: 'accessing' stamp: 'monty 3/8/2016 16:58'!
decodedInvalidCharacters
	^ self decodedUnicodeInvalidCharacters! !

!XMLEncodingWriteStreamAdapterUTF32LETest methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:02'!
inputCollection
	^ self decodedUTF32String! !

!XMLEncodingWriteStreamAdapterUTF32LETest methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:03'!
outputCollection
	^ self encodedUTF32LEString! !

!XMLEncodingWriteStreamAdapterUTF32LETest methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:02'!
streamConverterClass
	^ XMLUTF32LittleEndianStreamConverter! !

XMLEncodingWriteStreamAdapterTest subclass: #XMLEncodingWriteStreamAdapterUTF8Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLEncodingWriteStreamAdapterUTF8Test methodsFor: 'accessing' stamp: 'monty 3/8/2016 16:59'!
decodedInvalidCharacters
	^ self decodedUnicodeInvalidCharacters! !

!XMLEncodingWriteStreamAdapterUTF8Test methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:03'!
inputCollection
	^ self decodedUTF8String! !

!XMLEncodingWriteStreamAdapterUTF8Test methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:03'!
outputCollection
	^ self encodedUTF8String! !

!XMLEncodingWriteStreamAdapterUTF8Test methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:04'!
streamConverterClass
	^ XMLUTF8StreamConverter! !

XMLEncodingWriteStreamAdapterTest subclass: #XMLPercentEncodingWriteStreamAdapterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLPercentEncodingWriteStreamAdapterTest methodsFor: 'converting' stamp: 'monty 2/17/2016 11:58'!
contentsFromCharacter: aCharacter
	^ self percentEncodeCharacter: aCharacter! !

!XMLPercentEncodingWriteStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:05'!
inputCollection
	^ self decodedUTF8String! !

!XMLPercentEncodingWriteStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:05'!
outputCollection
	^ self percentEncodedUTF8String! !

!XMLPercentEncodingWriteStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:38'!
streamAdapterClass
	^ XMLPercentEncodingWriteStreamAdapter! !

!XMLPercentEncodingWriteStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/12/2016 00:05'!
streamConverterClass
	^ XMLUTF8StreamConverter! !

!XMLPercentEncodingWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testNextPutByteOrderMark
	self
		should: [self newStreamAdapter nextPutByteOrderMark]
		raise: Error! !

!XMLWriteStreamAdapterTest methodsFor: 'converting' stamp: 'monty 2/12/2016 06:00'!
contentsFromCharacter: aCharacter
	^ aCharacter asString! !

!XMLWriteStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:20'!
inputCollection
	^ 'test'! !

!XMLWriteStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:20'!
outputCollection
	^ 'test'! !

!XMLWriteStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:38'!
streamAdapterClass
	^ XMLWriteStreamAdapter! !

!XMLWriteStreamAdapterTest methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:22'!
streamClass
	^ WriteStream! !

!XMLWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testCR
	| adapter |

	adapter := self newStreamAdapter.
	self
		assert: adapter cr == Character cr;
		assert: adapter contents = (self contentsFromCharacter: Character cr).! !

!XMLWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testLF
	| adapter |

	adapter := self newStreamAdapter.
	self
		assert: adapter lf == Character lf;
		assert: adapter contents = (self contentsFromCharacter: Character lf).! !

!XMLWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testNextPut
	| adapter |

	adapter := self newStreamAdapter.
	self inputCollection do: [:each |
		self assert: (adapter nextPut: each) == each].
	self assert: adapter contents = self outputCollection.! !

!XMLWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testNextPutAll
	| adapter |

	adapter := self newStreamAdapter.
	self
		assert: (adapter nextPutAll: self inputCollection) = self inputCollection;
		assert: adapter contents = self outputCollection.! !

!XMLWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testPosition
	| adapter startPosition position |

	adapter := self newStreamAdapter.
	startPosition := adapter position. "1 or 0 depending on the stream on GS"
	position := startPosition.
	self inputCollection do: [:each |
		adapter nextPut: each.
		self assert: adapter position > position.
		position := adapter position].

	adapter position: startPosition.
	self assert: adapter position = startPosition.! !

!XMLWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testReset
	| adapter startPosition |

	adapter := self newStreamAdapter.
	startPosition := adapter position. "1 or 0 depending on the stream on GS"
	adapter nextPutAll: self inputCollection.
	self
		assert: adapter reset == adapter;
		assert: adapter position = startPosition.! !

!XMLWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testSpace
	| adapter |

	adapter := self newStreamAdapter.
	self
		assert: adapter space == Character space;
		assert: adapter contents = (self contentsFromCharacter: Character space).! !

!XMLWriteStreamAdapterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:39'!
testTab
	| adapter |

	adapter := self newStreamAdapter.
	self
		assert: adapter tab == Character tab;
		assert: adapter contents = (self contentsFromCharacter: Character tab).! !

TestCase subclass: #XMLStreamConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

XMLStreamConverterTest subclass: #XMLByteStreamConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLByteStreamConverterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:43'!
streamConverterClass
	^ XMLByteStreamConverter! !

XMLStreamConverterTest subclass: #XMLNullStreamConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLNullStreamConverterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:43'!
streamConverterClass
	^ XMLNullStreamConverter! !

!XMLNullStreamConverterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:43'!
testIsNull
	self assert: self newStreamConverter isNull! !

!XMLStreamConverterTest class methodsFor: 'testing' stamp: 'monty 3/5/2016 12:50'!
shouldInheritSelectors
	^ true! !

!XMLStreamConverterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 16:17'!
encodingNames
	^ self streamConverterClass encodingNames! !

!XMLStreamConverterTest methodsFor: 'instance creation' stamp: 'monty 3/5/2016 12:40'!
newStreamConverter
	^ self streamConverterClass new! !

!XMLStreamConverterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:40'!
streamConverterClass
	^ XMLStreamConverter! !

!XMLStreamConverterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 16:30'!
testAsXMLStreamConverter
	| streamConverter |

	streamConverter := self newStreamConverter.
	self assert: streamConverter asXMLStreamConverter == streamConverter.! !

!XMLStreamConverterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:58'!
testEncoding
	self encodingNames do: [:each |
		self assert: each asXMLStreamConverter encoding = each].
	self assert:
		((self encodingNames includes: self newStreamConverter encoding)
			or: [self encodingNames isEmpty]).! !

!XMLStreamConverterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:41'!
testIsDefault
	self deny: self newStreamConverter isDefault! !

!XMLStreamConverterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:41'!
testIsNull
	self deny: self newStreamConverter isNull! !

XMLStreamConverterTest subclass: #XMLUTF16BigEndianStreamConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLUTF16BigEndianStreamConverterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:46'!
streamConverterClass
	^ XMLUTF16BigEndianStreamConverter! !

XMLStreamConverterTest subclass: #XMLUTF16LittleEndianStreamConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLUTF16LittleEndianStreamConverterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:46'!
streamConverterClass
	^ XMLUTF16LittleEndianStreamConverter! !

XMLStreamConverterTest subclass: #XMLUTF32BigEndianStreamConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLUTF32BigEndianStreamConverterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:47'!
streamConverterClass
	^ XMLUTF32BigEndianStreamConverter! !

XMLStreamConverterTest subclass: #XMLUTF32LittleEndianStreamConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLUTF32LittleEndianStreamConverterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:46'!
streamConverterClass
	^ XMLUTF32LittleEndianStreamConverter! !

XMLStreamConverterTest subclass: #XMLUTF8StreamConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Streams'!

!XMLUTF8StreamConverterTest methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:44'!
streamConverterClass
	^ XMLUTF8StreamConverter! !

!XMLUTF8StreamConverterTest methodsFor: 'tests' stamp: 'monty 3/5/2016 12:50'!
testIsDefault
	self assert: self newStreamConverter isDefault! !

TestCase subclass: #XMLStringMethodsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLStringMethodsTest methodsFor: 'tests' stamp: 'monty 2/15/2016 23:37'!
testAsXMLEscapedString
	self assert:
		'<test name="&name;">''&value;''</test>' asXMLEscapedString =
			'&lt;test name="&amp;name;"&gt;''&amp;value;''&lt;/test&gt;'! !

!XMLStringMethodsTest methodsFor: 'tests' stamp: 'monty 3/4/2016 16:32'!
testAsXMLFileHandle
	XMLFileHandle hasSupportedImplementation
		ifTrue: [| path fileHandle |
			path := 'test.xml'.
			fileHandle := path asXMLFileHandle.
			self
				assert: (fileHandle isKindOf: XMLFileHandle);
				assert: fileHandle path = path]! !

!XMLStringMethodsTest methodsFor: 'tests' stamp: 'monty 3/6/2016 10:42'!
testAsXMLHTTPRequest
	XMLHTTPRequest hasSupportedImplementation
		ifTrue: [| url request |
			url := 'test.xml'.
			request := url asXMLHTTPRequest.
			self
				assert: (request isKindOf: XMLHTTPRequest);
				assert: request url = url]! !

!XMLStringMethodsTest methodsFor: 'tests' stamp: 'monty 3/5/2016 16:20'!
testAsXMLStreamConverter
	XMLStreamConverter allSubclassesDo: [:class |
		class encodingNames do: [:each |
			self assert: each asXMLStreamConverter class == class]]! !

!XMLStringMethodsTest methodsFor: 'tests' stamp: 'monty 2/15/2016 23:38'!
testAsXMLString
	| node |

	node := 'test' asXMLString.
	self
		assert: node class == XMLString;
		assert: node string = 'test'.! !

!XMLStringMethodsTest methodsFor: 'tests' stamp: 'monty 11/22/2014 20:00'!
testHasPrefixBeforeLocalName
	self
		deny: 'name' hasPrefixBeforeLocalName;
		deny: ':name' hasPrefixBeforeLocalName;
		assert: 'prefix:name' hasPrefixBeforeLocalName;
		assert: '::name' hasPrefixBeforeLocalName;
		assert: 'invalid:prefix:name' hasPrefixBeforeLocalName;
		assert: ':::name' hasPrefixBeforeLocalName;
		assert: ':invalid:prefix:name' hasPrefixBeforeLocalName.! !

!XMLStringMethodsTest methodsFor: 'tests' stamp: 'monty 10/24/2014 15:07'!
testIsQualifiedNameOrLocalName
	self
		assert: ('name' isQualifiedName: 'name' orLocalName: 'name');
		assert: ('name' isQualifiedName: 'prefix:name' orLocalName: 'name');
		assert: ('prefix:name' isQualifiedName: 'prefix:name' orLocalName: 'name');
		deny: ('name' isQualifiedName: 'different' orLocalName: 'different');
		deny: ('name' isQualifiedName: 'prefix:different' orLocalName: 'different');
		deny: ('prefix:name' isQualifiedName: 'prefix:different' orLocalName: 'different').! !

!XMLStringMethodsTest methodsFor: 'tests' stamp: 'monty 11/16/2014 20:22'!
testLocalNameAfterPrefix
	self
		assert: 'name' localNameAfterPrefix = 'name';
		assert: ':name' localNameAfterPrefix = ':name';
		assert: 'prefix:name' localNameAfterPrefix = 'name';
		assert: '::name' localNameAfterPrefix = 'name';
		assert: 'invalid:prefix:name' localNameAfterPrefix = 'name';
		assert: ':::name' localNameAfterPrefix = 'name';
		assert: ':invalid:prefix:name' localNameAfterPrefix = 'name'.! !

!XMLStringMethodsTest methodsFor: 'tests' stamp: 'monty 3/7/2016 20:11'!
testParseXML
	| xml |

	xml := XMLParserTest addressBookXML.
	self assert: xml parseXML printString = (XMLDOMParser parse: xml) printString.! !

!XMLStringMethodsTest methodsFor: 'tests' stamp: 'monty 11/17/2014 06:36'!
testPrefixBeforeLocalName
	self
		assert: 'name' prefixBeforeLocalName = '';
		assert: ':name' prefixBeforeLocalName = '';
		assert: 'prefix:name' prefixBeforeLocalName = 'prefix';
		assert: '::name' prefixBeforeLocalName = ':';
		assert: 'invalid:prefix:name' prefixBeforeLocalName = 'invalid:prefix';
		assert: ':::name' prefixBeforeLocalName = '::';
		assert: ':invalid:prefix:name' prefixBeforeLocalName = ':invalid:prefix'.! !

TestCase subclass: #XMLTokenizerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLTokenizerTest class methodsFor: 'accessing' stamp: 'monty 3/14/2016 21:06'!
badNCNames
	"all NC (non-qualified) names are names, but not all names are NC names"
	^ self badNames, self nonNCNameNames! !

!XMLTokenizerTest class methodsFor: 'accessing' stamp: 'monty 3/14/2016 21:05'!
badNames
	"all names are Nmtokens, but not all Nmtokesn are names"
	^ self badNmtokens, self nonNameNmtokens! !

!XMLTokenizerTest class methodsFor: 'accessing' stamp: 'monty 3/14/2016 21:26'!
badNmtokenChars
	"16r00 is not tested for GS compatibility, neither is 16r110000"
	^ #(16r01 16r0F 16r10
			16r2C 16r2F 16r40 16r5B
			16r5E 16r60 16r7B 16r80
			16rBF 16rD7 16rF7 16r37E
			16r2000 16r200B 16r200E 16r206F
			16r2190 16r2BFF 16r2FFF 16r3000
			16rD800 16rF8FF 16rFDD0 16rFDEF
			16rFFFE 16r0FFFF 16rF0000 16r10FFFF)
			collect: [:each | each asCharacter]! !

!XMLTokenizerTest class methodsFor: 'accessing' stamp: 'monty 3/14/2016 19:43'!
badNmtokens
	^ #('' ' ' '	'), (self badNmtokenChars collect: [:each | 'test' copyWith: each])! !

!XMLTokenizerTest class methodsFor: 'accessing' stamp: 'monty 3/14/2016 21:02'!
nonNCNameNames
	^ #(':' 'test:test' 'test:' ':test')! !

!XMLTokenizerTest class methodsFor: 'accessing' stamp: 'monty 3/14/2016 21:08'!
nonNameNmtokens
	^ self nonNameStartCharNameChars collect: [:each | 'test' copyWithFirst: each]! !

!XMLTokenizerTest class methodsFor: 'accessing' stamp: 'monty 3/14/2016 21:08'!
nonNameStartCharNameChars
	"NameChars that aren't NameStartChars"
	^ #($- $.
			$0 $1 $2 $3 $4 $5 $6 $7 $8 $9
			16rB7 16r0300 16r036F 16r203F 16r2040)
			collect: [:each | each asCharacter]! !

!XMLTokenizerTest methodsFor: 'source' stamp: 'monty 1/12/2014 23:38'!
addressBookXML
	^ XMLParserTest addressBookXML! !

!XMLTokenizerTest methodsFor: 'source' stamp: 'monty 1/12/2014 23:38'!
addressBookXMLWithDTD
	^ XMLParserTest addressBookXMLWithDTD! !

!XMLTokenizerTest methodsFor: 'accessing' stamp: 'monty 3/14/2016 13:37'!
badNCNames
	^ self class badNCNames! !

!XMLTokenizerTest methodsFor: 'accessing' stamp: 'monty 3/14/2016 13:36'!
badNames
	^ self class badNames! !

!XMLTokenizerTest methodsFor: 'accessing' stamp: 'monty 3/14/2016 14:15'!
badNmtokens
	^ self class badNmtokens! !

!XMLTokenizerTest methodsFor: 'accessing' stamp: 'monty 3/14/2016 21:02'!
nonNCNameNames
	^ self class nonNCNameNames! !

!XMLTokenizerTest methodsFor: 'accessing' stamp: 'monty 3/14/2016 14:31'!
nonNameNmtokens
	^ self class nonNameNmtokens! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 1/16/2014 09:08'!
testChangingNormalizedLineEnding
	| xml domParser parser tokenizer |

	xml :=
		'<root>{1}one{2}two{3}three{1}</root>'
			format: (Array with: String lf with: String cr with: String crlf).
	domParser := XMLDOMParser on: xml.

	parser := domParser parser.
	tokenizer := parser tokenizer.
	self assert: tokenizer normalizedLineEndingChar == Character lf.
	
	tokenizer normalizedLineEndingChar: $N.
	self assert: tokenizer normalizedLineEndingChar == $N.

	parser parseDocument.
	self assert: domParser document root contentString = 'NoneNtwoNthreeN'.! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 3/14/2016 13:38'!
testIsNCName
	self badNCNames do: [:each |
		self deny: (self tokenizerClass isNCName: each)].! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 3/14/2016 21:02'!
testIsName
	self nonNCNameNames do: [:each |
		self assert: (self tokenizerClass isName: each)].
	self badNames do: [:each |
		self deny: (self tokenizerClass isName: each)].! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 3/14/2016 21:02'!
testIsNmtoken
	self nonNCNameNames do: [:each |
		self assert: (self tokenizerClass isNmtoken: each)].
	self nonNameNmtokens do: [:each |
		self assert: (self tokenizerClass isNmtoken: each)].
	self badNmtokens do: [:each |
		self deny: (self tokenizerClass isNmtoken: each)].! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 3/14/2016 13:25'!
testNextMatchAll
	| xml tokenizer streamReader readLimit |

	xml := '<test>
test
</test>'.
	readLimit := xml size.
	tokenizer :=
		self tokenizerClass
			driver: nil
			on: xml readStream
			readLimit: readLimit.
	streamReader := tokenizer streamReader.
	self
		deny: (streamReader nextMatchAll: '!!<test>');
		assert: streamReader currentLineNumber = 1;
		assert: streamReader readLimit = readLimit.
	self
		deny: (streamReader nextMatchAll: '<test>!!');
		assert: streamReader currentLineNumber = 1;
		assert: streamReader readLimit = readLimit.
	self
		assert: (streamReader nextMatchAll: '<test>');
		assert: streamReader currentLineNumber = 1;
		assert: streamReader readLimit = (readLimit - '<test>' size).

	tokenizer :=
		self tokenizerClass
			driver: nil
			on: xml readStream
			readLimit: nil.
	streamReader := tokenizer streamReader.
	self
		deny: (streamReader nextMatchAll: '!!<test>');
		assert: streamReader currentLineNumber = 1;
		assert: streamReader readLimit isNil.
	self
		assert: (streamReader nextMatchAll: '<test>');
		assert: streamReader currentLineNumber = 1;
		assert: streamReader readLimit isNil.! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 3/14/2016 14:46'!
testNextNCNameFrom
	| readStream writeStream |

	self badNCNames do: [:each |
		readStream := each readStream.
		writeStream := String new writeStream.
		(self tokenizerClass
			nextNCNameFrom: readStream
			on: writeStream)
			ifTrue: [self deny: writeStream contents = each]].! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 3/14/2016 21:02'!
testNextNameFromOn
	| readStream writeStream |

	self nonNCNameNames do: [:each |
		readStream := each readStream.
		writeStream := String new writeStream.
		self assert:
			(self tokenizerClass
				nextNameFrom: readStream
				on: writeStream).
		self assert: writeStream contents = each].

	self badNames do: [:each |
		readStream := each readStream.
		writeStream := String new writeStream.
		(self tokenizerClass
			nextNameFrom: readStream
			on: writeStream)
			ifTrue: [self deny: writeStream contents = each]].! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 3/14/2016 21:02'!
testNextNmtokenFromOn
	| readStream writeStream |

	self nonNCNameNames, self nonNameNmtokens do: [:each |
		readStream := each readStream.
		writeStream := String new writeStream.
		self assert:
			(self tokenizerClass
				nextNmtokenFrom: readStream
				on: writeStream).
		self assert: writeStream contents = each].

	self badNmtokens do: [:each |
		readStream := each readStream.
		writeStream := String new writeStream.
		(self tokenizerClass
			nextNmtokenFrom: readStream
			on: writeStream)
			ifTrue: [self deny: writeStream contents = each]].! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 3/14/2016 13:27'!
testSkipSeparators
	| xml tokenizer streamReader readLimit |

	xml := '
	<test/>'.
	readLimit := xml size.
	tokenizer :=
		self tokenizerClass
			driver: nil
			on: xml readStream
			readLimit: readLimit.
	streamReader := tokenizer streamReader.
	2 timesRepeat: [
		streamReader skipSeparators.
		self
			assert: streamReader currentLineNumber = 2;
			assert: streamReader readLimit = (readLimit - ((xml copyUpTo: $<) size))].

	tokenizer :=
		self tokenizerClass
			driver: nil
			on: xml readStream
			readLimit: nil.
	streamReader := tokenizer streamReader.
	2 timesRepeat: [
		streamReader skipSeparators.
		self
			assert: streamReader currentLineNumber = 2;
			assert: streamReader readLimit isNil].! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 3/14/2016 13:26'!
testStreamReadingPortability
	| tokenizer stream |

	tokenizer :=
		self tokenizerClass
			driver: nil
			on: (stream := XMLExceptionRaisingReadStream on: self addressBookXML).
	self
		shouldnt: [
			[tokenizer streamReader
				peek;
				next] doWhileFalse: [tokenizer streamReader atEnd]]
		raise: Error.

	self assert: stream atEnd.
	self should: [stream peek] raise: Error.
	self should: [stream next] raise: Error.! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 3/14/2016 13:26'!
testTokenizerAddressBook
	| tokenizer |

	tokenizer :=
		self tokenizerClass
			driver: nil
			on: self addressBookXML readStream.
	self
		shouldnt: [[tokenizer streamReader next notNil] whileTrue]
		raise: XMLParserException.! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 3/14/2016 13:26'!
testTokenizerAddressBookWithDTD
	| tokenizer |

	tokenizer :=
		self tokenizerClass
			driver: nil
			on: self addressBookXMLWithDTD readStream.
	self
		shouldnt: [[tokenizer streamReader next notNil] whileTrue]
		raise: XMLParserException.! !

!XMLTokenizerTest methodsFor: 'tests' stamp: 'monty 3/14/2016 13:26'!
testTokenizerCharacters
	| tokenizer |
	"This test is actually not that useful. This is not the proper way of using the parser.
	This test is here just for specification purpose"
	"self debug: #testParsingCharacters"

	tokenizer :=
		self tokenizerClass
			driver: nil
			on: self addressBookXML readStream.

	self
		assert: tokenizer streamReader next == $<;
		assert: ('address' allSatisfy: [:each | tokenizer streamReader next == each]);
		assert: tokenizer streamReader peek == $b.

	tokenizer streamReader pushBack: 'test'.
	self
		assert: ('test' allSatisfy: [:each | tokenizer streamReader next == each]);
		assert: tokenizer streamReader peek == $b;
		assert: ('book' allSatisfy: [:each | tokenizer streamReader next == each]).

	tokenizer streamReader
		pushBack: 'book';
		pushBack: 'test'.
	self
		assert: ('test' allSatisfy: [:each | tokenizer streamReader next == each]);
		assert: ('book' allSatisfy: [:each | tokenizer streamReader next == each]).! !

!XMLTokenizerTest methodsFor: 'accessing' stamp: 'monty 3/14/2016 13:25'!
tokenizerClass
	^ XMLWellFormedParserTokenizer! !

TestCase subclass: #XMLURITest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-URI'!

!XMLURITest class methodsFor: 'private' stamp: 'monty 3/22/2015 03:15'!
generateStandardURICombiningTest
	"self generateStandardURICombiningTest"
	| readStream isFirstTest source |

	readStream := self standardURICombiningTestTemplate readStream.
	isFirstTest := true.
	source :=
		String streamContents: [:writeStream |
			writeStream nextPutAll: 'testStandardURICombining
	| baseURI |

	baseURI := ''', self standardURICombiningTestBaseURI, ''' asXMLURI.
	self'.
			[readStream atEnd]
				whileFalse: [| argument result |
					readStream skipTo: $".
					argument := readStream upTo: $".
					readStream skipTo: $".
					result := readStream upTo: $".
					(argument isEmpty and: [result isEmpty])
						ifFalse: [
							isFirstTest
								ifTrue: [isFirstTest := false]
								ifFalse: [writeStream nextPut: $;].
							writeStream nextPutAll: '
		assert: (baseURI / ''', argument, ''' asXMLURI) = ''', result, ''' asXMLURI']]].

	self
		compile: source
		classified: 'tests'.! !

!XMLURITest class methodsFor: 'private' stamp: 'monty 2/9/2015 07:06'!
standardURICombiningTestBaseURI
	^ 'http://a/b/c/d;p?q'! !

!XMLURITest class methodsFor: 'private' stamp: 'monty 2/9/2015 07:06'!
standardURICombiningTestTemplate
	^ '
      "g:h"           =  "g:h"
      "g"             =  "http://a/b/c/g"
      "./g"           =  "http://a/b/c/g"
      "g/"            =  "http://a/b/c/g/"
      "/g"            =  "http://a/g"
      "//g"           =  "http://g"
      "?y"            =  "http://a/b/c/d;p?y"
      "g?y"           =  "http://a/b/c/g?y"
      "#s"            =  "http://a/b/c/d;p?q#s"
      "g#s"           =  "http://a/b/c/g#s"
      "g?y#s"         =  "http://a/b/c/g?y#s"
      ";x"            =  "http://a/b/c/;x"
      "g;x"           =  "http://a/b/c/g;x"
      "g;x?y#s"       =  "http://a/b/c/g;x?y#s"
      ""              =  "http://a/b/c/d;p?q"
      "."             =  "http://a/b/c/"
      "./"            =  "http://a/b/c/"
      ".."            =  "http://a/b/"
      "../"           =  "http://a/b/"
      "../g"          =  "http://a/b/g"
      "../.."         =  "http://a/"
      "../../"        =  "http://a/"
      "../../g"       =  "http://a/g"

      "../../../g"    =  "http://a/g"
      "../../../../g" =  "http://a/g"


      "/./g"          =  "http://a/g"
      "/../g"         =  "http://a/g"
      "g."            =  "http://a/b/c/g."
      ".g"            =  "http://a/b/c/.g"
      "g.."           =  "http://a/b/c/g.."
      "..g"           =  "http://a/b/c/..g"

      "./../g"        =  "http://a/b/g"
      "./g/."         =  "http://a/b/c/g/"
      "g/./h"         =  "http://a/b/c/g/h"
      "g/../h"        =  "http://a/b/c/h"
      "g;x=1/./y"     =  "http://a/b/c/g;x=1/y"
      "g;x=1/../y"    =  "http://a/b/c/y"

      "g?y/./x"       =  "http://a/b/c/g?y/./x"
      "g?y/../x"      =  "http://a/b/c/g?y/../x"
      "g#s/./x"       =  "http://a/b/c/g#s/./x"
      "g#s/../x"      =  "http://a/b/c/g#s/../x"

      "http:g"        =  "http:g"
'! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/8/2014 18:17'!
absolute
	^ '/absolute/path'! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/8/2014 18:17'!
absoluteDir
	^ '/absolute/path/'! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/8/2014 18:17'!
absoluteDirRelativeCombined
	^ '/absolute/path/relative/path'! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/6/2016 02:46'!
absoluteDirRelativeCombinedSegments
	^ #('' 'absolute' 'path' 'relative' 'path') asOrderedCollection! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/6/2016 02:46'!
absoluteDirSegments
	^ #('' 'absolute' 'path' '') asOrderedCollection! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/8/2014 18:17'!
absoluteRelativeCombined
	^ '/absolute/relative/path'! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/6/2016 02:47'!
absoluteRelativeCombinedSegments
	^ #('' 'absolute' 'relative' 'path') asOrderedCollection! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/6/2016 02:46'!
absoluteSegments
	^ #('' 'absolute' 'path') asOrderedCollection! !

!XMLURITest methodsFor: 'enumerating' stamp: 'monty 1/13/2016 06:39'!
allURIStringsDo: aBlock
	aBlock
		value: '';
		value: '/';
		value: self absolute;
		value: self absoluteDir;
		value: self absoluteDirRelativeCombined;
		value: self absoluteRelativeCombined;
		value: self domainURIString;
		value: self domainURIWithPortString;
		value: self emailURIString;
		value: self fileURIWithHostString;
		value: self fileURIString;
		value: self fragmentURIString;
		value: self fullURIString;
		value: self fullURIWithPortString;
		value: self queryURIString;
		value: self relative;
		value: self relativeAbsoluteCombined;
		value: self schemelessURIString;
		value: self schemelessURIWithPortString! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/11/2016 08:12'!
alphaNumericCharacters
	^ 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789'! !

!XMLURITest methodsFor: 'enumerating' stamp: 'monty 1/7/2016 05:00'!
asciiCharactersDo: aBlock
	0 to: 127 do: [:each | aBlock value: each asCharacter]! !

!XMLURITest methodsFor: 'asserting' stamp: 'monty 1/7/2016 03:22'!
assertURI: aFirstURI atHostEquals: aSecondURI
	self assert: aFirstURI host = aSecondURI host.
	self
		assertURI: aFirstURI
		atPortEquals: aSecondURI.! !

!XMLURITest methodsFor: 'asserting' stamp: 'monty 1/7/2016 03:23'!
assertURI: aFirstURI atPathEquals: aSecondURI
	self assert: aFirstURI pathSegments = aSecondURI pathSegments.
	self
		assertURI: aFirstURI
		atQueryEquals: aSecondURI.! !

!XMLURITest methodsFor: 'asserting' stamp: 'monty 1/7/2016 03:23'!
assertURI: aFirstURI atPortEquals: aSecondURI
	self assert: aFirstURI port = aSecondURI port.
	self
		assertURI: aFirstURI
		atPathEquals: aSecondURI.! !

!XMLURITest methodsFor: 'asserting' stamp: 'monty 1/7/2016 03:24'!
assertURI: aFirstURI atQueryEquals: aSecondURI
	self
		assert: aFirstURI query = aSecondURI query;
		assert: aFirstURI fragment = aSecondURI fragment! !

!XMLURITest methodsFor: 'asserting' stamp: 'monty 1/7/2016 03:22'!
assertURI: aFirstURI atUserInfoEquals: aSecondURI
	self  assert: aFirstURI userInfo = aSecondURI userInfo.
	self
		assertURI: aFirstURI
		atHostEquals: aSecondURI.! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/8/2016 05:18'!
domainURIString
	^ 'http://www.example.com'! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/8/2016 05:18'!
domainURIWithPortString
	^ self domainURIString, ':80'! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/5/2016 06:17'!
emailURIString
	^ 'mailto:userName@', self host! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/8/2014 18:17'!
fileURIString
	^ 'file://', self absolute! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/8/2016 05:19'!
fileURIWithHostString
	^ 'file://', self host, self absolute! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/13/2016 06:38'!
fragment
	^ self fragmentURIString allButFirst! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/13/2016 06:31'!
fragmentURIString
	^ '#fragmentID'! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/13/2016 06:39'!
fullURIString
	^ self domainURIString, self absolute, self queryURIString, self fragmentURIString! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/13/2016 06:39'!
fullURIWithPortString
	^ self domainURIWithPortString, self absolute, self queryURIString, self fragmentURIString! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/8/2016 05:18'!
host
	^ self domainURIString copyAfterLast: $/! !

!XMLURITest methodsFor: 'testing' stamp: 'monty 1/11/2016 08:12'!
isUnreservedCharacter: aCharacter
	^ self unreservedCharacters includes: aCharacter! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/11/2016 08:13'!
nonPercentEncodedPathCharacters
	^ self unreservedCharacters, self subDelimiterCharacters, ':@'! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/12/2016 10:04'!
query
	^ OrderPreservingStringDictionary
		with: 'name1' -> 'value1'
		with: 'name2' -> 'value2'! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/13/2016 06:39'!
queryURIString
	^ '?name1=value1&name2=value2'! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/8/2014 18:17'!
relative
	^ 'relative/path'! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 2/9/2015 00:47'!
relativeAbsoluteCombined
	^ 'relative/absolute/path'! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/6/2016 02:50'!
relativeAbsoluteCombinedSegments
	^ #('relative' 'absolute' 'path') asOrderedCollection! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/6/2016 02:50'!
relativeSegments
	^ #('relative' 'path') asOrderedCollection! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/13/2016 06:39'!
schemelessURIString
	^ (self domainURIString copyAfter: $:), self absolute, self queryURIString, self fragmentURIString! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/13/2016 06:39'!
schemelessURIWithPortString
	^ (self domainURIWithPortString copyAfter: $:), self absolute, self queryURIString, self fragmentURIString! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/11/2016 08:13'!
subDelimiterCharacters
	^ '!!$&''()*+,;='! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 3/4/2016 16:31'!
testAsRetrievableResource
	(XMLFileHandle hasSupportedImplementation
		and: [XMLHTTPRequest hasSupportedImplementation])
		ifFalse: [^ self].

	self allURIStringsDo: [:each | | uri retrievableResource |
		uri := each asXMLURI.
		retrievableResource := uri asRetrievableResource.
		uri isLocalURI
			ifTrue: [
				self
					assert: (retrievableResource isKindOf: XMLFileHandle);
					assert: (retrievableResource path = uri path)]
			ifFalse: [
				self
					assert: (retrievableResource isKindOf: XMLHTTPRequest);
					assert: (retrievableResource url = uri printString)]]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/7/2016 04:23'!
testAsString
	self allURIStringsDo: [:each | | uri |
		uri := each asXMLURI.
		self assert: uri asString = uri printString]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/5/2016 23:54'!
testAsXMLURI
	self allURIStringsDo: [:each | | uri |
		uri := XMLURI fromString: each.
		self
			assert: uri class == XMLURI;
			assert: uri uriString = each;
			assert: uri asXMLURI == uri]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 11:39'!
testCombining
	self allURIStringsDo: [:uriString | | uri |
		uri := uriString asXMLURI.
		self allURIStringsDo: [:uriStringToCombineWith | | uriToCombineWith combinedURI |
			uriToCombineWith := uriStringToCombineWith asXMLURI.
			combinedURI := uri / uriToCombineWith.
			uriToCombineWith isRelative
				ifTrue: [
					self assert: combinedURI scheme = uri scheme.
					uriToCombineWith hasUserInfo
						ifTrue: [
							self
								assertURI: combinedURI
								atUserInfoEquals: uriToCombineWith]
						ifFalse: [
							self assert: combinedURI userInfo = uri userInfo.
							uriToCombineWith hasHost
								ifTrue: [
									self
										assertURI: combinedURI
										atHostEquals: uriToCombineWith]
								ifFalse: [
									self assert: combinedURI host = uri host.
									uriToCombineWith hasPort
										ifTrue: [
											self
												assertURI: combinedURI
												atPortEquals: uriToCombineWith]
										ifFalse: [
											self assert: combinedURI port = uri port.
											uriToCombineWith hasPath
												ifTrue: [
													uriToCombineWith hasAbsolutePath
														ifTrue: [
															self
																assertURI: combinedURI
																atPathEquals: uriToCombineWith]
														ifFalse: [self deny: combinedURI = uri path]]
												ifFalse: [
													self assert: combinedURI pathSegments = uri pathSegments.
													uriToCombineWith hasQuery
														ifTrue: [
															self
																assertURI: combinedURI
																atQueryEquals: uriToCombineWith]
														ifFalse: [
															self assert: combinedURI query = uri query.
															uriToCombineWith hasFragment
																ifTrue: [
																	self assert:
																		combinedURI fragment =
																			uriToCombineWith fragment]
																ifFalse: [
																	self assert:
																		combinedURI fragment =
																			uri fragment]]]]]].
					uriToCombineWith isEmpty
						ifTrue: [self assert: combinedURI == uri]
						ifFalse: [self deny: combinedURI == uri]]
				ifFalse: [
					"if the arg was an absolute URI, it should have been returned"
					self assert: combinedURI == uriToCombineWith].
			"receiver and arg should be unchanged by combining"
			self
				assert: uri = uriString asXMLURI;
				assert: uriToCombineWith = uriStringToCombineWith asXMLURI]]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 05:18'!
testCombiningAbsoluteURIs
	| domainURI fullURI fileURI |

	domainURI := self domainURIString asXMLURI.
	fullURI := self fullURIString asXMLURI.
	fileURI := self fileURIString asXMLURI.
	self
		assert: domainURI / self fullURIString = fullURI;
		assert: domainURI / self domainURIString = domainURI;
		assert: domainURI / self fileURIString = fileURI;
		assert: fullURI / self fullURIString = fullURI;
		assert: fullURI / self domainURIString = domainURI;
		assert: fullURI / self fileURIString = fileURI;
		assert: fileURI / self fullURIString = fullURI;
		assert: fileURI / self domainURIString = domainURI;
		assert: fileURI / self fileURIString = fileURI.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 05:18'!
testCombiningDotPathURIs
	| relativePathURI absolutePathURI fullURI |

	relativePathURI := self relative asXMLURI.
	self
		assert: relativePathURI / '.' = 'relative/' asXMLURI;
		assert: relativePathURI / './.' = 'relative/' asXMLURI;
		assert: relativePathURI / './path' = relativePathURI;
		assert: relativePathURI / ('.' , self absolute) = self relativeAbsoluteCombined asXMLURI.

	absolutePathURI := self absolute asXMLURI.
	self
		assert: absolutePathURI / '.' = '/absolute/' asXMLURI;
		assert: absolutePathURI / './.' = '/absolute/' asXMLURI;
		assert: absolutePathURI / './path' = absolutePathURI;
		assert: absolutePathURI / ('./' , self relative) = self absoluteRelativeCombined asXMLURI.

	fullURI := self fullURIString asXMLURI.
	self
		assert: fullURI / '.' = (self domainURIString , '/absolute/') asXMLURI;
		assert: fullURI / './.' = (self domainURIString , '/absolute/') asXMLURI;
		assert: fullURI / './path' = (self domainURIString , self absolute) asXMLURI;
		assert: fullURI / ('./' , self relative) = (self domainURIString , self absoluteRelativeCombined) asXMLURI.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 05:18'!
testCombiningDoubleDotPathURIs
	| relativePathURI absolutePathURI fullURI |

	relativePathURI := self relative asXMLURI.
	self
		assert: (relativePathURI / '..') isEmpty;
		assert: (relativePathURI / '../..') isEmpty;
		assert: relativePathURI / ('..' , self absolute) = 'absolute/path' asXMLURI;
		assert: relativePathURI / ('../..' , self absolute) = 'absolute/path' asXMLURI;
		assert: relativePathURI / ('../..' , self absolute , '/..') = 'absolute/' asXMLURI;
		assert: (relativePathURI / ('../..' , self absolute , '/../..')) isEmpty.

	absolutePathURI := self absolute asXMLURI.
	self
		assert: (absolutePathURI / '..') hasRootPath;
		assert: (absolutePathURI / '../..') isEmpty;
		assert: absolutePathURI / ('..' , self absolute) = absolutePathURI;
		assert: absolutePathURI / ('../..' , self absolute) = 'absolute/path' asXMLURI;
		assert: absolutePathURI / ('../..' , self absolute , '/..') = 'absolute/' asXMLURI;
		assert: (absolutePathURI / ('../..' , self absolute , '/../..')) isEmpty.

	fullURI := self fullURIString asXMLURI.
	self
		assert: (fullURI / '..') hasRootPath;
		deny: (fullURI / '../..') hasPath;
		assert: fullURI / ('..' , self absolute) = (self domainURIString , self absolute) asXMLURI;
		assert: fullURI / ('../..' , self absolute) = (self domainURIString , self absolute) asXMLURI;
		assert: fullURI / ('../..' , self absolute , '/..') = (self domainURIString , '/absolute/') asXMLURI;
		assert: fullURI / ('../..' , self absolute , '/../') = (self domainURIString , '/absolute/') asXMLURI.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testCombiningRelativeURIs
	| domainURI domainURIWithPort absolutePathURI absoluteDirPathURI |

	domainURI := self domainURIString asXMLURI.
	domainURIWithPort := self domainURIWithPortString asXMLURI.
	absolutePathURI := (self domainURIString , self absolute) asXMLURI.
	absoluteDirPathURI := (self domainURIString , self absoluteDir) asXMLURI.
	self
		assert: domainURI / self relative = (self domainURIString , '/' , self relative) asXMLURI;
		assert: domainURI / self absolute = (self domainURIString , self absolute) asXMLURI;
		assert: domainURI / self absoluteDir = (self domainURIString , self absoluteDir) asXMLURI;
		assert: domainURIWithPort / self relative = (self domainURIWithPortString , '/' , self relative) asXMLURI;
		assert: domainURIWithPort / self absolute = (self domainURIWithPortString , self absolute) asXMLURI;
		assert: domainURIWithPort / self absoluteDir = (self domainURIWithPortString , self absoluteDir) asXMLURI;
		assert: absolutePathURI / self relative = (self domainURIString , self absoluteRelativeCombined) asXMLURI;
		assert: absoluteDirPathURI / self relative = (self domainURIString , self absoluteDirRelativeCombined) asXMLURI;
		assert: absolutePathURI / self queryURIString = (self domainURIString , self absolute , self queryURIString) asXMLURI;
		assert: absolutePathURI / self fragmentURIString = (self domainURIString , self absolute , self fragmentURIString) asXMLURI;
		assert: domainURI / self schemelessURIString = self fullURIString asXMLURI.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/4/2016 08:07'!
testCopy
	self allURIStringsDo: [:each | | uri copy |
		uri := each asXMLURI.
		copy := uri copy.
		self
			assert: uri uriString = each;
			assert: copy uriString isNil;
			assert: uri = each asXMLURI;
			deny: uri == copy;
			assert: uri = copy;
			assert: uri scheme = copy scheme;
			assert: uri hasAuthorityPrefix = copy hasAuthorityPrefix;
			assert: uri userInfo = copy userInfo;
			assert: uri host = copy host;
			assert: uri port = copy port;
			assert: uri pathSegments = copy pathSegments;
			assert: uri query = copy query;
			assert: uri fragment = copy fragment]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/6/2016 18:40'!
testCopyWithFragment
	self allURIStringsDo: [:each | | uri oldFragment newFragment copy |
		uri := each asXMLURI.
		oldFragment := uri fragment.
		newFragment := oldFragment, 'new'.
		copy := uri copyWithFragment: newFragment.
		self
			assert: uri = each asXMLURI;
			assert: uri uriString = each;
			assert: copy uriString isNil;
			deny: uri == copy;
			deny: uri = copy;
			assert: uri scheme = copy scheme;
			assert: uri hasAuthorityPrefix = copy hasAuthorityPrefix;
			assert: uri userInfo = copy userInfo;
			assert: uri host = copy host;
			assert: uri port = copy port;
			assert: uri pathSegments = copy pathSegments;
			assert: uri query = copy query;
			deny: uri fragment = copy fragment;
			assert: uri fragment = oldFragment;
			assert: copy fragment = newFragment]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/6/2016 18:40'!
testCopyWithHost
	self allURIStringsDo: [:each | | uri oldHost newHost copy |
		uri := each asXMLURI.
		oldHost := uri host.
		newHost := oldHost, 'new'.
		copy := uri copyWithHost: newHost.
		self
			assert: uri = each asXMLURI;
			assert: uri uriString = each;
			assert: copy uriString isNil;
			deny: uri == copy;
			deny: uri = copy;
			assert: uri scheme = copy scheme;
			assert: uri hasAuthorityPrefix = copy hasAuthorityPrefix;
			assert: uri userInfo = copy userInfo;
			deny: uri host = copy host;
			assert: uri host = oldHost;
			assert: copy host = newHost;
			assert: uri port = copy port;
			assert: uri pathSegments = copy pathSegments;
			assert: uri query = copy query;
			assert: uri fragment = copy fragment]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/12/2016 07:28'!
testCopyWithPath
	self allURIStringsDo: [:each | | uri oldPath newPath copy |
		uri := each asXMLURI.
		oldPath := uri path.
		newPath := oldPath, '/new'.
		copy := uri copyWithPath: newPath.
		self
			assert: uri = each asXMLURI;
			assert: uri uriString = each;
			assert: copy uriString isNil;
			deny: uri == copy;
			deny: uri = copy;
			assert: uri scheme = copy scheme;
			assert: uri hasAuthorityPrefix = copy hasAuthorityPrefix;
			assert: uri userInfo = copy userInfo;
			assert: uri host = copy host;
			assert: uri port = copy port;
			deny: uri path = copy path;
			assert: uri path = oldPath;
			assert: copy path = newPath;
			assert: uri query = copy query;
			assert: uri fragment = copy fragment]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/6/2016 18:45'!
testCopyWithPathSegments
	self allURIStringsDo: [:each | | uri oldPathSegments newPathSegments copy |
		uri := each asXMLURI.
		oldPathSegments := uri pathSegments.
		newPathSegments := oldPathSegments copyWith: 'new'.
		copy := uri copyWithPathSegments: newPathSegments.
		self
			assert: uri = each asXMLURI;
			assert: uri uriString = each;
			assert: copy uriString isNil;
			deny: uri == copy;
			deny: uri = copy;
			assert: uri scheme = copy scheme;
			assert: uri hasAuthorityPrefix = copy hasAuthorityPrefix;
			assert: uri userInfo = copy userInfo;
			assert: uri host = copy host;
			assert: uri port = copy port;
			deny: uri pathSegments = copy pathSegments;
			assert: uri pathSegments = oldPathSegments;
			assert: copy pathSegments = newPathSegments;
			assert: uri query = copy query;
			assert: uri fragment = copy fragment]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/6/2016 18:41'!
testCopyWithPort
	self allURIStringsDo: [:each | | uri oldPort newPort copy |
		uri := each asXMLURI.
		oldPort := uri port.
		newPort := (oldPort ifNil: [0]) + 1.
		copy := uri copyWithPort: newPort.
		self
			assert: uri = each asXMLURI;
			assert: uri uriString = each;
			assert: copy uriString isNil;
			deny: uri == copy;
			deny: uri = copy;
			assert: uri scheme = copy scheme;
			assert: uri hasAuthorityPrefix = copy hasAuthorityPrefix;
			assert: uri userInfo = copy userInfo;
			assert: uri host = copy host;
			deny: uri port = copy port;
			assert: uri port = oldPort;
			assert: copy port = newPort;
			assert: uri pathSegments = copy pathSegments;
			assert: uri query = copy query;
			assert: uri fragment = copy fragment]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/6/2016 18:44'!
testCopyWithQuery
	self allURIStringsDo: [:each | | uri oldQuery newQuery copy |
		uri := each asXMLURI.
		oldQuery := uri query.
		(newQuery := oldQuery copy) add: 'new' -> 'new'.
		copy := uri copyWithQuery: newQuery.
		self
			assert: uri = each asXMLURI;
			assert: uri uriString = each;
			assert: copy uriString isNil;
			deny: uri == copy;
			deny: uri = copy;
			assert: uri scheme = copy scheme;
			assert: uri hasAuthorityPrefix = copy hasAuthorityPrefix;
			assert: uri userInfo = copy userInfo;
			assert: uri host = copy host;
			assert: uri port = copy port;
			assert: uri pathSegments = copy pathSegments;
			deny: uri query = copy query;
			assert: uri query = oldQuery;
			assert: copy query = newQuery;
			assert: uri fragment = copy fragment]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/12/2016 11:39'!
testCopyWithScheme
	self allURIStringsDo: [:each | | uri oldScheme oldHasAuthorityPrefix newScheme copy |
		uri := each asXMLURI.
		oldScheme := uri scheme.
		oldHasAuthorityPrefix := uri hasAuthorityPrefix.
		newScheme := oldScheme, 'new'.
		copy := uri copyWithScheme: newScheme.
		self
			assert: uri = each asXMLURI;
			assert: uri uriString = each;
			assert: copy uriString isNil;
			deny: uri == copy;
			deny: uri = copy;
			deny: uri scheme = copy scheme;
			assert: uri scheme = oldScheme;
			assert: copy scheme = newScheme;
			assert: uri hasAuthorityPrefix = oldHasAuthorityPrefix.
		"If the original had no scheme and no // prefix, then the copy should
		gain one. Otherwise it should only have one if the original did."
		(oldScheme isEmpty
			and: [oldHasAuthorityPrefix not])
			ifTrue: [self assert: copy hasAuthorityPrefix]
			ifFalse: [self assert: copy hasAuthorityPrefix = oldHasAuthorityPrefix].
		self
			assert: uri userInfo = copy userInfo;
			assert: uri host = copy host;
			assert: uri port = copy port;
			assert: uri pathSegments = copy pathSegments;
			assert: uri query = copy query;
			assert: uri fragment = copy fragment]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/6/2016 18:43'!
testCopyWithUserInfo
	self allURIStringsDo: [:each | | uri oldUserInfo newUserInfo copy |
		uri := each asXMLURI.
		oldUserInfo := uri userInfo.
		newUserInfo := oldUserInfo, 'new'.
		copy := uri copyWithUserInfo: newUserInfo.
		self
			assert: uri = each asXMLURI;
			assert: uri uriString = each;
			assert: copy uriString isNil;
			deny: uri == copy;
			deny: uri = copy;
			assert: uri scheme = copy scheme;
			assert: uri hasAuthorityPrefix = copy hasAuthorityPrefix;
			deny: uri userInfo = copy userInfo;
			assert: uri userInfo = oldUserInfo;
			assert: copy userInfo = newUserInfo;
			assert: uri host = copy host;
			assert: uri port = copy port;
			assert: uri pathSegments = copy pathSegments;
			assert: uri query = copy query;
			assert: uri fragment = copy fragment]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testDefaultAuthorityPrefixPrinting
	| uri copy |

	uri := '' asXMLURI.

	copy := uri copyWithUserInfo: 'userInfo'.
	self assert: copy printString = ('//userInfo@').

	copy := uri copyWithHost: self host.
	self assert: copy printString = ('//', self host).

	copy := uri copyWithPort: 80.
	self assert: copy printString = '//:80'.

	copy := uri copyWithPath: self relative.
	self assert: copy printString = self relative.

	copy := uri copyWithPath: self absolute.
	self assert: copy printString = self absolute.

	copy := uri copyWithQuery: self query.
	self assert: copy printString = self queryURIString.

	copy := uri copyWithFragment: self fragment.
	self assert: copy printString = self fragmentURIString.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testDefaultPathPrefixPrinting
	| uri copy |

	uri := self relative asXMLURI.

	copy := uri copyWithUserInfo: 'userInfo'.
	self assert: copy printString = ('//userInfo@/', self relative).

	copy := uri copyWithHost: self host.
	self assert: copy printString = ('//', self host, '/', self relative).

	copy := uri copyWithPort: 80.
	self assert: copy printString = ('//:80/', self relative).

	copy := uri copyWithScheme: 'http'.
	self assert: copy printString = ('http:///', self relative).

	copy := uri copyWithQuery: self query.
	self assert: copy printString = (self relative, self queryURIString).

	copy := uri copyWithFragment: self fragment.
	self assert: copy printString = (self relative, self fragmentURIString).! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 11:48'!
testEquality
	self allURIStringsDo: [:each | | uri otherURI |
		uri := each asXMLURI.

		otherURI := each asXMLURI.
		self
			assert: uri = otherURI;
			assert: otherURI = uri.
		otherURI scheme: uri scheme, 'new'.
		self
			deny: uri = otherURI;
			deny: otherURI = uri.

		otherURI := each asXMLURI.
		self
			assert: uri = otherURI;
			assert: otherURI = uri.
		otherURI userInfo: uri userInfo, 'new'.
		self
			deny: uri = otherURI;
			deny: otherURI = uri.

		otherURI := each asXMLURI.
		self
			assert: uri = otherURI;
			assert: otherURI = uri.
		otherURI host: uri host, 'new'.
		self
			deny: uri = otherURI;
			deny: otherURI = uri.

		otherURI := each asXMLURI.
		self
			assert: uri = otherURI;
			assert: otherURI = uri.
		otherURI pathSegments: (uri pathSegments copyWith: 'new').
		self
			deny: uri = otherURI;
			deny: otherURI = uri.

		otherURI := each asXMLURI.
		self
			assert: uri = otherURI;
			assert: otherURI = uri.
		otherURI query:
			(uri query copy
				add: 'new' -> '';
				yourself).
		self
			deny: uri = otherURI;
			deny: otherURI = uri.

		otherURI := each asXMLURI.
		self
			assert: uri = otherURI;
			assert: otherURI = uri.
		otherURI fragment: uri fragment, 'new'.
		self
			deny: uri = otherURI;
			deny: otherURI = uri]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testFragment
	self
		assert: '' asXMLURI fragment isEmpty;
		assert: '/' asXMLURI fragment isEmpty;
		assert: self absolute asXMLURI fragment isEmpty;
		assert: self absoluteDir asXMLURI fragment isEmpty;
		assert: self absoluteDirRelativeCombined asXMLURI fragment isEmpty;
		assert: self absoluteRelativeCombined asXMLURI fragment isEmpty;
		assert: self domainURIString asXMLURI fragment isEmpty;
		assert: self domainURIWithPortString asXMLURI fragment isEmpty;
		assert: self emailURIString asXMLURI fragment isEmpty;
		assert: self fileURIWithHostString asXMLURI fragment isEmpty;
		assert: self fileURIString asXMLURI fragment isEmpty;
		assert: self fragmentURIString asXMLURI fragment = self fragment;
		assert: self fullURIString asXMLURI fragment = self fragment;
		assert: self fullURIWithPortString asXMLURI fragment = self fragment;
		assert: self queryURIString asXMLURI fragment isEmpty;
		assert: self relative asXMLURI fragment isEmpty;
		assert: self relativeAbsoluteCombined asXMLURI fragment isEmpty;
		assert: self schemelessURIString asXMLURI fragment = self fragment;
		assert: self schemelessURIWithPortString asXMLURI fragment = self fragment! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/11/2016 08:14'!
testFragmentNonPercentEncodedCharacters
	| fragment uri |

	fragment := self nonPercentEncodedPathCharacters , '/?'.
	uri := ('#', fragment) asXMLURI.
	self assert: uri fragment = fragment.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/11/2016 08:14'!
testFragmentPercentEncodedCharacters
	| uri |

	self asciiCharactersDo: [:each | | encodedURIString |
		encodedURIString := '#', (XMLURI percentEncodeCharacter: each).
		uri := encodedURIString asXMLURI.
		self assert: uri fragment = each asString.

		uri uriString: nil.
		(self isUnreservedCharacter: each)
			ifTrue: [self assert: uri printString = ('#' copyWith: each)]
			ifFalse: [self assert: uri printString = encodedURIString]].

	uri := '#+' asXMLURI.
	self assert: uri fragment = '+'.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/15/2016 13:30'!
testFromStream
	self allURIStringsDo: [:each | | stream uri |
		stream := each readStream.
		uri := XMLURI fromStream: stream.
		self
			assert: uri class == XMLURI;
			assert: uri uriString isNil;
			assert: uri = each asXMLURI;
			assert: stream atEnd]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/5/2016 09:19'!
testFromString
	self allURIStringsDo: [:each | | uri |
		uri := XMLURI fromString: each.
		self
			assert: uri class == XMLURI;
			assert: uri uriString = each;
			assert: uri = each asXMLURI]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testHasAbsolutePath
	self
		deny: '' asXMLURI hasAbsolutePath;
		assert: '/' asXMLURI hasAbsolutePath;
		assert: self absolute asXMLURI hasAbsolutePath;
		assert: self absoluteDir asXMLURI hasAbsolutePath;
		assert: self absoluteDirRelativeCombined asXMLURI hasAbsolutePath;
		assert: self absoluteRelativeCombined asXMLURI hasAbsolutePath;
		deny: self domainURIString asXMLURI hasAbsolutePath;
		deny: self domainURIWithPortString asXMLURI hasAbsolutePath;
		deny: self emailURIString asXMLURI hasAbsolutePath;
		assert: self fileURIWithHostString asXMLURI hasAbsolutePath;
		assert: self fileURIString asXMLURI hasAbsolutePath;
		deny: self fragmentURIString asXMLURI hasAbsolutePath;
		assert: self fullURIString asXMLURI hasAbsolutePath;
		assert: self fullURIWithPortString asXMLURI hasAbsolutePath;
		deny: self queryURIString asXMLURI hasAbsolutePath;
		deny: self relative asXMLURI hasAbsolutePath;
		deny: self relativeAbsoluteCombined asXMLURI hasAbsolutePath;
		assert: self schemelessURIString asXMLURI hasAbsolutePath;
		assert: self schemelessURIWithPortString asXMLURI hasAbsolutePath! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testHasAuthority
	self
		deny: '' asXMLURI hasAuthority;
		deny: '/' asXMLURI hasAuthority;
		deny: self absolute asXMLURI hasAuthority;
		deny: self absoluteDir asXMLURI hasAuthority;
		deny: self absoluteDirRelativeCombined asXMLURI hasAuthority;
		deny: self absoluteRelativeCombined asXMLURI hasAuthority;
		assert: self domainURIString asXMLURI hasAuthority;
		assert: self domainURIWithPortString asXMLURI hasAuthority;
		assert: self emailURIString asXMLURI hasAuthority;
		assert: self fileURIWithHostString asXMLURI hasAuthority;
		deny: self fileURIString asXMLURI hasAuthority;
		deny: self fragmentURIString asXMLURI hasAuthority;
		assert: self fullURIString asXMLURI hasAuthority;
		assert: self fullURIWithPortString asXMLURI hasAuthority;
		deny: self queryURIString asXMLURI hasAuthority;
		deny: self relative asXMLURI hasAuthority;
		deny: self relativeAbsoluteCombined asXMLURI hasAuthority;
		assert: self schemelessURIString asXMLURI hasAuthority;
		assert: self schemelessURIWithPortString asXMLURI hasAuthority! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testHasAuthorityPrefix
	self
		deny: '' asXMLURI hasAuthorityPrefix;
		deny: '/' asXMLURI hasAuthorityPrefix;
		deny: self absolute asXMLURI hasAuthorityPrefix;
		deny: self absoluteDir asXMLURI hasAuthorityPrefix;
		deny: self absoluteDirRelativeCombined asXMLURI hasAuthorityPrefix;
		deny: self absoluteRelativeCombined asXMLURI hasAuthorityPrefix;
		assert: self domainURIString asXMLURI hasAuthorityPrefix;
		assert: self domainURIWithPortString asXMLURI hasAuthorityPrefix;
		deny: self emailURIString asXMLURI hasAuthorityPrefix;
		assert: self fileURIWithHostString asXMLURI hasAuthorityPrefix;
		assert: self fileURIString asXMLURI hasAuthorityPrefix;
		deny: self fragmentURIString asXMLURI hasAuthorityPrefix;
		assert: self fullURIString asXMLURI hasAuthorityPrefix;
		assert: self fullURIWithPortString asXMLURI hasAuthorityPrefix;
		deny: self queryURIString asXMLURI hasAuthorityPrefix;
		deny: self relative asXMLURI hasAuthorityPrefix;
		deny: self relativeAbsoluteCombined asXMLURI hasAuthorityPrefix;
		assert: self schemelessURIString asXMLURI hasAuthorityPrefix;
		assert: self schemelessURIWithPortString asXMLURI hasAuthorityPrefix! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testHasDirectoryPath
	self
		deny: '' asXMLURI hasDirectoryPath;
		assert: '/' asXMLURI hasDirectoryPath;
		deny: self absolute asXMLURI hasDirectoryPath;
		assert: self absoluteDir asXMLURI hasDirectoryPath;
		deny: self absoluteDirRelativeCombined asXMLURI hasDirectoryPath;
		deny: self absoluteRelativeCombined asXMLURI hasDirectoryPath;
		deny: self domainURIString asXMLURI hasDirectoryPath;
		deny: self domainURIWithPortString asXMLURI hasDirectoryPath;
		deny: self emailURIString asXMLURI hasDirectoryPath;
		deny: self fileURIWithHostString asXMLURI hasDirectoryPath;
		deny: self fileURIString asXMLURI hasDirectoryPath;
		deny: self fragmentURIString asXMLURI hasDirectoryPath;
		deny: self fullURIString asXMLURI hasDirectoryPath;
		deny: self fullURIWithPortString asXMLURI hasDirectoryPath;
		deny: self queryURIString asXMLURI hasDirectoryPath;
		deny: self relative asXMLURI hasDirectoryPath;
		deny: self relativeAbsoluteCombined asXMLURI hasDirectoryPath;
		deny: self schemelessURIString asXMLURI hasDirectoryPath;
		deny: self schemelessURIWithPortString asXMLURI hasDirectoryPath! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 12:17'!
testHasFragment
	self allURIStringsDo: [:each | | uri |
		uri := each asXMLURI.
		(each includes: $#)
			ifTrue: [self assert: uri hasFragment]
			ifFalse: [self deny: uri hasFragment]]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testHasHost
	self
		deny: '' asXMLURI hasHost;
		deny: '/' asXMLURI hasHost;
		deny: self absolute asXMLURI hasHost;
		deny: self absoluteDir asXMLURI hasHost;
		deny: self absoluteDirRelativeCombined asXMLURI hasHost;
		deny: self absoluteRelativeCombined asXMLURI hasHost;
		assert: self domainURIString asXMLURI hasHost;
		assert: self domainURIWithPortString asXMLURI hasHost;
		assert: self emailURIString asXMLURI hasHost;
		assert: self fileURIWithHostString asXMLURI hasHost;
		deny: self fileURIString asXMLURI hasHost;
		deny: self fragmentURIString asXMLURI hasHost;
		assert: self fullURIString asXMLURI hasHost;
		assert: self fullURIWithPortString asXMLURI hasHost;
		deny: self queryURIString asXMLURI hasHost;
		deny: self relative asXMLURI hasHost;
		deny: self relativeAbsoluteCombined asXMLURI hasHost;
		assert: self schemelessURIString asXMLURI hasHost;
		assert: self schemelessURIWithPortString asXMLURI hasHost! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testHasNonDirectoryPath
	self
		deny: '' asXMLURI hasNonDirectoryPath;
		deny: '/' asXMLURI hasNonDirectoryPath;
		assert: self absolute asXMLURI hasNonDirectoryPath;
		deny: self absoluteDir asXMLURI hasNonDirectoryPath;
		assert: self absoluteDirRelativeCombined asXMLURI hasNonDirectoryPath;
		assert: self absoluteRelativeCombined asXMLURI hasNonDirectoryPath;
		deny: self domainURIString asXMLURI hasNonDirectoryPath;
		deny: self domainURIWithPortString asXMLURI hasNonDirectoryPath;
		deny: self emailURIString asXMLURI hasNonDirectoryPath;
		assert: self fileURIWithHostString asXMLURI hasNonDirectoryPath;
		assert: self fileURIString asXMLURI hasNonDirectoryPath;
		deny: self fragmentURIString asXMLURI hasNonDirectoryPath;
		assert: self fullURIString asXMLURI hasNonDirectoryPath;
		assert: self fullURIWithPortString asXMLURI hasNonDirectoryPath;
		deny: self queryURIString asXMLURI hasNonDirectoryPath;
		assert: self relative asXMLURI hasNonDirectoryPath;
		assert: self relativeAbsoluteCombined asXMLURI hasNonDirectoryPath;
		assert: self schemelessURIString asXMLURI hasNonDirectoryPath;
		assert: self schemelessURIWithPortString asXMLURI hasNonDirectoryPath! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testHasPath
	self
		deny: '' asXMLURI hasPath;
		assert: '/' asXMLURI hasPath;
		assert: self absolute asXMLURI hasPath;
		assert: self absoluteDir asXMLURI hasPath;
		assert: self absoluteDirRelativeCombined asXMLURI hasPath;
		assert: self absoluteRelativeCombined asXMLURI hasPath;
		deny: self domainURIString asXMLURI hasPath;
		deny: self domainURIWithPortString asXMLURI hasPath;
		deny: self emailURIString asXMLURI hasPath;
		assert: self fileURIWithHostString asXMLURI hasPath;
		assert: self fileURIString asXMLURI hasPath;
		deny: self fragmentURIString asXMLURI hasPath;
		assert: self fullURIString asXMLURI hasPath;
		assert: self fullURIWithPortString asXMLURI hasPath;
		deny: self queryURIString asXMLURI hasPath;
		assert: self relative asXMLURI hasPath;
		assert: self relativeAbsoluteCombined asXMLURI hasPath;
		assert: self schemelessURIString asXMLURI hasPath;
		assert: self schemelessURIWithPortString asXMLURI hasPath! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testHasPort
	self
		deny: '' asXMLURI hasPort;
		deny: '/' asXMLURI hasPort;
		deny: self absolute asXMLURI hasPort;
		deny: self absoluteDir asXMLURI hasPort;
		deny: self absoluteDirRelativeCombined asXMLURI hasPort;
		deny: self absoluteRelativeCombined asXMLURI hasPort;
		deny: self domainURIString asXMLURI hasPort;
		assert: self domainURIWithPortString asXMLURI hasPort;
		deny: self emailURIString asXMLURI hasPort;
		deny: self fileURIWithHostString asXMLURI hasPort;
		deny: self fileURIString asXMLURI hasPort;
		deny: self fragmentURIString asXMLURI hasPort;
		deny: self fullURIString asXMLURI hasPort;
		assert: self fullURIWithPortString asXMLURI hasPort;
		deny: self queryURIString asXMLURI hasPort;
		deny: self relative asXMLURI hasPort;
		deny: self relativeAbsoluteCombined asXMLURI hasPort;
		deny: self schemelessURIString asXMLURI hasPort;
		assert: self schemelessURIWithPortString asXMLURI hasPort! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 12:15'!
testHasQuery
	self allURIStringsDo: [:each | | uri |
		uri := each asXMLURI.
		(each includes: $?)
			ifTrue: [self assert: uri hasQuery]
			ifFalse: [self deny: uri hasQuery]]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/5/2016 08:01'!
testHasRootPath
	self allURIStringsDo: [:each | | uri |
		uri := each asXMLURI.
		each = '/'
			ifTrue: [self assert: uri hasRootPath]
			ifFalse: [self deny: uri hasRootPath]]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testHasScheme
	self
		deny: '' asXMLURI hasScheme;
		deny: '/' asXMLURI hasScheme;
		deny: self absolute asXMLURI hasScheme;
		deny: self absoluteDir asXMLURI hasScheme;
		deny: self absoluteDirRelativeCombined asXMLURI hasScheme;
		deny: self absoluteRelativeCombined asXMLURI hasScheme;
		assert: self domainURIString asXMLURI hasScheme;
		assert: self domainURIWithPortString asXMLURI hasScheme;
		assert: self emailURIString asXMLURI hasScheme;
		assert: self fileURIWithHostString asXMLURI hasScheme;
		assert: self fileURIString asXMLURI hasScheme;
		deny: self fragmentURIString asXMLURI hasScheme;
		assert: self fullURIString asXMLURI hasScheme;
		assert: self fullURIWithPortString asXMLURI hasScheme;
		deny: self queryURIString asXMLURI hasScheme;
		deny: self relative asXMLURI hasScheme;
		deny: self relativeAbsoluteCombined asXMLURI hasScheme;
		deny: self schemelessURIString asXMLURI hasScheme;
		deny: self schemelessURIWithPortString asXMLURI hasScheme! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 12:14'!
testHasUserInfo
	self allURIStringsDo: [:each | | uri |
		uri := each asXMLURI.
		uri scheme = 'mailto'
			ifTrue: [self assert: uri hasUserInfo]
			ifFalse: [self deny: uri hasUserInfo]]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testHost
	self
		assert: '' asXMLURI host isEmpty;
		assert: '/' asXMLURI host isEmpty;
		assert: self absolute asXMLURI host isEmpty;
		assert: self absoluteDir asXMLURI host isEmpty;
		assert: self absoluteDirRelativeCombined asXMLURI host isEmpty;
		assert: self absoluteRelativeCombined asXMLURI host isEmpty;
		assert: self domainURIString asXMLURI host = self host;
		assert: self domainURIWithPortString asXMLURI host = self host;
		assert: self emailURIString asXMLURI host = self host;
		assert: self fileURIWithHostString asXMLURI host = self host;
		assert: self fileURIString asXMLURI host isEmpty;
		assert: self fragmentURIString asXMLURI host isEmpty;
		assert: self fullURIString asXMLURI host = self host;
		assert: self fullURIWithPortString asXMLURI host = self host;
		assert: self queryURIString asXMLURI host isEmpty;
		assert: self relative asXMLURI host isEmpty;
		assert: self relativeAbsoluteCombined asXMLURI host isEmpty;
		assert: self schemelessURIString asXMLURI host = self host;
		assert: self schemelessURIWithPortString asXMLURI host = self host! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/11/2016 08:14'!
testHostNonPercentEncodedCharacters
	| host uri |

	host := self unreservedCharacters, self subDelimiterCharacters.
	uri := ('http://', host) asXMLURI.
	self
		assert: uri scheme = 'http';
		assert: uri host = host asLowercase.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/11/2016 08:14'!
testHostPercentEncodedCharacters
	| prefix uri |

	prefix := 'http://'.
	self asciiCharactersDo: [:each | | lowercasedChar |
		uri := (prefix, (XMLURI percentEncodeCharacter: each)) asXMLURI.
		lowercasedChar := each asLowercase.
		self assert: uri host = lowercasedChar asString.

		uri uriString: nil.
		(self isUnreservedCharacter: each)
			ifTrue: [
				self assert:
					uri printString =
						(prefix copyWith: lowercasedChar)]
			ifFalse: [
				self assert:
					uri printString =
						(prefix, (XMLURI percentEncodeCharacter: lowercasedChar))]].

	uri := (prefix, '+') asXMLURI.
	self assert: uri host = '+'.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 11:57'!
testIPv6Hosts
	| ipv6Host uriString uri |

	ipv6Host := '[FEDC:BA98:7654:3210:FEDC:BA98:7654:3210]'.
	uriString := 'http://', ipv6Host, ':80', self absolute.
	uri := uriString asXMLURI.
	self
		assert: uri scheme = 'http';
		assert: uri host = ipv6Host asLowercase;
		assert: uri port = 80;
		assert: uri path = self absolute;
		assert: uri query isEmpty;
		assert: uri fragment isEmpty.
	uri uriString: nil.
	self assert: uri printString = uriString asLowercase.

	uriString := 'http://', ipv6Host, self absolute.
	uri := uriString asXMLURI.
	self
		assert: uri scheme = 'http';
		assert: uri host = ipv6Host asLowercase;
		assert: uri port isNil;
		assert: uri path = self absolute;
		assert: uri query isEmpty;
		assert: uri fragment isEmpty.
	uri uriString: nil.
	self assert: uri printString = uriString asLowercase.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testInvalidPort
	| uri |

	uri := (self domainURIString, ':invalid', self absolute) asXMLURI.
	self
		assert: uri host = self host;
		assert: uri port isNil;
		assert: uri path = self absolute.

	uri := (self domainURIString, ':invalid', self queryURIString) asXMLURI.
	self
		assert: uri host = self host;
		assert: uri port isNil;
		assert: uri pathSegments isEmpty.

	uri := (self domainURIString, ':invalid', self fragmentURIString) asXMLURI.
	self
		assert: uri host = self host;
		assert: uri port isNil;
		assert: uri pathSegments isEmpty.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testInvalidPortSuffix
	| uri |

	uri := (self domainURIWithPortString, 'invalid', self absolute) asXMLURI.
	self
		assert: uri host = self host;
		assert: uri port = 80;
		assert: uri path = self absolute.

	uri := (self domainURIWithPortString, 'invalid', self queryURIString) asXMLURI.
	self
		assert: uri host = self host;
		assert: uri port = 80;
		assert: uri pathSegments isEmpty.

	uri := (self domainURIWithPortString, 'invalid', self fragmentURIString) asXMLURI.
	self
		assert: uri host = self host;
		assert: uri port = 80;
		assert: uri pathSegments isEmpty.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/15/2016 13:31'!
testIsEmpty
	| uri |

	self allURIStringsDo: [:each |
		uri := each asXMLURI.
		each isEmpty
			ifTrue: [self assert: uri isEmpty]
			ifFalse: [self deny: uri isEmpty].

		uri
			parseURIStringIfUnparsed;
			uriString: nil.
		each isEmpty
			ifTrue: [self assert: uri isEmpty]
			ifFalse: [self deny: uri isEmpty]].

	uri := '//:?#' asXMLURI.
	self assert: uri isEmpty.
	uri
		parseURIStringIfUnparsed;
		uriString: nil.
	self assert: uri isEmpty.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 12:13'!
testIsFileURI
	self allURIStringsDo: [:each | | uri |
		uri := each asXMLURI.
		uri scheme = 'file'
			ifTrue: [self assert: uri isFileURI]
			ifFalse: [self deny: uri isFileURI]]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testIsLocalURI
	self
		assert: '' asXMLURI isLocalURI;
		assert: '/' asXMLURI isLocalURI;
		assert: self absolute asXMLURI isLocalURI;
		assert: self absoluteDir asXMLURI isLocalURI;
		assert: self absoluteDirRelativeCombined asXMLURI isLocalURI;
		assert: self absoluteRelativeCombined asXMLURI isLocalURI;
		deny: self domainURIString asXMLURI isLocalURI;
		deny: self domainURIWithPortString asXMLURI isLocalURI;
		deny: self emailURIString asXMLURI isLocalURI;
		assert: self fileURIWithHostString asXMLURI isLocalURI;
		assert: self fileURIString asXMLURI isLocalURI;
		assert: self fragmentURIString asXMLURI isLocalURI;
		deny: self fullURIString asXMLURI isLocalURI;
		deny: self fullURIWithPortString asXMLURI isLocalURI;
		assert: self queryURIString asXMLURI isLocalURI;
		assert: self relative asXMLURI isLocalURI;
		assert: self relativeAbsoluteCombined asXMLURI isLocalURI;
		deny: self schemelessURIString asXMLURI isLocalURI;
		deny: self schemelessURIWithPortString asXMLURI isLocalURI! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testIsRelative
	self
		assert: '' asXMLURI isRelative;
		assert: '/' asXMLURI isRelative;
		assert: self absolute asXMLURI isRelative;
		assert: self absoluteDir asXMLURI isRelative;
		assert: self absoluteDirRelativeCombined asXMLURI isRelative;
		assert: self absoluteRelativeCombined asXMLURI isRelative;
		deny: self domainURIString asXMLURI isRelative;
		deny: self domainURIWithPortString asXMLURI isRelative;
		deny: self emailURIString asXMLURI isRelative;
		deny: self fileURIWithHostString asXMLURI isRelative;
		deny: self fileURIString asXMLURI isRelative;
		assert: self fragmentURIString asXMLURI isRelative;
		deny: self fullURIString asXMLURI isRelative;
		deny: self fullURIWithPortString asXMLURI isRelative;
		assert: self queryURIString asXMLURI isRelative;
		assert: self relative asXMLURI isRelative;
		assert: self relativeAbsoluteCombined asXMLURI isRelative;
		assert: self schemelessURIString asXMLURI isRelative;
		assert: self schemelessURIWithPortString asXMLURI isRelative! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/4/2016 23:39'!
testNew
	| uri |

	uri := XMLURI new.
	self
		assert: uri uriString = '';
		assert: uri isEmpty.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/15/2016 13:32'!
testNotEmpty
	| uri |

	self allURIStringsDo: [:each |
		uri := each asXMLURI.
		each notEmpty
			ifTrue: [self assert: uri notEmpty]
			ifFalse: [self deny: uri notEmpty].

		uri
			parseURIStringIfUnparsed;
			uriString: nil.
		each notEmpty
			ifTrue: [self assert: uri notEmpty]
			ifFalse: [self deny: uri notEmpty]].

	uri := '//:?#' asXMLURI.
	self deny: uri notEmpty.
	uri
		parseURIStringIfUnparsed;
		uriString: nil.
	self deny: uri notEmpty.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/1/2016 21:41'!
testParsingQueryString
	#('' '?' '?&' '?&&') do: [:each |
		self assert:
			each asXMLURI query =
				OrderPreservingStringDictionary new].

	#('?a=b' '?&a=b&' '?&&a=b&&') do: [:each |
		self assert:
			each asXMLURI query =
				(OrderPreservingStringDictionary with: 'a' -> 'b')].

	#('?=&a=b' '?&=&a=b' '?&&=&&a=b' '?a=b&=' '?a=b&=&' '?a=b&&=&&') do: [:each |
		self assert:
			each asXMLURI query =
				(OrderPreservingStringDictionary with: '' -> '' with: 'a' -> 'b')].

	#('?c=&a=b' '?&c=&a=b' '?&&c=&&a=b' '?a=b&c=' '?a=b&c=&' '?a=b&&c=&&') do: [:each |
		self assert:
			each asXMLURI query =
				(OrderPreservingStringDictionary with: 'c' -> '' with: 'a' -> 'b')].! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testPath
	self
		assert: '' asXMLURI path isEmpty;
		assert: '/' asXMLURI path = '/';
		assert: self absolute asXMLURI path = self absolute;
		assert: self absoluteDir asXMLURI path = self absoluteDir;
		assert: self absoluteDirRelativeCombined asXMLURI path = self absoluteDirRelativeCombined;
		assert: self absoluteRelativeCombined asXMLURI path = self absoluteRelativeCombined;
		assert: self domainURIString asXMLURI path isEmpty;
		assert: self domainURIWithPortString asXMLURI path isEmpty;
		assert: self emailURIString asXMLURI path isEmpty;
		assert: self fileURIWithHostString asXMLURI path = self absolute;
		assert: self fileURIString asXMLURI path = self absolute;
		assert: self fragmentURIString asXMLURI path isEmpty;
		assert: self fullURIString asXMLURI path = self absolute;
		assert: self fullURIWithPortString asXMLURI path = self absolute;
		assert: self queryURIString asXMLURI path isEmpty;
		assert: self relative asXMLURI path = self relative;
		assert: self relativeAbsoluteCombined asXMLURI path = self relativeAbsoluteCombined;
		assert: self schemelessURIString asXMLURI path = self absolute;
		assert: self schemelessURIWithPortString asXMLURI path = self absolute! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/11/2016 08:15'!
testPathNonPercentEncodedCharacters
	| firstSegment secondSegment uri |

	firstSegment := self nonPercentEncodedPathCharacters copyWithout: $:.
	secondSegment := self nonPercentEncodedPathCharacters.
	uri := (firstSegment, '/', secondSegment) asXMLURI.
	self assert:
		uri pathSegments =
			(OrderedCollection with: firstSegment with: secondSegment).! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/11/2016 08:15'!
testPathPercentEncodedCharacters
	| uri |

	self asciiCharactersDo: [:each | | encodedURIString |
		encodedURIString := XMLURI percentEncodeCharacter: each.
		uri := encodedURIString asXMLURI.
		self assert: uri path = each asString.

		uri uriString: nil.
		(self isUnreservedCharacter: each)
			ifTrue: [self assert: uri printString = each asString]
			ifFalse: [self assert: uri printString = encodedURIString]].

	uri := '+' asXMLURI.
	self assert: uri path = '+'.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testPathSegments
	self
		assert: '' asXMLURI pathSegments isEmpty;
		assert: '/' asXMLURI pathSegments = #('' '') asOrderedCollection;
		assert: self absolute asXMLURI pathSegments = self absoluteSegments;
		assert: self absoluteDir asXMLURI pathSegments = self absoluteDirSegments;
		assert: self absoluteDirRelativeCombined asXMLURI pathSegments = self absoluteDirRelativeCombinedSegments;
		assert: self absoluteRelativeCombined asXMLURI pathSegments = self absoluteRelativeCombinedSegments;
		assert: self domainURIString asXMLURI pathSegments isEmpty;
		assert: self domainURIWithPortString asXMLURI pathSegments isEmpty;
		assert: self emailURIString asXMLURI pathSegments isEmpty;
		assert: self fileURIWithHostString asXMLURI pathSegments = self absoluteSegments;
		assert: self fileURIString asXMLURI pathSegments = self absoluteSegments;
		assert: self fragmentURIString asXMLURI pathSegments isEmpty;
		assert: self fullURIString asXMLURI pathSegments = self absoluteSegments;
		assert: self fullURIWithPortString asXMLURI pathSegments = self absoluteSegments;
		assert: self queryURIString asXMLURI pathSegments isEmpty;
		assert: self relative asXMLURI pathSegments = self relativeSegments;
		assert: self relativeAbsoluteCombined asXMLURI pathSegments = self relativeAbsoluteCombinedSegments;
		assert: self schemelessURIString asXMLURI pathSegments = self absoluteSegments;
		assert: self schemelessURIWithPortString asXMLURI pathSegments = self absoluteSegments! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/6/2016 07:54'!
testPathSegmentsDo
	self allURIStringsDo: [:uriString | | uri visited |
		uri := uriString asXMLURI.
		visited := OrderedCollection new.
		uri pathSegmentsDo: [:each | visited addLast: each].
		self assert: visited = uri pathSegments]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/7/2016 05:00'!
testPercentEncodeCharacter
	self asciiCharactersDo: [:each | | encoded |
		encoded := XMLURI percentEncodeCharacter: each.
		self
			assert: encoded size = 3;
			assert: encoded first == $%;
			assert:
				(XMLIntegerReader
					readFrom: encoded allButFirst
					withBase: 16) = each asciiValue]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/7/2016 05:00'!
testPercentOrPlusEncodeCharacter
	self asciiCharactersDo: [:each | | encoded |
		encoded := XMLURI percentOrPlusEncodeCharacter: each.
		each = Character space
			ifTrue: [self assert: encoded = '+']
			ifFalse: [
				self
					assert: encoded size = 3;
					assert: encoded first == $%;
					assert:
						(XMLIntegerReader
							readFrom: encoded allButFirst
							withBase: 16) = each asciiValue]]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testPort
	self
		assert: '' asXMLURI port isNil;
		assert: '/' asXMLURI port isNil;
		assert: self absolute asXMLURI port isNil;
		assert: self absoluteDir asXMLURI port isNil;
		assert: self absoluteDirRelativeCombined asXMLURI port isNil;
		assert: self absoluteRelativeCombined asXMLURI port isNil;
		assert: self domainURIString asXMLURI port isNil;
		assert: self domainURIWithPortString asXMLURI port = 80;
		assert: self emailURIString asXMLURI port isNil;
		assert: self fileURIWithHostString asXMLURI port isNil;
		assert: self fileURIString asXMLURI port isNil;
		assert: self fragmentURIString asXMLURI port isNil;
		assert: self fullURIString asXMLURI port isNil;
		assert: self fullURIWithPortString asXMLURI port = 80;
		assert: self queryURIString asXMLURI port isNil;
		assert: self relative asXMLURI port isNil;
		assert: self relativeAbsoluteCombined asXMLURI port isNil;
		assert: self schemelessURIString asXMLURI port isNil;
		assert: self schemelessURIWithPortString asXMLURI port = 80! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/6/2016 04:07'!
testPrintOn
	self allURIStringsDo: [:each | | uri |
		uri := each asXMLURI.
		self assert: uri printString = each.

		uri
			parseURIStringIfUnparsed;
			uriString: nil.
		self
			assert: uri uriString isNil;
			assert: uri printString = each;
			assert: uri uriString = each]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testQuery
	self
		assert: '' asXMLURI query isEmpty;
		assert: '/' asXMLURI query isEmpty;
		assert: self absolute asXMLURI query isEmpty;
		assert: self absoluteDir asXMLURI query isEmpty;
		assert: self absoluteDirRelativeCombined asXMLURI query isEmpty;
		assert: self absoluteRelativeCombined asXMLURI query isEmpty;
		assert: self domainURIString asXMLURI query isEmpty;
		assert: self domainURIWithPortString asXMLURI query isEmpty;
		assert: self emailURIString asXMLURI query isEmpty;
		assert: self fileURIWithHostString asXMLURI query isEmpty;
		assert: self fileURIString asXMLURI query isEmpty;
		assert: self fragmentURIString asXMLURI query isEmpty;
		assert: self fullURIString asXMLURI query = self query;
		assert: self fullURIWithPortString asXMLURI query = self query;
		assert: self queryURIString asXMLURI query = self query;
		assert: self relative asXMLURI query isEmpty;
		assert: self relativeAbsoluteCombined asXMLURI query isEmpty;
		assert: self schemelessURIString asXMLURI query = self query;
		assert: self schemelessURIWithPortString asXMLURI query = self query! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 04:48'!
testQueryAt
	self allURIStringsDo: [:each | | uri |
		uri := each asXMLURI.
		uri query keysAndValuesDo: [:key :value |
			self assert: (uri queryAt: key) = value].
		self assert: (uri queryAt: 'absent') = '']! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 04:53'!
testQueryAtIfAbsent
	self allURIStringsDo: [:each | | uri |
		uri := each asXMLURI.
		uri query keysAndValuesDo: [:key :value |
			self assert: (uri queryAt: key) = value.
			self assert:
				(uri
					queryAt: key
					ifAbsent: ['absentValue']) = value].
		self assert:
			(uri
				queryAt: 'absentKey'
				ifAbsent: ['absentValue']) = 'absentValue']! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 04:51'!
testQueryIncludesKey
	self allURIStringsDo: [:uriString | | uri |
		uri := uriString asXMLURI.
		uri query keysDo: [:each |
			self assert: (uri queryIncludesKey: each)].
		self deny: (uri queryIncludesKey: 'absent')]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 04:53'!
testQueryKeys
	self allURIStringsDo: [:each | | uri |
		uri := each asXMLURI.
		self assert: uri queryKeys asArray = uri query keys asArray]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 04:52'!
testQueryKeysAndValuesDo
	self allURIStringsDo: [:each | | uri visited |
		uri := each asXMLURI.
		visited := OrderedCollection new.
		uri queryKeysAndValuesDo: [:key :value |
			visited addLast: key -> value].
		self assert: visited = uri query associations asOrderedCollection]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/11/2016 08:15'!
testQueryNonPercentOrPlusEncodedCharacters
	| queryKey queryValue uri |

	queryKey := (self nonPercentEncodedPathCharacters copyWithoutAll: '=&+'), '/?'.
	queryValue :=  (self nonPercentEncodedPathCharacters copyWithoutAll: '&+'), '/?'.
	uri := ('?', queryKey, '=', queryValue) asXMLURI.
	self assert:
		uri query =
			(OrderPreservingStringDictionary with: queryKey -> queryValue).! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/11/2016 08:15'!
testQueryPercentOrPlusEncodedCharacters
	| uri |

	self asciiCharactersDo: [:each | | encodedURIString |
		encodedURIString :=
			'?', (XMLURI percentEncodeCharacter: each), '=', (XMLURI percentEncodeCharacter: each).
		uri := encodedURIString asXMLURI.
		self assert:
			uri query =
				(OrderPreservingStringDictionary with: each asString -> each asString).

		uri uriString: nil.
		(self isUnreservedCharacter: each)
			ifTrue: [self assert: uri printString = ('?', each asString, '=', each asString)]
			ifFalse: [
				each = Character space
					ifTrue: [self assert: uri printString = '?+=+']
					ifFalse: [self assert: uri printString = encodedURIString]]].

	uri := '?+=+' asXMLURI.
	self assert: uri query = (OrderPreservingStringDictionary with: ' ' -> ' ').! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/5/2016 23:55'!
testQueryWithEmptyKeysOrValues
	#('?' '?&' '?&&') do: [:each |
		self assert: each asXMLURI query isEmpty].

	#('?a=b' '?&a=b&' '?&&a=b&&') do: [:each |
		self assert:
			each asXMLURI query =
				(OrderPreservingStringDictionary with: 'a' -> 'b')].

	#('?=&a=b' '?&=&a=b' '?&&=&&a=b' '?a=b&=' '?a=b&=&' '?a=b&&=&&') do: [:each |
		self assert:
			each asXMLURI query =
				(OrderPreservingStringDictionary with: '' -> '' with: 'a' -> 'b')].

	#('?c=&a=b' '?&c=&a=b' '?&&c=&&a=b' '?a=b&c=' '?a=b&c=&' '?a=b&&c=&&') do: [:each |
		self assert:
			each asXMLURI query =
				(OrderPreservingStringDictionary with: 'c' -> '' with: 'a' -> 'b')].! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/15/2016 13:30'!
testReadFrom
	self allURIStringsDo: [:each | | stream uri |
		stream := each readStream.
		uri := XMLURI readFrom: stream.
		self
			assert: uri class == XMLURI;
			assert: uri uriString isNil;
			assert: uri = each asXMLURI;
			assert: stream atEnd.

		uri := XMLURI readFrom: each.
		self
			assert: uri class == XMLURI;
			assert: uri uriString = each;
			assert: uri = each asXMLURI]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/13/2016 06:39'!
testScheme
	self
		assert: '' asXMLURI scheme isEmpty;
		assert: '/' asXMLURI scheme isEmpty;
		assert: self absolute asXMLURI scheme isEmpty;
		assert: self absoluteDir asXMLURI scheme isEmpty;
		assert: self absoluteDirRelativeCombined asXMLURI scheme isEmpty;
		assert: self absoluteRelativeCombined asXMLURI scheme isEmpty;
		assert: self domainURIString asXMLURI scheme = 'http';
		assert: self domainURIWithPortString asXMLURI scheme = 'http';
		assert: self emailURIString asXMLURI scheme = 'mailto';
		assert: self fileURIWithHostString asXMLURI scheme = 'file';
		assert: self fileURIString asXMLURI scheme  = 'file';
		assert: self fragmentURIString asXMLURI scheme isEmpty;
		assert: self fullURIString asXMLURI scheme = 'http';
		assert: self fullURIWithPortString asXMLURI scheme = 'http';
		assert: self queryURIString asXMLURI scheme isEmpty;
		assert: self relative asXMLURI scheme isEmpty;
		assert: self relativeAbsoluteCombined asXMLURI scheme isEmpty;
		assert: self schemelessURIString asXMLURI scheme isEmpty;
		assert: self schemelessURIWithPortString asXMLURI scheme isEmpty! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/5/2016 23:55'!
testSchemeAndHostCaseInsensitivity
	self allURIStringsDo: [:each | | uri uppercaseURI |
		uri := each asXMLURI.
		uppercaseURI := each asUppercase asXMLURI.
		self
			assert: uri scheme = uppercaseURI scheme;
			assert: uri hasAuthorityPrefix = uppercaseURI hasAuthorityPrefix.
		uri hasUserInfo
			ifTrue: [self deny: uri userInfo = uppercaseURI userInfo].
		self
			assert: uri host = uppercaseURI host;
			assert: uri port = uppercaseURI port.
		(uri hasPath
			and: [uri hasRootPath not])
			ifTrue: [self deny: uri pathSegments = uppercaseURI pathSegments].
		uri hasQuery
			ifTrue: [self deny: uri query = uppercaseURI query].
		uri hasFragment
			ifTrue: [self deny: uri fragment = uppercaseURI fragment]]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/11/2016 08:21'!
testSchemeCharacters
	| scheme uri |

	scheme := self alphaNumericCharacters, '+-.'.
	uri := (scheme, '://', self host) asXMLURI.
	self
		assert: uri scheme = scheme asLowercase;
		assert: uri host = self host.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/5/2016 02:48'!
testStandardURICombining
	"Generated from RFC 3986, page 34-36"

	| baseURI |

	baseURI := 'http://a/b/c/d;p?q' asXMLURI.
	self
		assert: (baseURI / 'g:h' asXMLURI) = 'g:h' asXMLURI;
		assert: (baseURI / 'g' asXMLURI) = 'http://a/b/c/g' asXMLURI;
		assert: (baseURI / './g' asXMLURI) = 'http://a/b/c/g' asXMLURI;
		assert: (baseURI / 'g/' asXMLURI) = 'http://a/b/c/g/' asXMLURI;
		assert: (baseURI / '/g' asXMLURI) = 'http://a/g' asXMLURI;
		assert: (baseURI / '//g' asXMLURI) = 'http://g' asXMLURI;
		assert: (baseURI / '?y' asXMLURI) = 'http://a/b/c/d;p?y' asXMLURI;
		assert: (baseURI / 'g?y' asXMLURI) = 'http://a/b/c/g?y' asXMLURI;
		assert: (baseURI / '#s' asXMLURI) = 'http://a/b/c/d;p?q#s' asXMLURI;
		assert: (baseURI / 'g#s' asXMLURI) = 'http://a/b/c/g#s' asXMLURI;
		assert: (baseURI / 'g?y#s' asXMLURI) = 'http://a/b/c/g?y#s' asXMLURI;
		assert: (baseURI / ';x' asXMLURI) = 'http://a/b/c/;x' asXMLURI;
		assert: (baseURI / 'g;x' asXMLURI) = 'http://a/b/c/g;x' asXMLURI;
		assert: (baseURI / 'g;x?y#s' asXMLURI) = 'http://a/b/c/g;x?y#s' asXMLURI;
		assert: (baseURI / '' asXMLURI) = 'http://a/b/c/d;p?q' asXMLURI;
		assert: (baseURI / '.' asXMLURI) = 'http://a/b/c/' asXMLURI;
		assert: (baseURI / './' asXMLURI) = 'http://a/b/c/' asXMLURI;
		assert: (baseURI / '..' asXMLURI) = 'http://a/b/' asXMLURI;
		assert: (baseURI / '../' asXMLURI) = 'http://a/b/' asXMLURI;
		assert: (baseURI / '../g' asXMLURI) = 'http://a/b/g' asXMLURI;
		assert: (baseURI / '../..' asXMLURI) = 'http://a/' asXMLURI;
		assert: (baseURI / '../../' asXMLURI) = 'http://a/' asXMLURI;
		assert: (baseURI / '../../g' asXMLURI) = 'http://a/g' asXMLURI;
		assert: (baseURI / '../../../g' asXMLURI) = 'http://a/g' asXMLURI;
		assert: (baseURI / '../../../../g' asXMLURI) = 'http://a/g' asXMLURI;
		assert: (baseURI / '/./g' asXMLURI) = 'http://a/g' asXMLURI;
		assert: (baseURI / '/../g' asXMLURI) = 'http://a/g' asXMLURI;
		assert: (baseURI / 'g.' asXMLURI) = 'http://a/b/c/g.' asXMLURI;
		assert: (baseURI / '.g' asXMLURI) = 'http://a/b/c/.g' asXMLURI;
		assert: (baseURI / 'g..' asXMLURI) = 'http://a/b/c/g..' asXMLURI;
		assert: (baseURI / '..g' asXMLURI) = 'http://a/b/c/..g' asXMLURI;
		assert: (baseURI / './../g' asXMLURI) = 'http://a/b/g' asXMLURI;
		assert: (baseURI / './g/.' asXMLURI) = 'http://a/b/c/g/' asXMLURI;
		assert: (baseURI / 'g/./h' asXMLURI) = 'http://a/b/c/g/h' asXMLURI;
		assert: (baseURI / 'g/../h' asXMLURI) = 'http://a/b/c/h' asXMLURI;
		assert: (baseURI / 'g;x=1/./y' asXMLURI) = 'http://a/b/c/g;x=1/y' asXMLURI;
		assert: (baseURI / 'g;x=1/../y' asXMLURI) = 'http://a/b/c/y' asXMLURI;
		assert: (baseURI / 'g?y/./x' asXMLURI) = 'http://a/b/c/g?y/./x' asXMLURI;
		assert: (baseURI / 'g?y/../x' asXMLURI) = 'http://a/b/c/g?y/../x' asXMLURI;
		assert: (baseURI / 'g#s/./x' asXMLURI) = 'http://a/b/c/g#s/./x' asXMLURI;
		assert: (baseURI / 'g#s/../x' asXMLURI) = 'http://a/b/c/g#s/../x' asXMLURI;
		assert: (baseURI / 'http:g' asXMLURI) = 'http:g' asXMLURI! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/8/2016 12:23'!
testUserInfo
	self allURIStringsDo: [:each | | uri |
		uri := each asXMLURI.
		uri scheme = 'mailto'
			ifTrue: [self assert: uri userInfo = 'userName']
			ifFalse: [self assert: uri userInfo isEmpty]]! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/11/2016 08:16'!
testUserInfoNonPercentEncodedCharacters
	| userInfo uri |

	userInfo := self unreservedCharacters, self subDelimiterCharacters, ':'.
	uri := ('mailto:', userInfo, '@', self host) asXMLURI.
	self
		assert: uri scheme = 'mailto';
		assert: uri userInfo = userInfo;
		assert: uri host = self host.! !

!XMLURITest methodsFor: 'tests' stamp: 'monty 1/11/2016 08:16'!
testUserInfoPercentEncodedCharacters
	| prefix suffix uri |

	prefix := 'mailto:'.
	suffix := '@', self host.
	self asciiCharactersDo: [:each | | encodedURIString |
		encodedURIString := prefix, (XMLURI percentEncodeCharacter: each), suffix.
		uri := encodedURIString asXMLURI.
		self assert: uri userInfo = each asString.

		uri uriString: nil.
		(self isUnreservedCharacter: each)
			ifTrue: [self assert: uri printString = (prefix, each asString, suffix)]
			ifFalse: [self assert: uri printString = encodedURIString]].

	uri := (prefix, '+', suffix) asXMLURI.
	self assert: uri userInfo = '+'.! !

!XMLURITest methodsFor: 'accessing' stamp: 'monty 1/11/2016 08:12'!
unreservedCharacters
	^ self alphaNumericCharacters, '-._~'! !

XMLString subclass: #XMLFString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLFString commentStamp: '<historical>' prior: 0!
A dummy subclass of XMLString!

Object subclass: #SAXHandlerProxy
	instanceVariableNames: 'saxHandler messageRecorder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!SAXHandlerProxy commentStamp: 'monty 6/18/2015 00:54' prior: 0!
This class wraps a SAXHandler and records handler event messages using a message recorder. Non-handler messages are forwarded to the SAXHandler.!

SAXHandlerProxy subclass: #SAXHandlerPluggableProxy
	instanceVariableNames: 'handlerCallbacks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!SAXHandlerPluggableProxy commentStamp: 'monty 6/18/2015 00:43' prior: 0!
A pluggable version that allows blocks to be used to respond to handlers.!

!SAXHandlerPluggableProxy methodsFor: 'reflective operations' stamp: 'monty 11/26/2014 06:34'!
doesNotUnderstand: aMessage
	| result |

	result := super doesNotUnderstand: aMessage.

	^ (self handlerCallbacks
		at: aMessage selector
		ifAbsent: [^ result]) value: aMessage! !

!SAXHandlerPluggableProxy methodsFor: 'accessing' stamp: 'monty 11/26/2014 06:25'!
handle: aSelector with: aBlock
	self handlerCallbacks
		at: aSelector
		put: aBlock! !

!SAXHandlerPluggableProxy methodsFor: 'accessing' stamp: 'monty 11/26/2014 06:24'!
handlerCallbacks
	^ handlerCallbacks ifNil: [handlerCallbacks := Dictionary new]! !

!SAXHandlerProxy class methodsFor: 'instance creation' stamp: 'monty 11/26/2014 06:28'!
on: aStringOrStream
	^ self saxHandler: (SAXHandler on: aStringOrStream)! !

!SAXHandlerProxy class methodsFor: 'instance creation' stamp: 'monty 12/6/2013 22:23'!
saxHandler: aSaxHandler
	^ self new saxHandler: aSaxHandler! !

!SAXHandlerProxy methodsFor: 'accessing' stamp: 'monty 4/12/2015 08:22'!
allSentMessagesNamed: aName
	^ self messageRecorder messagesSelect: [:each | each selector = aName asSymbol]! !

!SAXHandlerProxy methodsFor: 'accessing' stamp: 'monty 4/12/2015 08:23'!
argumentsSentWith: aSelector
	self messageRecorder messagesDo: [:each |
		each selector = aSelector
			ifTrue: [^ each arguments]].
	^ #().! !

!SAXHandlerProxy methodsFor: 'reflective operations' stamp: 'monty 12/4/2015 08:28'!
doesNotUnderstand: aMessage
	| result |

	(self isHandlerMessage: aMessage)
		ifTrue: [self messageRecorder recordMessage: aMessage].

	^ (result := aMessage sendTo: saxHandler) == saxHandler
		ifTrue: [self]
		ifFalse: [result].! !

!SAXHandlerProxy methodsFor: 'testing' stamp: 'monty 12/4/2015 08:31'!
isHandlerMessage: aMessage
	| category |

	category := saxHandler class whichCategoryIncludesSelector: aMessage selector.

	^ category asString beginsWith: 'handling'.! !

!SAXHandlerProxy methodsFor: 'accessing' stamp: 'monty 4/12/2015 08:09'!
messageRecorder
	^ messageRecorder ifNil: [messageRecorder := XMLMessageRecorder new]! !

!SAXHandlerProxy methodsFor: 'parsing' stamp: 'monty 12/6/2013 22:38'!
parseDocument
	self parser parseDocument! !

!SAXHandlerProxy methodsFor: 'parsing' stamp: 'monty 12/6/2013 22:37'!
parser
	^ XMLParser
		driver: (self driverClass saxHandler: self)
		on: saxHandler input! !

!SAXHandlerProxy methodsFor: 'accessing' stamp: 'monty 12/6/2013 22:23'!
saxHandler: aSaxHandler
	saxHandler := aSaxHandler! !

!SAXHandlerProxy methodsFor: 'testing' stamp: 'monty 4/13/2015 02:45'!
wasSentSelectors: aSelectorArray
	^ (self messageRecorder messagesCollect: [:each | each selector]) asArray =
		aSelectorArray asArray! !

Object subclass: #XMLExceptionRaisingReadStream
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLExceptionRaisingReadStream commentStamp: 'monty 6/18/2015 00:58' prior: 0!
This is a read stream that raises errors if sent peek or next when atEnd like streams on other platforms . Used to test portability.!

!XMLExceptionRaisingReadStream class methodsFor: 'instance creation' stamp: 'monty 2/15/2016 23:29'!
on: aCollection
	^ self new on: aCollection! !

!XMLExceptionRaisingReadStream methodsFor: 'testing' stamp: 'monty 10/13/2014 21:56'!
atEnd
	^ stream atEnd! !

!XMLExceptionRaisingReadStream methodsFor: 'closing' stamp: 'monty 10/13/2014 22:00'!
close
	stream close! !

!XMLExceptionRaisingReadStream methodsFor: 'testing' stamp: 'monty 5/26/2015 01:46'!
isBinary
	^ false! !

!XMLExceptionRaisingReadStream methodsFor: 'testing' stamp: 'monty 10/13/2014 22:00'!
isStream
	^ true! !

!XMLExceptionRaisingReadStream methodsFor: 'accessing' stamp: 'monty 10/13/2014 21:57'!
next
	stream atEnd
		ifTrue: [Error signal: 'Cannot call next on this stream when atEnd'].
	^ stream next.! !

!XMLExceptionRaisingReadStream methodsFor: 'initialization' stamp: 'monty 2/15/2016 23:29'!
on: aCollection
	stream := aCollection readStream! !

!XMLExceptionRaisingReadStream methodsFor: 'accessing' stamp: 'monty 10/13/2014 22:01'!
peek
	stream atEnd
		ifTrue: [Error signal: 'Cannot call peek on this stream when atEnd'].
	^ stream peek.! !

Object subclass: #XMLMessageRecorder
	instanceVariableNames: 'messageQueue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLMessageRecorder commentStamp: 'monty 4/12/2015 08:28' prior: 0!
This class records sent messages in a queue.!

!XMLMessageRecorder methodsFor: 'accessing' stamp: 'monty 4/12/2015 07:56'!
clearMessages
	^ self messageQueue removeAll! !

!XMLMessageRecorder methodsFor: 'testing' stamp: 'monty 4/12/2015 07:56'!
hasMessages
	^ self messageQueue notEmpty! !

!XMLMessageRecorder methodsFor: 'private' stamp: 'monty 4/12/2015 07:53'!
messageQueue
	^ messageQueue ifNil: [messageQueue := OrderedCollection new]! !

!XMLMessageRecorder methodsFor: 'enumerating' stamp: 'monty 4/12/2015 08:24'!
messagesCollect: aBlock
	^ self messageQueue collect: aBlock! !

!XMLMessageRecorder methodsFor: 'enumerating' stamp: 'monty 4/12/2015 08:21'!
messagesDo: aBlock
	self messageQueue do: aBlock! !

!XMLMessageRecorder methodsFor: 'enumerating' stamp: 'monty 4/12/2015 08:22'!
messagesSelect: aBlock
	^ self messageQueue select: aBlock! !

!XMLMessageRecorder methodsFor: 'accessing' stamp: 'monty 4/12/2015 07:56'!
nextMessage
	^ self messageQueue removeFirst! !

!XMLMessageRecorder methodsFor: 'recording' stamp: 'monty 4/12/2015 08:05'!
recordMessage: aMessage
	self messageQueue addLast: aMessage! !

!XMLMessageRecorder methodsFor: 'recording' stamp: 'monty 4/12/2015 08:05'!
recordMessageReceiver: anObject selector: aSelector arguments: anArgumentCollection
	self recordMessage:
		(MessageSend
			receiver: anObject
			selector: aSelector
			arguments: anArgumentCollection)! !

Object subclass: #XMLRecordingListObserver
	instanceVariableNames: 'messageRecorder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLRecordingListObserver commentStamp: 'monty 4/12/2015 08:27' prior: 0!
This observer records notifications using an XMLMessageRecorder.!

XMLRecordingListObserver subclass: #XMLRecordingAttributeListObserver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLRecordingAttributeListObserver commentStamp: 'monty 1/17/2015 11:03' prior: 0!
This observer records notifications from XMLAttributeLists.!

!XMLRecordingAttributeListObserver methodsFor: 'notifying' stamp: 'monty 4/12/2015 08:02'!
addedAttribute: anAttribute
	self messageRecorder
		recordMessageReceiver: self
		selector: #addedAttribute:
		arguments: (Array with: anAttribute)! !

!XMLRecordingAttributeListObserver methodsFor: 'notifying' stamp: 'monty 4/12/2015 08:02'!
addingAttribute: anAttribute
	self messageRecorder
		recordMessageReceiver: self
		selector: #addingAttribute:
		arguments: (Array with: anAttribute)! !

!XMLRecordingAttributeListObserver methodsFor: 'notifying' stamp: 'monty 4/12/2015 08:03'!
removedAttribute: anAttribute
	self messageRecorder
		recordMessageReceiver: self
		selector: #removedAttribute:
		arguments: (Array with: anAttribute)! !

!XMLRecordingListObserver methodsFor: 'notifying' stamp: 'monty 4/12/2015 08:00'!
added: anObject to: aList
	self messageRecorder
		recordMessageReceiver: self
		selector: #added:to:
		arguments: (Array with: anObject with: aList)! !

!XMLRecordingListObserver methodsFor: 'notifying' stamp: 'monty 4/12/2015 08:01'!
adding: anObject to: aList
	self messageRecorder
		recordMessageReceiver: self
		selector: #adding:to:
		arguments: (Array with: anObject with: aList)! !

!XMLRecordingListObserver methodsFor: 'accesing' stamp: 'monty 4/12/2015 07:59'!
clearNotifications
	self messageRecorder clearMessages! !

!XMLRecordingListObserver methodsFor: 'testing' stamp: 'monty 4/12/2015 08:01'!
hasNotifications
	^ self messageRecorder hasMessages! !

!XMLRecordingListObserver methodsFor: 'accesing' stamp: 'monty 4/12/2015 07:59'!
messageRecorder
	^ messageRecorder ifNil: [messageRecorder := XMLMessageRecorder new]! !

!XMLRecordingListObserver methodsFor: 'accesing' stamp: 'monty 4/12/2015 07:59'!
nextNotification
	^ self messageRecorder nextMessage! !

!XMLRecordingListObserver methodsFor: 'notifying' stamp: 'monty 4/12/2015 08:02'!
removed: anObject from: aList
	self messageRecorder
		recordMessageReceiver: self
		selector: #removed:from:
		arguments: (Array with: anObject with: aList)! !

XMLRecordingListObserver subclass: #XMLRecordingNodeListObserver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-DOM'!

!XMLRecordingNodeListObserver commentStamp: 'monty 1/17/2015 10:45' prior: 0!
This observer records notifications from XMLNodeLists.!

!XMLRecordingNodeListObserver methodsFor: 'notifying' stamp: 'monty 4/12/2015 08:03'!
addedElement: anElement
	self messageRecorder
		recordMessageReceiver: self
		selector: #addedElement:
		arguments: (Array with: anElement)! !

!XMLRecordingNodeListObserver methodsFor: 'notifying' stamp: 'monty 4/12/2015 08:03'!
addedNode: aNode
	self messageRecorder
		recordMessageReceiver: self
		selector: #addedNode:
		arguments: (Array with: aNode)! !

!XMLRecordingNodeListObserver methodsFor: 'notifying' stamp: 'monty 4/12/2015 08:03'!
addingNode: aNode
	self messageRecorder
		recordMessageReceiver: self
		selector: #addingNode:
		arguments: (Array with: aNode)! !

!XMLRecordingNodeListObserver methodsFor: 'notifying' stamp: 'monty 4/12/2015 08:03'!
removedElement: anElement
	self messageRecorder
		recordMessageReceiver: self
		selector: #removedElement:
		arguments: (Array with: anElement)! !

!XMLRecordingNodeListObserver methodsFor: 'notifying' stamp: 'monty 4/12/2015 08:02'!
removedNode: aNode
	self messageRecorder
		recordMessageReceiver: self
		selector: #removedNode:
		arguments: (Array with: aNode)! !

XMLComment subclass: #XMLFComment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLFComment commentStamp: '<historical>' prior: 0!
A dummy subclass of XMLComment!

SAXHandler subclass: #SAXElementEventRecordingHandler
	instanceVariableNames: 'messageRecorder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!SAXElementEventRecordingHandler commentStamp: 'monty 6/18/2015 00:41' prior: 0!
This class records the final element events (startElement:attributes: and #endElement:) used when no others are defined.!

!SAXElementEventRecordingHandler methodsFor: 'handling - content' stamp: 'monty 4/12/2015 08:07'!
endElement: aQualifiedName
	self messageRecorder
		recordMessageReceiver: self
		selector: #endElement:
		arguments: (Array with: aQualifiedName)! !

!SAXElementEventRecordingHandler methodsFor: 'accessing' stamp: 'monty 4/12/2015 08:04'!
messageRecorder
	^ messageRecorder ifNil: [messageRecorder := XMLMessageRecorder new]! !

!SAXElementEventRecordingHandler methodsFor: 'handling - content' stamp: 'monty 4/12/2015 08:06'!
startElement: aQualifiedName attributes: aDictionary
	self messageRecorder
		recordMessageReceiver: self
		selector: #startElement:attributes:
		arguments: (Array with: aQualifiedName with: aDictionary)! !

SAXElementEventRecordingHandler subclass: #SAXElementPrefixURIEventRecordingHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!SAXElementPrefixURIEventRecordingHandler commentStamp: 'monty 6/18/2015 00:41' prior: 0!
This class records the second element events (#startElement:prefix:uri:attributes: and #endElement:prefix:uri:) used when the first aren't defined.!

!SAXElementPrefixURIEventRecordingHandler methodsFor: 'handling - content' stamp: 'monty 4/12/2015 08:08'!
endElement: aQualifiedName prefix: aPrefix uri: aURI
	self messageRecorder
		recordMessageReceiver: self
		selector: #endElement:prefix:uri:
		arguments:
			(Array
				with: aQualifiedName
				with: aPrefix
				with: aURI)! !

!SAXElementPrefixURIEventRecordingHandler methodsFor: 'handling - content' stamp: 'monty 4/12/2015 08:08'!
startElement: aQualifiedName prefix: aPrefix uri: aURI attributes: aDictionary
	self messageRecorder
		recordMessageReceiver: self
		selector: #startElement:prefix:uri:attributes:
		arguments:
			(Array
				with: aQualifiedName
				with: aPrefix
				with: aURI
				with: aDictionary)! !

SAXElementPrefixURIEventRecordingHandler subclass: #SAXElementPrefixURILocalNameEventRecordingHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!SAXElementPrefixURILocalNameEventRecordingHandler commentStamp: 'monty 6/18/2015 00:57' prior: 0!
This class records the first element events (#startElement:prefix:uri:attributes:localName: and #endElement:prefix:uri:localName:) sent before the others.!

!SAXElementPrefixURILocalNameEventRecordingHandler methodsFor: 'handling - content' stamp: 'monty 4/12/2015 10:09'!
endElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName
	self messageRecorder
		recordMessageReceiver: self
		selector: #endElement:prefix:uri:localName:
		arguments:
			(Array
				with: aQualifiedName
				with: aPrefix
				with: aURI
				with: aLocalName)! !

!SAXElementPrefixURILocalNameEventRecordingHandler methodsFor: 'handling - content' stamp: 'monty 4/13/2015 02:31'!
startElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName attributes: aDictionary
	self messageRecorder
		recordMessageReceiver: self
		selector: #startElement:prefix:uri:localName:attributes:
		arguments:
			(OrderedCollection new
				add: aQualifiedName;
				add: aPrefix;
				add: aURI;
				add: aLocalName;
				add: aDictionary;
				asArray)! !

XMLElement subclass: #XMLFDateElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLFDateElement commentStamp: '<historical>' prior: 0!
A dummy subclass of XMLElement!

XMLElement subclass: #XMLFElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLFElement commentStamp: '<historical>' prior: 0!
A dummy subclass of XMLElement!

XMLElement subclass: #XMLFRealNameElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLFRealNameElement commentStamp: '<historical>' prior: 0!
A dummy subclass of XMLElement!

XMLElement subclass: #XMLFSerializationFormatElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLFSerializationFormatElement commentStamp: '<historical>' prior: 0!
A dummy subclass of XMLElement!

XMLElement subclass: #XMLFTableElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLFTableElement commentStamp: '<historical>' prior: 0!
A dummy subclass of XMLElement!

XMLElement subclass: #XMLFUserElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLFUserElement commentStamp: '<historical>' prior: 0!
A dummy subclass of XMLElement!

XMLElement subclass: #XMLFUsernameElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLFUsernameElement commentStamp: '<historical>' prior: 0!
A dummy subclass of XMLElement!

XMLElement subclass: #XMLFWrongElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLFWrongElement commentStamp: '<historical>' prior: 0!
A dummy subclass of XMLElement!

XMLCData subclass: #XMLFCData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLFCData commentStamp: '<historical>' prior: 0!
A dummy subclass of XMLCData!

XMLPI subclass: #XMLFPI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Factories'!

!XMLFPI commentStamp: '<historical>' prior: 0!
A dummy subclass of XMLPI!
